Package: rstan
Type: Package
Title: R Interface to Stan
Version: 2.32.3
Authors@R: c(person("Jiqiang", "Guo", email = "guojq28@gmail.com", role = "aut"),
             person("Jonah", "Gabry", email = "jsg2201@columbia.edu", role = "aut"),
             person("Ben", "Goodrich", email = "benjamin.goodrich@columbia.edu", role = c("cre", "aut")),
             person("Andrew", "Johnson", email = "andrew.johnson@arjohnsonau.com", role = "aut"),
             person("Sebastian", "Weber", email = "sdw.post@waebers.de", role = "aut"),
             person("Hamada S.", "Badr", role = c("aut"),
                    email = "badr@jhu.edu",
                    comment = c(ORCID = "0000-0002-9808-2344")),
             person("Daniel", "Lee", role = "ctb"),
             person("Krzysztof", "Sakrejda", role = "ctb"),
             person("Modrak", "Martin", role = "ctb"),
             person("Trustees of", "Columbia University", role = "cph"),
             person("Oleg", "Sklyar", role = "cph", comment = "R/cxxfunplus.R"),
             person("The R", "Core Team", role = "cph", comment = "R/pairs.R, R/dynGet.R"),
             person("Jens", "Oehlschlaegel-Akiyoshi", role = "cph", comment = "R/pairs.R"),
             person("John", "Maddock", role = "cph", comment = "gamma.hpp"),
             person("Paul", "Bristow", role = "cph", comment = "gamma.hpp"),
             person("Nikhar", "Agrawal", role = "cph", comment = "gamma.hpp"),
             person("Christopher", "Kormanyos", role = "cph", comment = "gamma.hpp"),
             person("Bronder", "Steve", role = "ctb"))
Description: User-facing R functions are provided to parse, compile, test,
    estimate, and analyze Stan models by accessing the header-only Stan library
    provided by the 'StanHeaders' package. The Stan project develops a probabilistic
    programming language that implements full Bayesian statistical inference
    via Markov Chain Monte Carlo, rough Bayesian inference via 'variational'
    approximation, and (optionally penalized) maximum likelihood estimation via
    optimization. In all three cases, automatic differentiation is used to quickly
    and accurately evaluate gradients without burdening the user with the need to
    derive the partial derivatives.
License: GPL (>= 3)
NeedsCompilation: yes
Imports: methods, stats4, inline (>= 0.3.19), gridExtra (>= 2.3), Rcpp
        (>= 1.0.7), RcppParallel (>= 5.1.4), loo (>= 2.4.1), pkgbuild
        (>= 1.2.0), QuickJSR, ggplot2 (>= 3.3.5)
Depends: R (>= 3.4.0), StanHeaders (>= 2.26.28)
LinkingTo: Rcpp (>= 1.0.7), RcppEigen (>= 0.3.3.9.3), BH (>= 1.75.0-0),
        StanHeaders (>= 2.26.28), RcppParallel (>= 5.1.4)
Suggests: testthat (>= 3.0.4), parallel, KernSmooth, shinystan,
        bayesplot, rmarkdown, rstantools, rstudioapi, Matrix, knitr,
        coda, V8
URL: https://mc-stan.org/rstan/, https://discourse.mc-stan.org
BugReports: https://github.com/stan-dev/rstan/issues/
VignetteBuilder: knitr
SystemRequirements: GNU make, pandoc
Encoding: UTF-8
RoxygenNote: 7.1.2
Config/testthat/edition: 3
Packaged: 2023-10-15 02:09:37 UTC; ben
Author: Jiqiang Guo [aut],
  Jonah Gabry [aut],
  Ben Goodrich [cre, aut],
  Andrew Johnson [aut],
  Sebastian Weber [aut],
  Hamada S. Badr [aut] (<https://orcid.org/0000-0002-9808-2344>),
  Daniel Lee [ctb],
  Krzysztof Sakrejda [ctb],
  Modrak Martin [ctb],
  Trustees of Columbia University [cph],
  Oleg Sklyar [cph] (R/cxxfunplus.R),
  The R Core Team [cph] (R/pairs.R, R/dynGet.R),
  Jens Oehlschlaegel-Akiyoshi [cph] (R/pairs.R),
  John Maddock [cph] (gamma.hpp),
  Paul Bristow [cph] (gamma.hpp),
  Nikhar Agrawal [cph] (gamma.hpp),
  Christopher Kormanyos [cph] (gamma.hpp),
  Bronder Steve [ctb]
Maintainer: Ben Goodrich <benjamin.goodrich@columbia.edu>
Repository: CRAN
Date/Publication: 2023-10-15 05:00:02 UTC

useDynLib(rstan, .registration = TRUE)
# exportPattern("^[[:alpha:]]+")

importFrom("graphics", "hist", "lines", "mtext", "par", "points", "rect", "segments", "smoothScatter", "strheight", "strwidth", "text")
importFrom("grDevices", "devAskNewPage", "hcl", "rgb")
importFrom("stats", "acf", "aggregate", "density", "dnorm", "fft", "median", "optimHess", "pacf", "qbeta", "qnorm", "quantile", "reshape", "rnorm", "runif", "sd", "setNames", "var")
importFrom("utils", "apropos", "assignInMyNamespace", "capture.output", "download.file", "head", "menu", "object.size", "packageDate", "packageVersion", "read.csv", "relist", "select.list", "str", "tail", "unzip")
importFrom("stats4", summary, plot)
importFrom("inline", cxxfunction, getDynLib, getPlugin)
importFrom("pkgbuild", with_build_tools)
#importFrom("Rcpp", cpp_object_initializer)
importFrom("QuickJSR", JSContext)
import("methods")
import(Rcpp)
import(StanHeaders)

importFrom("gridExtra","arrangeGrob","grid.arrange")
import(ggplot2)

importFrom("loo", "loo", "loo_moment_match", "nlist")

export(
  sbc,
  stan_model,
  stanc,
  stanc_builder,
  stan_version,
  stan,
  stan_rdump,
  read_rdump,
  makeconf_path,
  sflist2stanfit,
  read_stan_csv,
  monitor,
  lookup,
  expose_stan_functions,
  stan_demo,
  rstan_options,
  As.mcmc.list,
  set_cppo,
  stan_plot,
  stan_trace,
  stan_hist,
  stan_dens,
  stan_scat,
  stan_ac,
  stan_diag,
  stan_par,
  stan_rhat,
  stan_ess,
  stan_mcse,
  rstan_gg_options,
  rstan_ggtheme_options,
  quietgg,
  extract_sparse_parts,
  rstan.package.skeleton,
  cpp_object_initializer,
# get_rstan.options
  check_hmc_diagnostics,
  check_divergences,
  check_treedepth,
  check_energy,
  get_divergent_iterations,
  get_max_treedepth_iterations,
  get_num_leapfrog_per_iteration,
  get_num_divergent,
  get_num_max_treedepth,
  get_bfmi,
  get_low_bfmi_chains,
  get_rng,
  get_stream,
  RNG, OUT,
  Rhat, ess_bulk, ess_tail,
  loo,
  loo_moment_match,
  nlist
)


exportClasses(
  stanmodel, stanfit
)
exportMethods(
#   print, plot,
#   extract,
  optimizing, vb, gqs,
  get_cppcode, get_cxxflags, # for stanmodel
  show, sampling, summary, extract,
  traceplot, plot, get_stancode, get_inits, get_seed, get_cppo_mode,
  log_prob, grad_log_prob,
  unconstrain_pars, constrain_pars, get_num_upars,
  get_seeds,
  get_adaptation_info,
  get_sampler_params,
  get_logposterior,
  get_posterior_mean,
  get_elapsed_time,
  get_stanmodel
)
S3method(print, stanfit)
S3method(as.array, stanfit)
S3method(as.matrix, stanfit)
S3method(as.data.frame, stanfit)
S3method(is.array, stanfit)
S3method(dim, stanfit)
S3method(dimnames, stanfit)
S3method(names, stanfit)
S3method(`names<-`, stanfit)
S3method(pairs, stanfit)
S3method(loo, stanfit)
S3method(loo_moment_match, stanfit)
S3method(print, sbc)
S3method(plot, sbc)
S3method(`[`, simsummary)

## ----setup, include = FALSE---------------------------------------------------
library(rstan)
knitr::opts_chunk$set(eval = .Platform$OS.type != "windows")

## ----eval = FALSE-------------------------------------------------------------
#  output <- sbc(beta_binomial, data = list(N = 10, a = 1, b = 1), M = 500, refresh = 0)

## ----include = FALSE----------------------------------------------------------
# This fakes what would happen if we actually took the time to run Stan.
N <- 10
M <- 500
pars_ <- rbeta(M, 1, 1)
y_ <- matrix(rbinom(pars_, size = N, prob = pars_), ncol = M)
post_ <- matrix(rbeta(M * 1000L, 1 + y_, 1 + N - y_), ncol = M)
ranks_ <- lapply(1:M, FUN = function(m) {
  matrix(post_[ , m] > pars_[m], ncol = 1, 
         dimnames = list(NULL, "pi"))
})
log_lik <- t(sapply(1:M, FUN = function(m) {
  c(dbinom(rep(1, y_[m]), size = 1, prob = pars_[m], log = TRUE),
    dbinom(rep(1, N - y_[m]), size = 1, prob = pars_[m], log = TRUE))
}))
sampler_params <- array(0, dim = c(1000, 6, M))
colnames(sampler_params) <- c("accept_stat__", "stepsize__", "treedepth__",   
                              "n_leapfrog__",  "divergent__",   "energy__")
output <- list(ranks = ranks_, Y = y_, pars = pars_, 
               log_lik = log_lik, sampler_params = sampler_params)
class(output) <- "sbc"

## -----------------------------------------------------------------------------
print(output)
plot(output, bins = 10) # it is best to specify the bins argument yourself

## ----setup, include=FALSE-----------------------------------------------------
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE, eval = FALSE, error = TRUE
)

## ----eval = TRUE--------------------------------------------------------------
mc <- 
'
functions { int fib(int n); }
model {} // use the fib() function somehow
'
try(stan_model(model_code = mc, model_name = "parser_error"), silent = TRUE)

## -----------------------------------------------------------------------------
#  stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
#             includes = paste0('\n#include "',
#                               file.path(getwd(), 'fib.hpp'), '"\n'))

## ----echo = FALSE, eval = TRUE, comment = NA----------------------------------
cat(readLines(system.file("include", "src", "stan", "model", "model_header.hpp", 
                          package = "StanHeaders")), sep = "\n")

## ----eval = TRUE--------------------------------------------------------------
mc <- 
'
functions { real sinc(real x); }
transformed data { real sinc_pi = sinc(pi()); }
'
stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
           includes = paste0('\n#include "', 
                             file.path(getwd(), 'sinc.hpp'), '"\n'))

## ----echo = FALSE, eval = TRUE, comment=""------------------------------------
cat(readLines("sinc.hpp"), sep = "\n")

## ----eval = TRUE--------------------------------------------------------------
try(readLines(stanc(model_code = mc, allow_undefined = TRUE)$cppcode))

library(rstan) 


inc <- '
#include <iostream>
#include <Rcpp.h>
#include <R.h>
#include <Rinternals.h>

#include <rstan/io/r_ostream.hpp> 

RcppExport SEXP test_r_ostream(SEXP params) {
  std::string str = Rcpp::as<std::string>(params);
  rstan::io::rcout << "[rcout] " << str << std::endl; 
  rstan::io::rcerr << "[rcerr] " << str; // << std::endl;
  rstan::io::rcerr.flush();
  rstan::io::rcerr << std::endl;
  return Rcpp::wrap(true);
} 

RCPP_MODULE(rstantest){
  using namespace Rcpp;
  function("test_r_ostream", &test_r_ostream); 
}
' 

fx <- cxxfunction(signature(), body = 'return R_NilValue;\n',
                  includes = inc,
                  plugin = "rstan", verbose = TRUE) 

mod <- Module("rstantest", getDynLib(fx)) 
mod$test_r_ostream('hello1')
mod$test_r_ostream('hello2')
mod$test_r_ostream('hello3')
mod$test_r_ostream('hello4')
mod$test_r_ostream('hello5')

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
# Copyright (C) 2005-2010 Oleg Sklyar
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

cxxfun_from_dll <- function(sig, code, DLL, check_dll = TRUE) {
  # Create function objects from dll (most of the code are copied from
  # cxxfunction in package inline).
  #
  # Args:
  #  sig: a list of function signatures
  #  DLL: object of class "DLLInfo"
  #  check_dll: check if the dll is loaded: When it is not
  #    loaded, the function call might result in a segfault.

  f <- DLL[['name']]
  if (check_dll) {
    dlls <- getLoadedDLLs()
    if (!f %in% names(dlls))
      stop(paste("dso ", DLL[['path']], " is not loaded", sep = ''))
  }

  res <- vector("list", length(sig))
  names(res) <- names(sig)
  res <- new("CFuncList", res)

  for(i in seq_along(sig)) {
    res[[i]] <- new("CFunc", code = code)
    fn <- function(arg) { NULL }

    ## Modify the function formals to give the right argument list
    args <- formals(fn)[rep(1, length(sig[[i]]))]
    names(args) <- names(sig[[i]])
    formals(fn) <- args

    ## create .Call function call that will be added to 'fn'
    body <- quote(CALL_PLACEHOLDER(EXTERNALNAME, ARG))[c(1:2, rep(3, length(sig[[i]])))]
    for (j in seq_along(sig[[i]])) body[[j + 2]] <- as.name(names(sig[[i]])[j])

    body[[1L]] <- .Call
    body[[2L]] <- getNativeSymbolInfo(names(sig)[[i]], DLL)$address
    ## update the body of 'fn'
    body(fn) <- body
    ## set fn as THE function in CFunc of res[[i]]
    res[[i]]@.Data <- fn
  }
  ## clear the environment
  rm(j)
  convention <- ".Call"
  if (identical(length(sig), 1L)) res[[1L]] else res
}

cxxfun_from_dso_bin <- function(dso) {
  # Create function objects from dll (most of the code are copied from
  # cxxfunction in package inline).
  #
  # Args:
  #  dso: object of class cxxdso
  #
  # Note: we are assuming that the dso is not loaded so
  #   we create the dso file from the raw vector
  #   and then loaded the dso. .

  sig <- dso@sig
  code <- dso@.CXXDSOMISC$cxxfun@code
  tfile <- tempfile()
  f <- basename(tfile)
  libLFile <- paste(tfile, ".", filename_ext(dso@.CXXDSOMISC$dso_last_path), sep = '')
  # write the raw vector containing the dso file to temporary file
  writeBin(dso@.CXXDSOMISC$dso_bin, libLFile)
  cleanup <- function(env) {
    if (file.exists(libLFile) && f %in% names(getLoadedDLLs()))
      dyn.unload(libLFile)
    unlink(libLFile)
  }
#  reg.finalizer(environment(), cleanup, onexit = FALSE)
  DLL <- dyn.load(libLFile)
  assign('dso_last_path', libLFile, dso@.CXXDSOMISC)
  res <- vector("list", length(sig))
  names(res) <- names(sig)
  res <- new("CFuncList", res)
  for(i in seq_along(sig)) {
    res[[i]] <- new("CFunc", code = code)
    fn <- function(arg) { NULL }

    ## Modify the function formals to give the right argument list
    args <- formals(fn)[rep(1, length(sig[[i]]))]
    names(args) <- names(sig[[i]])
    formals(fn) <- args

    ## create .Call function call that will be added to 'fn'
    body <- quote(CALL_PLACEHOLDER(EXTERNALNAME, ARG))[c(1:2, rep(3, length(sig[[i]])))]
    for (j in seq_along(sig[[i]])) body[[j + 2]] <- as.name(names(sig[[i]])[j])

    body[[1L]] <- .Call
    body[[2L]] <- getNativeSymbolInfo(names(sig)[[i]], DLL)$address
    ## update the body of 'fn'
    body(fn) <- body
    ## set fn as THE function in CFunc of res[[i]]
    res[[i]]@.Data <- fn
  }
  ## clear the environment
  rm(j)
  rm(tfile)
  convention <- ".Call"
  if (identical(length(sig), 1L)) res[[1L]] else res
}


dso_path <- function(fx) {
  # find the path for the dynamic shared objects associated with
  # the returned object from cxxfunction
  #
  # Args:
  #   fx: returned object from cxxfunction in package inline
  dllinfo <- getDynLib(fx)
  dllinfo[['path']]
}

read_dso <- function(path) {
  n <- file.info(path)$size
  readBin(path, what = 'raw', n = n)
}

cxxfunctionplus <- function(sig = character(), body = character(),
                            plugin = "default", includes = "",
                            settings = getPlugin(plugin),
                            save_dso = FALSE, module_name = "MODULE",
                            ..., verbose = FALSE) {
  R_version <- with(R.version, paste(major, minor, sep = "."))
  WINDOWS <- .Platform$OS.type == "windows"
  if (WINDOWS && R.version$major < 4) {
    stop("cxxfunctionplus requires R >= 4.0 on Windows to use C++17")
  } else {
    has_USE_CXX17 <- Sys.getenv("USE_CXX17") != ""
    Sys.setenv(USE_CXX17 = 1)
    if (!has_USE_CXX17) on.exit(Sys.unsetenv("USE_CXX17"))
  }
  if (rstan_options("required"))
    pkgbuild::has_build_tools(debug = FALSE) ||
    pkgbuild::has_build_tools(debug = TRUE)

  # compiling with -march=native on windows can cause segfaults
  if (WINDOWS) {
    has_march = .warn_march_makevars()
    if (has_march) {
      user_makevar = Sys.getenv("R_MAKEVARS_USER")
      Sys.setenv(R_MAKEVARS_USER = NULL)
      on.exit(Sys.setenv(R_MAKEVARS_USER = user_makevar))
    }
  }
  if (!isTRUE(verbose)) {
    tf <- tempfile(fileext = ".warn")
    zz <- file(tf, open = "wt")
    sink(zz, type = "output")
    on.exit(close(zz), add = TRUE)
    on.exit(sink(type = "output"), add = TRUE)
  }
  fx <- pkgbuild::with_build_tools(
    cxxfunction(sig = sig, body = body, plugin = plugin, includes = includes,
                settings = settings, ..., verbose = verbose),
    required = rstan_options("required") &&
    # workaround for packages with src/install.libs.R
      !identical(Sys.getenv("WINDOWS"), "TRUE") &&
      !identical(Sys.getenv("R_PACKAGE_SOURCE"), "") )
  if (!isTRUE(verbose)) {
    sink(type = "output")
    close(zz)
    try(file.remove(tf), silent = TRUE)
    on.exit(NULL)
  }
  dso_last_path <- dso_path(fx)
  if (grepl("^darwin", R.version$os) && grepl("clang", get_CXX(FALSE))) {
    CLANG_DIR = tail(n = 1, grep("clang[456789]", value = TRUE,
                                 x = list.dirs("/usr/local", recursive = FALSE)))
    Rv <- R.version
    GOOD <- file.path("/Library", "Frameworks", "R.framework", "Versions",
                      paste(Rv$major, substr(Rv$minor, 1, 1), sep = "."),
                      "Resources", "lib", "libc++.1.dylib")
    if (length(CLANG_DIR) == 1L && file.exists(GOOD)) {
      cmd <- paste("install_name_tool", "-change", CLANG_DIR, GOOD, dso_last_path)
      system(cmd)
      dyn.unload(dso_last_path)
      dyn.load(dso_last_path)
    }
  }
  dso_bin <- if (save_dso) read_dso(dso_last_path) else raw(0)
  dso_filename <- sub("\\.[^.]*$", "", basename(dso_last_path))
  if (!is.list(sig))  {
    sig <- list(sig)
    names(sig) <- dso_filename
  }
  cxxflags <- try(get_makefile_flags("CXXFLAGS"))
  if (!is.character(cxxflags)) cxxflags <- NA_character_
  dso <- new('cxxdso', sig = sig, dso_saved = save_dso,
             dso_filename = dso_filename,
             modulename = module_name,
             system = R.version$system,
             cxxflags = cxxflags,
             .CXXDSOMISC = new.env(parent = emptyenv()))
  assign("cxxfun", fx, envir = dso@.CXXDSOMISC)
  assign("dso_last_path", dso_last_path, envir = dso@.CXXDSOMISC)
  assign("dso_bin", dso_bin, envir = dso@.CXXDSOMISC)
  if (!is.null(module_name) && module_name != '')
    assign("module", Module(module_name, getDynLib(fx)), envir = dso@.CXXDSOMISC)
  return(dso)
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

##
stan_model <- function(file,
                       model_name = "anon_model",
                       model_code = '',
                       stanc_ret = NULL,
                       boost_lib = NULL,
                       eigen_lib = NULL,
                       save_dso = TRUE,
                       verbose = FALSE,
                       auto_write = rstan_options("auto_write"),
                       obfuscate_model_name = TRUE,
                       allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
                       allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
                       standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
                       use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
                       warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
                       warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE)),
                       includes = NULL,
                       isystem = c(if (!missing(file)) dirname(file), getwd())) {
  if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
    Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
  }

  # Construct a stan model from stan code
  #
  # Args:
  #   file: the file that has the model in Stan model language.
  #   model_name: a character for naming the model.
  #   stanc_ret: An alternative way to specify the model
  #     by using returned results from stanc.
  #   model_code: if file is not specified, we can used
  #     a character to specify the model.

  if (is.null(stanc_ret)) {
    model_name2 <- deparse(substitute(model_code))
    if (is.null(attr(model_code, "model_name2")))
      attr(model_code, "model_name2") <- model_name2
    if (missing(model_name)) model_name <- NULL

    if(missing(file)) {
      tf <- tempfile()
      writeLines(model_code, con = tf)
      file <- file.path(dirname(tf), paste0(tools::md5sum(tf), ".stan"))
      if(!file.exists(file)) file.rename(from = tf, to = file)
      else file.remove(tf)
    }
    else file <- normalizePath(file)

    stanc_ret <- stanc(file = file, model_code = model_code,
                       model_name = model_name, verbose = verbose,
                       obfuscate_model_name = obfuscate_model_name,
                       allow_undefined = allow_undefined,
                       allow_optimizations = allow_optimizations,
                       standalone_functions = standalone_functions,
                       use_opencl = use_opencl,
                       warn_pedantic = warn_pedantic,
                       warn_uninitialized = warn_uninitialized,
                       isystem = isystem)

    # find possibly identical stanmodels
    model_re <- "(^[[:alnum:]]{2,}.*$)|(^[A-E,G-S,U-Z,a-z].*$)|(^[F,T].+)"
    if(!is.null(model_name))
      if(!grepl(model_re, model_name))
        stop("model name must match ", model_re)
    S4_objects <- apropos(model_re, mode="S4", ignore.case = FALSE)
    if (length(S4_objects) > 0) {
      e <- environment()
      stanfits <- sapply(mget(S4_objects, envir = e, inherits = TRUE),
                         FUN = is, class2 = "stanfit")
      stanmodels <- sapply(mget(S4_objects, envir = e, inherits = TRUE),
                           FUN = is, class2 = "stanmodel")
      if (any(stanfits)) for (i in names(which(stanfits))) {
        obj <- get_stanmodel(get(i, envir = e, inherits = TRUE))
        if (identical(obj@model_code[1], stanc_ret$model_code[1])) return(obj)
      }
      if (any(stanmodels)) for (i in names(which(stanmodels))) {
        obj <- get(i, envir = e, inherits = TRUE)
        if (identical(obj@model_code[1], stanc_ret$model_code[1])) return(obj)
      }
    }

    mtime <- file.info(file)$mtime
    file.rds <- gsub("stan$", "rds", file)
    md5 <- tools::md5sum(file)
    if (!file.exists(file.rds)) {
      file.rds <- file.path(tempdir(), paste0(md5, ".rds"))
    }
    if(!file.exists(file.rds) ||
       (mtime.rds <- file.info(file.rds)$mtime) <
       as.POSIXct(packageDate("rstan")) ||
       !is(obj <- readRDS(file.rds), "stanmodel") ||
       !is_sm_valid(obj) ||
       (!identical(stanc_ret$model_code, obj@model_code) && is.null(
        message("hash mismatch so recompiling; make sure Stan code ends with a blank line"))) ||
       avoid_crash(obj@dso@.CXXDSOMISC$module) && is.null(
        message("recompiling to avoid crashing R session"))) {

        # do nothing
    }
    else return(invisible(obj))
  }
  if (!is.list(stanc_ret)) {
    stop("stanc_ret needs to be the returned object from stanc.")
  }
  m <- match(c("cppcode", "model_name", "status"), names(stanc_ret))
  if (any(is.na(m))) {
    stop("stanc_ret does not have element `cppcode', `model_name', and `status'")
  } else {
    if (!stanc_ret$status)
      stop("stanc_ret is not a successfully returned list from stanc")
  }

  # check for compilers
  if (.Platform$OS.type != "windows") {
    CXX <- get_CXX()
    if (!is.null(attr(CXX, "status")) || nchar(CXX) == 0) {
      WIKI <- "https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started"
      warning(paste("C++ compiler not found on system. If absent, see\n", WIKI))
    }
    else if (grepl("69", CXX, fixed = TRUE))
      warning("You may need to launch Xcode once to accept its license")
  } else CXX <- "g++"

  model_cppname <- stanc_ret$model_cppname
  model_name <- stanc_ret$model_name
  model_code <- stanc_ret$model_code
  model_cppcode <- stanc_ret$cppcode
  model_cppcode <- paste("#ifndef MODELS_HPP",
                         "#define MODELS_HPP",
                         "#define STAN__SERVICES__COMMAND_HPP",
                         "#include <rstan/rstaninc.hpp>",
                         model_cppcode,
                         "#endif",
                         sep = '\n')

  inc <- paste("#include <Rcpp.h>\n",
               "using namespace Rcpp;\n",
               if(is.null(includes)) model_cppcode else
                 sub("(class[[:space:]]+[A-Za-z_][A-Za-z0-9_]*[[:space:]]*)",
                     paste(includes, "\\1"), model_cppcode),
               "\n",
               get_Rcpp_module_def_code(model_cppname),
               sep = '')

  if (verbose && interactive())
    cat("COMPILING THE C++ CODE FOR MODEL '", model_name, "' NOW.\n", sep = '')
  if (verbose) cat(system_info(), "\n")
  if (!is.null(boost_lib)) {
    old.boost_lib <- rstan_options(boost_lib = boost_lib)
    on.exit(rstan_options(boost_lib = old.boost_lib))
  }
  if (!file.exists(rstan_options("boost_lib")))
    stop("Boost not found; call install.packages('BH')")

  if (!is.null(eigen_lib)) {
    old.eigen_lib <- rstan_options(eigen_lib = eigen_lib)
    on.exit(rstan_options(eigen_lib = old.eigen_lib), add = TRUE)
  }
  if (!file.exists(rstan_options("eigen_lib")))
    stop("Eigen not found; call install.packages('RcppEigen')")

  dso <- cxxfunctionplus(signature(), body = paste(" return Rcpp::wrap(\"", model_name, "\");", sep = ''),
                         includes = inc, plugin = "rstan", save_dso = save_dso | auto_write,
                         module_name = paste('stan_fit4', model_cppname, '_mod', sep = ''),
                         verbose = verbose)
  # bod <- paste0("return Rcpp::XPtr<stan_model>(new stan_model(",
  #               "Rcpp::as<rstan::io::rlist_ref_var_context>(context__), ",
  #               "Rcpp::as<unsigned int>(seed), ",
  #               "&Rcpp::Rcout), true);")
  # dso <- cxxfunctionplus(sig = signature(context__ = "list", seed = "integer"),
  #                        body = bod, plugin = "rstan", includes = inc,
  #                        save_dso = save_dso | auto_write, verbose = verbose,
  #                        module_name = paste('stan_fit4', model_cppname, '_mod', sep = ''))

  obj <- new("stanmodel", model_name = model_name,
             model_code = model_code,
             dso = dso, # keep a reference to dso
             mk_cppmodule = mk_cppmodule,  # mk_cppmodule function is defined in file stanmodel-class.R
             model_cpp = list(model_cppname = model_cppname,
                              model_cppcode = model_cppcode))

  if(missing(file) || (file.access(dirname(file), mode = 2) != 0) || !isTRUE(auto_write)) {
    tf <- tempfile()
    writeLines(model_code, con = tf)
    file <- file.path(tempdir(), paste0(tools::md5sum(tf), ".stan"))
    if(!file.exists(file)) file.rename(from = tf, to = file)
    else file.remove(tf)
    saveRDS(obj, file = gsub("stan$", "rds", file))
  }
  else if(isTRUE(auto_write)) {
    file <- gsub("stan$", "rds", file)
    if (file.exists(file)) {
      rds <- try(readRDS(file), silent = TRUE)
      if (!is(rds, "stanmodel"))
        warning(rds, " exists but is not a 'stanmodel' so not overwriting")
      else saveRDS(obj, file = file)
    }
    else saveRDS(obj, file = file)
  }

  invisible(obj)
  ## We keep a reference to *dso* above to avoid dso to be
  ## deleted by R's garbage collection. Note that if dso
  ## is freed, we can lose the compiled shared object, which
  ## can cause segfault later.
}

is_sm_valid <- function(sm) {
  # Test if a stan model (compiled object) is still valid.
  # It could become invalid when the user do not specify
  # save_dso when calling stan_model. So when the user
  # use the model created in another R session, the dso
  # is lost.
  #
  # This function works only for a stanmodel that is
  # created from function stan_model in package rstan.
  #
  # Args:
  #   sm: the stanmodel object
  #
  if (isS4(sm@dso) && is_dso_loaded(sm@dso)) return(TRUE)
  sm@dso@dso_saved && identical(sm@dso@system, R.version$system)
}

##
##
##

stan <- function(file, model_name = "anon_model",
                 model_code = '',
                 fit = NA,
                 data = list(),
                 pars = NA,
                 chains = 4, iter = 2000,
                 warmup = floor(iter / 2),
                 thin = 1,
                 init = "random",
                 seed = sample.int(.Machine$integer.max, 1),
                 algorithm = c("NUTS", "HMC", "Fixed_param"), #, "Metropolis"),
                 control = NULL,
                 sample_file = NULL, # the file to which the samples are written
                 diagnostic_file = NULL, # the file to which diagnostics are written
                 save_dso = TRUE,
                 verbose = FALSE, include = TRUE,
                 cores = getOption("mc.cores", 1L),
                 open_progress = interactive() && !isatty(stdout()) &&
                                 !identical(Sys.getenv("RSTUDIO"), "1"),
                 ...,
                 boost_lib = NULL,
                 eigen_lib = NULL) {
  # Return a fitted model (stanfit object)  from a stan model, data, etc.
  # A wrap of method stan_model and sampling of class stanmodel.
  #
  # Args:
  #
  # Returns:
  #   A S4 class stanfit object
  if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
    Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
  }

  dot_arg_names <- names(list(...))
  is_arg_recognizable(dot_arg_names,
                      c("chain_id", "init_r", "test_grad",
                        "append_samples", "refresh", "control",
                        "enable_random_init", "save_warmup",
                        "obfuscate_model_name"),
                      pre_msg = "passing unknown arguments: ",
                      call. = FALSE)

  if (!is.list(data) && !is.environment(data) && !is.character(data))
    stop("'data' must be a list, environment, or character vector")

  if (is(fit, "stanfit")) sm <- get_stanmodel(fit)
  else {
    attr(model_code, "model_name2") <- deparse(substitute(model_code))
    if (missing(model_name)) model_name <- NULL
    sm <- stan_model(file, model_name = model_name,
                     model_code = model_code, stanc_ret = NULL,
                     boost_lib = boost_lib, eigen_lib = eigen_lib,
                     save_dso = save_dso, verbose = verbose)
  }

  if (is.null(sample_file))  sample_file <- NA
  if (is.null(diagnostic_file))  diagnostic_file <- NA

  sampling(sm, data, pars, chains, iter, warmup, thin, seed, init,
           check_data = TRUE, sample_file = sample_file,
           diagnostic_file = diagnostic_file,
           verbose = verbose, algorithm = match.arg(algorithm),
           control = control, check_unknown_args = FALSE,
           cores = cores, open_progress = open_progress, include = include, ...)
}

test_that("Running stan() in parallel works",  {
  skip("Backwards compatibility + stan error")

  code <- "
  data {
    int N;
  } parameters {
    array[N] real y;
  }  model {
    y ~ normal(0, 1);
  }
  "
  N <- 3
  # D
  fit <- stan(model_code = code,
              data = "N",
              chains = 0,
              cores = 2,
              open_progress = FALSE)
  expect_true(fit@mode != 0)

  fit2 <- stan(fit = fit,
               data = "N",
               cores = 2,
               chains = 4,
               open_progress = FALSE)
  expect_true(fit2@mode == 0)
  expect_equal(fit2@sim$chains, 4L)

  # This should fail because check_data is false so that integer N cannot be
  # found.
  fit3 <- sampling(fit@stanmodel,
                   data = list(N = 2),
                   chains = 2,
                   check_data = FALSE,
                   iter = 40,
                   cores = 2,
                   open_progress = FALSE)
  expect_true(fit3@mode != 0)
})

test_that("stan() arguments works", {
  skip("Backwards compatibility")

  csv_fname <- "tsfa.csv"
  csv_fname2 <- "tsfa2.csv"
  diag_fname2 <- "diag2.csv"
  model_code <- "
    parameters {
      array[2] real y;
      real alpha;
    }
    transformed parameters {
      array[2, 2] real y2;
      y2[1, 1] = y[1];
      y2[1, 2] = -y[1];
      y2[2, 1] = -y[2];
      y2[2, 2] = y[2];
    }
    model {
      y ~ normal(0, 1);
    }
  "
  fit <- stan(model_code = model_code,
              iter = 100, chains = 1, thin = 3,
              sample_file = csv_fname)
  fit2 <- stan(fit = fit, iter = 1001, chains = 3, thin = 2,
               sample_file = csv_fname2, diagnostic_file = diag_fname2)
  fit3 <- stan(fit = fit, iter = 1001, chains = 3, thin = 2,
               control = list())
  fit4 <- stan(fit = fit, iter = 1001, chains = 3, thin = 2,
               control = list(adapt_gamma = .7))
  y_ii <- rnorm(2)
  fit5 <- stan(fit = fit, init = list(list(y = y_ii)), chains = 1, iter = 100)
  expect_equal(attr(fit2@sim$samples[[1]], "args")$iter, 1001)
  expect_equal(attr(fit2@sim$samples[[1]], "args")$control$adapt_delta, 0.8)
  expect_equal(attr(fit3@sim$samples[[1]], "args")$control$adapt_delta, 0.8)
  expect_equal(attr(fit4@sim$samples[[1]], "args")$control$adapt_gamma, 0.7)
  expect_equal(attr(fit5@sim$samples[[1]], "inits")[1:2], y_ii)

  fit6 <- stan(fit = fit, pars = "y", chains = 1, iter = 100)
  expect_equal(fit6@sim$pars_oi, c("y", "lp__"))

  fit6 <- stan(fit = fit, pars = c("y", "y2"), include = FALSE, chains = 1,
               iter = 100)
  expect_equal(fit6@sim$pars_oi, c("alpha", "lp__"))

  # Just test if we can specify refresh.
  fit7 <- stan(fit = fit, refresh = 10, chains = 1, iter = 100)
  fit8 <- stan(fit = fit, refresh = -1)

  csv_fname <- "tsfa*.csv"
  csv_fname2 <- "tsfa2*.csv"
  diag_fname2 <- "diag2*.csv"
  on.exit(system(paste('rm -rf ', csv_fname)), add = TRUE)
  on.exit(system(paste('rm -rf ', csv_fname2)), add = TRUE)
  on.exit(system(paste('rm -rf ', diag_fname2)), add = TRUE)
})

test_that("partial_inits works", {
  skip("Backwards compatibility")

  stan_model_code <- "
    data {
      int<lower=0> N;
      array[N] real y;
    }

    parameters {
      real mu;
      real alpha;
      simplex[3] delta;
      real eta;
      real<lower=0> sigma;
    }

    transformed parameters {
      real beta;
      beta = -alpha;
    }

    model {
      mu ~ normal(0, 10);
      y ~ normal(mu, 1);
      alpha ~ normal(0, 1);
      eta ~ normal(0, 1);
      sigma ~ exponential(2);
    }

    generated quantities {
      real gamma;
      gamma = mu + alpha;
    }
  "

  rr <- stan_model(
    model_code = stan_model_code,
    model_name = "m1",
    verbose = FALSE
  )

  y <- rnorm(20)
  dat <- list(N = length(y), y = y)
  f <- sampling(rr, data = dat, init = 0, iter = 10)
  i1 <- get_inits(f)
  # Check 1-20.
  for (i in seq_along(i1)) {
    expect_equal(i1[[i]]$mu, 0)
    expect_equal(i1[[i]]$alpha, 0)
    expect_equal(i1[[i]]$eta, 0)
    expect_equal(i1[[i]]$sigma, 1)
    expect_equal(i1[[i]]$delta, as.array(rep(1, 3) / 3))
  }

  f3 <- sampling(
    rr,
    data = dat,
    iter = 10,
    chains = 1,
    init = list(list(mu = 2)), seed = 3, thin = 1,
    enable_random_init = TRUE
  )
  i3 <- get_inits(f3)
  # Check 21-29.
  for (i in seq_along(i3)) {
    expect_equal(i3[[i]]$mu, 2)
    # This assumes that it is not possible that the following is initialized to
    # the expect randomly.
    expect_false(identical(i3[[i]]$alpha, 0))
    expect_false(identical(i3[[i]]$alpha, 1))
    expect_false(identical(i3[[i]]$eta, 0))
    expect_false(identical(i3[[i]]$eta, 1))
    expect_false(identical(i3[[i]]$sigma, 1))
    expect_false(identical(i3[[i]]$sigma, 2))
    expect_false(identical(i3[[i]]$delta, as.array(rep(1, 3) / 3)))
    expect_false(identical(i3[[i]]$delta, as.array(c(.1, .2, .7))))
  }

  # Allow partial inits, but mu is not specified correctly.
  f4 <- sampling(rr,
    data = dat, iter = 10, chains = 1,
    init = list(list(mu = rep(2, 2))), seed = 3, thin = 1,
    enable_random_init = TRUE
  )
  emsg <- geterrmessage()

  # Check 30-31.
  # checkTrue(grepl('.*mismatch.*', emsg))
  # checkTrue(grepl('.*().*(2).*', emsg))
  expect_true(grepl("Error", emsg))

  f5 <- sampling(rr,
    data = dat, iter = 10, chains = 1,
    seed = 3, thin = 1, init_r = 2,
    enable_random_init = TRUE
  )

  f6 <- sampling(rr,
    data = dat, iter = 10, chains = 1,
    seed = 3, thin = 1, init_r = 4,
    enable_random_init = TRUE
  )

  f7 <- sampling(rr,
    data = dat, iter = 10, chains = 1,
    seed = 4, thin = 1, init_r = 4,
    enable_random_init = TRUE
  )

  i5 <- get_inits(f5)[[1]]
  i6 <- get_inits(f6)[[1]]
  i7 <- get_inits(f7)[[1]]

  # Check 32-34
  expect_equal(i5$alpha * 2, i6$alpha)
  expect_equal(log(i5$sigma) * 2, log(i6$sigma))
  expect_false(identical(i7$alpha, i6$alpha))

  f8 <- sampling(rr,
    data = dat, iter = 10, chains = 1,
    init = list(list(mu = 2, sigma = 2)),
    enable_random_init = TRUE
  )
  i8 <- get_inits(f8)[[1]]
  # Check 35.
  expect_equal(i8$sigma, 2)

  f9 <- sampling(rr,
    data = dat, iter = 10, chains = 2, seed = 3,
    init = list(list(mu = 2), list(mu = 1, sigma = 2)),
    enable_random_init = TRUE
  )
  f10 <- sampling(rr,
    data = dat, iter = 10, chains = 2, seed = 3,
    init = list(list(mu = 2, eta = 1), list(mu = 1, sigma = 2, eta = 1)),
    enable_random_init = TRUE
  )
  i9 <- get_inits(f9)
  i10 <- get_inits(f10)

  # Check 36, this check is based on the implementation in which
  # though say mu is specified by the user, a random init is still
  # generated but not used.
  expect_equal(i9[[1]]$delta, i10[[1]]$delta)
})

test_that("get_elapsed_time works", {
  skip("Backwards compatibility")

  code <- "
    parameters {
      real y;
    }
    model {
      y ~ normal(0,1);
    }
  "
  nchains <- 5L
  sm <- stan_model(model_code = code)
  fit <- sampling(sm, chains = nchains, iter = 30)
  expect_equal(dim(get_elapsed_time(fit)), c(nchains, 2L))
  nchains <- 1L
  fit2 <- sampling(sm, chains = nchains, iter = 30)
  expect_equal(dim(get_elapsed_time(fit2)), c(nchains, 2L))
})

test_that("get_adaptation_info works", {
  skip("Backwards compatibility")

  code <- "
    parameters {
      real y;
    }
    model {
      y ~ normal(0,1);
    }
  "
  fit <- stan(model_code = code, chains = 5L, iter = 30L)
  info <- get_adaptation_info(fit)
  expect_true(all(!is.na(info)))
  expect_equal(length(info), 5L)
})

test_that("outputing csv and extracting work", {
  skip("Backwards compatibility")

  csv_fname <- "teststanfit.csv"
  model_code <- "
    transformed data {
      int n;
      n = 0;
    }
    parameters {
      array[2] real y;
      array[n] real zeroleny;
    }
    transformed parameters {
      array[2, 2] real y2;
      y2[1, 1] = y[1];
      y2[1, 2] = -y[1];
      y2[2, 1] = -y[2];
      y2[2, 2] = y[2];
    }
    model {
      y ~ normal(0, 1);
    }
    generated quantities {
      array[3, 2, 3] real y3;
      y3[1,1,1] = 1;   y3[1,1,2] = 2;   y3[1,1,3] = 3;
      y3[1,2,1] = 4;   y3[1,2,2] = 5;   y3[1,2,3] = 6;
      y3[2,1,1] = 7;   y3[2,1,2] = 8;   y3[2,1,3] = 9;
      y3[2,2,1] = 10;  y3[2,2,2] = 11;  y3[2,2,3] = 12;
      y3[3,1,1] = 13;  y3[3,1,2] = 14;  y3[3,1,3] = 15;
      y3[3,2,1] = 16;  y3[3,2,2] = 17;  y3[3,2,3] = 18;
    }
  "

  fit <- stan(model_code = model_code,
              iter = 100, chains = 1, thin = 3,
              sample_file = csv_fname)

  pmean <- get_posterior_mean(fit)[ ,1]
  pmean2 <- summary(fit)$c_summary[ , "mean", 1]
  expect_equal(pmean, pmean2, tolerance = 0.00001)

  expect_true(file.exists(csv_fname))
  d <- read.csv(file = csv_fname, comment.char = '#',
                header = TRUE)
  y_names <- paste0("y.", 1:2)
  cat2 <- function(a, b) paste0(a, ".", b)
  y2_names <- paste0("y2.", t(outer(1:2, 1:2, cat2)))
  y3_names <- paste0("y3.", t(outer(as.vector(t(outer(1:3, 1:2, cat2))), 1:3, cat2)))

  dfit <- as.data.frame(fit)
  iter1 <- unlist(d[1,])
  names(iter1) <-  sqrfnames_to_dotfnames(names(iter1))
  expect_equal(iter1["y.1"], iter1["y2.1.1"], ignore_attr = "names")
  expect_equal(iter1["y.1"], -iter1["y2.1.2"], ignore_attr = "names")
  expect_equal(iter1["y.2"], iter1["y2.2.2"], ignore_attr = "names")
  expect_equal(iter1["y.2"], -iter1["y2.2.1"], ignore_attr = "names")
  expect_equal(iter1[y3_names], 1:18, ignore_attr = "names")

  # FIXME Uncomment the following line:
  #expect_equal(colnames(d), c("lp__", "treedepth__", "stepsize__", y_names, y2_names, y3_names))

  iter1 <- unlist(d[1,])
  # to check if the order is also row-major
  expect_equal(iter1["y.1"], iter1["y2.1.1"], ignore_attr = "names")
  expect_equal(iter1["y.1"], -iter1["y2.1.2"], ignore_attr = "names")
  expect_equal(iter1["y.2"], iter1["y2.2.2"], ignore_attr = "names")
  expect_equal(iter1["y.2"], -iter1["y2.2.1"], ignore_attr = "names")
  expect_equal(iter1[y3_names], 1:18, ignore_attr = "names")

  fit2 <- read_stan_csv(csv_fname)
  expect_equal(is_sf_valid(fit), TRUE)
  expect_equal(is_sf_valid(fit2), FALSE)
  fit2_m <- as.vector(get_posterior_mean(fit2))
  fit2_m2 <- summary(fit2)$summary[,"mean"]
  expect_equal(fit2_m, fit2_m2, ignore_attr = "names")
  unlink(csv_fname)

  # test extract
  fitb <- stan(fit = fit, iter = 105, warmup = 100, chains = 3, thin = 3)
  e1 <- extract(fitb)
  expect_equal(e1$y3[1,1,1,1], 1)
  expect_equal(e1$y3[1,2,2,2], 11)
  expect_equal(e1$y3[1,2,1,2], 8)
  expect_equal(e1$y3[1,3,1,2], 14)

  m1 <- as.matrix(fitb, pars = "y3")
  expect_equal(dim(m1), c(6L, 18L))
  old_asmatrix_stanfit <- function(x, ...) {
    if (x@mode != 0) return(numeric(0))
    e <- extract(x, permuted = FALSE, inc_warmup = FALSE, ...)
    out <- apply(e, 3, FUN = function(y) y)
    if (length(dim(out)) < 2L) out <- t(as.matrix(out))
    dimnames(out) <- dimnames(e)[-2]
    return(out)
  }
  expect_true(identical(m1, old_asmatrix_stanfit(fitb, pars = 'y3')))
  m2 <- as.matrix(fitb)
  expect_true(identical(m2, old_asmatrix_stanfit(fitb)))
})

test_that("init zero gradient fails", {
  skip("Backwards compatibility + error")

  code <- "
    parameters {
      real x;
    }
    model {
      x ~ normal(0, -1);
    }
  "
  sm <- stan_model(model_code = code)
  mod <- sm@dso@.CXXDSOMISC$module
  model_cppname <- sm@model_cpp$model_cppname
  sf_mod <- eval(call("$", mod, paste0('stan_fit4', model_cppname)))
  dat <- list()
  # FIXME: This errors. (No valid constructor)
  sampler <- new(sf_mod, dat, 12345, sm@dso@.CXXDSOMISC$cxxfun)
  args <- list(init = "0", iter = 1)
  expect_error({s <- sampler$call_sampler(args)})
  errmsg <- geterrmessage()
  #expect_true(grepl('.*Rejecting initial value.*', errmsg))
  expect_true(grepl("Error", errmsg))
})

test_that("init zero throws exception for infinite lp", {
  skip("Backwards compatibility + error")
  code <- "
    parameters {
      real x;
    }
    model {
      target += 1 / x;
    }
  "
  sm <- stan_model(model_code = code)
  mod <- sm@dso@.CXXDSOMISC$module
  model_cppname <- sm@model_cpp$model_cppname
  sf_mod <- eval(call("$", mod, paste0('stan_fit4', model_cppname)))
  dat <- list()
  # FIXME: This errors (No valid constructor)
  sampler <- new(sf_mod, dat, 12345, sm@dso@.CXXDSOMISC$cxxfun)
  args <- list(init = "0", iter = 1)
  expect_error({s <- sampler$call_sampler(args)})
  errmsg <- geterrmessage()
  #expect_true(grepl('.*negative infinity.*', errmsg))
  expect_true(grepl("Error", errmsg))
})

test_that("init zero throws exception for infinite gradient", {
  skip("Backwards compatibility + error")

  code <- "
    parameters {
      real x;
    }
    model {
      target += 1 / log(x);
    }
  "
  sm <- stan_model(model_code = code)
  mod <- sm@dso@.CXXDSOMISC$module
  model_cppname <- sm@model_cpp$model_cppname
  sf_mod <- eval(call("$", mod, paste0('stan_fit4', model_cppname)))
  dat <- list()
  # FIXME: This errors (No valid constructor)
  sampler <- new(sf_mod, dat, 12345, sm@dso@.CXXDSOMISC$cxxfun)
  args <- list(init = "0", iter = 1)
  expect_exception({s <- sampler$call_sampler(args)})
  errmsg <- geterrmessage()
  #expect_error(grepl('.*not finite.*', errmsg))
  expect_true(grepl("Error", errmsg))
})

test_that("grad log is correct", {
  skip("Backwards compatibility + failure")

  y <- c(0.70,  -0.16,  0.77, -1.37, -1.99,  1.35, 0.08,
         0.02,  -1.48, -0.08,  0.34,  0.03, -0.42, 0.87,
         -1.36,  1.43,  0.80, -0.48, -1.61, -1.27)

  code <- "
    data {
      array[20] real y;
    }
    parameters {
      real mu;
      real<lower=0> sigma;
    }
    model {
      y ~ normal(mu, sigma);
    }
  "

  log_prob_fun <- function(mu, log_sigma, adjust = TRUE) {
    sigma <- exp(log_sigma)
    lp <- -sum((y - mu)^2) / (2 * (sigma^2)) - length(y) * log(sigma)
    if (adjust) lp <- lp + log(sigma)
    lp
  }

  log_prob_grad_fun <- function(mu, log_sigma, adjust = TRUE) {
    sigma <- exp(log_sigma)
    g_lsigma <- sum((y - mu)^2) * sigma^(-2) - length(y)
    if (adjust) g_lsigma <- g_lsigma + 1
    g_mu <- sum(y - mu) * sigma^(-2)
    c(g_mu, g_lsigma)
  }

  sf <- stan(model_code = code, data = list(y = y), iter = 200)
  mu <- 0.1
  sigma <- 2

  # FIXME: This test fails.
  expect_equal(
    log_prob(sf, unconstrain_pars(sf, list(mu = mu, sigma = sigma))),
    log_prob_fun(mu, log(sigma))
  )
  # FIXME: This test fails.
  expect_equal(
    log_prob(sf, unconstrain_pars(sf, list(mu = mu, sigma = sigma)), FALSE),
    log_prob_fun(mu, log(sigma), adjust = FALSE)
  )
  lp1 <- log_prob(sf, unconstrain_pars(sf, list(mu = mu, sigma = sigma)), FALSE, TRUE)
  expect_equal(attr(lp1, 'gradient'), log_prob_grad_fun(mu, log(sigma), FALSE))

  g1 <- grad_log_prob(sf, unconstrain_pars(sf, list(mu = mu, sigma = sigma)), FALSE)
  expect_equal(attr(g1, 'log_prob'), log_prob_fun(mu, log(sigma), adjust = FALSE))

  attributes(g1) <- NULL
  expect_equal(g1, log_prob_grad_fun(mu, log(sigma), adjust = FALSE))
})

test_that("Specifying arguments and data works", {
  skip("Backwards compatibility")
  y <- c(0.70,  -0.16,  0.77, -1.37, -1.99,  1.35, 0.08,
         0.02,  -1.48, -0.08,  0.34,  0.03, -0.42, 0.87,
         -1.36,  1.43,  0.80, -0.48, -1.61, -1.27)

  code <- "
    data {
      int N;
      array[N] real y;
    }
    parameters {
      real mu;
      real<lower=0> sigma;
    }
    model {
      y ~ normal(mu, sigma);
    }
  "
  stepsize0 <- 0.15
  sf <- stan(model_code = code, data = list(y = y, N = length(y)), iter = 200,
             control = list(adapt_engaged = FALSE, stepsize = stepsize0))
  expect_equal(
    attr(sf@sim$samples[[1]], "sampler_params")$stepsize__[1],
    stepsize0
  )

  sf2 <- stan(fit = sf, iter = 20, algorithm = 'HMC',
              data = list(y = y, N = length(y)),
              control = list(adapt_engaged = FALSE, stepsize = stepsize0))
  expect_equal(
    attr(sf2@sim$samples[[1]],"sampler_params")$stepsize__[1],
    stepsize0
  )

  sf3 <- stan(fit = sf, iter = 1, data = list(y = y, N = length(y)),
              init = 0, chains = 1)
  i_u <- unconstrain_pars(sf3, get_inits(sf3)[[1]])
  expect_equal(i_u, rep(0, 2))

  # Data in a function frame.
  tfun2 <- function() {
    y <- rnorm(20)
    N <- length(y)
    sm <- sf@stanmodel
    fit <- sampling(sm, data = c("y", "N"))
    fit
  }
  s1 <- tfun2()
  expect_equal(s1@mode, 0L)

  tfun <- function() {
    y <- rnorm(20)
    N <- length(y)
    fit <- stan(fit = sf, data = c("y", "N"))
    fit
  }
  s <- tfun()
  expect_equal(s@mode, 0L)
})

test_that("loo moment matching works", {
  skip("Backwards compatibility")

  stan_model_code <- "
    data {
      int<lower=0> N;
      vector[N] x;
    }
    parameters {
      real mu;
      real<lower=0> sigma;
    }
    model {
      x ~ normal(mu,sigma);
    }
    generated quantities {
      vector[N] log_lik;
      for (n in 1:N) {
        log_lik[n] = normal_lpdf(x[n] | mu, sigma);
      }
   }
  "

  rr <- stan_model(
    model_code = stan_model_code,
    model_name = "m1",
    verbose = FALSE
  )

  CHAINS <- 4
  ITER <- 2000
  SEED <- 12
  set.seed(SEED)

  # Generate toy data.
  data_sd <- 1
  n <- as.integer(30)
  x <- rnorm(n = n, mean = 0, sd = data_sd)
  x[[1]] <- 11

  stan_data <- list(N = n, x = x, loo = 0)

  fit_rstan <- sampling(
    rr,
    data = stan_data,
    chains = CHAINS,
    seed = SEED,
    iter = ITER
  )

  expect_error(loo(fit_rstan, moment_match = TRUE, k_threshold = "error"))
  expect_error(loo(fit_rstan, moment_match = "error"))

  loo_3 <- loo(fit_rstan, moment_match = TRUE, save_psis = TRUE)
  expect_equal(loo_3$diagnostics, loo_3$psis_object$diagnostics)
})

test_that("optimizing works", {
  skip("Backwards compatibility")

  code <- "
    parameters {
      real y;
    } model {
      y ~ normal(0,1);
    }
  "
  m <- stan_model(model_code = code)
  o <- optimizing(m, hessian = TRUE)
  expect_equal(o$par[1], 0, tolerance = 0.1, ignore_attr = "names")
  expect_equal(o$hessian[1,1], -1, tolerance = 0.1)

  mc <- "
    parameters {
      simplex[2] a;
      real y;
    }
    model {
      y ~ normal(0,1);
      target += -square(a[1]) - square(a[2]);
    }
    generated quantities {
      matrix[4, 5] B;
      for (i in 1:4) for (j in 1:5) B[i,j] = i * 10 + j;
    }
  "
  m2 <- stan_model(model_code = mc)
  set.seed(1287)
  o2 <- optimizing(m2, hessian = TRUE, seed = 4)
  # Check the names of the returns.
  expect_equal(o2$par["B[3,4]"], 34, ignore_attr = "names")
  expect_equal(o2$par["B[1,4]"], 14, ignore_attr = "names")
  expect_equal(o2$par["B[4,5]"], 45, ignore_attr = "names")
  expect_equal(o2$par["B[4,1]"], 41, ignore_attr = "names")

  set.seed(1287)
  o3 <- optimizing(m2, as_vector = FALSE, seed = 4)
  s <- list(a = 1:2, y = 3, B = array(0, dim = c(4,5)))
  attr(s$a, "dim") <- 2
  expect_equal(o3$par, rstan_relist(o2$par, s))
  expect_equal(o2$par[3], 0, tolerance = 0.1, checkNames = FALSE)
})

doctor_cppcode <- function(stanc_ret,
                           ..., # unused but forces exact name matching
                           add_license = character(),
                           use_R_PRNG = FALSE,
                           use_Rcout = FALSE,
                           promote_args_to_auto = FALSE,
                           detemplate = FALSE,
                           double_only = detemplate,
                           propto__ = FALSE,
                           jacobian__ = FALSE,
                           make_data_public = FALSE,
                           default_constructor = FALSE,
                           drop_Eigen = FALSE,
                           drop_log_prob = FALSE,
                           drop_model_header = FALSE,
                           add_toplevel_headers = character(),
                           includes_for_user_defined_functions = character(),
                           return_names = FALSE,
                           return_dims = FALSE,
                           add_methods = list(),
                           add_gradient_method = FALSE,
                           add_hessian_method = FALSE,
                           add_hessian_times_gradient_method = FALSE,
                           add_newton_step = FALSE,
                           methods_for_user_defined_functions = FALSE,
                           friends = character()) {
  dots <- list(...)
  if (length(dots) > 0L) {
    print(str(dots))
    stop("found the above mistakenly passed through ...")
  }
  if (!is.character(add_license))
    stop("'add_license' must be a (possibly empty) character vector")
  if (!is.list(add_methods))
    stop("'add_methods' must be a (possibly empty) list")
  if (!is.character(add_toplevel_headers))
    stop("'add_toplevel_headers' must be a character vector, possibly of length zero")
  if (!is.character(includes_for_user_defined_functions))
    stop("'includes_for_user_defined_functions' must be a character vector, possibly of length zero")
  if (!is.character(friends))
    stop("'friends' must be a (possibly empty) character vector")

  if (!is.list(stanc_ret) || is.null(stanc_ret$cppcode) ||
      !is.character(stanc_ret$cppcode))
    stop("'stanc_ret' must be a list (like) that is returned by 'stanc'")

  valid <- "[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_]"

  lines <- scan(what = character(), sep = "\n", quiet = TRUE, text = stanc_ret$cppcode)

  check_logical_scalar_first <- function(x, na_ok = FALSE) {
    end <- if (na_ok) "must be TRUE, FALSE, or NA" else "must be either TRUE or FALSE"
    msg <- paste(deparse(substitute(x)), end)
    if (!is.logical(x) || length(x) != 1L) stop(msg)
    if (!na_ok && is.na(x)) stop(msg)
    x
  }

  four_spaces <- "    "
  protected <- list()

  if (check_logical_scalar_first(use_R_PRNG)) {
    protected$PRNG <- paste0(four_spaces,
                             "boost_random_R base_rng__ = boost_random_R();")
  } else { # this could be dangerous to use with parallel chains
    protected$PRNG <- paste0(four_spaces,
                             "mutable boost::ecuyer1988 base_rng__;")
  }

  if (check_logical_scalar_first(use_Rcout)) {
    pstream__ <- "&Rcpp::Rcout"
    lines <- gsub("std::ostream* pstream__ = 0", "std::ostream* pstream__ = &Rcpp::Rcout",
                  lines, fixed = TRUE)
    lines <- gsub("std::ostream* pstream__) const {",
                  "std::ostream* pstream__ = &Rcpp::Rcout) const {",
                  lines, fixed = TRUE)
  } else {
    pstream__ <- "nullptr"
    lines <- gsub("std::ostream* pstream__ = 0", "std::ostream* pstream__ = nullptr",
                  lines, fixed = TRUE)
    lines <- gsub("std::ostream* pstream__) const {",
                  "std::ostream* pstream__ = nullptr) const {",
                  lines, fixed = TRUE)
  }
  protected$pstream <- paste0(four_spaces, "std::ostream* pstream__ = ",
                              pstream__, ";")

  if (check_logical_scalar_first(promote_args_to_auto)) {
    lines <- gsub("typename boost::math::tools::promote_args<.*$", "auto", lines)
    lines <- gsub("typedef typename boost::math::tools::promote_args<(.*)>.* ",
                  "typedef decltype(\\1) ", lines)
    mark <- grep("typedef decltype", lines)
    lines[mark] <- gsub(", ", " + ", lines[mark], fixed = TRUE)
  }

  if (check_logical_scalar_first(detemplate)) {
    if (length(friends) > 0L)
      stop("cannot 'detemplate' the C++ code if 'friends' is specified")
    mark <- grep("template <", lines, fixed = TRUE)
    for (m in rev(mark)) # cannot actually detemplate the RNG for some reason
      if (!grepl("template <typename RNG>", lines[m], fixed = TRUE)) lines <- lines[-m]

      if (!isTRUE(double_only)) {
        double_only <- TRUE
        message("'double_only' changed to TRUE due to 'detemplate' being TRUE")
      }
      if (!isTRUE(propto__)) {
        propto__ <- TRUE
        message("'propto__' changed to TRUE due to 'detemplate' being TRUE")
      }
      if (is.na(jacobian__)) {
        jacobian__ <- TRUE
        message("'jacobian__' changed to FALSE due to 'detemplate' being TRUE")
      }
  } else {
    lines <- gsub("typename T__>$", "typename T__ = double>", lines)
    lines <- gsub("typename T_>$",  "typename T_ = double>",  lines)
    lines <- gsub("typename T([[:digit:]]+)__",
                  "typename T\\1__ = double", lines)
    lines <- gsub("typename T_lp__", "T_lp__ = double", lines)
    lines <- gsub("typename T_lp_accum__", "typename T_lp_accum__ = double", lines)
    lines <- gsub("Class RNG",
                  paste0("Class RNG = ", ifelse(use_R_PRNG, "boost_random_R",
                                                "boost::ecuyer1988")), lines)
  }

  if (check_logical_scalar_first(double_only)) {
    if (!detemplate)
      stop("if 'double_only' is TRUE, then 'detemplate' must also be TRUE")
    lines <- gsub("T_lp_accum__", "accumulator<double>", lines)
    lines <- gsub("T_lp__", "double", lines)
    lines <- gsub("T[[:digit:]]*_+", "double", lines)
  }

  if (check_logical_scalar_first(propto__) || !propto__) {
    if (!detemplate) {
      lines <- gsub("bool propto__",
                    paste("bool propto__ =", ifelse(propto__, "true", "false")),
                    lines, fixed = TRUE)
      lines <- gsub("bool propto,",
                    paste("bool propto =", ifelse(propto__, "true,", "false,")),
                    lines, fixed = TRUE)
      mark <- grep("const static bool propto", lines)
      lines[mark] <- paste0(four_spaces, "const static bool propto__ = true;")
    }
    else {
      protected$propto__ <- paste0(four_spaces, "const bool propto__ = ",
                                   ifelse(propto__, "true", "false"), ";")
      protected$propto <- paste0(four_spaces, "const bool propto = ",
                                 ifelse(propto__, "true", "false"), ";")
    }
  }

  if (check_logical_scalar_first(jacobian__) || !jacobian__) {
    if (!detemplate) {
      lines <- gsub("bool jacobian__",
                    paste("bool jacobian__ =", ifelse(jacobian__, "true", "false")),
                    lines, fixed = TRUE)
      lines <- gsub("bool jacobian,",
                    paste("bool jacobian =", ifelse(jacobian__, "true,", "false,")),
                    lines, fixed = TRUE)
    }
    else {
      protected$jacobian__ <- paste0(four_spaces, "const bool jacobian__ = ",
                                     ifelse(jacobian__, "true", "false"), ";")
      protected$jacobian <- paste0(four_spaces, "const bool jacobian = ",
                                   ifelse(jacobian__, "true", "false"), ";")
    }
  }

  if (check_logical_scalar_first(default_constructor)) {
    if (!isTRUE(make_data_public)) {
      make_data_public <- TRUE
      message("'make_data_public' changed to TRUE due to 'default_constructor' being TRUE")
    }
  }

  if (check_logical_scalar_first(make_data_public)) {
    lines <- sub("^private:", "public:", lines)
  }

  find_closing <- function(lines, mark) {
    if (length(mark) != 1 || is.na(mark)) stop("mark is wrong")
    opening <- lines[mark]
    chars <- strsplit(opening, split = NULL, fixed = TRUE)[[1]]
    count <- 0L
    pos <- 1L
    while (pos <= length(chars) && chars[pos] == " ") {
      count <- count + 1L
      pos <- pos + 1L
    }
    closing <- grep(paste0("^[[:space:]]{", count, "}", "\\}"), lines[-c(1:mark)])[1]
    if (length(closing) == 0) stop("no closing found")
    closing <- closing + mark
    return(closing)
  }

  if (check_logical_scalar_first(drop_Eigen)) {
    mark <- grep("log_prob(Eigen::Matrix<", lines, fixed = TRUE)
    if (!detemplate) mark <- mark - 1L
    closing <- find_closing(lines, mark)
    lines <- lines[-c(mark:closing)]
    mark <- grep("Eigen::Matrix<double,Eigen::Dynamic,1>& params_r,",
                 lines, fixed = TRUE)[1] - 1L
    closing <- find_closing(lines, mark)
    lines <- lines[-c(mark:closing)]
    mark <- grep("Eigen::Matrix<double, Eigen::Dynamic, 1>& params_r,",
                 lines, fixed = TRUE)[1] - 1L
    closing <- find_closing(lines, mark)
    lines <- lines[-c(mark:closing)]
  }

  if (check_logical_scalar_first(drop_log_prob)) {
    mark <- grep("log_prob(", lines, fixed = TRUE)
    mark <- mark[!grepl("}", lines[mark])]
    if (!detemplate) mark <- mark - 1L
    closing <- find_closing(lines, mark)
    lines <- lines[-c(mark:closing)]
  }

  necessary_headers <- c("#include <rstan/boost_random_R.hpp>",
                         "#include <rstan/io/rlist_ref_var_context.hpp>",
                         "#include <stan/model/model_base_crtp.hpp>")
  if (check_logical_scalar_first(drop_model_header)) {
    lines <- grep("#include <stan/model/model_header.hpp>", lines,
                  fixed = TRUE, invert = TRUE, value = TRUE)
    necessary_headers <- c(necessary_headers,
                           ifelse(double_only,
                                  "#include <stan/math/prim.hpp>",
                                  "#include <stan/math/math.hpp>"),
                           "#include <stan/model/prob_grad.hpp>",
                           "#include <stan/io/array_var_context.hpp>",
                           "#include <stan/io/cmd_line.hpp>",
                           "#include <stan/io/dump.hpp>",
                           "#include <stan/io/reader.hpp>",
                           "#include <stan/io/writer.hpp>",
                           "#include <stan/lang/rethrow_located.hpp>",
                           "#include <stan/model/prob_grad.hpp>",
                           "#include <stan/model/indexing.hpp>",
                           "#include <boost/exception/all.hpp>")
    if (!use_R_PRNG)
      necessary_headers <- c(necessary_headers,
                             "#include <boost/random/additive_combine.hpp>",
                             "#include <boost/random/linear_congruential.hpp>")

  }

  if (check_logical_scalar_first(return_names)) {
    get_param_names <- c(
      "() const {",
      "  std::vector<std::string> names__;",
      "  get_param_names(names__);",
      "  return names__;",
      "}")
    constrained_param_names <- c(
      "(bool include_tparams__ = true, bool include_gqs__ = true) const {",
      "  std::vector<std::string> param_names__;",
      "  constrained_param_names(param_names__, include_tparams__, include_gqs__);",
      "  return param_names__;",
      "}")
    unconstrained_param_names <- sub("constrained_param_names",
                                     "unconstrained_param_names",
                                     constrained_param_names)
    add_methods$get_param_names_ <- get_param_names
    add_methods$constrained_param_names_ <- constrained_param_names
    add_methods$unconstrained_param_names_ <- unconstrained_param_names
  }

  if (check_logical_scalar_first(return_dims)) {
    get_dims <- c(
      "() const {",
      "  std::vector<std::vector<size_t> > dimss__;",
      "  get_dims(dimss__);",
      "  return dimss__;",
      "}")
    add_methods$get_dims_ <- get_dims
  }

  if (!isTRUE(use_R_PRNG)) {
    set_Boost_seed <- c(
      "(const unsigned int seed) const {",
      "  base_rng__.seed(seed);",
      "  return;",
      "}")
    add_methods$set_Boost_seed <- set_Boost_seed
  }

  if (!drop_log_prob) { # this is easily exportable to R
    log_prob <- if (detemplate) c(
      "(std::vector<double>& params_r__) const {",
      "  std::vector<int> params_i__;",
      "  return log_prob(params_r__, params_i__, pstream__);",
      "}") else c(
        "(std::vector<double>& params_r__) const {",
        "  std::vector<int> params_i__;",
        "  return log_prob<>(params_r__, params_i__, pstream__);",
        "}")

    add_methods$log_prob_ <- log_prob
  }

  if (check_logical_scalar_first(add_gradient_method)) {
    if (drop_log_prob)
      stop("if 'add_gradient_method' is TRUE, 'drop_log_prob' must be FALSE")
    if (double_only)
      stop("if 'add_gradient_method' is TRUE, 'double_only' must be FALSE")
    gradient <- c(
      "(Eigen::VectorXd params_r__) {",
      "  std::vector<int> params_i__;",
      "  double fx;",
      "  std::vector<double> grad_fx;",
      "  auto stan::math::gradient([&params_i__](auto theta) {",
      "    return log_prob<propto__, jacobian__>(to_array_1d(theta), params_i__, pstream__);",
      "  }, params_r__, fx, grad_fx);",
      "  return grad_fx;",
      "}")
    add_methods$gradient <- gradient
  }

  if (check_logical_scalar_first(add_hessian_method)) {
    if (drop_log_prob)
      stop("if 'add_gradient_method' is TRUE, 'drop_log_prob' must be FALSE")
    if (detemplate)
      stop("if 'add_gradient_method' is TRUE, 'detemplate' must be FALSE")
    hessian <- c(
      "(std::vector<double> params_r__) {",
      "  std::vector<int> params_i__;",
      "  std::vector<double> grad_fx;",
      "  std::vector<double> hess_fx;",
      paste0("  auto stan::model::grad_hess_log_prob<propto__, jacobian__>(",
             "this, params_r__, params_i__, grad_fx, hess_fx, pstream__);"),
      "  return hess_fx;",
      "}")
    add_methods$hessian <- hessian
    necessary_headers <- c(necessary_headers,
                           "#include <stan/model/grad_hess_log_prob.hpp>")
  }

  if (check_logical_scalar_first(add_hessian_times_gradient_method)) {
    if (drop_log_prob)
      stop("if 'add_hessian_times_gradient_method' is TRUE, 'drop_log_prob' must be FALSE")
    if (detemplate)
      stop("if 'add_hessian_times_gradient_method' is TRUE, 'detemplate' must be FALSE")
    hessian_times_gradient <- c(
      "(std::vector<double> params_r__) {",
      "  std::vector<int> params_i__;",
      "  std::vector<double> grad_fx;",
      "  std::vector<double> hess_fx;",
      paste0("  auto stan::model::grad_hess_log_prob<propto__, jacobian__>(",
             "this, params_r__, params_i__, grad_fx, hess_fx, pstream__);"),
             "  return (hess_fx.selfadjointView<Eigen::Lower>() * grad_fx).eval();",
      "}")
    add_methods$hessian_times_gradient <- hessian_times_gradient
    if (!add_hessian_method)
      necessary_headers <- c(necessary_headers,
                             "#include <stan/model/grad_hess_log_prob.hpp>")
  }

  if (check_logical_scalar_first(add_newton_step)) {
    if (isTRUE(drop_log_prob))
      stop("if 'add_newton_step' is TRUE, 'drop_log_prob' must be FALSE")
    newton_step <- c(
      "(std::vector<double> params_r__) {",
      "  std::vector<int> params_i__;",
      paste0("  auto stan::optimization::newton_step",
             "(this, params_r, params_i, pstream__);"),
      "  return params_r__;",
      "}")
    add_methods$newton_step <- newton_step
    necessary_headers <- c(necessary_headers,
                           "#include <stan/services/optimization/newton_step.hpp>")
  }

  make_sig_bod <- function(sig, friend) {
    if (detemplate) {
      start <- if (friend) "(this, " else "("
    } else start <- if (friend) "<>(this, " else "<>("
    if (grepl("lp__", sig)) {
      sig <- sub("(^.*lp__).*$", "\\1) {", sig)
      inputs <- strsplit(sig, split = ",")[[1]]
      inputs <- sub(") {", "", inputs, fixed = TRUE)
      inputs <- sub(paste0("^.* (", valid, "+$)"), "\\1", inputs)
      if (friend) inputs <- inputs[-1L]
      inputs <- c(inputs, "lp_accum__", "pstream__") # dubious
      inputs <- paste(inputs, collapse = ", ")
      bod <- c("accumulator<double> lp_accum__;",
               paste0("return ", namespace_name, "::", method_name,
                      start, inputs, "); }"))
    } else if (grepl("base_rng__", sig)) {
      sig <- sub("(^.*), RNG.*$", "\\1) {", sig)
      inputs <- strsplit(sig, split = ",")[[1]]
      inputs <- sub(") {", "", inputs, fixed = TRUE)
      inputs <- sub(paste0("^.* (", valid, "+$)"), "\\1", inputs)
      if (friend) inputs <- inputs[-1L]
      inputs <- c(inputs, "base_rng__", "pstream__")
      inputs <- paste(inputs, collapse = ", ")
      bod <- paste0("return ", namespace_name, "::", method_name,
                    start, inputs, "); }")
    } else if (grepl(",", sig)) {
      sig <- sub("(^.*), std::ostream.*$", "\\1) {", sig)
      inputs <- strsplit(sig, split = ",")[[1]]
      inputs <- sub(") {", "", inputs, fixed = TRUE)
      inputs <- sub(paste0("^.* (", valid, "+$)"), "\\1", inputs)
      if (friend) inputs <- inputs[-1L]
      inputs <- c(inputs, "pstream__")
      inputs <- paste(inputs, collapse = ", ")
      bod <- paste0("return ", namespace_name, "::", method_name,
                    start, inputs, "); }")
    } else {
      if (friend) stop("no template arguments in friend function")
      sig <- "() {"
      bod <- paste0("return ", namespace_name, "::", method_name, "(); }")
    }
    return(list(sig = sig, bod = bod))
  }

  lines <- gsub("Eigen::Matrix<double, Eigen::Dynamic, 1>",
                "Eigen::VectorXd", lines, fixed = TRUE)
  lines <- gsub("Eigen::Matrix<double, Eigen::Dynamic, Eigen::Dynamic>",
                "Eigen::MatrixXd", lines, fixed = TRUE)

  mark <- grep("^namespace .* \\{$", lines)
  namespace_name <- sub("namespace ", "", lines[mark])
  namespace_name <- sub(" {", "", namespace_name, fixed = TRUE)
  if (check_logical_scalar_first(methods_for_user_defined_functions)) {
    mark <- grep(paste0("^", valid, "+\\("), lines)
    for (i in rev(mark)) {
      end <- i
      while (!grepl("{", lines[end], fixed = TRUE)) end <- end + 1L
      sig <- paste(lines[i:end], collapse = " ")
      method_name <- sub(paste0("^(", valid, "+)\\(.*$"), "\\1", sig)
      sig <- sub(method_name, "", sig, fixed = TRUE)
      sig_bod <- make_sig_bod(sig, friend = FALSE)
      if (!detemplate)
        sig_body$sig <- gsub("T[[:digit:]]+__", "double", sig_bod$sig)
      add_methods[[method_name]] <- c(sig_bod$sig, sig_bod$bod)
    }
  }

  if (length(friends) > 0L) {
    mark <- grep(paste0("^", valid, "+\\(.*\\);$"), lines)
    if (length(mark) == 0L)
      stop("'friends' is specified but no undefined functions found")
    friends_list <- list()
    for (i in rev(mark)) {
      sig <- lines[i]
      method_name <- sub(paste0("^(", valid, "+\\(.*$"), "\\1", sig)
      if (!(method_name %*% names(friends))) next
      sig <- sub(method_name, "", sig, fixed = TRUE)
      sig_body <- make_sig_bod(sig, friend = TRUE)
      add_methods[[method_name]] <- c(sig_bod$sig, sig_bod$bod)
      sig <- sub("(", "(prob_grad&, ", sig_bod$sig, fixed = TRUE)
      sig <- sub("{", ";", sig, fixed = TRUE)
      friends_list[[method_name]] <- sig
    }
  }

  for (i in seq_along(add_methods)) {
    mark <- grep("^}; // model$", lines) - 1L
    method <- add_methods[[i]]
    lines <- append(lines, after = mark,
                    values = c(paste0(four_spaces, "auto ",
                                      names(add_methods)[i], method[1]),
                               paste0(four_spaces, four_spaces, method[-1])))
  }

  if (length(friends)) {
    mark <- grep("^}; // model$", lines) - 1L
    lines <- append(lines, values = "friend:", after = mark)
    for (i in seq_along(friends_list)) {
      mark <- grep("^}; // model$", lines) - 1L
      method <- friends_list[[i]]
      lines <- append(lines, after = mark,
                      values = c(paste0(four_spaces, "auto ",
                                        names(friends_list)[i], method[1]),
                                 paste0(four_spaces, four_spaces, method[-1])))
    }
  }

  # do not create base_rng__ in ctor_body
  mark <- grep("boost::ecuyer1988 base_rng__ =", lines, fixed = TRUE)
  lines <- lines[-c(mark:(mark + 2L))]

  # deal with constructor
  mark <- grep("^class[[:space:]]+", lines)
  class_name <- sub("^class (.*)", "\\1", lines[mark])
  lines <- append(lines, after = mark + 1L,
                  values = c("protected:", unlist(protected)))
  lines <- append(lines, after = mark - 1L,
                  values = includes_for_user_defined_functions)
  mark <- grep(paste0(class_name, "(stan::io::var_context& context__,"),
               lines, fixed = TRUE)[1]
  closing <- find_closing(lines, mark)
  lines <- lines[-c(mark:closing)]
  mark <- grep(paste0(class_name, "(stan::io::var_context& context__,"),
               lines, fixed = TRUE)[1]
  closing <- find_closing(lines, mark)
  lines <- lines[-c(mark:closing)]
  if (default_constructor) {
    mark <- grep("void ctor_body(stan::io::var_context& context__,",
                 lines, fixed = TRUE)
    closing <- find_closing(lines, mark)
    lines <- lines[-c(mark:closing)]
    ctor <- paste0(four_spaces, class_name, "() : prob_grad(0) { }")
  } else {
    ctor <- c(paste0(four_spaces, class_name,
                     "(rstan::io::rlist_ref_var_context& context__) ",
                     ": model_base_crtp(0) {"),
              paste0(four_spaces, four_spaces,
                     "ctor_body(context__, 0, ", pstream__, ");"),
              paste0(four_spaces, "}"))
  }
  mark <- grep("^public:", lines)[1L + make_data_public]
  lines <- append(lines, values = ctor, after = mark)
  lines <- append(lines, after = 1L, c(necessary_headers,
                                       add_toplevel_headers))

  if (length(add_license) == 1L) {
    license <- file.path(R.home("share"), "licenses", add_license)
    if (!file.exists(license))
      stop("'add_license' not among license abbreviations known to R")
    lines < c("/*", readLines(license), "*/", "", lines)
  }

  stanc_ret$cppcode <- lines
  stanc_ret$methods <- add_methods
  stanc_ret$friends <- if (length(friends)) friends_list
  return(stanc_ret)
}

require(Rcpp)
require(inline)
require(rstan) 

inc <- paste(readLines('test_rlist_ref_var_context.cpp'), collapse = '\n')

fx <- cxxfunction(signature(), "" , includes = inc, 
                  # settings = myplugin, verbose = TRUE); 
                  plugin = "rstan", verbose = TRUE)
mod <- Module("rstantest", getDynLib(fx))

lst1 <- list(foo = c(1L, 2L)); 
lst2 <- list(foo = c(1L, 2L), bar = 1.0) 
lst3 <- list(foo = c(1L, 2L),
             bar = 1L, 
             bing = structure(c(1.0, 2.0, 2.0, 5.0, 3.0, 6.0), .Dim = c(2, 3))); 

mod$test_rlist_ref_var_context1(lst1); 
mod$test_rlist_ref_var_context2(lst2); 
mod$test_rlist_ref_var_context3(lst3);

.add_aesthetics <- function(dots, to_add) {
  # Add some default aesthetics if not included in ...
  # @param dots list(...)
  # @param to_add = character vector
  add_names <- gsub("pt_", "", to_add)
  for (j in seq_along(to_add)) {
    if (!add_names[j] %in% names(dots))
      dots[[add_names[j]]] <- rstanvis_aes_ops(to_add[j])
  }
  dots
}

`%ifNULL%` <- function(x, replacement) {
  if (!is.null(x)) x
  else replacement
}

is.stanreg <- function(x) inherits(x, "stanreg")
is.stanfit <- function(x) inherits(x, "stanfit")

.get_stanreg_parnames <- function(x) {
  stopifnot(is.stanreg(x))
  pars <- rownames(x$stan_summary)
  sel <- grepl("mean_ppd|lp__|log-posterior", pars, ignore.case = TRUE)
  c(pars[!sel], pars[sel])
}

.check_object <- function(object, unconstrain = FALSE) {
  if (is.stanreg(object)) {
    if (object$algorithm == "optimizing")
      stop("Plots not yet available for optimization (algorithm='optimizing')",
           call. = FALSE)
    if (unconstrain)
      stop("Option 'unconstrain' not yet available for stanreg objects.",
           call. = FALSE)
    .mode_check(object$stanfit)
  }
  else .mode_check(object)
}

.mode_check <- function(object) {
  stopifnot(is.stanfit(object))
  if (object@mode == 1L) {
    stop("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
         "sampling is not conducted.\n",
         call. = FALSE)
  } else if (object@mode == 2L) {
    stop("Stan model '", object@model_name, "' does not contain samples.\n",
         call. = FALSE)
  }
  invisible(TRUE)
}

.vb_check <- function(x) {
  if (is.stanreg(x)) x <- x$stanfit
  if (isTRUE(x@stan_args[[1]]$method == "variational"))
    stop("Plot only available for models estimated using MCMC", call. = FALSE)
}

.reshape_sample <- function(x) {
  res <- lapply(seq_along(x), function(i) {
    data.frame(value = unlist(x[[i]], use.names = FALSE),
               parameter = rep(names(x[[i]]), each = length(x[[i]][[1L]])),
               chain = i)
  })
  res <- do.call(rbind, res)
  res$parameter <- as.character(res$parameter)
  res
}
.make_plot_data <- function(object, pars, include = TRUE,
                            inc_warmup = FALSE, unconstrain = FALSE) {

  window <- NULL
  if (is.stanreg(object)) {
    sim <- object$stanfit@sim
  }
  else sim <- object@sim

  nopars <- missing(pars)
  if (is.stanfit(object) && !nopars) {
    if ("log-posterior" %in% pars)
      pars[which(pars == "log-posterior")] <- "lp__"
  }
  if (!include) {
    if (nopars) stop("pars must be specified if include=FALSE.", call. = FALSE)
    else {
      if (is.stanreg(object))
        pars <- setdiff(sim$fnames_oi, pars)
      else
        pars <- setdiff(sim$pars_oi, pars)
    }
  }
  if (nopars) {
    if (is.stanreg(object))
      pars <- names(object$coefficients)
    else
      pars <- setdiff(sim$pars_oi, c("lp__", "log-posterior"))
  }
  else {
    if (!is.stanreg(object))
      pars <- check_pars_second(sim, pars)
  }

  pars <- remove_empty_pars(pars, sim$dims_oi)
  if (unconstrain && "lp__" %in% pars) {
    if (length(pars) == 1L) stop("Can't unconstrain lp__", call. = FALSE)
    pars <- pars[-which(pars == "lp__")]
  }
  tidx <- pars_total_indexes(sim$pars_oi,
                             sim$dims_oi,
                             sim$fnames_oi,
                             pars)
  tidx <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)
  num_plots <- length(tidx)

  if (nopars && num_plots > 10) {
    # if pars is not specified then default to showing 10 of the parameters
    tidx <- tidx[1:10]
    message("'pars' not specified. Showing first 10 parameters by default.")
  }

  if (!is.null(window)) {
    window <- sort(window)
    if (window[1] < 1) window[1] <- 1
    if (window[1] > sim$iter[1])
      stop("wrong specification of argument window", call. = FALSE)
    if (is.na(window[2]) || window[2] > sim$iter[1])
      window[2] <- sim$iter[1]
  } else {
    window <- c(1, sim$iter[1])
  }
  if ((all(sim$warmup2 == 0) || !inc_warmup) && window[1] <= sim$warmup[1]) {
    window[1] <- sim$warmup[1] + 1
  }
  if (window[1] > window[2]) {
    stop("the given window does not include sample")
  }
  if (window[1] > sim$warmup[1]) inc_warmup <- FALSE

  thin <- sim$thin
  warmup2 <- sim$warmup2[1]
  warmup <- sim$warmup
  start_i <- window[1]
  window_size <- (window[2] - start_i) %/% thin
  id <- seq.int(start_i, by = thin, length.out = window_size)
  start_idx <- (if (warmup2 == 0) (start_i - warmup) else start_i) %/% thin
  if (start_idx < 1)  start_idx <- 1
  idx <- seq.int(start_idx, by = 1, length.out = window_size)

  if (unconstrain) {
    sim$samples <- .upars(object)
    sel <- grep(paste(pars, collapse ="|"), names(sim$samples[[1]]), value = TRUE)
    samp_use <- lapply(sim$samples, function(chain) {
      out <- lapply(chain[names(chain) %in% sel], function(x) x[idx])
      names(out) <- sel
      out
    })
  } else {
    samp_use <- lapply(sim$samples, function(chain) {
      out <- lapply(chain[tidx], function(x) x[idx])
      names(out) <- sim$fnames_oi[tidx]
      out
    })
  }
  nchains <- length(samp_use)

  if (unconstrain) {
    if (is.stanreg(object))
      object <- object$stanfit
    pblock <- .get_stan_params(object)
    pars2 <- unlist(lapply(strsplit(pars, "\\["), "[[", 1))
    not_pblock <- length(setdiff(unique(pars2), pblock))
    if (not_pblock)
      stop("If 'unconstrain' is TRUE only variables declared in the ",
           "'parameters' block can be included in 'pars'.",
           call. = FALSE)
  }

  dat <- .reshape_sample(samp_use)
  dat$iteration <- idx * thin
  dat$chain <- factor(dat$chain)
  fnames <- if (unconstrain)
    names(samp_use[[1]]) else sim$fnames_oi[tidx]
  lp <- which(dat$parameter == "lp__")
  if (!identical(lp, integer(0))) {
    dat$parameter[lp] <- "log-posterior"
    fnames[which(fnames == "lp__")] <- "log-posterior"
  }
  dat$parameter <- factor(dat$parameter, levels = fnames)
  list(samp = dat,
       nchains = nchains,
       nparams = length(fnames),
       warmup = warmup)
}

# get parameter names for parameters block only
.get_stan_params <- function(object) {
  stopifnot(is.stanfit(object))
  params <- grep("context__.vals_r", fixed = TRUE, value = TRUE,
                 x = strsplit(get_cppcode(get_stanmodel(object)), "\n")[[1]])
  params <- sapply(strsplit(params, "\""), FUN = function(x) x[[2]])
  intersect(params, object@model_pars)
}

# unconstrain
.upars <- function(object, permuted = FALSE, inc_warmup = TRUE) {
  if (is.stanreg(object))
    object <- object$stanfit
  pars <- extract(object, permuted = permuted, inc_warmup = inc_warmup)
  nchains <- ncol(pars)
  pn <- dimnames(pars)$parameters
  param_names <- pn[pn != "lp__"]
  sel <- object@model_pars != "lp__"
  skeleton <- create_skeleton(object@model_pars, object@par_dims)[sel]
  upars <- apply(pars, 1:2, FUN = function(theta) {
    unconstrain_pars(object, rstan_relist(theta, skeleton))
  })
  if (length(dim(upars)) == 2) {
    upars <- array(upars, dim = c(dim(upars)[1], dim(upars)[2], 1))
    upars <- aperm(upars, c(2, 3, 1))
  }
  else upars <- aperm(upars, c(3, 1, 2))

  pblock <- .get_stan_params(object)
  mark <- c()
  for (p in pblock) {
    patt <- paste0("^", p, "|^", p, "\\[")
    sel <- grep(patt, param_names)
    if (length(sel))
      mark <- c(mark, sel)
  }
  param_names <- param_names[sort(mark)]

  cpp_code <- strsplit(get_cppcode(get_stanmodel(object)), "\n")[[1]]
  param_names <- .remove_udiag_pars(cpp_code, pblock, param_names)

  lapply(seq_len(nchains), function(chain) {
    x <- upars[chain,, ]
    if (NCOL(x) == 1) x <- rbind(x)
    plist <- lapply(seq_len(nrow(x)), function(param) x[param, ])
    names(plist) <- param_names
    plist
  })
}

.remove_udiag_pars <- function(cpp_code, pblock, param_names) {
  patts <- c("cholesky_corr", "cholesky_cov", "corr_matrix", "cov_matrix")
  for (patt in patts) {
    to_drop <- c()
    for (p in pblock) {
      par_mentions = grep(paste0("(",p,")"), x = cpp_code,
                          fixed = TRUE, value = TRUE)
      if (length(grep(patt, par_mentions))) {
        for (v in grep(p, param_names)) {
          ij <- strsplit(x = param_names[v], split = c("\\[|,|\\]"))[[1]][2:3]
          if (diff(as.numeric(ij)) > -1)
            to_drop = c(v,to_drop)
        }
      }
    }
    if (length(to_drop))
      param_names <- param_names[-to_drop]
  }

  return(param_names)
}

color_vector <- function(n) {
  hues = seq(15, 375, length=n+1)
  hcl(h=hues, l=50, c=50)[1:n]
}
color_vector_chain <- function(n) {
  hues = seq(15, 375, length=n+1)
  hcl(h=hues, l=80, c=50)[1:n]
}



# autocorrelation ---------------------------------------------------------
.ac_fun <- function(x, lag.max, partial = FALSE) {
  if (!partial)
    acf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
  else
    pacf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
}
.ac_plot_data <- function(dat, lags, partial = FALSE) {
  ch <- dat[, grep("chain", colnames(dat))]
  nc <- length(unique(ch))
  ac_list <- tapply(dat$value, INDEX = ch, FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(1:nc, each = nl), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc)
  data.frame(chains = ch, ac = do.call(c, ac_list), lag = ll)
}
.ac_plot_data_multi <- function(dat, lags, partial = FALSE) {
  ch <- dat[, grep("chain", colnames(dat))]
  nc <- length(unique(ch))
  pa <- factor(dat[, grep("parameter", colnames(dat))])
  np <- length(unique(pa))
  ac_list <- tapply(dat$value, INDEX = list(ch, pa),
                    FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(rep(1:nc, each = nl), np), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc * np)
  pp <- factor(rep(1:np, each = nc * nl), labels = levels(pa))
  data.frame(parameters = pp, chains = ch, ac = do.call(c, ac_list), lag = ll)
}



# rhat, ess, mcse ---------------------------------------------------------
.rhat_neff_mcse_hist <- function(which = c("rhat", "n_eff_ratio", "mcse_ratio"),
                                 object, pars=NULL, ...) {
  if (is.stanreg(object)) object <- object$stanfit
  thm <- rstanvis_hist_theme()
  dots <- .add_aesthetics(list(...), c("fill", "color"))
  if (which == "n_eff_ratio") {
    lp <- suppressWarnings(get_logposterior(object, inc_warmup = FALSE))
    SS <- prod(length(lp), length(lp[[1L]]))
  }
  if (!is.null(pars)) smry <- summary(object, pars = pars)$summary
  else smry <- summary(object)$summary

  xlab <- switch(which,
                 rhat = "Rhat statistic",
                 n_eff_ratio = "Effective sample size / Sample size",
                 mcse_ratio = "Monte Carlo SE / Posterior SD"
  )
  stat <- switch(which,
                 rhat = smry[, "Rhat"],
                 n_eff_ratio = smry[, "n_eff"] / SS,
                 mcse_ratio = smry[, "se_mean"] / smry[, "sd"]
  )
  df <- data.frame(stat)
  plot_labs <- ggplot2::labs(y = "", x = xlab)
  base <- ggplot2::ggplot(df, ggplot2::aes(x = stat))
  base +
    do.call(ggplot2::geom_histogram, dots) +
    plot_labs +
    thm
}




# nuts stuff --------------------------------------------------------------
.LP_NAME <- "log-posterior"
.LP_LAB <- "Log Posterior"
.METROP_LAB <- "Mean Metrop. Acceptance"
.STEPSIZE_LAB <- "Sampled Step Size"
.TREEDEPTH_LAB <- "Treedepth"
.NDIVERGENT_LAB <- "Divergent"

.NUTS_VLINE_CLR <- "#222222"
.NUTS_FILL <- "#66a7e0"
.NUTS_CLR <- "#006dcc"
.NUTS_PT_CLR <- "#328ad6"
.NDIVERGENT_FILL <- "#ae0001"
.MAXTD_FILL <- "#eeba30"
.NDIVERGENT_CLR <-  "black"
.MAXTD_CLR <- "black"
.DIV_AND_MAXTD_SHAPE <- 21

.nuts_return <- function(graphs, ...) {
  gtable <- do.call(gridExtra::arrangeGrob, c(graphs, ...))
  plot(gtable)
  invisible(graphs)
}

.nuts_args_check <- function(...) {
  if ("par" %in% names(dots <- list(...)))
    stop("'par' argument should not be specified.", call. = FALSE)
}

.max_td <- function(x) {
  if (is.stanreg(x))
    x <- x$stanfit
  cntrl <- x@stan_args[[1L]]$control
  if (is.null(cntrl)) 11
  else {
    max_td <- cntrl$max_treedepth
    if (is.null(max_td)) 11
    else max_td
  }
}

.sampler_params_post_warmup <- function(object, which = "stepsize__", as.df = FALSE) {
  if (which == "divergent__" && utils::packageVersion("rstan") < "2.10")
    which <- "n_divergent__"
  if (is.stanreg(object))
    object <- object$stanfit
  sampler_params <- suppressWarnings(get_sampler_params(object))
  warmup_val <- floor(object@sim$warmup / object@sim$thin)
  warmup_saved <- object@stan_args[[1L]][["save_warmup"]]
  if (!is.null(warmup_saved)) {
    if (!warmup_saved) warmup_val <- 0
  }
  sp <-lapply(1:length(sampler_params), function(i) {
    out <- sampler_params[[i]]
    out <- if (warmup_val == 0) out[, which] else out[-(1:warmup_val), which]
    names(out) <- (warmup_val + 1):(warmup_val + NROW(out))
    out
  })
  if (length(which) == 1L && as.df) .sampler_param_df(sp, warmup_val)
  else sp
}

.sampler_param_df <- function(sp, warmup_val) {
  sp_mat <- do.call("cbind", sp)
  colnames(sp_mat) <- paste0("chain:", 1:ncol(sp_mat))
  sp_mat <- cbind(iterations = (warmup_val+1):(warmup_val + nrow(sp_mat)), sp_mat)
  as.data.frame(sp_mat)
}

.reshape_df <- function(df) {
  iter_col <- grep("iteration", colnames(df))
  mdf <- reshape(df, direction = "long",
                 v.names = "value",
                 varying = list(colnames(df)[-iter_col]),
                 timevar = "variable",
                 times = colnames(df)[-iter_col],
                 idvar = colnames(df)[iter_col])
  rownames(mdf) <- 1:nrow(mdf)
  mdf$variable <- as.factor(mdf$variable)
  attr(mdf, "reshapeLong") <- NULL
  mdf
}

.sampler_param_vs_param <- function(p, sp, divergent = NULL, hit_max_td = NULL,
                                    p_lab, sp_lab, chain = 0, violin = FALSE,
                                    smoother = FALSE, ...) {
  dots <- .add_aesthetics(list(...), c("alpha", "shape"))
  dots$alpha <- 0.5 * dots$alpha
  dots$color <- .NUTS_PT_CLR
  dots$fill <- .NUTS_FILL
  xy_labs <- ggplot2::labs(
    y = if (missing(p_lab)) NULL else p_lab,
    x = if (missing(sp_lab)) NULL else sp_lab
  )
  df <- data.frame(sp = do.call("c", sp), p = c(p))
  if (violin) df$sp <- as.factor(round(df$sp, 4))
  if (!is.null(divergent)) df$divergent <- do.call("c", divergent)
  if (!is.null(hit_max_td)) df$hit_max_td <- do.call("c", hit_max_td)

  base <- ggplot2::ggplot(df, ggplot2::aes(x = .data$sp, y = .data$p)) + xy_labs
  if (chain == 0) {
    if (violin)
      graph <- base + ggplot2::geom_violin(color = .NUTS_CLR, fill = .NUTS_FILL)
    else {
      graph <- base + do.call(ggplot2::geom_point, dots)
      if (smoother) graph <- graph + ggplot2::stat_smooth(se = FALSE)
      if (!is.null(divergent))
        graph <-
          graph + ggplot2::geom_point(
            data = subset(df, divergent == 1),
            mapping = ggplot2::aes(x = .data$sp, y = .data$p),
            color = .NDIVERGENT_CLR,
            fill = .NDIVERGENT_FILL,
            alpha = 0.8,
            size = 3,
            shape = .DIV_AND_MAXTD_SHAPE
          )
      if (!is.null(hit_max_td))
        graph <-
          graph + ggplot2::geom_point(
            data = subset(df, hit_max_td == 1),
            mapping = ggplot2::aes(x = .data$sp, y = .data$p),
            color = .MAXTD_CLR,
            fill = .MAXTD_FILL,
            alpha = 0.8,
            size = 3,
            shape = .DIV_AND_MAXTD_SHAPE
          )
    }
    return(graph)
  }
  chain_data <- data.frame(sp = sp[, chain], p = p[, chain])
  if (!is.null(divergent)) chain_data$div <- divergent[, chain]
  if (!is.null(hit_max_td)) chain_data$hit <- hit_max_td[, chain]
  chain_clr <- color_vector_chain(ncol(sp))[chain]
  chain_fill <- chain_clr
  if (violin) {
    chain_data$sp <- as.factor(round(chain_data$sp, 4))
    graph <- base +
      ggplot2::geom_violin(color = .NUTS_CLR, fill = .NUTS_FILL, ...) +
      ggplot2::geom_violin(
        data = chain_data,
        mapping = ggplot2::aes(x = .data$sp, y = .data$p),
        color = chain_clr,
        fill = chain_fill,
        ...
      )
    return(graph)
  }
  graph <- base + do.call(ggplot2::geom_point, dots)
  if (smoother) graph <- graph + ggplot2::stat_smooth(se = FALSE)
  graph <- graph +
    ggplot2::geom_point(
      data = chain_data,
      mapping = ggplot2::aes(x = .data$sp, y = .data$p),
      color = chain_fill,
      ...
    )
  if (smoother) graph <- graph +
    ggplot2::stat_smooth(
      data = chain_data,
      mapping = ggplot2::aes(x = .data$sp, y = .data$p),
      color = chain_fill,
      se = FALSE
    )
  if (!is.null(divergent))
    graph <- graph +
     ggplot2::geom_point(
      data = chain_data[chain_data$div == 1, , drop = FALSE],
      mapping = ggplot2::aes(x = .data$sp, y = .data$p),
      color = .NDIVERGENT_CLR,
      fill = .NDIVERGENT_FILL,
      size = 3,
      shape = .DIV_AND_MAXTD_SHAPE
     )
  if (!is.null(hit_max_td))
    graph <-
    graph + ggplot2::geom_point(
      data = chain_data[chain_data$hit == 1, , drop = FALSE],
      mapping = ggplot2::aes(x = .data$sp, y = .data$p),
      color = .MAXTD_CLR,
      fill = .MAXTD_FILL,
      size = 3,
      shape = .DIV_AND_MAXTD_SHAPE
    )
  graph
}

.sampler_param_vs_sampler_param_violin <- function(df_x, df_y, lab_x, lab_y,
                                                   chain = 0) {

  xy_labs <- ggplot2::labs(y = lab_y, x = lab_x)
  df <- data.frame(x = do.call("c", df_x), y = do.call("c", df_y))
  df$x <- as.factor(df$x)

  base <- ggplot2::ggplot(df, ggplot2::aes(x = .data$x, y = .data$y)) + xy_labs
  graph <- base + ggplot2::geom_violin(color = .NUTS_CLR, fill = .NUTS_FILL)
  if (chain == 0) return(graph)
  chain_clr <- color_vector_chain(ncol(df_x))[chain]
  chain_fill <- chain_clr
  chain_data <- data.frame(x = as.factor(df_x[, chain]), y = df_y[, chain])
  graph + ggplot2::geom_violin(
    data = chain_data,
    mapping = ggplot2::aes(x = .data$x, y = .data$y),
    color = chain_clr,
    fill = chain_fill,
    alpha = 0.5
  )
}

.p_hist <- function(df, lab, chain = 0, ...) {
  mdf <- .reshape_df(df) # reshape2::melt(df, id.vars = grep("iteration", colnames(df), value = TRUE))
  dots <- .add_aesthetics(list(...), "linewidth")
  dots$binwidth <- diff(range(mdf$value))/30
  dots$fill <- .NUTS_FILL
  dots$color <- .NUTS_CLR
  base <- ggplot2::ggplot(mdf, ggplot2::aes(x = .data$value)) +
    do.call(ggplot2::geom_histogram, dots) +
    ggplot2::labs(x = if (missing(lab)) NULL else lab, y = "")
  if (chain == 0) {
    graph <- base +
      ggplot2::geom_vline(xintercept = mean(mdf$value), color = .NUTS_VLINE_CLR, linewidth = .8) +
      ggplot2::geom_vline(xintercept = median(mdf$value), color = .NUTS_VLINE_CLR, lty = 2, linewidth = 1)
    return(graph)
  }
  chain_data <- mdf[mdf$variable == paste0("chain:",chain), ]
  chain_clr <- color_vector_chain(ncol(df) - 1)[chain]
  chain_fill <- chain_clr
  base + ggplot2::geom_histogram(data = chain_data,
                        binwidth = diff(range(chain_data$value))/30,
                        fill = chain_fill, alpha = 0.5) +
    ggplot2::geom_vline(xintercept = mean(chain_data$value), color = chain_clr, linewidth = .8) +
    ggplot2::geom_vline(xintercept = median(chain_data$value),
               color = chain_clr, lty = 2, linewidth = 1)
}

.treedepth_ndivergent_hist <- function(df_td, df_nd, chain = 0,
                                       divergent = c("All", 0, 1), ...) {
  x_lab <- if (divergent == "All")
    "Treedepth" else paste0("Treedepth (Divergent = ", divergent,")")
  plot_labs <- ggplot2::labs(x = x_lab, y = "")

  mdf_td <- .reshape_df(df_td) #reshape2::melt(df_td, id.vars = grep("iteration", colnames(df_td), value = TRUE))
  mdf_nd <- .reshape_df(df_nd) #reshape2::melt(df_nd, id.vars = grep("iteration", colnames(df_nd), value = TRUE))
  mdf <- cbind(mdf_td, div = mdf_nd$value)
  plot_data <- if (divergent == "All") mdf else mdf[mdf$div == divergent, , drop = FALSE]
  if (nrow(plot_data) == 0) return(NULL)
  count <- NaN

  graph <- ggplot2::ggplot(
    data = plot_data,
    mapping = ggplot2::aes(x = factor(.data$value)),
    na.rm = TRUE
  ) +
    ggplot2::geom_bar(
      mapping = ggplot2::aes(
        y = ggplot2::after_stat(count) / sum(ggplot2::after_stat(count))
      ),
      width = 1, fill = .NUTS_FILL, color = .NUTS_CLR
    ) +
    plot_labs
  if (chain == 0) return(graph)
  chain_clr <- color_vector_chain(ncol(df_td) - 1)[chain]
  chain_fill <- chain_clr
  chain_data <- plot_data[plot_data$variable == paste0("chain:", chain), , drop = FALSE]
  graph + ggplot2::geom_bar(
    data = chain_data,
    mapping = ggplot2::aes(
      y = ggplot2::after_stat(count) / sum(ggplot2::after_stat(count))
    ),
    fill = chain_fill, width = 1
  )
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

expose_stan_functions_hacks <- function(code, includes = NULL) {
  code <- paste("// [[Rcpp::depends(StanHeaders)]]",
                "// [[Rcpp::depends(rstan)]]",
                "// [[Rcpp::plugins(rstan)]]",
                "// [[Rcpp::depends(RcppEigen)]]",
                "// [[Rcpp::depends(BH)]]",
                "#include <stan/math/prim/fun/Eigen.hpp>",
                "#include <stan/math/prim/meta.hpp>",
                "#include <boost/integer/integer_log2.hpp>",
                "#include <exporter.h>",
                "#include <RcppEigen.h>",
                includes,
                code, sep = "\n")
  code <- gsub("// [[stan::function]]",
               "// [[Rcpp::export]]", code, fixed = TRUE)
  code <- gsub("stan::math::accumulator<double>& lp_accum__,(\\n)?(\\s*)?std::ostream\\*(\\n)?(\\s*)?pstream__(\\n)?(\\s*)?=(\\n)?\\s*)?(nullptr|0))(\\s*)?\\{",
               "std::ostream* pstream__ = nullptr){\nstan::math::accumulator<double> lp_accum__;",
               code)
  code <- gsub("pstream__(\\s*|)=(\\s*|)nullptr", "pstream__ = 0", code)
  return(code)
}

expose_stan_functions <- function(stanmodel, includes = NULL,
                                  show_compiler_warnings = FALSE, ...) {
  mc <- NULL
  if(is(stanmodel, "stanfit")) {
    mc <- get_stancode(get_stanmodel(stanmodel))
  }
  else if(is.list(stanmodel)) {
    mc <- stanmodel$model_code
  }
  else if(is(stanmodel, "stanmodel")) {
    mc <- get_stancode(stanmodel)
  }
  else if(is.character(stanmodel)) {
    if(length(stanmodel) == 1) mc <- get_model_strcode(stanmodel, NULL)
    else mc <- get_model_strcode(model_code = stanmodel)
  }
  else stop("'stanmodel' is not a valid object")

  tf <- tempfile(fileext = ".stan")
  writeLines(mc, con = tf)
  md5 <- paste("user", tools::md5sum(tf), sep = "_")
  stopifnot(stanc(model_code = mc, model_name = "User-defined functions",
                  allow_undefined = TRUE)$status)
  r <- stanc(model_code = mc, model_name = "User-defined functions",
             allow_undefined = TRUE,
             standalone_functions = TRUE)
  code <- expose_stan_functions_hacks(r$cppcode, includes)

  WINDOWS <- .Platform$OS.type == "windows"
  if (WINDOWS && R.version$major < 4) {
    stop("expose_stan_functions requires R >= 4.0 on Windows to use C++17")
  } else {
    has_USE_CXX17 <- Sys.getenv("USE_CXX17") != ""
    Sys.setenv(USE_CXX17 = 1)
    if (!has_USE_CXX17) on.exit(Sys.unsetenv("USE_CXX17"))
  }

  if (rstan_options("required"))
    pkgbuild::has_build_tools(debug = FALSE) ||
    pkgbuild::has_build_tools(debug = TRUE)

  if (WINDOWS) {
    has_march = .warn_march_makevars()
    if (has_march) {
      user_makevar = Sys.getenv("R_MAKEVARS_USER")
      Sys.setenv(R_MAKEVARS_USER = NULL)
      on.exit(Sys.setenv(R_MAKEVARS_USER = user_makevar))
    }
  }

  if (!isTRUE(show_compiler_warnings)) {
    tf <- tempfile(fileext = ".warn")
    zz <- file(tf, open = "wt")
    sink(zz, type = "output")
    on.exit(close(zz), add = TRUE)
    on.exit(sink(type = "output"), add = TRUE)
  }
  Rcpp::registerPlugin("rstan", rstanplugin)
  compiled <- pkgbuild::with_build_tools(try(suppressWarnings(
    Rcpp::sourceCpp(code = paste(code, collapse = "\n"), ...)), silent = TRUE),
    required = rstan_options("required") &&
    # workaround for packages with src/install.libs.R
      identical(Sys.getenv("WINDOWS"), "TRUE") &&
      !identical(Sys.getenv("R_PACKAGE_SOURCE"), "") )
  if (!isTRUE(show_compiler_warnings)) {
    sink(type = "output")
    close(zz)
    try(file.remove(tf), silent = TRUE)
    on.exit(NULL)
  }
  DOTS <- list(...)
  if (isTRUE(DOTS$dryRun)) return(code)
  if (inherits(compiled, "try-error")) stop("Compilation failed!")
  ENV <- DOTS$env
  if (is.null(ENV)) ENV <- globalenv()
  for (x in compiled$functions) {
    FUN <- get(x, envir = ENV)
    args <- formals(FUN)
    args$pstream__ <- get_stream()
    if ("lp__" %in% names(args)) args$lp__ <- 0
    if ("base_rng__" %in% names(args)) args$base_rng__ <- get_rng()
    formals(FUN) <- args
    assign(x, FUN, envir = ENV)
  }
  return(invisible(compiled$functions))
}

test_that("expose_functions works with dryRun", {
  funcode <- "
    functions {
      real rtn_real(real x) {
        return x;
      }
    }"

  expect_no_error(expose_stan_functions(stanc(model_code = funcode), dryRun = TRUE))
})

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

lookup <- function(FUN, ReturnType = character()) {
  if(length(ReturnType) > 0) {
    if(ReturnType == "ANY") return(rosetta[,-1])
    if(ReturnType %in% rosetta$ReturnType) {
      return(rosetta[rosetta$ReturnType == ReturnType,-1,drop=FALSE])
    }
    else return(paste("no matching Stan functions; ReturnType must be 'ANY' or one of:",
                      paste(sort(unique(rosetta$ReturnType)), collapse = ", ")))
  }
  if(is.function(FUN)) FUN <- deparse(substitute(FUN))
  if(!is.character(FUN)) stop("'FUN' must be a character string for a function")
  if(length(FUN) != 1) stop("'FUN' must be of length one")
  
  if(FUN == "nrow") FUN <- "NROW"
  if(FUN == "ncol") FUN <- "NCOL"
  keep_cols <- colnames(rosetta) != "RFunction"
  if(exists(FUN)) {
    matches <- as.logical(charmatch(rosetta$RFunction, FUN, nomatch = 0L))
    if(any(matches)) return(rosetta[matches, keep_cols, drop = FALSE])
  }
  matches <- grepl(FUN, rosetta$StanFunction, ignore.case = TRUE)
  if(any(matches)) return(rosetta[matches, keep_cols, drop = FALSE])
  else return("no matching Stan functions")
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

## Use an environment to keep some options, especially,
## for plotting.

.rstan_opt_env <- new.env()

init_rstan_opt_env <- function(e) {
  tmat <- matrix(c(254, 237, 222,
                   253, 208, 162,
                   253, 174, 107,
                   253, 141, 60,
                   230, 85, 13,
                   166, 54, 3),
                 byrow = TRUE, ncol = 3)

  rhat_cols <- rgb(tmat, alpha = 150, names = paste(1:nrow(tmat)),
                   maxColorValue = 255)

  assign("plot_rhat_breaks", c(1.1, 1.2, 1.5, 2), e)
  # in this default setting,
  # if rhat < rhat.breaks[i], the color is rhat_cols[i]
  assign("plot_rhat_cols", rhat_cols[1:4], e)

  # when R-hat is NA, NaN, or Inf
  assign("plot_rhat_nan_col", rhat_cols[6] , e)
  # when R-hat is large than max(rhat.breaks)
  assign("plot_rhat_large_col", rhat_cols[6], e)

  # color for indicating or important info.
  # for example, the color of star and text saying
  # the variable is truncated in stan_plot_inferences
  assign("rstan_alert_col", rgb(230, 85, 13, maxColorValue = 255), e)

  # color for plot chains in trace plot and stan_plot_inferences
  assign("rstan_chain_cols", rstancolc, e)

  # set the default number of parameters we are considered
  # for plot of stanfit: when the number of parameters in a
  # vector/array parameter is less than
  # what is set here, we would have empty space. But when the
  # number of parameters is larger than max, they are truncated.
  assign('plot_standard_npar', 30, e)
  assign('plot_max_npar', 40, e)

  # color for shading the area of warmup trace plot
  assign("rstan_warmup_bg_col", rstancolgrey[3], e)

#   stan_lib_path  <- system.file('include', 'stanlib', package = 'rstan')
#   boost_dir <- dir(stan_lib_path, pattern = 'boost.*')
  # boost lib path
#   boost_lib_path <- file.path(stan_lib_path, boost_dir)
  boost_lib_path <- system.file('include', package = 'BH')
  eigen_lib_path <- system.file('include', package = 'RcppEigen')
#   eigen_dir <- dir(stan_lib_path, pattern = 'eigen.*')
#   eigen_lib_path <- file.path(stan_lib_path, eigen_dir)
  assign("eigen_lib", eigen_lib_path, e)
  assign("boost_lib", boost_lib_path, e)

  ya_boost  <- system.file('include', 'boost_not_in_BH', package = 'rstan')
  assign('boost_lib2', ya_boost, e)

  assign('auto_write', FALSE, e)

  assign('required', TRUE, e)

  assign('disable_march_warning', FALSE, e)

  assign('threads_per_chain', 1L, e)

  # cat("init_rstan_opt_env called.\n")
  invisible(e)
}

# init_rstan_opt_env(.rstan_opt_env)

rstan_options <- function(...) {
  # Set/get options in RStan
  # Args: any options can be defined, using 'name = value'
  #
  # e <- rstan:::.rstan_opt_env
  e <- .rstan_opt_env
  if (length(as.list(e)) == 0)
    init_rstan_opt_env(e)

  a <-  list(...)
  len <- length(a)
  if (len < 1) return(NULL)
  a_names <- names(a)
  if (is.null(a_names)) { # case like rstan_options("a", "b")
    empty <- rep(TRUE, len)
    empty_len <- len
  } else { # case like rstan_options(a = 3, b = 4, "c")
    empty <- (a_names == '')
    empty_len <- sum(empty)
  }
  for (i in which(empty)) {
    if (!is.character(a[[i]])) stop("rstan_options only accepts arguments as 'name=value' or 'name'")
  }

  r <- if (empty_len < len) mget(a_names[!empty], envir = e, ifnotfound = NA)
  if (empty_len > 0)
    r <- c(r, mget(unlist(a[empty]), envir = e,
                   ifnotfound = list(function(x) { warning("rstan option '", x, "' not found"); NA })))

  # set options
  for (n in a_names[!empty]) {
    if (n == 'plot_rhat_breaks') {
      assign(n, sort(a[[n]]), e)
    } else if (n == 'threads_per_chain') {
      assign(n, max(1L, as.integer(a[[n]]), na.rm = TRUE), e)
      Sys.setenv("STAN_NUM_THREADS" = max(1L, as.integer(a[[n]]), na.rm = TRUE))
    } else {
      assign(n, a[[n]], e)
    }
  }

  if (len == 1) return(invisible(r[[1]]))
  invisible(r)
}

test_that("Getting/setting 0 or 1 options works", {
  rstan:::rstan_options(a = 22, b = 23)
  on.exit(rstan::rstan_options(a = NULL, b = NULL))

  o <- rstan:::rstan_options()
  expect_null(o)

  rstan:::rstan_options(testname = 22)
  expect_equal(rstan:::rstan_options("testname"), 22)
})

test_that("Getting/setting multiple options works", {
  rstan:::rstan_options(a = 22, b = 23)
  on.exit(rstan::rstan_options(a = NULL, b = NULL, c = NULL, d = NULL))

  o <- rstan:::rstan_options("a", "b")
  expect_equal(o$a, 22)
  expect_equal(o$b, 23)
  ov <- rstan:::rstan_options(a = 34)
  expect_equal(ov, 22)
  o <- rstan:::rstan_options("a", "b")
  expect_equal(o$a, 34)
  # Warning that "rstan option 'c' not found"
  # FIXME: Should the warning be tested?
  o <- suppressWarnings(rstan:::rstan_options("a", "b", "c"))
  expect_equal(o$c, NA)
  # Warning that "rstan option 'c' not found"
  # FIXME: Should the warning be tested?
  o <- suppressWarnings(rstan:::rstan_options("a", "b", "c", d = 38))
  expect_equal(o$d, NA)
  expect_equal(rstan:::rstan_options("d"), 38)
})

test_that("Option plot_rhat_breaks works", {
  rstan_options(plot_rhat_breaks = c(2, 1.2))
  o <- rstan_options("plot_rhat_breaks")
  expect_equal(o, c(1.2, 2))

  rstan_options(plot_rhat_breaks = c(1.5, 2, 1.2))
  o <- rstan_options("plot_rhat_breaks")
  expect_equal(o, c(1.2, 1.5, 2))

  on.exit(rstan_options(plot_rhat_break = NULL))
})

# All options used in rstan.
# Note: In the RUnit tests this test case had to checks, so I decided to skip
# it when converting to testthat.
test_that("All rstan options works", {
  skip("Need to add expectations")

  rhat_nan_col <- rstan_options("plot_rhat_nan_col")
  rhat_large_col <- rstan_options("plot_rhat_large_col")
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  rhat_colors <- rstan_options("plot_rhat_cols")
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  rhat_colors <- rstan_options("plot_rhat_cols")
  rhat_legend_cols <- c(
    rhat_colors, rstan_options("plot_rhat_large_col"),
    rstan_options("plot_rhat_nan_col")
  )
  alert_col <- rstan_options("rstan_alert_col")
  chain_cols <- rstan_options("rstan_chain_cols")
  standard_width <- rstan_options("plot_standard_npar")
  max_width <- rstan_options("plot_max_npar")
  rstan_options("eigen_lib")
  rstan_options("boost_lib")
  rstan_options("rstan_chain_cols")
  warmup_col <- rstan_options("rstan_warmup_bg_col")
})

.setUp <- function() {
  rstan:::rstan_options(a = 22, b = 23) 
} 

test_options1 <- function() {
  o <- rstan:::rstan_options() 
  checkTrue(is.null(o)) 
  rstan:::rstan_options(testname = 22) 
  checkEquals(rstan:::rstan_options("testname"), 22) 
} 

test_options2 <- function() {
  o <- rstan:::rstan_options('a', 'b') 
  checkEquals(o$a, 22) 
  checkEquals(o$b, 23) 
  ov <- rstan:::rstan_options(a = 34)
  checkEquals(ov, 22) 
  o <- rstan:::rstan_options('a', 'b') 
  checkEquals(o$a, 34) 
  o <- rstan:::rstan_options('a', 'b', 'c') 
  checkEquals(o$c, NA) 
  o <- rstan:::rstan_options('a', 'b', 'c', d = 38) 
  checkEquals(o$d, NA) 
  checkEquals(rstan:::rstan_options("d"), 38)
} 

test_plot_rhat_breaks <- function() {
  rstan_options(plot_rhat_breaks = c(2, 1.2))
  o <- rstan_options("plot_rhat_breaks")
  checkEquals(o, c(1.2, 2)) 
  rstan_options(plot_rhat_breaks = c(1.5, 2, 1.2))
  o <- rstan_options("plot_rhat_breaks")
  checkEquals(o, c(1.2, 1.5, 2)) 
} 


# all options used in rstan 
test_rstan_options <- function() { 
  rhat_nan_col <- rstan_options("plot_rhat_nan_col")
  rhat_large_col <- rstan_options("plot_rhat_large_col")
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  rhat_colors <- rstan_options("plot_rhat_cols")
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  rhat_colors <- rstan_options("plot_rhat_cols")
  rhat_legend_cols <- c(rhat_colors, rstan_options('plot_rhat_large_col'),
                        rstan_options("plot_rhat_nan_col"))
  alert_col <- rstan_options("rstan_alert_col")
  chain_cols <- rstan_options("rstan_chain_cols")
  standard_width <- rstan_options('plot_standard_npar') 
  max_width <- rstan_options('plot_max_npar') 
  rstan_options("eigen_lib")
  rstan_options("boost_lib")
  rstan_options("rstan_chain_cols")
  warmup_col <- rstan_options("rstan_warmup_bg_col") 
} 

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

setMethod("show", "stanmodel",
          function(object) {
            cat("S4 class stanmodel '", object@model_name, "' coded as follows:\n" ,sep = '')
            cat(object@model_code, "\n")
          })

setGeneric(name = 'optimizing',
           def = function(object, ...) {

             if (is.sparc()) {
               msg <- "optimizing() will likely crash on SPARC."
               if (interactive()) stop(msg, " Run in batch mode to test.")
               else message(msg)
             }
             standardGeneric("optimizing")
})

setGeneric(name = 'vb',
           def = function(object, ...) {

             if (is.sparc()) {
               msg <- "vb() will likely crash on SPARC."
               if (interactive()) stop(msg, " Run in batch mode to test.")
               else message(msg)
             }
             standardGeneric("vb")
})

setGeneric(name = "sampling",
           def = function(object, ...) { standardGeneric("sampling")})

setMethod('get_stancode', signature = "stanmodel",
          function(object, print = FALSE) {
            code <- object@model_code
            if (print) cat(code, "\n")
            return(code)
          })

setGeneric(name = "get_cppcode",
           def = function(object, ...) { standardGeneric("get_cppcode") })

setMethod("get_cppcode", "stanmodel",
          function(object) {
            object@model_cpp$model_cppcode
          })

setGeneric(name = "get_cxxflags",
           def = function(object, ...) { standardGeneric("get_cxxflags") })
setMethod("get_cxxflags", "stanmodel", function(object) { object@dso@cxxflags })

new_empty_stanfit <- function(stanmodel, miscenv = new.env(parent = emptyenv()),
                              model_pars = character(0), par_dims = list(),
                              mode = 2L, sim = list(),
                              inits = list(), stan_args = list()) {
  new("stanfit",
      model_name = stanmodel@model_name,
      model_pars = model_pars,
      par_dims = par_dims,
      mode = mode,
      sim = sim,
      inits = inits,
      stan_args = stan_args,
      stanmodel = stanmodel,
      date = date(),
      .MISC = miscenv)
}


prep_call_sampler <- function(object) {
  if (!is_sm_valid(object))
    stop(paste("the compiled object from C++ code for this model is invalid, possible reasons:\n",
               "  - compiled with save_dso=FALSE;\n",
               "  - compiled on a different platform;\n",
               "  - does not exist (created from reading csv files).", sep = ''))
  if (!is_dso_loaded(object@dso)) {
    # load the dso if available
    grab_cxxfun(object@dso)
  }
}

# This function gets used when a stanmodel instance is created
# in function stan_model.
mk_cppmodule <- function(object) {
  prep_call_sampler(object)
  model_cppname <- object@model_cpp$model_cppname
  mod <- get("module", envir = object@dso@.CXXDSOMISC, inherits = FALSE)
  eval(call("$", mod, paste('stan_fit4', model_cppname, sep = '')))
}

setMethod("vb", "stanmodel",
          function(object, data = list(), pars = NA, include = TRUE,
                   seed = sample.int(.Machine$integer.max, 1),
                   init = 'random',
                   check_data = TRUE, sample_file = tempfile(fileext = '.csv'),
                   algorithm = c("meanfield", "fullrank"),
                   importance_resampling = FALSE,
                   keep_every = 1,
                   ...) {
            if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
              Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
            }

            if (is.list(data) & !is.data.frame(data)) {
              parsed_data <- with(data, parse_data(get_cppcode(object)))
              if (!is.list(parsed_data)) {
                message("failed to get names of data from the model; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
              # for (nm in names(data)) parsed_data[[nm]] <- data[[nm]]
              # parsed_data <- parsed_data[!sapply(parsed_data, is.null)]
              data <- parsed_data
            } else if (is.character(data)) { # names of objects
              data <- try(mklist(data))
              if (is(data, "try-error")) {
                message("failed to create the data; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
            }

            if (check_data) {
              data <- try(force(data))
              if (is(data, "try-error")) {
                message("failed to evaluate the data; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }

              if (!missing(data) && length(data) > 0) {
                data <- try(data_preprocess(data))
                if (is(data, "try-error")) {
                  message("failed to preprocess the data; variational Bayes not done")
                  return(invisible(new_empty_stanfit(object)))
                }
              } else data <- list()
            }
            cxxfun <- grab_cxxfun(object@dso)
            stan_fit_cpp_module <- object@mk_cppmodule(object)
            if (test_221(object@model_cpp$model_cppcode) &&
                stan_fit_cpp_module@constructors[[1]]$nargs == 2L) {
              mod <- try(new(stan_fit_cpp_module, data, as.integer(seed)))
              if (is(mod, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
              sampler <- try(new(stan_fit, mod$fit_ptr()))
              if (is(sampler, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
            } else {
              sampler <- try(new(stan_fit_cpp_module, data, as.integer(seed), cxxfun))
              if (is(sampler, "try-error")) {
                message('failed to create the optimizer; optimization not done')
                return(invisible(list(stanmodel = object)))
              }
            }

            if (is.numeric(init)) init <- as.character(init)
            if (is.function(init)) init <- init()
            if (!is.list(init) && !is.character(init)) {
              message("wrong specification of initial values")
              return(invisible(new_empty_stanfit(object)))
            }
            seed <- check_seed(seed, warn = 1)
            if (is.null(seed))
              return(invisible(list(stanmodel = object)))
            args <- list(init = init, seed = seed, chain_id = 1L,
                         method = "variational",
                         algorithm = match.arg(algorithm))

            if (!is.null(sample_file) && !is.na(sample_file))
              args$sample_file <- writable_sample_file(sample_file)
            dotlist <- list(...)
            is_arg_recognizable(names(dotlist),
                                c("iter", "init_r",
                                  "append_samples",
                                  "elbo_samples",
                                  "eta",
                                  "adapt_engaged",
                                  "eval_elbo",
                                  "grad_samples",
                                  "output_samples",
                                  "adapt_iter",
                                  "tol_rel_obj",
                                  "refresh"),
                                 pre_msg = "passing unknown arguments: ",
                                 call. = FALSE)
            if (!is.null(dotlist$method))  dotlist$method <- NULL
            if (is.null(dotlist$eta)) dotlist$eta <- 1.0
            else {
              if (dotlist$eta <= 0) stop("'eta' must be > 0")
            }

            sfmiscenv <- new.env(parent = emptyenv())
            assign("stan_fit_instance", sampler, envir = sfmiscenv)

            m_pars <- sampler$param_names()
            p_dims <- sampler$param_dims()
            if(!include) {
              if (length(pars) == 1 && is.na(pars)) pars <- "lp__"
              else pars <- setdiff(m_pars, pars)
              if (length(pars) == 0) pars <- "lp__"
            }

            if (!missing(pars) && !any(is.na(pars)) && length(pars) > 0) {
              sampler$update_param_oi(pars)
              m <- which(match(pars, m_pars, nomatch = 0) == 0)
              if (length(m) > 0) {
                message("no parameter ", paste(pars[m], collapse = ', '), "; sampling not done")
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv, m_pars, p_dims, 2L)))
              }
            }
            else pars <- m_pars

            skeleton <- create_skeleton(m_pars, p_dims)
            cC <- sapply(names(skeleton), simplify = FALSE, FUN = function(x) {
              param <- skeleton[[x]]
              if (x == "lp__") TRUE
              else if (x %in% pars) rep(TRUE, length(param))
              else rep(FALSE, length(param))
            })

            vbres <- sampler$call_sampler(c(args, dotlist))
            samples <- read_one_stan_csv(attr(vbres, "args")$sample_file)
            diagnostic_columns <- which(grepl('__$',colnames(samples)))[-1]
            if (length(diagnostic_columns)>0) {
              diagnostics <- samples[-1,diagnostic_columns]
              samples <- samples[,-diagnostic_columns]
            } else {
              diagnostics <- NULL
            }
            pest <- rstan_relist(as.numeric(samples[1,-1]), skeleton[-length(skeleton)])
            means <- sapply(samples[-1,], mean)
            means <- as.matrix(c(means[-1], means[1]))
            colnames(means) <- "chain:1"
            assign("posterior_mean_4all", means, envir = sfmiscenv)
            inits_used <- rstan_relist(attr(vbres, "inits"), skeleton)
            if ("lp__" %in% names(inits_used))  inits_used$lp__ <- NULL
            samples <- cbind(samples[-1,-1,drop=FALSE],
                             "lp__" = samples[-1,1])[,unlist(cC)]
            cC <- cC[sapply(cC, any)] # any(logical()) is FALSE
            count <- 1L
            for (i in seq_along(cC)) {
              len <- length(cC[[i]])
              cC[[i]] <- count:(count + len - 1L)
              count <- count + len
            }
            samples <- samples[,unlist(cC[c(pars, "lp__")]), drop = FALSE]
            fnames_oi <- sampler$param_fnames_oi()
            n_flatnames <- length(fnames_oi)
            iter <- nrow(samples)
            if ("log_g__" %in% colnames(diagnostics)) {
              if (length(extralp <- which(grepl('lp__.1', colnames(samples)))) > 0)
                  samples <- samples[,-extralp]
              lr <- diagnostics$log_p-diagnostics$log_g
              lr[lr == -Inf] <- -800
              p <- suppressWarnings(loo::psis(lr, r_eff = 1))
              p$log_weights <- p$log_weights - log_sum_exp(p$log_weights)
              theta_pareto_k <- suppressWarnings(apply(samples, 2L, function(col) {
                if (all(is.finite(col))) loo::psis(log1p(col^2) / 2 + lr, r_eff = 1)$diagnostics$pareto_k
                else NaN
              }))
              ## todo: change fixed threshold to an option
              if (p$diagnostics$pareto_k > 1) {
                warning("Pareto k diagnostic value is ",
                        round(p$diagnostics$pareto_k,2),
                        ". Resampling is disabled.",
                        " Decreasing tol_rel_obj may help if variational algorithm has terminated prematurely.",
                        " Otherwise consider using sampling instead.", call. = FALSE, immediate. = TRUE)
                #importance_resampling <- FALSE
              } else if (p$diagnostics$pareto_k > 0.7) {
                warning("Pareto k diagnostic value is ",
                        round(p$diagnostics$pareto_k,2),
                        ". Resampling is unreliable.",
                        " Increasing the number of draws or decreasing tol_rel_obj may help.",
                        call. = FALSE, immediate. = TRUE)
              }
              psis <- loo::nlist(pareto_k = p$diagnostics$pareto_k,
                                 n_eff = p$diagnostics$n_eff / keep_every)
              ## importance_resampling
              if (importance_resampling) {
                iter <- ceiling(dim(samples)[1] / keep_every)
                ir_idx <- sample_indices(exp(p$log_weights),
                                         n_draws = iter)
                samples <- samples[ir_idx,]
                ## SIR mcse and n_eff
                w_sir <- as.numeric(table(ir_idx)) / length(ir_idx)
                mcse <- apply(samples[!duplicated(ir_idx),], 2L, function(col) {
                  if (all(is.finite(col))) sqrt(sum(w_sir^2*(col-mean(col))^2))
                  else NaN
                })
                n_eff <- round(apply(samples[!duplicated(ir_idx),], 2L, var) / (mcse^2), digits = 0)
              } else {
                ir_idx <- NULL
                mcse <- rep(NaN, length(theta_pareto_k))
                n_eff <- rep(NaN, length(theta_pareto_k))
              }
              diagnostics <- list(as.list(diagnostics), psis = psis,
                                  theta_pareto_k = theta_pareto_k,
                                  ir_idx = ir_idx, mcse = mcse, n_eff = n_eff)
            } else {
              diagnostics <- list(as.list(diagnostics))
            }
            sim <- list(samples = list(as.list(samples)),
                        diagnostics = diagnostics,
                        iter = iter, thin = keep_every,
                        warmup = 0L,
                        chains = 1L,
                        n_save = iter,
                        est = pest, # point estimate
                        warmup2 = 0L, # number of warmup iters in n_save
                        permutation = list(1:iter),
                        pars_oi = sampler$param_names_oi(),
                        dims_oi = sampler$param_dims_oi(),
                        fnames_oi = fnames_oi,
                        n_flatnames = n_flatnames)
            nfit <- new("stanfit",
                        model_name = object@model_name,
                        model_pars = m_pars,
                        par_dims = p_dims,
                        mode = 0L,
                        sim = sim,
                        # keep a record of the initial values
                        inits = inits_used,
                        stan_args = list(args),
                        stanmodel = object,
                        # keep a ref to avoid garbage collection
                        # (see comments in fun stan_model)
                        date = date(),
                        .MISC = sfmiscenv)
            return(nfit)
          })

setMethod("optimizing", "stanmodel",
          function(object, data = list(),
                   seed = sample.int(.Machine$integer.max, 1),
                   init = 'random', check_data = TRUE, sample_file = NULL,
                   algorithm = c("LBFGS", "BFGS", "Newton"),
                   verbose = FALSE, hessian = FALSE, as_vector = TRUE,
                   draws = 0, constrained = TRUE,
                   importance_resampling = FALSE, ...) {
            if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
              Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
            }

            if (is.list(data) & !is.data.frame(data)) {
              parsed_data <- with(data, parse_data(get_cppcode(object)))
              if (!is.list(parsed_data)) {
                message("failed to get names of data from the model; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
              # for (nm in names(data)) parsed_data[[nm]] <- data[[nm]]
              # parsed_data <- parsed_data[!sapply(parsed_data, is.null)]
              data <- parsed_data
            } else if (is.character(data)) { # names of objects
              data <- try(mklist(data))
              if (is(data, "try-error")) {
                message("failed to create the data; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
            }

            if (check_data) {
              data <- try(force(data))
              if (is(data, "try-error")) {
                message("failed to evaluate the data; sampling not done")
                return(invisible(NULL))
              }

              if (!missing(data) && length(data) > 0) {
                data <- try(data_preprocess(data))
                if (is(data, "try-error")) {
                  message("failed to preprocess the data; optimization not done")
                  return(invisible(list(stanmodel = object)))
                }
              } else data <- list()
            }
            cxxfun <- grab_cxxfun(object@dso)
            sfmiscenv <- new.env(parent = emptyenv())
            stan_fit_cpp_module <- object@mk_cppmodule(object)
            if (test_221(object@model_cpp$model_cppcode) &&
                stan_fit_cpp_module@constructors[[1]]$nargs == 2L) {
              mod <- try(new(stan_fit_cpp_module, data, as.integer(seed)))
              if (is(mod, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
              sampler <- try(new(stan_fit, mod$fit_ptr()))
              if (is(sampler, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
            } else {
              sampler <- try(new(stan_fit_cpp_module, data, as.integer(seed), cxxfun))
              if (is(sampler, "try-error")) {
                message('failed to create the optimizer; optimization not done')
                return(invisible(list(stanmodel = object)))
              }
            }

            m_pars <- sampler$param_names()
            idx_wo_lp <- which(m_pars != "lp__")
            m_pars <- m_pars[idx_wo_lp]
            p_dims <- sampler$param_dims()[idx_wo_lp]
            if (is.numeric(init)) init <- as.character(init)
            if (is.function(init)) init <- init()
            if (!is.list(init) && !is.character(init)) {
              message("wrong specification of initial values")
              return(invisible(list(stanmodel = object)))
            }
            seed <- check_seed(seed, warn = 1)
            if (is.null(seed))
              return(invisible(list(stanmodel = object)))
            args <- list(init = init,
                         seed = seed,
                         method = "optim",
                         algorithm = match.arg(algorithm))

            if (!is.null(sample_file) && !is.na(sample_file))
              args$sample_file <- writable_sample_file(sample_file)
            dotlist <- list(...)
            is_arg_recognizable(names(dotlist),
                                c("iter",
                                  "save_iterations",
                                  "refresh",
                                  "init_alpha",
                                  "tol_obj",
                                  "tol_grad",
                                  "tol_param",
                                  "tol_rel_obj",
                                  "tol_rel_grad",
                                  "history_size"),
                                 pre_msg = "passing unknown arguments: ",
                                 call. = FALSE)
            if (!is.null(dotlist$method))  dotlist$method <- NULL
            if (!verbose && is.null(dotlist$refresh)) dotlist$refresh <- 0L
            optim <- sampler$call_sampler(c(args, dotlist))
            optim$return_code <- attr(optim, "return_code")
            if (optim$return_code != 0) warning("non-zero return code in optimizing")
            attr(optim, "return_code") <- NULL
            fnames <- sampler$param_fnames_oi()
            names(optim$par) <- fnames[-length(fnames)]
            skeleton <- create_skeleton(m_pars, p_dims)
            theta <- rstan_relist(optim$par, skeleton)
            theta <- sampler$unconstrain_pars(theta)
            if (hessian || draws) {
              fn <- function(theta) {
                sampler$log_prob(theta, FALSE, FALSE)
              }
              gr <- function(theta) {
                sampler$grad_log_prob(theta, FALSE)
              }
              H <- optimHess(theta, fn, gr, control = list(fnscale = -1))
              colnames(H) <- rownames(H) <- sampler$unconstrained_param_names(FALSE, FALSE)
              if (hessian) optim$hessian <- H
            }
            if (draws > 0 && is.matrix(R <- try(chol(-H)))) {
                K <- ncol(R)
                R_inv <- backsolve(R, diag(K))
                Z <- matrix(rnorm(K * draws), K, draws)
                theta_tilde <- t(theta + R_inv %*% Z)
                if (importance_resampling) {
                  log_p <- apply(theta_tilde, 1, FUN = function(theta) {
                    sampler$log_prob(theta, adjust_transform = TRUE, gradient = FALSE)
                  })
                  log_g <- colSums(dnorm(Z, log = TRUE)) - sum(log(diag(R_inv)))
                  optim$log_p <- log_p
                  optim$log_g <- log_g
                } else {
                  optim$log_p <- rep(NaN, length(theta))
                  optim$log_g <- rep(NaN, length(theta))
                }
                colnames(theta_tilde) <- colnames(H)
                optim$log_prob <- sampler$log_prob
                optim$grad_log_prob <- sampler$grad_log_prob
            } else {
              theta_tilde <- t(theta)
            }
            if (constrained) {
              theta_tilde <- t(apply(theta_tilde, 1, FUN = function(theta) {
                sampler$constrain_pars(theta)
              }))
              if (NCOL(theta_tilde) != length(optim$par)) theta_tilde <- t(theta_tilde)
            }
            colnames(theta_tilde) <- names(optim$par)
            optim$theta_tilde <- theta_tilde
            if (!as_vector) optim$par <- rstan_relist(optim$par, skeleton)
            return(optim)
          })

setMethod("sampling", "stanmodel",
          function(object, data = list(), pars = NA, chains = 4, iter = 2000,
                   warmup = floor(iter / 2),
                   thin = 1, seed = sample.int(.Machine$integer.max, 1),
                   init = "random", check_data = TRUE,
                   sample_file = NULL, diagnostic_file = NULL, verbose = FALSE,
                   algorithm = c("NUTS", "HMC", "Fixed_param"), #, "Metropolis"),
                   control = NULL, include = TRUE,
                   cores = getOption("mc.cores", 1L),
                   open_progress = interactive() && !isatty(stdout()) &&
                     !identical(Sys.getenv("RSTUDIO"), "1"),
                   show_messages = TRUE, ...) {
            is_arg_deprecated(names(list(...)),
                              c("enable_random_init"),
                              pre_msg = "passing deprecated arguments: ")

            if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
              Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
            }

            objects <- ls()
            if (is.list(data) & !is.data.frame(data)) {
              parsed_data <- with(data, parse_data(get_cppcode(object)))
              if (!is.list(parsed_data)) {
                message("failed to get names of data from the model; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
              # for (nm in names(data)) parsed_data[[nm]] <- data[[nm]]
              # parsed_data <- parsed_data[!sapply(parsed_data, is.null)]
              data <- parsed_data
            } else if (is.character(data)) { # names of objects
              data <- try(mklist(data))
              if (is(data, "try-error")) {
                message("failed to create the data; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
            }
            # check data and preprocess
            if (verbose)
              cat('\n', "CHECKING DATA AND PREPROCESSING FOR MODEL '", object@model_name,
                    "' NOW.\n", sep = '')
            if (check_data) {
              data <- try(force(data))
              if (is(data, "try-error")) {
                message("failed to evaluate the data; sampling not done")
                return(invisible(new_empty_stanfit(object)))
              }
              if (!missing(data) && length(data) > 0) {
                data <- try(data_preprocess(data))
                if (is(data, "try-error")) {
                  message("failed to preprocess the data; sampling not done")
                  return(invisible(new_empty_stanfit(object)))
                }
              } else data <- list()
            }
            if (verbose)
              cat('\n', "COMPILING MODEL '", object@model_name,
                  "' NOW.\n", sep = '')
            dots <- list(...)
            data$CHAIN_ID <- dots$chain_id
            if (verbose)
              cat('\n', "STARTING SAMPLER FOR MODEL '", object@model_name,
                  "' NOW.\n", sep = '')
            sfmiscenv <- new.env(parent = emptyenv())
            stan_fit_cpp_module <- object@mk_cppmodule(object)
            cxxfun <- grab_cxxfun(object@dso)
            if (test_221(object@model_cpp$model_cppcode) &&
                stan_fit_cpp_module@constructors[[1]]$nargs == 2L) {
              mod <- try(new(stan_fit_cpp_module, data, as.integer(seed)))
              if (is(mod, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
              sampler <- try(new(stan_fit, mod$fit_ptr()))
              if (is(sampler, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
            } else {
              sampler <- try(new(stan_fit_cpp_module, data, as.integer(seed), cxxfun))
              if (is(sampler, "try-error")) {
                message('failed to create the sampler; sampling not done')
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
              }
            }

            assign("stan_fit_instance", sampler, envir = sfmiscenv)
            m_pars = sampler$param_names()
            p_dims = sampler$param_dims()
            mode <- if (!is.null(dots$test_grad) && dots$test_grad)
              "TESTING GRADIENT" else "SAMPLING"

            if (is.numeric(init)) init <- as.character(init)
            if (is.function(init)) {
              if ("chain_id" %in% names(formals(init)))
                init <- lapply(1:chains, FUN = init)
              else
                init <- lapply(1:chains, function(id) init())
            }
            if (!is.list(init) && !is.character(init)) {
              message("wrong specification of initial values")
              return(invisible(new_empty_stanfit(object)))
            }
            if (is.list(list)) init <- lapply(init, function(x) x)

            if (cores > 1 && mode == "SAMPLING" && chains > 1) {
              .dotlist <- c(sapply(objects, simplify = FALSE, FUN = get,
                                  envir = environment()), list(...))
              .dotlist$chains <- 1L
              .dotlist$cores <- 0L
              .dotlist$open_progress <- FALSE
              callFun <- function(i) {
                .dotlist$chain_id <- i
                if(is.list(.dotlist$init)) .dotlist$init <- .dotlist$init[i]
                if(is.character(.dotlist$sample_file)) {
                  if (grepl("\\.csv$", .dotlist$sample_file))
                    .dotlist$sample_file <- sub("\\.csv$", paste0("_", i, ".csv"),
                                                .dotlist$sample_file)
                  else .dotlist$sample_file <- paste0(.dotlist$sample_file,
                                                      "_", i, ".csv")
                }
                if(is.character(.dotlist$diagnostic_file)) {
                  if (grepl("\\.csv$", .dotlist$diagnostic_file))
                    .dotlist$diagnostic_file <- sub("\\.csv$", paste0("_", i, ".csv"),
                                                    .dotlist$diagnostic_file)
                  else
                    .dotlist$diagnostic_file <- paste0(.dotlist$diagnostic_file,
                                                       "_", i, ".csv")
                }
                out <- do.call(rstan::sampling, args = .dotlist)
                return(out)
              }
              if ( .Platform$OS.type == "unix" &&
                   (!interactive() || isatty(stdout())) ) {
                nfits <- parallel::mclapply(1:chains, FUN = callFun,
                                            mc.preschedule = FALSE,
                                            mc.cores = min(chains, cores))
              }
              else {
                tfile <- tempfile()
                sinkfile <- paste0(tfile, "_StanProgress.txt")
                cat("Click the Refresh button to see progress of the chains\n", file = sinkfile)
                if (open_progress &&
                    !identical(browser <- getOption("browser"), "false")) {
                  if (identical(Sys.getenv("RSTUDIO"), "1"))
                    stop("you cannot specify 'open_progress = TRUE' when using RStudio")
                  sinkfile_html <- paste0(tfile, "_StanProgress.html")
                  create_progress_html_file(sinkfile_html, sinkfile)
                  utils::browseURL(paste0("file://", sinkfile_html))
                }
                else if (identical(Sys.getenv("RSTUDIO"), "1") &&
                         .Platform$OS.type == "windows" && interactive()) {
                  if (!requireNamespace("rstudioapi"))
                    stop("must install the rstudioapi package when using RStan in parallel via RStudio")
                  if (rstudioapi::isAvailable("0.98.423")) {
                    v <- rstudioapi::getVersion()
                    if (v > "0.99.100") rstudioapi::viewer(sinkfile, height = "maximize")
                    else if (v >= "0.98.423") rstudioapi::viewer(sinkfile)
                  }
                  else sinkfile <- ""
                }
                else sinkfile <- ""
                if (!is.null(dots$refresh) && dots$refresh == 0)
                  cl <- parallel::makeCluster(min(cores, chains), useXDR = FALSE,
                                              setup_strategy = "sequential")
                else
                  cl <- parallel::makeCluster(min(cores, chains), outfile = sinkfile, 
                                              useXDR = FALSE, setup_strategy = "sequential")
                on.exit(parallel::stopCluster(cl))
                dependencies <- c("rstan", "Rcpp", "ggplot2")
                .paths <- unique(c(.libPaths(), sapply(dependencies, FUN = function(d) {
                  dirname(system.file(package = d))
                })))
                .paths <- .paths[.paths != ""]
                parallel::clusterExport(cl, varlist = ".paths", envir = environment())
                parallel::clusterEvalQ(cl, expr = .libPaths(.paths))
                parallel::clusterEvalQ(cl, expr =
                                      suppressPackageStartupMessages(require(rstan, quietly = TRUE)))
                parallel::clusterExport(cl, varlist = ".dotlist", envir = environment())
                data_e <- as.environment(data)
                parallel::clusterExport(cl, varlist = names(data_e), envir = data_e)
                nfits <- parallel::parLapplyLB(cl, X = 1:chains, fun = callFun)
              }
              valid <- sapply(nfits, is, class2 = "stanfit") &
                       sapply(nfits, FUN = function(x) x@mode == 0)
              if(all(valid)) {
                nfits <- sflist2stanfit(nfits)
                nfits@.MISC <- sfmiscenv
                throw_sampler_warnings(nfits)
                return(nfits)
              }
              else {
                warning("some chains had errors; consider specifying chains = 1 to debug")
                message("here are whatever error messages were returned")
                print(nfits[!valid])
                if(any(valid)) {
                  nfits <- sflist2stanfit(nfits[valid])
                  nfits@.MISC <- sfmiscenv
                  throw_sampler_warnings(nfits)
                  return(nfits)
                }
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv,
                                                   m_pars, p_dims, 2L)))
              }
            }
            check_unknown_args <- dots$check_unknown_args
            if (is.null(check_unknown_args) || check_unknown_args) {
              is_arg_recognizable(names(dots),
                                  c("chain_id", "init_r", "test_grad",
                                    "obfuscate_model_name",
                                    "enable_random_init",
                                    "append_samples", "refresh", "control",
                                    "include", "cores", "open_progress",
                                    "save_warmup"),
                                  pre_msg = "passing unknown arguments: ",
                                  call. = FALSE)
            }
            if(!include) {
              if (length(pars) == 1 && is.na(pars)) pars <- "lp__"
              else pars <- setdiff(m_pars, pars)
              if (length(pars) == 0) pars <- "lp__"
            }

            if (!missing(pars) && !any(is.na(pars)) && length(pars) > 0) {
              sampler$update_param_oi(pars)
              m <- which(match(pars, m_pars, nomatch = 0) == 0)
              if (length(m) > 0) {
                message("no parameter ", paste(pars[m], collapse = ', '), "; sampling not done")
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv, m_pars, p_dims, 2L)))
              }
            }

            if (chains < 1) {
              message("the number of chains is less than 1; sampling not done")
              return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv, m_pars, p_dims, 2L)))
            }

            args_list <- try(config_argss(chains = chains, iter = iter,
                                          warmup = warmup, thin = thin,
                                          init = init, seed = seed, sample_file = sample_file,
                                          diagnostic_file = diagnostic_file,
                                          algorithm = match.arg(algorithm), control = control, ...))

            if (is(args_list, "try-error")) {
              message('error in specifying arguments; sampling not done')
              return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv, m_pars, p_dims, 2L)))
            }

            # number of samples saved after thinning
            warmup2 <- 1 + (warmup - 1) %/% thin
            if (!is.null(dots$save_warmup) && !dots$save_warmup)
              warmup2 <- 0L
            n_kept <- 1 + (iter - warmup - 1) %/% thin
            n_save <- n_kept + warmup2

            samples <- vector("list", chains)

            for (i in 1:chains) {
              cid <- args_list[[i]]$chain_id
              if (is.null(dots$refresh) || dots$refresh > 0) {
                cat('\n', mode, " FOR MODEL '", object@model_name,
                    "' NOW (CHAIN ", cid, ").\n", sep = '')
              }
              if (is.character(show_messages))
                messages <- normalizePath(show_messages, mustWork = FALSE)
              else messages <- tempfile()
              mfile <- file(messages, open = "wt")
              sink(mfile, type = "message")
              samples_i <- try(sampler$call_sampler(args_list[[i]]))
              sink(NULL, type = "message")
              if (!file.exists(messages)) {
                report <- "log file disappeared"
              }
              else {
                close(mfile)
                report <- scan(file = messages, what = character(),
                               sep = "\n", quiet = TRUE)
                unlink(messages)
              }
              if (is(samples_i, "try-error") || is.null(samples_i)) {
                print(report)
                msg <- "error occurred during calling the sampler; sampling not done"
                if (.Platform$OS.type == "windows") print(msg)
                else message(msg)
                return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv,
                                                   m_pars, p_dims, 2L)))
              }
              if (length(report) > 0 && isTRUE(show_messages)) {
                end <- unique(grep("if ", report, ignore.case = TRUE, value = TRUE))
                report <- grep("if ", report, ignore.case = TRUE, value = TRUE, invert = TRUE)
                report <- gsub(" because of the following issue:", "", report, fixed = TRUE)
                report <- grep("^Exception thrown at line", report, value = TRUE)
                report <- gsub("stan::math::", "", report, fixed = TRUE)
                report <- strtrim(report, width = 100)
                if (length(report) > 0) {
                  tab <- sort(table(report), decreasing = TRUE)
                  msg <- paste("The following numerical problems occurred",
                               "the indicated number of times on chain",
                               cid)
                  if (.Platform$OS.type == "windows") print(msg)
                  else message(msg)
                  mat <- as.matrix(tab)
                  colnames(mat) <- "count"
                  if (.Platform$OS.type == "windows") {
                    print(mat)
                    print("When a numerical problem occurs, the Hamiltonian proposal gets rejected.")
                    print("See https://mc-stan.org/misc/warnings.html#exception-hamiltonian-proposal-rejected")
                    print(paste("If the number in the 'count' column is small, ",
                                "there is no need to ask about this message on stan-users."))
                  }
                  else {
                    message(paste(capture.output(print(mat)), collapse = "\n"))
                    message("When a numerical problem occurs, the Hamiltonian proposal gets rejected.")
                    message("See https://mc-stan.org/misc/warnings.html#exception-hamiltonian-proposal-rejected")
                    message("If the number in the 'count' column is small, ",
                            "there is no need to ask about this message on stan-users.")
                  }
                }
              }
              samples[[i]] <- samples_i
            }

            idx_wo_lp <- which(m_pars != 'lp__')
            skeleton <- create_skeleton(m_pars[idx_wo_lp], p_dims[idx_wo_lp])
            inits_used = lapply(lapply(samples, function(x) attr(x, "inits")),
                                function(y) rstan_relist(y, skeleton))

            # test_gradient mode: no sample
            if (attr(samples[[1]], 'test_grad')) {
              sim = list(num_failed = sapply(samples, function(x) x$num_failed))
              return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv,
                                                 m_pars, p_dims, 1L, sim = sim,
                                                 inits = inits_used,
                                                 stan_args = args_list)))
            }

            # perm_lst <- lapply(1:chains, function(id) rstan_seq_perm(n_kept, chains, seed, chain_id = id))
            ## sample_int is a little bit faster than our own rstan_seq_perm (one
            ## reason is that the RNG used in R is faster),
            ## but without controlling the seed
            perm_lst <- lapply(1:chains, function(id) sample.int(n_kept))

            fnames_oi <- sampler$param_fnames_oi()
            n_flatnames <- length(fnames_oi)
            sim = list(samples = samples,
                       iter = iter, thin = thin,
                       warmup = warmup,
                       chains = chains,
                       n_save = rep(n_save, chains),
                       warmup2 = rep(warmup2, chains), # number of warmup iters in n_save
                       permutation = perm_lst,
                       pars_oi = sampler$param_names_oi(),
                       dims_oi = sampler$param_dims_oi(),
                       fnames_oi = fnames_oi,
                       n_flatnames = n_flatnames)
            nfit <- new("stanfit",
                        model_name = object@model_name,
                        model_pars = m_pars,
                        par_dims = p_dims,
                        mode = 0L,
                        sim = sim,
                        # keep a record of the initial values
                        inits = inits_used,
                        stan_args = args_list,
                        stanmodel = object,
                          # keep a ref to avoid garbage collection
                          # (see comments in fun stan_model)
                        date = date(),
                        .MISC = sfmiscenv)
            if (cores > 0) throw_sampler_warnings(nfit)
            return(nfit)
          })


setGeneric(name = "gqs",
           def = function(object, ...) { standardGeneric("gqs") })
setMethod("gqs", "stanmodel",
          function(object, data = list(), draws,
                seed = sample.int(.Machine$integer.max, size = 1L)) {
  draws <- as.matrix(draws)
  objects <- ls()
  if (is.list(data) & !is.data.frame(data)) {
    parsed_data <- with(data, parse_data(get_cppcode(object)))
    if (!is.list(parsed_data)) {
      message("failed to get names of data from the model; sampling not done")
      return(invisible(new_empty_stanfit(object)))
    }
    data <- parsed_data
  } else if (is.character(data)) { # names of objects
    data <- try(mklist(data))
    if (is(data, "try-error")) {
      message("failed to create the data; sampling not done")
      return(invisible(new_empty_stanfit(object)))
    }
  }
  if (TRUE) { # check_data
    data <- try(force(data))
    if (is(data, "try-error")) {
      message("failed to evaluate the data; sampling not done")
      return(invisible(new_empty_stanfit(object)))
    }
    if (!missing(data) && length(data) > 0) {
      data <- try(data_preprocess(data))
      if (is(data, "try-error")) {
        message("failed to preprocess the data; sampling not done")
        return(invisible(new_empty_stanfit(object)))
      }
    } else data <- list()
  }
  stan_fit_cpp_module <- object@mk_cppmodule(object)
  cxxfun <- grab_cxxfun(object@dso)
  sfmiscenv <- new.env(parent = emptyenv())
  stan_fit_cpp_module <- object@mk_cppmodule(object)
  cxxfun <- grab_cxxfun(object@dso)
  if (test_221(object@model_cpp$model_cppcode) &&
      stan_fit_cpp_module@constructors[[1]]$nargs == 2L) {
    mod <- try(new(stan_fit_cpp_module, data, as.integer(seed)))
    if (is(mod, "try-error")) {
      message('failed to create the sampler; sampling not done')
      return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
    }
    sampler <- try(new(stan_fit, mod$fit_ptr()))
    if (is(sampler, "try-error")) {
      message('failed to create the sampler; sampling not done')
      return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
    }
  } else {
    sampler <- try(new(stan_fit_cpp_module, data, as.integer(seed), cxxfun))
    if (is(sampler, "try-error")) {
      message('failed to create the sampler; sampling not done')
      return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv)))
    }
  }
  
  assign("stan_fit_instance", sampler, envir = sfmiscenv)
  m_pars = sampler$param_names()
  p_dims = sampler$param_dims()
  p_names <- unique(sub("\\..*$", "", sampler$constrained_param_names(TRUE, FALSE)))
  all_names <- sampler$constrained_param_names(TRUE, TRUE)
  some_names <- sampler$constrained_param_names(TRUE, FALSE)
  # exclude transformed parameters from draws
  draws_colnames <- sub("\\.", "[", sampler$constrained_param_names(FALSE, FALSE))
  draws_colnames <- gsub("\\.", ",", draws_colnames)
  draws_colnames[grep("\\[", draws_colnames)] <- 
    paste0(draws_colnames[grep("\\[", draws_colnames)], "]")
  draws <- draws[, draws_colnames, drop = FALSE]
  gq_names <- unique(sub("\\..*$", "", setdiff(all_names, some_names)))
  sampler$update_param_oi(gq_names)
  samples <- try(sampler$standalone_gqs(draws, as.integer(seed)))
  if (is(samples, "try-error") || is.null(samples)) {
    msg <- "error occurred during calling the sampler; sampling not done"
    message(msg)
    return(invisible(new_empty_stanfit(object, miscenv = sfmiscenv,
                                       m_pars, p_dims, 2L)))
  }

  skeleton <- create_skeleton(gq_names, p_dims[gq_names])

  perm_lst <- list(1:nrow(draws)) # not actually permuted

  fnames_oi <- setdiff(sampler$param_fnames_oi(), "lp__")
  n_flatnames <- length(fnames_oi)
  sim = list(samples = list(samples),
             iter = nrow(draws), thin = 1L,
             warmup = 0L,
             chains = 1L,
             n_save = nrow(draws),
             warmup2 = 0L, # number of warmup iters in n_save
             permutation = perm_lst,
             pars_oi = gq_names,
             dims_oi = sampler$param_dims_oi()[gq_names],
             fnames_oi = fnames_oi,
             n_flatnames = n_flatnames)
  nfit <- new("stanfit",
              model_name = object@model_name,
              model_pars = gq_names,
              par_dims = p_dims[gq_names],
              mode = 0L,
              sim = sim,
              # keep a record of the initial values
              inits = list(),
              stan_args = list(list(seed = seed)),
              stanmodel = object,
              # keep a ref to avoid garbage collection
              # (see comments in fun stan_model)
              date = date(),
              .MISC = sfmiscenv)
  return(nfit)
})

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

rstan_load_time <- as.POSIXct("1970-01-01 00:00.00 UTC")
RNG <- 0
OUT <- 0

.onLoad <- function(libname, pkgname) {
  if (requireNamespace("V8", quietly = TRUE)) {
    assign("stanc_ctx", V8::v8(), envir = topenv())
  } else assign("stanc_ctx", QuickJSR::JSContext$new(stack_size = 4 * 1024 * 1024), envir = topenv())

  if (packageVersion("StanHeaders") == "2.26.28") {
    stanc_js <- system.file("exec", "stanc.js", package = "rstan", mustWork = TRUE)
  } else stanc_js <- system.file("stanc.js", package = "StanHeaders", mustWork = TRUE)
  stanc_ctx$source(stanc_js)
  assignInMyNamespace("rstan_load_time", value = Sys.time())
  set_rstan_ggplot_defaults()
  assignInMyNamespace("RNG", value = get_rng(0))
  assignInMyNamespace("OUT", value = get_stream())
  Rcpp::loadModule("class_model_base", what = TRUE)
  Rcpp::loadModule("class_stan_fit", what = TRUE)
}

.onAttach <- function(...) {
  packageStartupMessage("\nrstan version ",
                        utils::packageVersion("rstan"),
                        " (Stan version ",
                        stan_version(), ")\n")
  packageStartupMessage("For execution on a local, multicore CPU with excess RAM we recommend calling\n",
                        "options(mc.cores = parallel::detectCores()).\n",
                        "To avoid recompilation of unchanged Stan programs, we recommend calling\n",
                        "rstan_options(auto_write = TRUE)",
                        "\nFor within-chain threading using `reduce_sum()` or `map_rect()` Stan functions,\n",
                        "change `threads_per_chain` option:\n",
                        paste0("rstan_options(threads_per_chain = ",
                               rstan_options("threads_per_chain"), ")\n"))
  if (.Platform$OS.type == "windows") {
    packageStartupMessage("Do not specify '-march=native' in 'LOCAL_CPPFLAGS' or a Makevars file")
  }
}

.onUnload <- function(libpath) {
   # unload the package library
   library.dynam.unload("rstan", libpath)
}

gqs_light <- function(stanc_ret) {
  gqs <- c(
  "(Rcpp::NumericMatrix draws) {",
      "std::vector<std::string> param_names;",
      "constrained_param_names(param_names, false, true);",
      "std::vector<std::string> gq_names = param_names;",
      "size_t upper_bound = param_names.size();",
      "param_names.clear();",
      "constrained_param_names(param_names, false, false);",
      "size_t lower_bound = param_names.size();",
      "size_t num_gqs = upper_bound - lower_bound;",
      "std::vector<std::vector<size_t> > param_dimss;",
      "get_dims(param_dimss); // does this include lp__?",
      "param_dimss.erase(param_dimss.begin() + lower_bound, param_dimss.end());",
      "gq_names.erase(gq_names.begin(), gq_names.begin() + lower_bound);",

      "std::vector<int> dummy_params_i;",
      "std::vector<double> unconstrained_params_r;",
      "std::vector<double> gqs;",
      "std::vector<double> draws_i(draws.cols());",
      "std::stringstream msg;",
      "Rcpp::NumericMatrix output(draws.rows(), num_gqs);",
      "Rcpp::CharacterVector cn(num_gqs);",
      "for (size_t j = 0; j < num_gqs; ++j)",
        "cn(j) = gq_names[j];",
      "Rcpp::colnames(output) = cn;",
      
      "for (size_t i = 0; i < draws.rows(); ++i) {",
        "dummy_params_i.clear();",
        "unconstrained_params_r.clear();",
        "for (size_t j = 0; j < draws_i.size(); j++)",
          "draws_i[j] = draws(i, j);",
        "try {",
          "stan::io::array_var_context context(param_names, draws_i,",
                                              "param_dimss);",
          "transform_inits(context, dummy_params_i, unconstrained_params_r,",
                          "&msg);",
        "} catch (const std::exception& e) {",
          "throw std::runtime_error(e.what());",
        "}",
        "if (i % 100 == 0) Rcpp::checkUserInterrupt();",
        "write_array<boost_random_R>(base_rng__, unconstrained_params_r,",
                                    "dummy_params_i, gqs, false, true, pstream__);",
        "for (size_t j = 0; j < num_gqs; j++)",
          "output(i, j) = gqs[j];",
      "}",
      "return output;",
    "}")
  
  stanc_ret_ <- doctor_cppcode(stanc_ret, use_R_PRNG = TRUE, use_Rcout = TRUE,
                               detemplate = TRUE, double_only = TRUE, propto__ = TRUE,
                               make_data_public = FALSE, drop_Eigen = TRUE,
                               drop_log_prob = TRUE, drop_model_header = TRUE,
                               return_names = TRUE, return_dims = TRUE, 
                               add_methods = list(gqs = gqs),
                               methods_for_user_defined_functions = TRUE)
  out <- exposeStanClass(stanc_ret_, field_access = "read_write")
  return(out)
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

delete_stan_demo_folder <- function() {
  MODELS_HOME <- system.file('include', 'example-models-master', package = 'rstan')
  unlink(MODELS_HOME, recursive = TRUE) 
}

stan_demo <-
function(model = character(0), 
         method = c("sampling", "optimizing", "meanfield", "fullrank"), ...) {
  if(is.numeric(model)) {
    MODEL_NUM <- as.integer(model)
    model <- character(0)
  }
  else MODEL_NUM <- -1L
  MODELS_HOME <- system.file('include', 'example-models-master', package = 'rstan')
  if(MODELS_HOME == "") {
    MODELS_HOME <- file.path(tempdir(), "example-models-master")
    if(!file.exists(MODELS_HOME)) {
      writable <- file.access(system.file('include', package = 'rstan'), 
                              mode = 2) == 0
      choices <- c(if(writable) "Download to include directory of the rstan package",
                   "Download to the temporary directory", "Do not download")
      if(interactive()) choice <- menu(choices, title = "Do you want to download the example models?")
      else choice <- if(writable) 2L else 1L
      if(choice == 0 | choice == length(choices)) stop("cannot proceed without example models")
      if(choices[choice] == "Download to include directory of the rstan package") {
        FILE <- file.path(system.file('include', package = 'rstan'),
                          "example-models-master.zip")
      }
      else FILE <- file.path(tempdir(), "example-models-master.zip")
      URL <- "https://github.com/stan-dev/example-models/archive/master.zip"
      on.exit(cat(paste("Download of example-models failed. Try unziping\n", 
                        URL, "\nin", dirname(FILE))))
      code <- download.file(URL, FILE)
      if (code != 0) stop()
      else on.exit(NULL)
      unzip(FILE, exdir = dirname(FILE))
      MODELS_HOME <- file.path(dirname(FILE), "example-models-master")
    }
  }
  WD <- getwd()
  on.exit(setwd(WD))
  setwd(MODELS_HOME)
  MODELS <- dir(MODELS_HOME, pattern = paste0(model, ".stan", "$"), 
                recursive = TRUE, full.names = FALSE)
  if(length(MODELS) == 0) {
    stop("'model' not found; leave 'model' unspecified to see all choices")
  }
  else if(length(MODELS) > 1) {
    if(MODEL_NUM %in%  1:length(MODELS)) {
      MODELS <- MODELS[MODEL_NUM]
    }
    else if(MODEL_NUM == 0) MODELS <- ""
    else if(interactive()) MODELS <- select.list(MODELS)
    else MODELS <- MODELS[1]
    if(!nzchar(MODELS)) {
      return(dir(MODELS_HOME, pattern = paste0(model, ".stan", "$"), 
                 recursive = TRUE, full.names = FALSE))
    }
    model <- sub(".stan$", "", basename(MODELS))
  }
  MODEL_HOME <- dirname(MODELS)
  STAN_ENV <- new.env()
  if(file.exists(fp <- file.path(MODEL_HOME, paste0(model, ".data.R")))) {
    source(fp, local = STAN_ENV, verbose = FALSE, echo = TRUE)
  }
  else {
    Rfiles <- dir(MODEL_HOME, pattern = "R$", full.names = TRUE)
    if(length(Rfiles) > 0) 
      sapply(Rfiles, FUN = source, 
             local = STAN_ENV, verbose = FALSE, echo = TRUE)
  }
  method <- match.arg(method)
  dots <- list(...)
  if(is.null(dots$object)) dots$object <- stan_model(MODELS, model_name = model)
  dots$data <- STAN_ENV
  if(method == "sampling") return(do.call(sampling, args = dots))
  else if (method == "optimizing") return(do.call(optimizing, args = dots))
  else {
    dots$algorithm <- method
    return(do.call(vb, args = dots))
  }
}

# This file is part of RStan
# Copyright (C) 2015, 2022 Jiqiang Guo, Benjamin Goodrich, and Krzysztof Sakrejda
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

extract_sparse_parts <- function(A) {
  if (!requireNamespace("Matrix")) 
    stop("You have to install the Matrix package to call 'extract_sparse_parts'")
  if (!is(A, 'Matrix')) 
    A <- Matrix::Matrix(A, sparse = TRUE, doDiag = FALSE)
  A <- Matrix::t(A)
  A <- as(as(as(A, "CsparseMatrix"), "generalMatrix"), "dMatrix")
  return(.Call(extract_sparse_components, A))
}

test_that("extract_sparse_parts works", {
  A <- rbind(
    c(19L, 27L,  0L,  0L),
    c( 0L,  0L,  0L,  0L),
    c( 0L,  0L,  0L, 52L),
    c(81L,  0L, 95L, 33L)
  )
  w <- c(19, 27, 52, 81, 95, 33)
  v <- c(1, 2, 4, 1, 3, 4)
  u <- c(1, 3, 3, 4, 7)
  parts <- rstan::extract_sparse_parts(A)
  expect_equal(parts$w, w)
  expect_equal(parts$v, v)
  expect_equal(parts$u, u)

  parts <- rstan::extract_sparse_parts(A * 1.0)
  expect_equal(parts$w, w)
  expect_equal(parts$v, v)
  expect_equal(parts$u, u)

  parts <- rstan::extract_sparse_parts(Matrix::Matrix(A))
  expect_equal(parts$w, w)
  expect_equal(parts$v, v)
  expect_equal(parts$u, u)
})

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

setMethod("show", "stanfit", 
          function(object) {
            print.stanfit(x = object, pars = object@sim$pars_oi)
          })  

print.stanfit <- function(x, pars = x@sim$pars_oi, 
                          probs = c(0.025, 0.25, 0.5, 0.75, 0.975), 
                          digits_summary = 2, include = TRUE, ...) { 
  if (x@mode == 1L) { 
    cat("Stan model '", x@model_name, "' is of mode 'test_grad';\n",
        "sampling is not conducted.\n", sep = '')
    return(invisible(NULL)) 
  } else if (x@mode == 2L) {
    cat("Stan model '", x@model_name, "' does not contain samples.\n", sep = '') 
    return(invisible(NULL)) 
  } 

  if(!include) pars <- setdiff(x@sim$pars_oi, pars)
  s <- summary(x, pars, probs, ...)  
  if (is.null(s)) return(invisible(NULL))
  n_kept <- x@sim$n_save - x@sim$warmup2
  cat("Inference for Stan model: ", x@model_name, '.\n', sep = '')
  cat(x@sim$chains, " chains, each with iter=", x@sim$iter, 
      "; warmup=", x@sim$warmup, "; thin=", x@sim$thin, "; \n", 
      "post-warmup draws per chain=", n_kept[1], ", ", 
      "total post-warmup draws=", sum(n_kept), ".\n\n", sep = '')

  # round n_eff to integers
  s$summary[, 'n_eff'] <- round(s$summary[, 'n_eff'], 0)

  print(round(s$summary, digits_summary), ...) 

  sampler <- attr(x@sim$samples[[1]], "args")$sampler_t

  if (!is.null(x@stan_args[[1]]$method) && 
        isTRUE(x@stan_args[[1]]$method == "variational")) {
      if ("diagnostics" %in% names(x@sim)
          & "ir_idx" %in% names(x@sim$diagnostics)
          & !is.null(x@sim$diagnostics$ir_idx)) {
        cat("\nApproximate samples were drawn using VB(", x@stan_args[[1]]$algorithm, 
            ") + PSIS at ", x@date, ".\n", sep = '')
    } else {
      cat("\nApproximate samples were drawn using VB(", x@stan_args[[1]]$algorithm, 
          ") at ", x@date, ".\n", sep = '')
      message("We recommend genuine 'sampling' from the posterior distribution for final inferences!")
    }
    return(invisible(NULL))
  } else {
      cat("\nSamples were drawn using ", sampler, " at ", x@date, ".\n",
          "For each parameter, n_eff is a crude measure of effective sample size,\n", 
          "and Rhat is the potential scale reduction factor on split chains (at \n",
          "convergence, Rhat=1).\n", sep = '')
      return(invisible(NULL))
  }
}

setMethod("plot", signature(x = "stanfit", y = "missing"), 
          function(x, ..., plotfun) {
            if (x@mode == 1L) {
              cat("Stan model '", x@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (x@mode == 2L) {
              cat("Stan model '", x@model_name, 
                  "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 
            if (isTRUE(all.equal(x@sim$n_save, x@sim$warmup2,
                                 check.attributes = FALSE, check.names = FALSE))) {
              cat("Stan model '", x@model_name, 
                  "' does not contain samples after warmup.\n", sep = '')
              return(invisible(NULL))
            }
            
            if (missing(plotfun)) fun <- "stan_plot" 
            else {
              plotters <- paste0("stan_", c("plot", "trace", "scat", "hist", "dens", "ac",
                                            "diag", "rhat", "ess", "mcse", "par"))
              fun <- grep(paste0(plotfun, "$"), plotters, value = TRUE)
              if (!length(fun)) stop("Plotting function not found.")
              else fun <- match.fun(fun)
            }
            do.call(fun, list(object = x, ...))
          }) 

setGeneric(name = "get_stancode",
           def = function(object, ...) { standardGeneric("get_stancode")}) 

setGeneric(name = "get_cppo_mode", 
           def = function(object, ...) { standardGeneric("get_cppo_mode") }) 

setMethod('get_cppo_mode', signature = "stanfit", 
           function(object) { 
             cxxf <- get_cxxflags(object@stanmodel)
             if (identical(cxxf, character(0))) return(NA)
             l <- get_cxxo_level(cxxf)
             if ("" == l) l <- "0" 
             p <- match(l, c("3", "2", "1", "0")) 
             c("fast", "presentation2", "presentation1", "debug")[p]
           }) 

setMethod('get_stancode', signature = "stanfit", 
          function(object, print = FALSE) {
            return(get_stancode(object@stanmodel, print))
          }) 

setGeneric(name = 'get_stanmodel', 
           def = function(object, ...) { standardGeneric("get_stanmodel")})

setMethod("get_stanmodel", signature = "stanfit", 
          function(object) { 
            return(object@stanmodel)
          }) 

setGeneric(name = 'get_inits', 
           def = function(object, ...) { standardGeneric("get_inits")})

setMethod("get_inits", signature = "stanfit", 
          function(object, iter = NULL) { 
            if (is.null(iter)) return(object@inits)
            stopifnot(is.numeric(iter), iter > 0, 
                      iter <= length(object@sim$samples[[1]][[1]]))
            inits <- object@inits
            if (length(inits) == 0) {
              inits <- lapply(1:ncol(object), FUN = function(c) {
                sapply(object@par_dims, simplify = FALSE, FUN = function(p) {
                  if (length(p) == 0) return(NA_real_)
                  array(NA_real_, dim = p)
                })
              })
            }
            for (c in 1:ncol(object)) {
              vec <- as.matrix(as.data.frame(object@sim$samples[[c]]))[iter,]
              inits[[c]] <- relist(vec, skeleton = inits[[c]])
            }
            return(inits)                       
})

setGeneric(name = 'get_seed', 
           def = function(object, ...) { standardGeneric("get_seed")})

setMethod("get_seed", signature = "stanfit", 
          function(object) { 
            if (length(object@stan_args) < 1L) return(NULL) 
            object@stan_args[[1]]$seed })

setGeneric(name = 'get_seeds', 
           def = function(object, ...) { standardGeneric("get_seeds")})

setMethod("get_seeds", signature = "stanfit", 
          function(object) { 
            if (length(object@stan_args) < 1L) return(NULL) 
            sapply(object@stan_args, function(x) x$seed) })

### HELPER FUNCTIONS
### 


get_kept_samples <- function(n, sim) {
  #
  # Args:
  #  sim: the sim slot in object stanfit 
  #  n: the nth parameter (starting from 1) 
  # Note: 
  #  samples from different chains are merged. 

  # get chain kept samples (gcks) 
  gcks <- function(s, nw, permutation) {
    s <- s[[n]][-(1:nw)] 
    s[permutation] 
  } 
  ss <- mapply(gcks, sim$samples, sim$warmup2, sim$permutation,
               SIMPLIFY = FALSE, USE.NAMES = FALSE) 
  do.call(c, ss) 
} 

get_kept_samples2 <- function(n, sim) {
  # a different implementation of get_kept_samples 
  # It seems this one is faster than get_kept_samples 
  # TODO: to understand why it is faster? 
  lst <- vector("list", sim$chains)
  for (ic in 1:sim$chains) { 
    if (sim$warmup2[ic] > 0) 
      lst[[ic]] <- sim$samples[[ic]][[n]][-(1:sim$warmup2[ic])][sim$permutation[[ic]]]
    else 
      lst[[ic]] <- sim$samples[[ic]][[n]][sim$permutation[[ic]]]
  } 
  do.call(c, lst)
}

get_samples <- function(n, sim, inc_warmup = TRUE) {
  # get chain samples
  # Args:
  #   n: parameter index (integer)
  #   sim: the sim list in stanfit 
  # 
  # Returns:
  #   a list of chains for the nth parameter; each chain is an 
  #   element of the list.  
  if (all(sim$warmup2 == 0)) inc_warmup <- TRUE # for the case warmup sample is discarded
  gcs <- function(s, inc_warmup, nw) {
    # Args:
    #   s: samples of all chains 
    #   nw: number of warmup 
    if (inc_warmup)  return(s[[n]])
    else return(s[[n]][-(1:nw)]) 
  } 
  ss <- mapply(gcs, sim$samples, inc_warmup, sim$warmup2, 
               SIMPLIFY = FALSE, USE.NAMES = FALSE) 
  ss 
} 

get_samples2 <- function(n, sim, inc_warmup = TRUE) {
  # serves the same purpose with get_samples, but with 
  # different implementation 
  # It seems that this one is fast. 
  if (all(sim$warmup2 == 0)) inc_warmup <- TRUE # for the case warmup sample is discarded
  npar <- length(n)
  lst <- vector("list", sim$chains)
  for (ic in 1:sim$chains) {
    lst[[ic]] <- 
      if (inc_warmup) sim$samples[[ic]][[n]] else sim$samples[[ic]][[n]][-(1:sim$warmup2[ic])]
  }
  lst
} 

par_traceplot <- function(sim, n, par_name, inc_warmup = TRUE, window = NULL, ...) {
  # same thin, n_save, warmup2 for all the chains
  thin <- sim$thin
  warmup2 <- sim$warmup2[1] 
  warmup <- sim$warmup 
  main <- paste("Trace of ", par_name) 
  chain_cols <- rstan_options("rstan_chain_cols")
  warmup_col <- rstan_options("rstan_warmup_bg_col") 

  start_i <- window[1] 
  window_size <- (window[2] - start_i) %/% thin
  id <- seq.int(start_i, by = thin, length.out = window_size)
  start_idx <- (if (warmup2 == 0) (start_i - warmup) else start_i) %/% thin
  if (start_idx < 1)  start_idx <- 1
  idx <- seq.int(start_idx, by = 1, length.out = window_size)

  yrange <- NULL 
  for (i in 1:sim$chains)
    yrange <- range(yrange, sim$samples[[i]][[n]][idx]) 

  if (inc_warmup) {
    plot(range(id), yrange, type = 'n', bty = 'l',
         xlab = 'Iterations', ylab = "", main = main, ...)
    rect(par("usr")[1], par("usr")[3], sim$warmup, par("usr")[4], 
         col = warmup_col, border = NA)
  } else {  
    plot(range(id), yrange, type = 'n', bty = 'l',
         xlab = 'Iterations (without warmup)', ylab = "", main = main, ...)
  } 

  for (i in 1:sim$chains)  
    lines(id, sim$samples[[i]][[n]][idx], 
          xlab = '', ylab = '', col = chain_cols[(i-1) %% 6 + 1], ...) 
} 

######

setGeneric(name = 'get_adaptation_info', 
           def = function(object, ...) { standardGeneric("get_adaptation_info")})

setMethod("get_adaptation_info", 
          definition = function(object) {
            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 

            lai <- lapply(object@sim$samples, function(x) attr(x, "adaptation_info"))
            is_empty <- function(x) { 
              if (is.null(x)) return(TRUE) 
              if (is.character(x) && all(nchar(x) == 0)) return(TRUE)
              FALSE
            }
            if (all(sapply(lai, FUN = is_empty))) return(invisible(NULL))  
            return(lai) 
          }) 

setGeneric(name = "get_logposterior", 
           def = function(object, ...) { standardGeneric("get_logposterior")})


setMethod("get_logposterior", "stanfit",
          definition = function(object, inc_warmup = TRUE) {
            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 

            llp <- lapply(object@sim$samples, function(x) x[['lp__']]) 
            if (inc_warmup) return(llp)
            if (all(object@sim$warmup2 == 0)) return(llp)
            return(mapply(function(x, w) x[-(1:w)], 
                             llp, object@sim$warmup2,
                             SIMPLIFY = FALSE, USE.NAMES = FALSE)) 
          }) 

setGeneric(name = 'get_sampler_params', 
           def = function(object, ...) { standardGeneric("get_sampler_params")}) 

setMethod("get_sampler_params", 
          definition = function(object, inc_warmup = TRUE) {
            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            }
            
            if (isTRUE(object@stan_args[[1]]$method == "variational")) {
              stop("'get_sampler_params' not available for ",
                   "meanfield or fullrank algorithms.") 
            }

            ldf <- lapply(object@sim$samples, 
                          function(x) do.call(cbind, attr(x, "sampler_params")))   
            if (all(sapply(ldf, is.null))) return(invisible(NULL))  
            if (inc_warmup) {
              if (all(object@sim$warmup2 == 0))
                warning("warmup samples not saved")
              return(invisible(ldf))
            }
            else if (all(object@sim$warmup2 == 0)) return(invisible(ldf))
            return(mapply(function(x, w) 
              if (w > 0) x[-(1:w), , drop = FALSE] else x, 
                             ldf, object@sim$warmup2, 
                             SIMPLIFY = FALSE, USE.NAMES = FALSE)) 
          }) 

setGeneric(name = 'get_elapsed_time',
           def = function(object, ...) { standardGeneric("get_elapsed_time")})

setMethod("get_elapsed_time",
          definition = function(object, inc_warmup = TRUE) {
            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL))
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '')
              return(invisible(NULL))
            }

            ltime <- lapply(object@sim$samples,
                            function(x) attr(x, "elapsed_time"))
            t <- do.call(rbind, ltime)
            if (is.null(t)) return(t)
            cids <- sapply(object@stan_args, function(x) x$chain_id)
            rownames(t) <- paste0("chain:", cids)
            t
          })

setGeneric(name = 'get_posterior_mean', 
           def = function(object, ...) { standardGeneric("get_posterior_mean")}) 

setMethod("get_posterior_mean", signature = "stanfit",
          definition = function(object, pars) {
            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 
            fnames <- flatnames(object@model_pars, object@par_dims, col_major = TRUE)
            if (!exists("posterior_mean_4all", envir = object@.MISC, inherits = FALSE)) {
              mean_pars <- lapply(object@sim$samples, function(x) attr(x, "mean_pars"))
              if (all(sapply(mean_pars, is.null))) 
                mean_pars <- list(rapply(object@sim$samples, mean))
              mean_lp__ <- lapply(object@sim$samples, function(x) attr(x, "mean_lp__"))
              if (all(sapply(mean_lp__, is.null))) 
                mean_lp__ <- as.list(rep(0, length(mean_lp__)))
              m <- rbind(do.call(cbind, mean_pars), do.call(cbind, mean_lp__))
              name_allchains <- NULL
              if (ncol(m) > 1) {
                m <- cbind(m, apply(m, 1, mean))
                name_allchains <- "mean-all chains"
              } 
              cids <- sapply(object@stan_args, function(x) x$chain_id)
              colnames(m) <- c(paste0('mean-chain:', cids), name_allchains)
              rownames(m) <- fnames
              assign("posterior_mean_4all", m, envir = object@.MISC)
            }
            pars <- if (missing(pars)) object@model_pars else check_pars(c(object@model_pars, fnames), pars)
            pars <- remove_empty_pars(pars, object@sim$dims_oi)
            tidx <- pars_total_indexes(object@model_pars, object@par_dims, fnames, pars) 
            tidx <- lapply(tidx, function(x) attr(x, "row_major_idx"))
            return(object@.MISC$posterior_mean_4all[unlist(tidx), , drop = FALSE])
          })

setGeneric(name = "extract",
           def = function(object, ...) { standardGeneric("extract") }) 

setMethod("extract", signature = "stanfit",
          definition = function(object, pars, permuted = TRUE, 
                                inc_warmup = FALSE, include = TRUE) {
            # Extract the samples in different forms for different parameters. 
            #
            # Args:
            #   object: the object of "stanfit" class 
            #   pars: the names of parameters (including other quantiles) 
            #   permuted: if TRUE, the returned samples are permuted without
            #     warming up. And all the chains are merged. 
            #   inc_warmup: if TRUE, warmup samples are kept; otherwise, 
            #     discarded. If permuted is TRUE, inc_warmup is ignored. 
            #   include: if FALSE interpret pars as those to exclude
            #
            # Returns:
            #   If permuted is TRUE, return an array (matrix) of samples with each
            #   column being the samples for a parameter. 
            #   If permuted is FALSE, return array with dimensions
            #   (# of iter (with or w.o. warmup), # of chains, # of flat parameters). 

            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 

            if(!include) pars <- setdiff(object@sim$pars_oi, pars)
            pars <- if (missing(pars)) object@sim$pars_oi else check_pars_second(object@sim, pars) 
            pars <- remove_empty_pars(pars, object@sim$dims_oi)
            tidx <- pars_total_indexes(object@sim$pars_oi, 
                                       object@sim$dims_oi, 
                                       object@sim$fnames_oi, 
                                       pars) 

            n_kept <- object@sim$n_save - object@sim$warmup2
            fun1 <- function(par_i) {
              # sss <- sapply(tidx[[par_i]], get_kept_samples2, object@sim)
              # if (is.list(sss))  sss <- do.call(c, sss)
              # the above two lines are slower than the following line of code
              sss <- do.call(cbind, lapply(tidx[[par_i]], get_kept_samples2, object@sim)) 
              dim(sss) <- c(sum(n_kept), object@sim$dims_oi[[par_i]]) 
              dimnames(sss) <- list(iterations = NULL)
              sss 
            } 
           
            if (permuted) {
              slist <- lapply(pars, fun1) 
              names(slist) <- pars 
              return(slist) 
            } 

            tidx <- unlist(tidx, use.names = FALSE) 
            tidxnames <- object@sim$fnames_oi[tidx] 
            sss <- lapply(tidx, get_samples2, object@sim, inc_warmup) 
            sss2 <- lapply(sss, function(x) do.call(c, x))  # concatenate samples from different chains
            sssf <- unlist(sss2, use.names = FALSE) 
  
            n2 <- object@sim$n_save[1]  ## assuming all the chains have equal iter 
            if (!inc_warmup) n2 <- n2 - object@sim$warmup2[1] 
            dim(sssf) <- c(n2, object@sim$chains, length(tidx)) 
            cids <- sapply(object@stan_args, function(x) x$chain_id)
            dimnames(sssf) <- list(iterations = NULL, chains = paste0("chain:", cids), parameters = tidxnames)
            sssf 
          })  

setMethod("summary", signature = "stanfit", 
          function(object, pars, 
                   probs = c(0.025, 0.25, 0.50, 0.75, 0.975), use_cache = TRUE, ...) { 
            # Summarize the samples (that is, compute the mean, SD, quantiles, for 
            # the samples in all chains and chains individually after removing
            # warmup samples, and n_eff and split R hat for all the kept samples.)
            # 
            # Returns: 
            #   A list with elements:
            #   summary: the summary for all the kept samples 
            #   c_summary: the summary for individual chains. 
            # 
            # Note: 
            #   This function is not straight in terms of implementation as it
            #   saves some standard summaries including n_eff and Rhat in the
            #   environment of the object. The summaries and created upon 
            #   the first time standard summary is called and resued later if possible. 
            #   In addition, the indexes complicate the implementation as internally
            #   we use column major indexes for vector/array parameters. But it might
            #   be better to use row major indexes for the output such as print.

            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 

            if (isTRUE(all.equal(object@sim$n_save, object@sim$warmup2,
                check.attributes = FALSE, check.names = FALSE))) {
              cat("Stan model '", object@model_name, "' does not contain samples after warmup.\n", sep = '')
              return(invisible(NULL))
            }

            if (!exists("summary", envir = object@.MISC, inherits = FALSE) && use_cache) 
              assign("summary", summary_sim(object@sim), envir = object@.MISC)
           
            pars <- if (missing(pars)) object@sim$pars_oi else check_pars_second(object@sim, pars) 
            pars <- remove_empty_pars(pars, object@sim$dims_oi)
            if (missing(probs)) 
              probs <- c(0.025, 0.25, 0.50, 0.75, 0.975)  

            if (!use_cache) {
              # not using the cached (and not create cache, which takes time for too many pars)
              ss <-  summary_sim(object@sim, pars, probs) 
              if (isTRUE(object@stan_args[[1]]$method == "variational")) {
                s1 <- cbind(ss$msd[, 1, drop = FALSE], ss$sem, ss$msd[, 2, drop = FALSE], 
                            ss$quan, ss$ess, ss$khat)
                colnames(s1) <- c("mean", "se_mean", "sd", colnames(ss$quan), 'n_eff', 'khat')
              } else {
                s1 <- cbind(ss$msd[, 1, drop = FALSE], ss$sem, ss$msd[, 2, drop = FALSE], 
                            ss$quan, ss$ess, ss$rhat)
                colnames(s1) <- c("mean", "se_mean", "sd", colnames(ss$quan), 'n_eff', 'Rhat')
              }
              s2 <- combine_msd_quan(ss$c_msd, ss$c_quan) 
              idx2 <- match(attr(ss, "row_major_idx"), attr(ss, "col_major_idx"))
              sf <- list(summary = s1[idx2, , drop = FALSE],
                         c_summary = s2[idx2, , , drop = FALSE])
              return(sf)
            } 
            m <- match(probs, default_summary_probs())
            if (any(is.na(m))) { # unordinary quantiles are requested 
              ss <-  summary_sim_quan(object@sim, pars, probs) 
              col_idx <- attr(ss, "col_major_idx") 
              ss$ess <- object@.MISC$summary$ess[col_idx, drop = FALSE]
              if (isTRUE(object@stan_args[[1]]$method == "variational")) {
                ss$hat <- object@.MISC$summary$khat[col_idx, drop = FALSE]
                hatstr <- "khat"
              } else {
                ss$hat <- object@.MISC$summary$rhat[col_idx, drop = FALSE]
                hatstr <- "Rhat"
              }
              ss$mean <- object@.MISC$summary$msd[col_idx, 1, drop = FALSE] 
              ss$sd <- object@.MISC$summary$msd[col_idx, 2, drop = FALSE] 
              ss$sem <- object@.MISC$summary$sem[col_idx]  
              s1 <- cbind(ss$mean, ss$sem, ss$sd, 
                          ss$quan, ss$ess, ss$hat)
              colnames(s1) <- c("mean", "se_mean", "sd", colnames(ss$quan), 'n_eff', hatstr)
              s2 <- combine_msd_quan(object@.MISC$summary$c_msd[col_idx, , , drop = FALSE], ss$c_quan) 
              idx2 <- match(attr(ss, "row_major_idx"), col_idx)
              ss <- list(summary = s1[idx2, , drop = FALSE],
                         c_summary = s2[idx2, , , drop = FALSE])
              return(ss)
            }

            tidx <- pars_total_indexes(object@sim$pars_oi, 
                                       object@sim$dims_oi, 
                                       object@sim$fnames_oi, 
                                       pars) 
            tidx <- lapply(tidx, function(x) attr(x, "row_major_idx"))
            tidx <- unlist(tidx, use.names = FALSE)
            tidx_len <- length(tidx)

            ss <- object@.MISC$summary 
            qnames <- colnames(ss$quan)[m]

            if (!is.null(object@stan_args[[1]]$method) && 
                  isTRUE(object@stan_args[[1]]$method == "variational")) {
              s1 <- cbind(ss$msd[tidx, 1, drop = FALSE], 
                          ss$sem[tidx, drop = FALSE], 
                          ss$msd[tidx, 2, drop = FALSE], 
                          ss$quan[tidx, m, drop = FALSE], 
                          ss$ess[tidx, drop = FALSE],
                          ss$khat[tidx, drop = FALSE])
              dim(s1) <- c(length(tidx), length(m) + 5L)
              colnames(s1) <- c("mean", "se_mean", "sd", qnames, 'n_eff', 'khat')
            }
            else {
              s1 <- cbind(ss$msd[tidx, 1, drop = FALSE], 
                          ss$sem[tidx, drop = FALSE], 
                          ss$msd[tidx, 2, drop = FALSE], 
                          ss$quan[tidx, m, drop = FALSE], 
                          ss$ess[tidx, drop = FALSE],
                          ss$rhat[tidx, drop = FALSE])
              dim(s1) <- c(length(tidx), length(m) + 5L) 
              colnames(s1) <- c("mean", "se_mean", "sd", qnames, 'n_eff', 'Rhat')
            }
            pars_names <- rownames(ss$msd)[tidx] 
            rownames(s1) <- pars_names 
            s2 <- combine_msd_quan(ss$c_msd[tidx, , , drop = FALSE], ss$c_quan[tidx, m, , drop = FALSE]) 
            # dim(s2) <- c(tidx_len, length(m) + 2, object@sim$chains)
            # dimnames(s2) <- list(parameter = pars_names, 
            #                      stats = c("mean", "sd", qnames), NULL) 
            ss <- list(summary = s1, c_summary = s2) 
            return(ss)
          })  

if (!isGeneric("traceplot")) {
  setGeneric(name = "traceplot",
             def = function(object, ...) { standardGeneric("traceplot") }) 
} 

if (!isGeneric("log_prob")) {
  setGeneric(name = "log_prob", 
             def = function(object, ...) { standardGeneric("log_prob") }) 
} 

if (!isGeneric("unconstrain_pars")) {
  setGeneric(name = "unconstrain_pars", 
             def = function(object, ...) { standardGeneric("unconstrain_pars") }) 
} 

setMethod("unconstrain_pars", signature = "stanfit", 
          function(object, pars) {
            # pars is a list as in specifying inits for a chain
            if (!is_sfinstance_valid(object)) 
              stop("the model object is not created or not valid")
            object@.MISC$stan_fit_instance$unconstrain_pars(pars)
          })

if (!isGeneric("constrain_pars")) {
  setGeneric(name = "constrain_pars", 
             def = function(object, ...) { standardGeneric("constrain_pars") }) 
} 

setMethod("constrain_pars", signature = "stanfit", 
          function(object, upars) {
            # upars is a vector on the unconstrained space (R^N*), 
            # where N* is the number of unconstrained parameters. 
            if (!is_sfinstance_valid(object)) 
              stop("the model object is not created or not valid")
            p <- object@.MISC$stan_fit_instance$constrain_pars(upars)
            idx_wo_lp <- which(object@model_pars != 'lp__')
            rstan_relist(p, create_skeleton(object@model_pars[idx_wo_lp], object@par_dims[idx_wo_lp]))
          })


setMethod("log_prob", signature = "stanfit", 
          function(object, upars, adjust_transform = TRUE, gradient = FALSE) {
            if (!is_sfinstance_valid(object)) 
              stop("the model object is not created or not valid")
            return(object@.MISC$stan_fit_instance$log_prob(upars, adjust_transform, gradient))
          }) 

if (!isGeneric("get_num_upars")) {
  setGeneric(name = "get_num_upars", 
             def = function(object, ...) { standardGeneric("get_num_upars") }) 
} 

setMethod("get_num_upars", signature = "stanfit", 
          function(object) {
            if (!is_sfinstance_valid(object)) 
              stop("the model object is not created or not valid")
            object@.MISC$stan_fit_instance$num_pars_unconstrained()
          })

if (!isGeneric("grad_log_prob")) {
  setGeneric(name = "grad_log_prob", 
             def = function(object, ...) { standardGeneric("grad_log_prob") }) 
} 

setMethod("grad_log_prob", signature = "stanfit", 
          function(object, upars, adjust_transform = TRUE) {
            if (!is_sfinstance_valid(object)) 
              stop("the model object is not created or not valid")
            object@.MISC$stan_fit_instance$grad_log_prob(upars, adjust_transform) 
          }) 

setMethod("traceplot", signature = "stanfit", 
          function(object, pars, include = TRUE, unconstrain = FALSE,
                   inc_warmup = FALSE, window = NULL, nrow = NULL, ncol = NULL,
                   ...) { 

            if (object@mode == 1L) {
              cat("Stan model '", object@model_name, "' is of mode 'test_grad';\n",
                  "sampling is not conducted.\n", sep = '')
              return(invisible(NULL)) 
            } else if (object@mode == 2L) {
              cat("Stan model '", object@model_name, "' does not contain samples.\n", sep = '') 
              return(invisible(NULL)) 
            } 
            args <- list(object = object, include = include, 
                         unconstrain = unconstrain, inc_warmup = inc_warmup,
                         nrow = nrow, ncol = ncol, window = window, ...)
            if (!missing(pars)) { 
              if ("log-posterior" %in% pars)
                pars[which(pars == "log-posterior")] <- "lp__"
              args$pars <- pars
            }
            do.call("stan_trace", args)
          })  


is_sf_valid <- function(sf) {
  # Similar to is_sm_valid, this is only to test whether
  # the compiled DSO is loaded 
  return(is_sm_valid(sf@stanmodel)) 
} 

is_sfinstance_valid <- function(object) {
  # Args
  #  object: an instance of S4 class stanfit 
  exists("stan_fit_instance", envir = object@.MISC, inherits = FALSE) && 
  !(is_null_ptr(object@.MISC$stan_fit_instance@.xData$.pointer) ||
    is_null_ptr(object@.MISC$stan_fit_instance@.xData$.module) || 
    is_null_ptr(object@.MISC$stan_fit_instance@.xData$.cppclass)) 
} 


sflist2stanfit <- function(sflist) {
  # merge a list of stanfit objects into one
  # Args:
  #   sflist, a list of stanfit objects, each element of which 
  #   should have equal length of `iter`, `warmup`, and `thin`. 
  # Returns: 
  #   A new stanfit objects have all the chains in each element of sf_list. 
  #   The date would be where the new object is created. 
  # Note: 
  #   * method get_seed would not work well for this merged 
  #     stanfit object in that it only returns the seed used
  #     for the first object. But all the information is still there. 
  #   * When print function is called, the sampler info is obtained 
  #     only from the first chain. 
  #  
  sf_len <- length(sflist) 
  if (sf_len == 0) stop("'sflist' should have at least 1 element")
  if (!is.list(sflist) || 
      any(sapply(sflist, function(x) !is(x, "stanfit")))) {   
    stop("'sflist' must be a list of 'stanfit' objects")
  }

  non_zero_modes_idx <- which(sapply(sflist, function(x) x@mode) > 0)
  if (length(non_zero_modes_idx) > 0) { 
    stop("The following elements of 'sflist' do not contain samples: ",
         paste(non_zero_modes_idx, collapse = ', '), ".") 
  }   
  if (sf_len == 1) return(sflist[[1]])
  for (i in 2:sf_len) { 
    if (!identical(sflist[[i]]@sim$pars_oi, sflist[[1]]@sim$pars_oi) || 
        !identical(sflist[[i]]@sim$dims_oi, sflist[[1]]@sim$dims_oi))   
      stop("parameters in element ", i, " (stanfit object) are different from in element 1") 
    if (sflist[[i]]@sim$n_save[1] != sflist[[1]]@sim$n_save[1] ||  
        sflist[[i]]@sim$warmup2[1] != sflist[[1]]@sim$warmup2[1])
      stop("all 'stanfit' objects should have equal length of iterations and warmup") 
  } 
  n_chains = sum(sapply(sflist, function(x) x@sim$chains))  
  sim = list(samples = do.call(c, lapply(sflist, function(x) x@sim$samples)),  
             chains = n_chains, 
             iter = sflist[[1]]@sim$iter, 
             thin = sflist[[1]]@sim$thin, 
             warmup = sflist[[1]]@sim$warmup,
             n_save = rep(sflist[[1]]@sim$n_save[1], n_chains), 
             warmup2 = rep(sflist[[1]]@sim$warmup2[1], n_chains), 
             permutation = do.call(c, lapply(sflist, function(x) x@sim$permutation)), 
             pars_oi = sflist[[1]]@sim$pars_oi, 
             dims_oi = sflist[[1]]@sim$dims_oi, 
             fnames_oi = sflist[[1]]@sim$fnames_oi,
             n_flatnames = sflist[[1]]@sim$n_flatnames) 
  nfit <- new("stanfit", 
              model_name = sflist[[1]]@model_name, 
              model_pars = sflist[[1]]@model_pars,
              par_dims  = sflist[[1]]@par_dims, 
              mode = 0L, 
              sim = sim, 
              inits = do.call(c, lapply(sflist, function(x) x@inits)), 
              stan_args = do.call(c, lapply(sflist, function(x) x@stan_args)), 
              stanmodel = sflist[[1]]@stanmodel, 
              date = date(), 
              .MISC = new.env(parent = emptyenv())) 
  return(nfit)
} 
# sflist2stan(list(l1=ss1, l2=ss2))

names.stanfit <- function(x) dimnames(x)$parameters

`names<-.stanfit` <- function(x, value) {
  value <- as.character(value)
  len <- length(x@sim$fnames_oi)
  if(length(value) != len)
    stop(paste("parameter names must be of length", len))
  x@sim$fnames_oi <- value
  if(length(x@.MISC$summary)) {
    x@.MISC$summary <- rapply(x@.MISC$summary, f = function(y) {
      rownames(y) <- value
      return(y)
    }, how = "replace")
  }
  return(x)
}

as.array.stanfit <- function(x, ...) {
  if (x@mode != 0) return(numeric(0)) 
  out <- extract(x, permuted = FALSE, inc_warmup = FALSE, ...)
  # dimnames(out) <- dimnames(x)
  return(out)
} 
as.matrix.stanfit <- function(x, ...) {
  if (x@mode != 0) return(numeric(0)) 
  e <- extract(x, permuted = FALSE, inc_warmup = FALSE, ...) 
  if (is.null(e)) return(e)
  enames <- dimnames(e)
  edim <- dim(e)
  dim(e) <- c(edim[1] * edim[2], edim[3])
  dimnames(e) <- enames[-2]
  e
}
 
as.data.frame.stanfit <- function(x, ...) {
  return( as.data.frame(as.matrix(x, ...)) )
}

dim.stanfit <- function(x) {
  if (x@mode != 0) return(numeric(0)) 
  c(x@sim$n_save[1] - x@sim$warmup2[1], x@sim$chains, x@sim$n_flatnames)
}

setGeneric("as.mcmc.list", function(object, ...) standardGeneric("as.mcmc.list"))

as.mcmc.list.stanfit <- function(object, pars, ...) {
  pars <- if (missing(pars)) object@sim$pars_oi else check_pars_second(object@sim, pars) 
  pars <- remove_empty_pars(pars, object@sim$dims_oi)
  tidx <- pars_total_indexes(object@sim$pars_oi, object@sim$dims_oi, object@sim$fnames_oi, pars)
  tidx <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)

  lst <- vector("list", object@sim$chains)
  for (ic in 1:object@sim$chains) { 
    x <- do.call(cbind, object@sim$samples[[ic]])[,tidx,drop=FALSE]
    warmup2 <- object@sim$warmup2[ic] 
    if (warmup2 > 0) x <- x[-(1:warmup2), ]
    x <- as.matrix(x)
    if (is.null(colnames(x))) colnames(x) <- pars
    end <- object@sim$iter
    thin <- object@sim$thin
    start <- end - (nrow(x) - 1) * thin
    class(x) <- 'mcmc'
    attr(x, "mcpar") <- c(start, end, thin)
    lst[[ic]] <- x 
  }
  class(lst) <- "mcmc.list"
  return(lst)
}

setMethod("as.mcmc.list", "stanfit", as.mcmc.list.stanfit)

As.mcmc.list <- function(object, pars, include = TRUE, ...) {
  if (missing(pars)) pars <- object@sim$pars_oi
  else if (!include) pars <- setdiff(object@sim$pars_oi, pars)
  pars <-  check_pars_second(object@sim, pars)
  return(as.mcmc.list.stanfit(object, pars = pars))
}

dimnames.stanfit <- function(x) {
  if (x@mode != 0) return(character(0)) 
  cids <- sapply(x@stan_args, function(x) x$chain_id)
  list(iterations = NULL, chains = paste0("chain:", cids), parameters = x@sim$fnames_oi) 
}
is.array.stanfit <- function(x)  return(x@mode == 0)

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

filename_ext <- function(x) {
  # obtain the file extension
  # copied from tools package
  pos <- regexpr("\\.([[:alnum:]]+)$", x)
  ifelse(pos > -1L, substring(x, pos + 1L), "")
}

filename_rm_ext <- function(x) {
  # remove the filename's extension
  sub("\\.[^.]*$", "", x)
}

real_is_integer <- function(x) {
  if (length(x) < 1L) return(TRUE)
  if (any(is.infinite(x)) || any(is.nan(x))) return(FALSE)
  all(floor(x) == x)
}


list_as_integer_if_doable <- function(x) {
  # change the storage mode from 'real' to 'integer'
  # if applicable since by default R use real.
  #
  # Args:
  #  x: A list
  #
  # Note:
  # Ignore non-numeric vectors since we ignore
  # them in rlist_var_context
  #
  lapply(x,
         FUN = function(y) {
           if (!is.numeric(y)) return(y)
           if (is.integer(y)) return(y)
           ## this commented out is the idea in the function is.wholenumber in
           ## the help of is.integer
           # if (isTRUE(all.equal(y, round(y), check.attributes = FALSE)))
           if (real_is_integer(y)) storage.mode(y) <- "integer"
           return(y)
         })
}

mklist <- function(names) {
  # Make a list using names
  # Args:
  #   names: character strings of names of objects
  # Note:
  #   Only extracted are modes of numeric and list, which
  #   are enough for stan

  names <- unique(names)
  cenv <- environment()
  for (fn in rev(sys.parents())) {
    env1 <- sys.frame(fn)
    if (identical(env1, cenv)) next
    d1 <- mget(names, envir = env1, ifnotfound = NA, inherits = FALSE, mode = "numeric")
    d2 <- mget(names, envir = env1, ifnotfound = NA, inherits = FALSE, mode = "list")
    na_idx1 <- is.na(d1)
    na_idx2 <- is.na(d2)
    na_idx <- na_idx1 & na_idx2
    numf <- sum(na_idx)
    if (numf > 0 && numf < length(names))
      stop(paste("objects ", paste("'", names[na_idx], "'", collapse = ', ', sep = ''),
                 " of mode numeric and list not found", sep = ''))
    if (numf == length(names))  next
    r <- c(d1[!na_idx1], d2[na_idx1])
    names(r) <- c(names[!na_idx1], names[na_idx1])
    return(r)
  }
  stop(paste("objects ", paste("'", names, "'", collapse = ', ', sep = ''),
             " of mode numeric and list not found", sep = ''))
}

stan_kw1 <- c('for', 'in', 'while', 'repeat', 'until', 'if', 'then', 'else',
              'true', 'false')
stan_kw2 <- c('int', 'real', 'vector', 'simplex', 'ordered', 'positive_ordered',
              'row_vector', 'matrix', 'corr_matrix', 'cov_matrix', 'lower', 'upper')
stan_kw3 <- c('model', 'data', 'parameters', 'quantities', 'transformed', 'generated')

cpp_kw <- c("alignas", "alignof", "and", "and_eq", "asm", "auto", "bitand", "bitor", "bool",
            "break", "case", "catch", "char", "char16_t", "char32_t", "class", "compl",
            "const", "constexpr", "const_cast", "continue", "decltype", "default", "delete",
            "do", "double", "dynamic_cast", "else", "enum", "explicit", "export", "extern",
            "false", "float", "for", "friend", "goto", "if", "inline", "int", "long", "mutable",
            "namespace", "new", "noexcept", "not", "not_eq", "nullptr", "operator", "or", "or_eq",
            "private", "protected", "public", "register", "reinterpret_cast", "return",
            "short", "signed", "sizeof", "static", "static_assert", "static_cast", "struct",
            "switch", "template", "this", "thread_local", "throw", "true", "try", "typedef",
            "typeid", "typename", "union", "unsigned", "using", "virtual", "void", "volatile",
            "wchar_t", "while", "xor", "xor_eq")


is_legal_stan_vname <- function(name) {
  # Return:
  #   FALSE: not a lega variable name in Stan
  #   TRUE: maybe it is valid, but 100% sure
  if (grepl('\\.',  name)) return(FALSE)
  if (grepl('^\\d', name)) return(FALSE)
  if (grepl('__$',  name)) return(FALSE)
  if (name %in% stan_kw1) return(FALSE)
  if (name %in% stan_kw2) return(FALSE)
  if (name %in% stan_kw3) return(FALSE)
  !name %in% cpp_kw
}

data_list2array <- function(x) {
  # Turn a list of array to an array whose first dimension is the list
  # and other dimensions being the dimensions of the array element.
  # So this would allow data in Stan coded as `vector[J] y[I]`
  # to read data in form a list that has I elements of vector of length J, say
  #
  # # I <- 4; J <- 5
  # # y <- lapply(1:I, function(i) rnorm(J))
  #
  # Args:
  #   x: A list of numeric array with the same dimensions
  # Returns:
  #   An array with the first dimension indexes the list;
  #   other dimensions being the dimensions of the list element (an array)
  #
  len <- length(x)
  if (len == 0L)  return(NULL)

  dimx1 <- dim(x[[1]])

  if (any(sapply(x, function(xi) !is.numeric(xi))))
    stop("all elements of the list should be numeric")
  if (is.null(dimx1)) dimx1 <- length(x[[1]])
  lendimx1 <- length(dimx1)

  if (len > 1) {
    d <- sapply(x[-1],
                function(xi) {
                  dimxi <- dim(xi)
                  if (is.null(dimxi)) dimxi <- length(xi)
                  identical(dimxi, dimx1)
                })
    if (!all(d)) stop("the dimensions for all elements (array) of the list are not same")
  }

  # TODO(?): check if x is numeric or array.
  x <- do.call(c, x)
  dim(x) <- c(dimx1, len)
  aperm(x, c(lendimx1 + 1L, seq_len(lendimx1)))
}

data_preprocess <- function(data) { # , varnames) {
  # Preprocess the data (list or env) to list for stan
  #
  # Args:
  #  data: A list, an environment, or a vector of character strings for names
  #  of objects
  #   * stop if there is NA; no-name lists; duplicate names
  #   * stop if the objects given name is not found
  #   * remove NULL, non-numeric elements
  #   * change to integers when applicable

  #
  # if (is.environment(data)) {

  #   data <- mget(varnames, envir = data, mode = "numeric",
  #                ifnotfound = list(NULL))
  #   data <- data[!sapply(data, is.null)]
  # }
  if (is.environment(data)) {
    data <- as.list(data)
  } else if (is.list(data)) {
    v <- names(data)
    if (is.null(v))
      stop("data must be a named list")

    ## Stan would report error if variable is not found
    ## from the list
    # if (any(nchar(v) == 0))
    #   stop("unnamed variables in data list")
    #

    if (any(duplicated(v))) {
      stop("duplicated names in data list: ",
           paste(v[duplicated(v)], collapse = " "))
    }
  } else {
    stop("data must be a list or an environment")
  }

  names <- names(data)
  for (x in names) {
    if (!is_legal_stan_vname(x))
    stop(paste('data with name ', x, " is not allowed in Stan", sep = ''))
  }

  data <- lapply(names,
                 FUN = function(name) {
                   x <- data[[name]]
                   if (is.data.frame(x)) {
                     x <- data.matrix(x) # change data.frame to array
                   } else if (is.list(x)) {
                     x <- data_list2array(x) # list to array
                   } else if (is.logical(x)) {
                     storage.mode(x) <- "integer"
                   }

                   ## Now we stop whenever we have NA in the data
                   ## since we do not know what variables are needed
                   ## at this point.
                   if (any(is.na(x))) {
                     stop("Stan does not support NA (in ", name, ") in data")
                   }

                   # remove those not numeric data
                   if (!is.numeric(x)) {
                     warning("data with name ", name, " is not numeric and not used")
                     return(NULL)
                   }

                   if (is.integer(x)) return(x)

                   # change those integers stored as reals to integers
                   if (all(abs(x) < .Machine$integer.max) && real_is_integer(x))
                     storage.mode(x) <- "integer"
                   return(x)
                 })

  names(data) <- names
  data[!sapply(data, is.null)]
}


read_model_from_con <- function(con) {
  lines <- readLines(con, n = -1L, warn = FALSE)
  paste(lines, collapse = '\n')
}

get_model_strcode <- function(file, model_code = '') {
  # return the model code as a character string
  # Args:
  #   file: a file or connection
  #   model_code: character string for one of the following
  #     * the name of an object of character string
  #     * the model code itself
  #
  # Returns:
  #   the model code with attribute model_name2,
  #   a name implied from file or object name,
  #   which can be used later when model_name is not
  #   specified for function stan.

  if (!missing(file)) {
    if (is.character(file)) {
      fname <- file
      model_name2 <- sub("\\.[^.]*$", "", filename_rm_ext(basename(fname)))
      file <- try(file(fname, "rt"))
      if (inherits(file, "try-error")) {
        stop(paste("cannot open model file \"", fname, "\"", sep = ""))
      }
      on.exit(close(file))
    } else if (!inherits(file, "connection")) {
      stop("file must be a character string or connection")
    }
    model_code <- paste(readLines(file, warn = TRUE), collapse = '\n')
    # the model name implied from file name, which
    # will be used if model_name is not specified later
    attr(model_code, "model_name2") <- model_name2
    return(model_code)
  }

  model_name2 <- attr(model_code, "model_name2")
  if (is.null(model_name2))
    model_name2 <- deparse(substitute(model_code))
  if (model_code != '' && is.character(model_code)) {
    if (!grepl("\\{", model_code)) {
      # model_code points an object that includes the model
      model_name2 <- model_code
      if (exists(model_code, mode = 'character', envir = parent.frame()))
        model_code <- get(model_code, mode = 'character', envir = parent.frame())
    } else {
      # model_code includes the code itself, two cases of passing:
      #  1. using another object such as stan(mode_code = scode)`
      #  2. providing the string directly such stan(model_code = "")
      if (grepl("\\{", model_name2))
        model_name2 <- 'anon_model'
    }
    attr(model_code, "model_name2") <- model_name2
    return(model_code)
  }

  stop("model file missing and empty model_code")
}

# FIXEME: implement more check on the arguments
check_args <- function(argss) {
  if (FALSE) stop()
}

#
# model_code <- read_model_from_con('https://stan.googlecode.com/git/src/models/bugs_examples/vol1/dyes/dyes.stan')
# cat(model_code)


append_id <- function(file, id, suffix = '.csv') {
  fname <- basename(file)
  fpath <- dirname(file)
  fname2 <- gsub("\\.csv[[:space:]]*$",
                 paste("_", id, ".csv", sep = ''),
                 fname)
  if (fname2 == fname)
    fname2 <- paste(fname, "_", id, ".csv", sep = '')
  file.path(fpath, fname2)
}

check_seed <- function(seed, warn = 0) {
  if (is.character(seed) && grepl("[^0-9]", seed)) {
    if (warn == 0) stop("seed needs to be string of digits")
    else message("seed needs to be string of digits")
    return(NULL)
  }
  if (is.numeric(seed)) seed <- as.integer(seed)
  if (is.na(seed)) seed <- sample.int(.Machine$integer.max, 1)
  seed
}

is_named_list <- function(x) {
  # tell if list x is a named list
  if (!is.list(x)) return(FALSE)
  n <- names(x)
  if (is.null(n) || "" %in% n) return(FALSE)
  return(TRUE)
}


## from ../inst/include/rstan/stan_args.hpp
#
# enum sampling_algo_t { NUTS = 1, HMC = 2, Metroplos = 3};
# enum optim_algo_t { Newton = 1, BFGS = 3, LBFGS = 4};
# enum sampling_metric_t { UNIT_E = 1, DIAG_E = 2, DENSE_E = 3};
# enum stan_args_method_t { SAMPLING = 1, OPTIM = 2, TEST_GRADIENT = 3};


config_argss <- function(chains, iter, warmup, thin,
                         init, seed, sample_file, diagnostic_file, algorithm,
                         control, ...) {

  iter <- as.integer(iter)
  if (iter < 1)
    stop("parameter 'iter' should be a positive integer")
  thin <- as.integer(thin)
  if (thin < 1 || thin > iter)
    stop("parameter 'thin' should be a positive integer less than 'iter'")
  warmup <- max(0, as.integer(warmup))
  if (warmup >= iter)
    stop("parameter 'warmup' should be an integer less than 'iter'")
  chains <- as.integer(chains)
  if (chains < 1)
    stop("parameter 'chains' should be a positive integer")

  iters <- rep(iter, chains)
  thins <- rep(thin, chains)
  warmups <- rep(warmup, chains)

  inits_specified <- FALSE
  if (is.numeric(init)) init <- as.character(init)
  if (is.character(init)) {
    if (init[1] %in% c("0", "random")) inits <- rep(init[1], chains)
    else inits <- rep("random", chains)
    inits_specified <- TRUE
  }

  dotlist <- list(...)

  # use chain_id argument if specified
  chain_ids <- seq_len(chains)
  if (!is.null(dotlist$chain_id)) {
    chain_id <- as.integer(dotlist$chain_id)
    if (any(duplicated(chain_id))) stop("chain_id has duplicated elements")
    chain_id_len <- length(chain_id)
    chain_ids <- if (chain_id_len >= chains) chain_id else {
                   c(chain_id, max(chain_id) + seq_len(chains - chain_id_len))
                 }
    dotlist$chain_id <- NULL
  }

  if (!inits_specified && is.function(init)) {
    ## the function can take an argument named by chain_id
    if (any(names(formals(init)) == "chain_id")) {
      inits <- lapply(chain_ids, function(id) init(chain_id = id))
    } else {
      inits <- lapply(chain_ids, function(id) init())
    }
    if (!is_named_list(inits[[1]]))
      stop('the function for specifying initial values need return a named list')
    inits_specified <- TRUE
  }
  if (!inits_specified && is.list(init)) {
    if (length(init) != chains)
      stop("initial value list mismatchs number of chains")
    if (!any(sapply(init, is.list))) {
      stop("initial value list is not a list of lists")
    }
    inits <- init;
    for (i in 1:chains) {
      if (!is_named_list(inits[[i]]))
        stop('the list for specifying initial values need be a named list')
    }
    inits_specified <- TRUE
  }
  if (!inits_specified) stop("wrong specification of initial values")

  ## only one seed is needed by virtue of the RNG
  seed <- if (missing(seed)) sample.int(.Machine$integer.max, 1) else check_seed(seed)

  dotlist$method <- if (!is.null(dotlist$test_grad) && dotlist$test_grad) "test_grad" else "sampling"

  all_metrics <- c("unit_e", "diag_e", "dense_e")
  if (!is.null(control)) {
    if (!is.list(control))
      stop("'control' should be a named list")
      is_arg_recognizable(names(control),
                          c("adapt_engaged", "adapt_gamma",
                            "adapt_delta", "adapt_kappa", "adapt_t0",
                            "adapt_init_buffer", "adapt_term_buffer",
                            "adapt_window", "stepsize",
                            "stepsize_jitter", "metric", "int_time",
                            "max_treedepth",
                            "epsilon", "error"),
                          pre_msg = "'control' list contains unknown members of names: ",
                          call. = FALSE)
    metric <- control$metric
    if (!is.null(metric) && is.na(match(metric, all_metrics))) {
      stop("metric should be one of ", paste0(paste0('"', all_metrics, '"'), collapse = ", "))
    }
    dotlist$control <- control
  }

  argss <- vector("list", chains)
  ## the name of arguments in the list need to
  ## match those in include/rstan/stan_args.hpp
  for (i in 1:chains)
    argss[[i]] <- list(chain_id = chain_ids[i],
                       iter = iters[i], thin = thins[i], seed = seed,
                       warmup = warmups[i], init = inits[[i]],
                       algorithm = algorithm)

  if (!missing(sample_file) && !is.null(sample_file) && !is.na(sample_file)) {
    sample_file <- writable_sample_file(sample_file)
    if (chains == 1)
        argss[[1]]$sample_file <- sample_file
    if (chains > 1) {
      for (i in 1:chains)
        argss[[i]]$sample_file <- append_id(sample_file, i)
    }
  }

  if (!missing(diagnostic_file) && !is.null(diagnostic_file) && !is.na(diagnostic_file)) {
    diagnostic_file <- writable_sample_file(diagnostic_file)
    if (chains == 1)
        argss[[1]]$diagnostic_file <- diagnostic_file
    if (chains > 1) {
      for (i in 1:chains)
        argss[[i]]$diagnostic_file <- append_id(diagnostic_file, i)
    }
  }

  for (i in 1:chains)
    argss[[i]] <- c(argss[[i]], dotlist)
  check_args(argss)
  argss
}

is_dir_writable <- function(path) {
  (file.access(path, mode = 2) == 0) && (file.access(path, mode = 1) == 0)
}

writable_sample_file <-
function(file, warn = TRUE,
         wfun = function(x, x2) {
           paste('"', x, '" is not writable; use "', x2, '" instead', sep = '')
         }) {
  # Check if the path for file is writable, if not using tempdir()
  #
  # Args:
  #  file: The file interested.
  #  warning: TRUE give a warning.
  #  warningfun: A function that take two dirs for creating
  #    the warning message.
  #
  # Returns:
  #  If the specified file is writable, return itself.
  #  Otherwise, change the path to tempdir().

  dir <- dirname(file)
  if (is_dir_writable(dir)) return(file)

  dir2 <- tempdir()
  if (warn) warning(wfun(dir, dir2))
  file.path(dir2, basename(file))
}


stan_rdump <- function(list, file = "", append = FALSE,
                       envir = parent.frame(),
                       width = options("width")$width, quiet = FALSE) {
  # Dump an R list or environment for a model data
  # to the R dump file that Stan supports.
  #
  # Args:
  #   list: a vector of character for all variables interested
  #         (the same as in R's dump function)
  #   file: the output file for dumping the variables.
  #   append: then TRUE, the file is opened with
  #           mode of appending; otherwise, a new file
  #           is created.
  #   quiet: no warning if TRUE
  #
  # Return:

  if (is.character(file)) {
    ex <- sapply(list, exists, envir = envir)
    if (!all(ex)) {
      notfound_list <- list[!ex]
      if (!quiet)
        warning(paste("objects not found: ", paste(notfound_list, collapse = ', '), sep = ''))
    }
    list <- list[ex]
    if (!any(ex))
      return(invisible(character()))

    if (nzchar(file)) {
      file <- file(file, ifelse(append, "a", "w"))
      on.exit(close(file), add = TRUE)
    } else {
      file <- stdout()
    }
  }

  for (x in list) {
    if (!is_legal_stan_vname(x) & !quiet)
      warning(paste("variable name ", x, " is not allowed in Stan", sep = ''))
  }

  l2 <- NULL
  addnlpat <- paste0("(.{1,", width, "})(\\s|$)")
  for (v in list) {
    vv <- get(v, envir)

    if (is.data.frame(vv)) {
      vv <- data.matrix(vv)
    } else if (is.list(vv)) {
      vv <- data_list2array(vv)
    } else if (is.logical(vv)) {
      mode(vv) <- "integer"
    } else if (is.factor(vv)) {
      vv <- as.integer(vv)
    }

    if (!is.numeric(vv))  {
      if (!quiet)
        warning(paste0("variable ", v, " is not supported for dumping."))
      next
    }

    if (!is.integer(vv) && max(abs(vv)) < .Machine$integer.max && real_is_integer(vv))
      storage.mode(vv) <- "integer"

    if (is.vector(vv)) {
      if (length(vv) == 0) {
        cat(v, " <- integer(0)\n", file = file, sep = '')
        next
      }
      if (length(vv) == 1) {
        cat(v, " <- ", as.character(vv), "\n", file = file, sep = '')
        next
      }
      str <- paste0(v, " <- \nc(", paste(vv, collapse = ', '), ")")
      str <-  gsub(addnlpat, '\\1\n', str)
      cat(str, file = file)
      l2 <- c(l2, v)
      next
    }

    if (is.matrix(vv) || is.array(vv)) {
      l2 <- c(l2, v)
      vvdim <- dim(vv)
      cat(v, " <- \n", file = file, sep = '')
      if (length(vv) == 0) { 
        str <- paste0("structure(integer(0), ")
      } else {
        str <- paste0("structure(c(", paste(as.vector(vv), collapse = ', '), "),") 
      }
      str <- gsub(addnlpat, '\\1\n', str)
      cat(str,
          ".Dim = c(", paste(vvdim, collapse = ', '), "))\n", file = file, sep = '')
      next
    }
  }
  invisible(l2)
}

get_rhat_cols <- function(rhats) {
  #
  # Args:
  #   rhats: a vector of rhats
  #
  rhat_nan_col <- rstan_options("plot_rhat_nan_col")
  rhat_large_col <- rstan_options("plot_rhat_large_col")
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  # print(rhat_breaks)
  rhat_colors <- rstan_options("plot_rhat_cols")

  sapply(rhats,
         FUN = function(x) {
           if (is.na(x) || is.nan(x) || is.infinite(x))
             return(rhat_nan_col)
           for (i in 1:length(rhat_breaks)) {
             if (x >= rhat_breaks[i]) next
             return(rhat_colors[i])
           }
           rhat_large_col
         })
}

plot_rhat_legend <- function(x, y, cex = 1) {
  # Args
  #   x, y: left, bottom corner coordinates
  #   cex: cex for the labels
  rhat_breaks <- rstan_options("plot_rhat_breaks")
  n_breaks <- length(rhat_breaks)
  rhat_colors <- rstan_options("plot_rhat_cols")[1:n_breaks]
  rhat_legend_labels <- c(paste("< ", rhat_breaks, "  ", sep = ''),
                        paste(">= ", max(rhat_breaks), "  ", sep = ''),
                        "NaN/Inf")
  rhat_legend_cols <- c(rhat_colors, rstan_options('plot_rhat_large_col'),
                        rstan_options("plot_rhat_nan_col"))
  rhat_legend_width <- strwidth(rhat_legend_labels, cex = cex)
  rhat_rect_width <- strwidth("r-hat ", cex = cex)
  text(x, y, label = 'Rhat:  ', adj = c(0, 0), cex = cex)
  s1 <- strwidth('Rhat:  ', cex = cex)
  starts <- x + c(s1, s1 + cumsum(rhat_rect_width + rhat_legend_width))

  height <- strheight("0123456789<>=", cex = cex)

  for (i in 1:length(rhat_legend_cols)) {
    rect(starts[i], y, starts[i] + rhat_rect_width, y + height, col = rhat_legend_cols[i], border = NA)
    text(starts[i] + rhat_rect_width, y, adj = c(0, 0), label = rhat_legend_labels[i], cex = cex)
  }
}


read_rdump <- function(f, keep.source = FALSE, ...) {
  # Read data defined in an R dump file to an R list
  #
  # Args:
  #   f: the file to be sourced
  #   keep.source: see doc of function source
  # 
  # Returns:
  #   A list

  if (missing(f))
    stop("no file specified.")
  e <- new.env()
  source(file = f, local = e, keep.source = keep.source, ...)
  as.list(e)
}


idx_col2rowm <- function(d) {
  # Suppose an iteration of samples for an array parameter is ordered by
  # col-major. This function generates the indexes that can be used to change
  # the sequences to row-major.
  # Args:
  #   d: the dimension of the parameter
  len <- length(d)
  if (0 == len) return(1)
  if (1 == len) return(1:d)
  idx <- aperm(array(1:prod(d), dim = d))
  return(as.vector(idx))
}


idx_row2colm <- function(d) {
  # What if it is row-major and we want col_major?
  len <- length(d)
  if (0 == len) return(1)
  if (1 == len) return(1:d)
  idx <- aperm(array(1:prod(d), dim = rev(d)))
  return(as.vector(idx))
}

multi_idx_row2colm <- function(dims) {
  # Suppose we want to change a vector of parameter names (each of which is in
  # row major) to col major.  This function serves to get the indexes.
  # Args:
  #   dims: a list of dimensions for all the parameters
  #
  ## print(dims)
  shifts <- calc_starts(dims) - 1
  idx <- lapply(seq_along(shifts), function(i) shifts[i] + idx_row2colm(dims[[i]]))
  do.call(c, idx)
}


seq_array_ind <- function(d, col_major = FALSE) {
  #
  # Generate an array of indexes for an array parameter
  # in order of major or column.
  #
  # Args:
  #   d: the dimensions of an array parameter, for example,
  #     c(2, 3).
  #
  #   col_major: Determine what is the order of indexes.
  #   If col_major = TRUE, for d = c(2, 3), return
  #   [1, 1]
  #   [2, 1]
  #   [1, 2]
  #   [2, 2]
  #   [1, 3]
  #   [2, 3]
  #   If col_major = FALSE, for d = c(2, 3), return
  #   [1, 1]
  #   [1, 2]
  #   [1, 3]
  #   [2, 1]
  #   [2, 2]
  #   [2, 3]
  #
  # Returns:
  #   If length of d is 0, return empty vector.
  #   Otherwise, return an array of indexes, each
  #   row of which is an index.
  #
  # Note:
  #   R function arrayInd might be helpful sometimes.
  #
  if (length(d) == 0L)
    return(numeric(0L))

  total <- prod(d)
  if (total == 0L)
    return(array(0L, dim = 0L))

  len <- length(d)
  if (len == 1L)
    return(array(1:total, dim = c(total, 1)))

  res <- array(1L, dim = c(total, len))

  # Handle cases like 1x1 matrices
  if (total == 1)
    return(res)

  jidx <- if (col_major) 1L:len else len:1L
  for (i in 2L:total) {
    res[i, ] <- res[i - 1, ]
    for (j in jidx) {
      if (res[i - 1, j] < d[j]) {
        res[i, j] <- res[i - 1, j] + 1
        break
      }
      res[i, j] <- 1
    }
  }
  res
}

flat_one_par <- function(n, d, col_major = FALSE) {
  # Return all the elemenetwise parameters for a vector/array
  # parameter.
  #
  # Args:
  #  n: Name of the parameter. For example, n = "alpha"
  #  d: A vector indicates the dimensions of parameter n.
  #     For example, d = c(2, 3).  d could be empty
  #     as well when n is a scalar.
  #
  if (0 == length(d)) return(n)
  nameidx <- seq_array_ind(d, col_major)
  names <- apply(nameidx, 1, function(x) paste(n, "[", paste(x, collapse = ','), "]", sep = ''))
  as.vector(names)
}


flatnames <- function(names, dims, col_major = FALSE) {
  if (length(names) == 1)
    return(flat_one_par(names, dims[[1]], col_major = col_major))
  nameslst <- mapply(flat_one_par, names, dims,
                     MoreArgs = list(col_major = col_major),
                     SIMPLIFY = FALSE,
                     USE.NAMES = FALSE)
  if (is.vector(nameslst, "character"))
    return(nameslst)
  do.call(c, nameslst)
}

num_pars <- function(d) prod(d)

calc_starts <- function(dims) {
  len <- length(dims)
  s <- sapply(unname(dims), function(d)  num_pars(d), USE.NAMES = FALSE)
  cumsum(c(1, s))[1:len]
}

check_pars <- function(allpars, pars) {
  pars_wo_ws <- gsub('\\s+', '', pars)
  m <- which(match(pars_wo_ws, allpars, nomatch = 0) == 0)
  if (length(m) > 0)
    stop("no parameter ", paste(pars[m], collapse = ', '))
  if (length(pars_wo_ws) == 0)
    stop("no parameter specified (pars is empty)")
  unique(pars_wo_ws)
}

check_pars_first <- function(object, pars) {
  # Check if all parameters in pars are valid parameters of the model
  # Args:
  #   object: a stanfit object
  #   pars: a character vector of parameter names
  # Returns:
  #   pars without white spaces, if any, if all are valid
  #   otherwise stop reporting error
  allpars <- cbind(object@model_pars, flatnames(object@model_pars))
  check_pars(allpars, pars)
}

check_pars_second <- function(sim, pars) {
  #
  # Check if all parameters in pars are parameters for which we saved
  # their samples
  #
  # Args:
  #   sim: The sim slot of class stanfit
  #   pars: a character vector of parameter names
  #
  # Returns:
  #   pars without white spaces, if any, if all are valid
  #   otherwise stop reporting error
  if (missing(pars)) return(sim$pars_oi)
  allpars <- c(sim$pars_oi, sim$fnames_oi)
  check_pars(allpars, pars)
}

remove_empty_pars <- function(pars, model_dims) {
  #
  # Remove parameters that are actually empty, which
  # could happen when for exmample a user specify the
  # following stan model code:
  #
  # transformed data { int n; n <- 0; }
  # parameters { array[n] real y; }
  #
  # Args:
  #   pars: a character vector of parameters names
  #   model_dims: a named list of the parameter dimension
  #
  # Returns:
  #   A character vector of parameter names with empty parameter
  #   being removed.
  #
  ind <- rep(TRUE, length(pars))
  model_pars <- names(model_dims)
  if (is.null(model_pars)) stop("model_dims need be a named list")
  for (i in seq_along(pars)) {
    p <- pars[i]
    m <- match(p, model_pars)
    if (!is.na(m) && prod(model_dims[[p]]) == 0)  ind[i] <- FALSE
  }
  pars[ind]
}

pars_total_indexes <- function(names, dims, fnames, pars) {
  # Obtain the total indexes for parameters (pars) in the
  # whole sequences of names that is order by 'column major.'
  # Args:
  #   names: all the parameters names specifying the sequence of parameters
  #   dims:  the dimensions for all parameters, the order for all parameters
  #          should be the same with that in 'names'
  #   fnames: all the parameter names specified by names and dims
  #   pars:  the parameters of interest. This function assumes that
  #     pars are in names.
  # Note: inside each parameter (vector or array), the sequence is in terms of
  #   col-major. That means if we have parameter alpha and beta, the dims
  #   of which are [2,2] and [2,3] respectively.  The whole parameter sequence
  #   are alpha[1,1], alpha[2,1], alpha[1,2], alpha[2,2], beta[1,1], beta[2,1],
  #   beta[1,2], beta[2,2], beta[1,3], beta[2,3]. In addition, for the col-majored
  #   sequence, an attribute named 'row_major_idx' is attached, which could
  #   be used when row major index is favored.

  starts <- calc_starts(dims)
  par_total_indexes <- function(par) {
    # for just one parameter
    #
    p <- match(par, fnames)
    # note that here when `par' is a scalar, it would
    # match one of `fnames'
    if (!is.na(p)) {
      names(p) <- par
      attr(p, "row_major_idx") <- p
      return(p)
    }
    p <- match(par, names)
    np <- num_pars(dims[[p]])
    if (np == 0) return(NULL)
    idx <- starts[p] + seq(0, by = 1, length.out = np)
    names(idx) <- fnames[idx]
    attr(idx, "row_major_idx") <- starts[p] + idx_col2rowm(dims[[p]]) - 1
    idx
  }
  idx <- lapply(pars, FUN = par_total_indexes)
  nulls <- sapply(idx, is.null)
  idx <- idx[!nulls]
  names(idx) <- pars[!nulls]
  idx
}

rstancolgrey <- rgb(matrix(c(247, 247, 247, 204, 204, 204, 150, 150, 150, 82, 82, 82),
                           byrow = TRUE, ncol = 3),
                    alpha = 100,
                    names = paste(1:4), maxColorValue = 255)

# from https://colorbrewer2.org/, colorblind safe,
# 6 different colors, diverging
rstancolc <- rgb(matrix(c(230, 97, 1,
                          153, 142, 195,
                          84, 39, 136,
                          241, 163, 64,
                          216, 218, 235,
                          254, 224, 182),
                        byrow = TRUE, ncol = 3),
                 names = paste(1:6), maxColorValue = 255)

default_summary_probs <- function() c(0.025, 0.05, 0.10, 0.25, 0.50, 0.75, 0.90, 0.95, 0.975)

## summarize the chains merged and individually
get_par_summary <- function(sim, n, probs = default_summary_probs()) {
  ss <- lapply(1:sim$chains,
               function(i) {
                 if (sim$warmup2[i] == 0) sim$samples[[i]][[n]]
                 else sim$samples[[i]][[n]][-(1:sim$warmup2[i])]
               })
  msdfun <- function(chain) c(mean(chain, na.rm = TRUE), sd(chain, na.rm = TRUE))
  qfun <- function(chain) quantile(chain, probs = probs, na.rm = TRUE)
  c_msd <- unlist(lapply(ss, msdfun), use.names = FALSE)
  c_quan <- unlist(lapply(ss, qfun), use.names = FALSE)
  ass <- do.call(c, ss)
  msd <- msdfun(ass)
  quan <- qfun(ass)
  list(msd = msdfun(ass), quan = qfun(ass), c_msd = c_msd, c_quan = c_quan)
}

# mean and sd
get_par_summary_msd <- function(sim, n) {
  ss <- lapply(1:sim$chains,
               function(i) {
                 if (sim$warmup2[i] == 0) sim$samples[[i]][[n]]
                 else sim$samples[[i]][[n]][-(1:sim$warmup2[i])]
               })
  sumfun <- function(chain) c(mean(chain), sd(chain))
  cs <- lapply(ss, sumfun)
  as <- sumfun(do.call(c, ss))
  list(msd = as, c_msd = unlist(cs, use.names = FALSE))
}

# quantiles
get_par_summary_quantile <- function(sim, n, probs = default_summary_probs()) {
  ss <- lapply(1:sim$chains,
               function(i) {
                 if (sim$warmup2[i] == 0) sim$samples[[i]][[n]]
                 else sim$samples[[i]][[n]][-(1:sim$warmup2[i])]
               })
  sumfun <- function(chain) quantile(chain, probs = probs, na.rm = TRUE)
  cs <- lapply(ss, sumfun)
  as <- sumfun(do.call(c, ss))
  list(quan = as, c_quan = unlist(cs, use.names = FALSE))
}

combine_msd_quan <- function(msd, quan) {
  # Combine msd and quantiles for chain's summary
  # Args:
  #   msd: the array for mean and sd with dim num.par * 2 * chains
  #   cquan: the array for quantiles with dim num.par * n.quan * chains
  dim1 <- dim(msd)
  dim2 <- dim(quan)
  if (any(dim1[c(1, 3)] != dim2[c(1, 3)]))
    stop("numers of parameter/chains differ in msd and quan")
  chains <- dim1[3]
  n_par <- dim1[1]
  n_stat <- dim1[2] + dim2[2]
  par_names <- dimnames(msd)[[1]]
  stat_names <- c(dimnames(msd)[[2]], dimnames(quan)[[2]])
  chain_id_names <- dimnames(msd)[[3]]
  fun <- function(i) {
    # This is a bit ugly; one reason is that we need to
    # deal with the case that dim1[1] = 1, in which
    # a1 is a vector.
    a1 <- msd[, , i]
    a2 <- quan[, , i]
    dim(a1) <- dim1[1:2]
    dim(a2) <- dim2[1:2]
    cbind(a1, a2)
  }
  ll <- lapply(1:chains, fun)
  twodnames <- dimnames(ll[[1]])
  msdquan <- array(unlist(ll), dim = c(n_par, n_stat, chains))
  dimnames(msdquan) <- list(parameter = par_names, stats = stat_names,
                            chains = chain_id_names)
  msdquan
}


summary_sim <- function(sim, pars, probs = default_summary_probs()) {
  # cat("summary_sim is called.\n")
  probs_len <- length(probs)
  pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
  tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
  tidx_rowm <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)
  tidx_len <- length(tidx)
  tidx_rowm <- unlist(tidx_rowm, use.names = FALSE)
  lmsdq <- lapply(tidx, function(n) get_par_summary(sim, n, probs))
  msd <- do.call(rbind, lapply(lmsdq, function(x) x$msd))
  quan <- do.call(rbind, lapply(lmsdq, function(x) x$quan))
  probs_str <- colnames(quan)
  dim(msd) <- c(tidx_len, 2)
  dim(quan) <- c(tidx_len, probs_len)
  rownames(msd) <- sim$fnames_oi[tidx]
  rownames(quan) <- sim$fnames_oi[tidx]
  colnames(msd) <- c("mean", "sd")
  colnames(quan) <- probs_str

  c_msd <- do.call(rbind, lapply(lmsdq, function(x) x$c_msd))
  c_quan <- do.call(rbind, lapply(lmsdq, function(x) x$c_quan))
  dim(c_msd) <- c(tidx_len, 2, sim$chains)
  dim(c_quan) <- c(tidx_len, probs_len, sim$chains)

  sim_attr_args <- attr(sim, "args")
  cids <- if (is.null(sim_attr_args)) {
            cids <- 1:sim$chains
          } else {
            sapply(attr(sim, "args"), function(x) x$chain_id)
          }

  dimnames(c_msd) <- list(parameters = sim$fnames_oi[tidx],
                          stats = c("mean", "sd"),
                          chains = paste0("chain:", cids))
  dimnames(c_quan) <- list(parameters = sim$fnames_oi[tidx],
                           stats = probs_str,
                           chains = paste0("chains:", cids))

  if ("diagnostics" %in% names(sim) & "n_eff" %in% names(sim$diagnostics)) {
      #vb
      ess <- array(sim$diagnostics$n_eff, dim = c(tidx_len, 1))
      khat <- array(sim$diagnostics$theta_pareto_k, dim = c(tidx_len, 1))
      mcse <- array(sim$diagnostics$mcse, dim = c(tidx_len, 1))
      ss <- list(msd = msd, sem = mcse,
                 c_msd = c_msd, quan = quan, c_quan = c_quan,
                 ess = ess, khat = khat)
  } else {
      ess <-  array(sapply(tidx, function(n) rstan_ess(sim, n)), dim = c(tidx_len, 1))
      rhat <- array(sapply(tidx, function(n) rstan_splitrhat(sim, n)), dim = c(tidx_len, 1))
      ss <- list(msd = msd, sem = msd[, 2] / sqrt(ess),
                 c_msd = c_msd, quan = quan, c_quan = c_quan,
                 ess = ess, rhat = rhat)
  }
  attr(ss, "row_major_idx") <- tidx_rowm
  attr(ss, "col_major_idx") <- tidx
  ss
}

summary_sim_quan <- function(sim, pars, probs = default_summary_probs()) {
  probs_len <- length(probs)
  pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
  tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
  tidx_rowm <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)
  tidx_len <- length(tidx)
  tidx_rowm <- unlist(tidx_rowm, use.names = FALSE)
  lquan <- lapply(tidx, function(n) get_par_summary_quantile(sim, n, probs))
  quan <- do.call(rbind, lapply(lquan, function(x) x$quan))
  probs_str <- colnames(quan)
  dim(quan) <- c(tidx_len, probs_len)
  rownames(quan) <- sim$fnames_oi[tidx]
  colnames(quan) <- probs_str

  sim_attr_args <- attr(sim, "args")
  cids <- if (is.null(sim_attr_args)) {
            cids <- 1:sim$chains
          } else {
            sapply(attr(sim, "args"), function(x) x$chain_id)
          }

  c_quan <- do.call(rbind, lapply(lquan, function(x) x$c_quan))
  dim(c_quan) <- c(tidx_len, probs_len, sim$chains)
  dimnames(c_quan) <- list(parameters = sim$fnames_oi[tidx],
                           stats = probs_str,
                           chains = paste0("chains:", cids))

  ss <- list(quan = quan, c_quan = c_quan)
  attr(ss, "row_major_idx") <- tidx_rowm
  attr(ss, "col_major_idx") <- tidx
  ss
}

summary_sim_ess <- function(sim, pars) {
  pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
  tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
  tidx_rowm <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)
  tidx_rowm <- unlist(tidx_rowm, use.names = FALSE)
  ess <- sapply(tidx, function(n) rstan_ess(sim, n))
  names(ess) <- sim$fnames_oi[tidx]
  attr(ess, "row_major_idx") <- tidx_rowm
  attr(ess, "col_major_idx") <- tidx
  ess
}

summary_sim_rhat <- function(sim, pars) {
  pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
  tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)
  tidx_rowm <- lapply(tidx, function(x) attr(x, "row_major_idx"))
  tidx <- unlist(tidx, use.names = FALSE)
  tidx_rowm <- unlist(tidx_rowm, use.names = FALSE)
  rhat <- sapply(tidx, function(n) rstan_splitrhat(sim, n))
  names(rhat) <- sim$fnames_oi[tidx]
  attr(rhat, "row_major_idx") <- tidx_rowm
  attr(rhat, "col_major_idx") <- tidx
  rhat
}


create_skeleton <- function(pars, dims) {
  # for the purpose of using relist to convert
  # vector to list
  lst <- lapply(seq_along(pars),
                function(i) {
                  len_dims <- length(dims[[i]])
                  if (len_dims < 1) return(0)
                  return(array(0, dim = dims[[i]]))
                })
  names(lst) <- pars
  lst
}

rstan_relist <- function(x, skeleton) {
  lst <- relist(x, skeleton)
  for (i in seq_along(skeleton))
    dim(lst[[i]]) <- dim(skeleton[[i]])
  lst
}

# ported from bugs.plot.inferences in R2WinBUGS
#
stan_plot_inferences <- function(sim, summary, pars, model_info, display_parallel = FALSE, ...) {
  #
  # Args:
  #   sim: the sim list in stanfit object
  #   pars: parameters of interest
  #   model_info: names list with elements model_name and model_date
  #   display_parallel

  alert_col <- rstan_options("rstan_alert_col")
  chain_cols <- rstan_options("rstan_chain_cols")
  chain_cols.len <- length(chain_cols)

  if (.Device %in% c("windows", "X11cairo", 'quartz')) {
    cex.points <- .7
    min.width <- .02
  } else {
    cex.points <- .3
    min.width <- .01
  }

  cex_names <- .7
  cex.axis <- .6
  cex_tiny <- .4
  # the standard number of parameters in an array parameters.
  # we have this so that even the # of parameters are less than
  # 30, we still have equal space between parameters for
  # the whole plot.
  standard_width <- rstan_options('plot_standard_npar')
  max_width <- rstan_options('plot_max_npar')

  pars <- if (missing(pars)) sim$pars_oi else check_pars_second(sim, pars)
  n_pars <- length(pars)
  chains <- sim$chains

  tidx <- pars_total_indexes(sim$pars_oi, sim$dims_oi, sim$fnames_oi, pars)

  ## if in Splus, suppress printing of warnings during the plotting.
  ## otherwise a warning is generated
  if (!is.R()) {
    warn.settings <- options("warn")[[1]]
    options (warn = -1)
  }
  height <- .6
  # mar: c(bottom, left, top, right)
  par.old <- par(no.readonly = TRUE)
  on.exit(par(par.old))
  par(mar = c(1, 0, 1, 0))

  plot(c(0, 1), c(-n_pars - .5, -.4),
       ann = FALSE, bty = "n", xaxt = "n", yaxt = "n", type = "n")
  if (!is.R())
    options(warn = warn.settings)

  # plot the model general information
  header <- paste("Stan model '", model_info$model_name, "' (", chains,
                  " chains: iter=", sim$iter, "; warmup=", sim$warmup,
                  "; thin=", sim$thin, ") fitted at ",
                  model_info$model_date, sep = '')
  # side: (1=bottom, 2=left, 3=top, 4=right)
  mtext(header, side = 3, outer = TRUE, line = -1, cex = .7)

  W <- max(strwidth(pars, cex = cex_names))
  # the max width of the variable names

  # cex_names is defined at the beginning of this fun
  B <- (1 - W) / 3.8
  A <- 1 - 3.5 * B
  title <- if (display_parallel) "80% interval for each chain" else  "medians and 80% intervals"
  text(A, -.4, title, adj = 0, cex = cex_names)
  num_height <- strheight (1:9, cex = cex_tiny) * 1.2

  truncated <- FALSE
  for (k in 1:n_pars) {
    text (0, -k, pars[k], adj = 0, cex = cex_names)

    k_dim <- sim$dims_oi[[pars[k]]]
    k_dim_len <- length(k_dim)
    k_aidx <- seq_array_ind(k_dim, col_major = FALSE)

    # the index for the parameters in the whole
    # sequences of parameters
    index <- attr(tidx[[k]], "row_major_idx")

    # number of parameters we could plot for this
    # particular vector/array parameter
    k_num_p <- length(index)

    # number of parameter we would plot
    J <- min(k_num_p, max_width)
    spacing <- 3.5 / max(J, standard_width)

    # the medians for all the kept samples merged
    sprobs = default_summary_probs()
    mp <- match(0.5, sprobs)
    i80p <- match(c(0.1, 0.9), sprobs)
    med <- summary$quan[index, mp]
    med <- array(med, dim = c(k_num_p, 1))
    i80 <- summary$quan[index, i80p]
    i80 <- array(i80, dim = c(k_num_p, 2))
    rhats <- summary$rhat
    rhats_cols <- get_rhat_cols(rhats)

    med.chain <- summary$c_quan[index, mp, ]
    med.chain <- array(med.chain, dim = c(k_num_p, sim$chains))
    i80.chain <- summary$c_quan[index, i80p, ]
    i80.chain <- array(i80.chain, dim = c(k_num_p, 2, sim$chains))

    rng <- if (display_parallel) range(i80, i80.chain) else range(i80)
    p.rng <- pretty(rng, n = 2)
    b <- height / (max(p.rng) - min(p.rng))
    a <- -(k + height / 2) - b * p.rng[1]
    lines(A + c(0, 0), -k + 0.5 * height * c(-1, 1))

    # plot a line at zero (if zero is in the range of the mini-plot)
    if (min(p.rng) < 0 & max(p.rng) > 0) {
      lines(A + B * spacing * c(0, J + 1),
            rep(a, 2), lwd = .5, col = "gray")
    }
    # plot the breaks of the axis
    for (x in p.rng){
      text(A - B * .2, a + b * x, x, cex = cex.axis)
      lines(A + B * c(-.05, 0), rep(a + b * x, 2))
    }
    for (j in 1:J){
      if (display_parallel){
        for (m in 1:chains){
          interval <- a + b * i80.chain[j, , m]

          # When the interval is too tiny, we use the min.width instead
          # of the real one.
          if (interval[2] - interval[1] < min.width)
            interval <- mean(interval) + c(-.5, .5) * min.width
          segments(x0 = A + B * spacing * (j + .6 *(m - (chains + 1) / 2) / chains),
                   y0 = interval[1], y1 = interval[2], lwd = .5,
                   col = chain_cols[(m-1) %% chain_cols.len + 1])
        }
      } else {
        lines(A + B * spacing * rep(j, 2), a + b * i80[j,], lwd = .5)
        for (m in 1:chains)
          points(A + B * spacing * j, a + b * med.chain[j, m],
                 pch = 20, cex = cex.points,
                 col = chain_cols[(m-1) %% chain_cols.len + 1])
      }

      # draw an indicator for Rhat
      # (xleft, ybottom, xright, ytop)

      if (k_dim_len == 0)
        rect(A + B * spacing * (j - .5), -k - height / 2 - 0.05 + num_height * .5,
             A + B * spacing * (j + .5), -k - height / 2 - 0.05 - num_height * .5, col = rhats_cols[j], border = NA)

      # plot the dimension indexes for this parameter
      if (k_dim_len  >= 1) {
        rect(A + B * spacing * (j - .5), -k - height / 2 - 0.05 + num_height * .5,
             A + B * spacing * (j + .5), -k - height / 2 - 0.05 - num_height * (k_dim_len - .5), col = rhats_cols[j], border = NA)

        # k_dim: the dimension of parameter k
        for (m in 1:k_dim_len) {
          index0 <- k_aidx[j, m]
          if (j == 1)
            text(A+B*spacing*j, -k-height/2-.05-num_height*(m-1), index0, cex=cex_tiny)
          else if (index0 != k_aidx[j - 1, m] & (index0 %% (floor(log10(index0) + 1)) == 0))
            text(A+B*spacing*j, -k-height/2-.05-num_height*(m-1), index0, cex=cex_tiny)

          # Note for `(index0 %% (floor(log10(index0) + 1)) == 0) in the above condition.
          # When 10 <= index0 <= 99, floor(log10(index0) + 1) == 2,
          # so that one index would be drawn out of two consecutive.
          # That is, we would have 10, 12, 14, 16, etc.
          # Similarly, when 100 <= index0 <= 999, we draw one out of three
          # though in the case, we do not draw them at all since the max is
          # 40.
        }
      }
    }
    if (J < k_num_p) {
      text (-.015, -k, "*", cex = cex_names, col = alert_col)
      truncated <- TRUE
    }
  }
  plot_rhat_legend(0, -n_pars - .5, cex = cex_names)
  if (truncated) {
    text(0, -n_pars - .5 - num_height * 2.5, "*  array truncated for lack of space",
         adj = c(0, 0), cex = cex_names, col = alert_col)
  }
  invisible(NULL)
}

legitimate_model_name <- function(name, obfuscate_name = TRUE) {
  # To make model name be a valid name in C++.
  # obfuscate_name

  namep1 <- if (obfuscate_name)  basename(tempfile('model', '')) else 'model'
  name <- paste(namep1, '_', name, sep = '')
  gsub('[^[:alnum:]]', '_', name)
  # return("anon_model")

  # Note: why using different (ideally unique) name?
  #
  # The name returned from this function is used
  # as Rcpp module name and the name for the stan_fit class
  # for each model. Actually we need a unique name. The reason
  # is that it seems if the Rcpp modules have the same name, a newly
  # created model created from compiling the C++ code would replace
  # previous one though the DSO files are different. I guess
  # that Rcpp implement the module by call the C++ function using .Call, we
  # would always call the function with the same name loaded later. I am
  # not sure the real reason, but experiments do show that
  # later modules created would use previous one if the class name
  # in the module is the same. So if obfuscate_name = TRUE, we try
  # to generate a unique name, if FALSE, it is the user's responsibility
  # to keep the name unique and in the case, users might be able to
  # take advantage of tools such as ccache
}

boost_url <- function() {"https://www.boost.org/users/download/"}

makeconf_path <- function() {
  RMU <- Sys.getenv("R_MAKEVARS_USER")
  if (!is.na(RMU) && RMU != "") return(RMU)
  arch <- .Platform$r_arch
  if (arch == '')
    return(file.path(R.home(component = 'etc'), 'Makeconf'))
  return(file.path(R.home(component = 'etc'), arch, 'Makeconf'))
}

is_null_ptr <- function(ns) {
  .Call(is_Null_NS, ns)
}

is_null_cxxfun <- function(cx) {
  # Tell if the returned object from cxxfunction in package inline
  # contains null pointer
  add <- body(cx@.Data)[[2]]
  # add is of class NativeSymbol
  .Call(is_Null_NS, add)
}

obj_size_str <- function(x) {
  if (x >= 1024^3)       return(paste(round(x/1024^3, 1L), "Gb"))
  else if (x >= 1024^2)  return(paste(round(x/1024^2, 1L), "Mb"))
  else if (x >= 1024)    return(paste(round(x/1024, 1L), "Kb"))
  return(paste(x, "bytes"))
}

system_info <- function() {
  paste("OS: ", R.version$system,
        "; rstan: ",  packageVersion('rstan'),
        "; Rcpp: ", packageVersion('Rcpp'),
        "; inline: ", packageVersion('inline'), sep = '')
}

read_comments <- function(f, n = -1) {
  # Read comments beginning with `#`
  # Args:
  #   f: the filename
  #   n: max number of line; -1 means all
  # Returns:
  #   a vector of strings
  con <- file(f, 'r')
  comments <- list()
  iter <- 0
  while (length(input <- readLines(con, n = 1)) > 0) {
    if (n > 0 && n <= iter) break;
    if (grepl("#", input)) {
      comments <- c(comments, gsub("^.*#", "#", input))
      iter <- iter + 1
    }
  }
  close(con)
  do.call(c, comments)
}

sqrfnames_to_dotfnames <- function(fnames) {
  # change names such as alpha[1,1] to alpha.1.1
  gsub('\\]', '', gsub('\\[|,', '.', fnames))
}


dotfnames_to_sqrfnames <- function(fnames) {
  fnames <- sapply(fnames,
                   function(i) {
                     if (!grepl("\\.", i)) return(i)
                     i <- sub("\\.", "[", i)
                     i <- sub("\\s*$", "]", i)
                     i }, USE.NAMES = FALSE)
  gsub("\\.\\s*", ",", fnames)
}

unique_par <- function(fnames) {
  # obtain parameters from flat names in format of say alpha.1,
  # alpha.2, beta.1.1, ..., beta.3.4, --- in this case, return
  # c('alpha', 'beta')
  unique(gsub('\\..*', '', fnames))
}


get_dims_from_fnames <- function(fnames, pname) {
  # Get the dimension for a parameter from
  # the flatnames such as "alpha.1.1", ..., "alpha.3.4", the
  # format of names in the CSV files generated by Stan.
  # Currently, this function assume fnames are correctly given.
  # Args:
  #   fnames: a character of names for one (vector/array) parameter
  #   pname: the name for this vector/array parameter such as "alpha"
  #     for the above example

  if (missing(pname)) pname <- gsub('\\..*', '', fnames[1])

  if (length(fnames) == 1 && fnames == pname)
    return(integer(0)) # a scalar

  idxs <- sub(pname, '', fnames, fixed = TRUE)
  lp <- gregexpr('\\d+', idxs)

  tfun <- function(name, start, i) {
    last <- attr(start, 'match.length')[i] + start[i]
    # cat('name=', name, ', start=', start[i], ', last=', last, '.\n', sep = '')
    as.integer(substr(name, start[i], last))
  }

  dim_len <- length(lp[[1]])
  dims <- integer(dim_len)
  for (i in 1:dim_len) {
    dimi <- mapply(tfun, idxs, lp, MoreArgs = list(i = i), USE.NAMES = FALSE)
    dims[i] <- max(dimi)
  }
  dims
}

all_int_eq <- function(is) {
  # tell if all integers in 'is' are the same
  if (!all(is.integer(is)))
    stop("not all are integers")
  min(is) == max(is)
}

read_csv_header <- function(f, comment.char = '#') {
  # Read the header of a csv file (the first line not beginning with
  # comment.char). And the line number is return as attribute of name 'lineno'.
  con <- file(f, 'r')
  niter <- 0
  iter.count <- NA
  save.warmup <- FALSE
  sample.count <- NA_integer_
  thin <- NULL
  while (length(input <- readLines(con, n = 1)) > 0) {
    niter <- niter + 1
    if (!grepl(comment.char, input)) break;
    if (grepl("# iter=",input))
      iter.count <- as.integer(gsub("# iter=","",input))
    if (grepl("#.*num_samples",input)){
      sample.count <- as.integer(gsub("[^0-9]*([0-9]*).*","\\1",input))
    }
    if (grepl("#.*num_warmup",input)){
      warmup.count <- as.integer(gsub("[^0-9]*([0-9]*).*","\\1",input))
    } else {
      warmup.count <- 0L
    }
    if (grepl("#.*thin", input)){
      thin <- as.integer(gsub("[^0-9]*([0-9]*).*","\\1",input))
    }
    if (grepl("#.*save_warmup",input)){
      save.warmup <- !grepl("0",input)
    }
    if (grepl("#.*output_sample",input)){
      iter.count <- as.numeric(gsub("[^0-9]*([0-9]*).*","\\1",input))
    }

  }
  header <- input
  if(is.na(iter.count)){
    if(save.warmup)
      iter.count <- warmup.count + sample.count
    else
      iter.count <- sample.count
  }
  if(!is.null(thin)){
    iter.count <- iter.count %/% thin
  }
  attr(header, "iter.count") <- iter.count
  attr(header, "lineno") <- niter
  close(con)
  header
}

is_arg_recognizable <- function(x, y, pre_msg = '', post_msg = '', ...) {
  # check if all elements of x are in y.
  # x: a vector of characters
  # y: a vector of characters
  idx <- match(x, y)
  na_idx <- which(is.na(idx))
  if (length(na_idx) > 0) {
    stop(pre_msg, paste(x[na_idx], collapse = ', '), ".", post_msg, ...)
  }
  return(TRUE)
}

is_arg_deprecated <- function(x, y, pre_msg = '', post_msg = '', ...) {
  # check if any elements of x are in y.
  # x: a vector of characters
  # y: a vector of characters
  idx <- match(x, y)
  found_idx <- which(!is.na(idx))
  if (length(found_idx) > 0) {
    message(pre_msg, paste(x[found_idx], collapse = ', '), ".", post_msg, ...)
  }
  return(TRUE)
}

get_time_from_csv <- function(tlines) {
  # get the warmup time and sample time from the commented lines
  # about time in the CSV files
  # Args:
  #  tlines: character vector of length 3 (or 2 since the last one is not used)
  #          from the CSV File. For example, it could be
  #          # Elapsed Time: 0.005308 seconds (Warm-up)
  #                          0.003964 seconds (Sampling)
  #                          0.009272 seconds (Total)
  t <- rep(NA, 2)
  names(t) <- c("warmup", "sample")
  if (length(tlines) < 2) return(t)
  warmupt <- gsub("[^0-9.]", "", tlines[1])
  samplet <- gsub("[^0-9.]", "", tlines[2])
  t[1] <- as.double(warmupt)
  t[2] <- as.double(samplet)
  t
}

parse_data <- function(cppcode) {
  cppcode <- scan(what = character(), sep = "\n", quiet = TRUE,
                  text = cppcode)
  private <- grep("^private:$", cppcode) + 1L
  public <- grep("^public:$", cppcode) - 1L
  # pull out object names from the data block
  objects <- gsub("^.* ([0-9A-Za-z_]+).*;.*$", "\\1",
                  cppcode[private:public])
  # Remove model internal name _data__ suffix for stanc3 v2.30+
  objects <- gsub("_data__$", "", objects)
  # Remove model internal name underscores in case of Eigen::Maps
  objects <- gsub("__$", "\\1", objects)
  # Remove any bad regex matches that found the end of an Eigen::Map.
  objects <- gsub("^[[:digit:]]+", "\\1", objects)
  # Remove empty characters and trim whitespaces
  objects <- objects[nzchar(trimws(objects))]

  # Get them from the calling environment
  stuff <- list()
  for (int in seq_along(objects)) {
   stuff[[objects[int]]] <- dynGet(objects[int], inherits = FALSE, ifnotfound = NULL)
  }
  for (i in seq_along(stuff)) if (is.null(stuff[[i]])) {
    if (exists(objects[i], envir = globalenv(), mode = "numeric"))
      stuff[[i]] <- get(objects[i], envir = globalenv(), mode = "numeric")
    else if (exists(objects[i], envir = globalenv(), mode = "logical"))
      stuff[[i]] <- get(objects[i], envir = globalenv(), mode = "logical")
  }
  return(stuff)
}

set_cppo <- function(...) {
  warning("'set_cppo' is defunct; manually edit your Makevars file if necessary")
  return(invisible(NULL))
}

get_stan_param_names <- function(object) {
  stopifnot(is(object, "stanfit"))
  params <- grep("vals_r__ = context__.vals_r(", fixed = TRUE, value = TRUE,
                 x = strsplit(get_cppcode(get_stanmodel(object)), "\n")[[1]])
  params <- sapply(strsplit(params, "\""), FUN = function(x) x[[2]])
  params <- intersect(params, object@sim$pars_oi)
  stopifnot(length(params) > 0)
  return(params)
}

create_progress_html_file <- function(htmlfname, textfname) {
  # Args:
  #   htmlfname: the HTML file name
  #   textfname: the text file name
  template_file <- file.path(system.file('misc', package = 'rstan'), 'stan_progress.html')
  src <- paste(readLines(template_file), collapse = '\n')
  src2 <- sub("%filename%", textfname, sub("%title%", textfname, src, fixed = TRUE), fixed = TRUE)
  cat(src2, file = htmlfname)
}

get_CXX <- function(...) {
  if (.Platform$OS.type != "windows")
    return (system2(file.path(R.home(component = "bin"), "R"),
            args = "CMD config CXX17", stdout = TRUE, stderr = FALSE))

  ls_path <- Sys.which("ls")
  if (ls_path == "") return(NULL)

  install_path <- dirname(dirname(ls_path))
  file.path(install_path,
            paste0('mingw_', Sys.getenv('WIN')), 'bin', 'g++')
}

is.sparc <- function() {
  grepl("^sparc",  R.version$platform)
}

avoid_crash <- function(mod) {
  file.exists(get("packageName", envir = mod)[["path"]]) &&
  as(get("packageName", envir = mod)["info"][1], "character") %in% 
    c("<pointer: (nil)>", "<pointer: 0x0>")
}

# @param x numeric vector
log_sum_exp <- function(x) {
  max_x <- max(x)
  max_x + log(sum(exp(x - max_x)))
}

sample_indices <- function(wts, n_draws) {
  ## Stratified resampling
  ##   Kitagawa, G., Monte Carlo Filter and Smoother for Non-Gaussian
  ##   Nonlinear State Space Models, Journal of Computational and
  ##   Graphical Statistics, 5(1):1-25, 1996.
  K <- length(wts)
  w <- n_draws * wts # expected number of draws from each model
  idx <- rep(NA, n_draws)

  c <- 0
  j <- 0

  for (k in 1:K) {
    c <- c + w[k]
    if (c >= 1) {
      a <- floor(c)
      c <- c - a
      idx[j + 1:a] <- k
      j <- j + a
    }
    if (j < n_draws && c >= runif(1)) {
      c <- c - 1
      j <- j + 1
      idx[j] <- k
    }
  }
  return(idx)
}

test_221 <- function(cppcode) {
  grepl("Code generated by Stan version 2.2", cppcode, fixed = TRUE) ||
  grepl("Code generated by Stan version 3", cppcode, fixed = TRUE)  
}

.setUp <- function() {
  model_code <- "model { \n y ~ normal(0, 1); \n}"  
  cat(model_code, file = 'tmp.stan')  

  a <- c(1, 3, 5)
  b <- matrix(1:10, ncol = 2)
  c <- array(1:18, dim = c(2, 3, 3)) 
  dump(c("a", "b", "c"), file = 'dumpabc.Rdump')
  rstan:::stan_rdump(c("a", "b", "c"), file = 'standumpabc.Rdump')
  d <- factor(c("a", "b", "b", "a", "a"))
  e <- factor(c(1, 3, 3, 1, 1))
  f <- c(TRUE, TRUE, FALSE)
  g <- 1000000000
  h <- 1.1e+10
  rstan:::stan_rdump(c("d", "e", "f"), file = 'standumpabc.Rdump', append = TRUE)
  rstan:::stan_rdump(c("g", "h"), file = 'standumpabc.Rdump', append = TRUE)

  cc <- c("# comment line 1", 
          " no comments line 1", 
          "# comment line 2",
          "# comment line 3", 
          "# comment line 4", 
          "# comment line 5", 
          " not comments line 2", 
          "# comment line 6",
          "not comments #comment line 7",
          "not comments at the end of file")
  cat(file = 'cc.csv', paste(cc, collapse = '\n'), "\n")
  cc2 <- c("# line 1", "#line 2", "# line 3", "#line4", "c,a,b", "1,2,3")
  cat(file = 'cc2.csv', paste(cc2, collapse = '\n'), "\n")
} 

test_get_model_strcode <- function() {
  model_code <- "model { \n y ~ normal(0, 1); \n}"  
  code <- 'parameters { real y; } model{y ~ normal(0,1);}'
  str1 <- rstan:::get_model_strcode("tmp.stan") 
  str2 <- rstan:::get_model_strcode(model_code = code)
  str3 <- rstan:::get_model_strcode(model_code = 'code') 
  str4 <- rstan:::get_model_strcode(model_code = 'parameters {real y;} model {y ~ normal(0,1); }') 
 
  mname1 <- attr(str1, 'model_name2')
  mname2 <- attr(str2, 'model_name2')
  mname3 <- attr(str3, 'model_name2')
  mname4 <- attr(str4, 'model_name2')
  checkEquals(mname1, 'tmp')
  checkEquals(mname2, 'code') 
  checkEquals(mname3, 'code')
  checkEquals(mname4, 'anon_model')
   
  attributes(str1) <- NULL 
  attributes(str2) <- NULL 
  attributes(str3) <- NULL 
  checkEquals(str1, model_code) 
  checkEquals(str2, code) 
  checkEquals(str3, code) 

  model_code <- "model { \n y ~ normal(0, 1); \n}"  
  # cat(model_code, file = 'tmp.stan')  
  checkEquals(model_code, rstan:::read_model_from_con('tmp.stan'), 
              msg = "Read stan model from file") 
  attr(model_code, 'model_name2') <- 'tmp' 
  checkEquals(model_code, rstan:::get_model_strcode('tmp.stan'), 
              msg = "Read stan model from file") 
  attr(model_code, 'model_name2') <- 'model_code' 
  checkEquals(model_code, rstan:::get_model_strcode(model_code = model_code), 
              msg = "Read stan model from model_code") 
  checkException(rstan:::get_model_strcode(), 
                 msg = "Read stan model from model_code") 
} 

test_is_valid_stan_name <- function() {
  checkTrue(!rstan:::is_legal_stan_vname('7dd'))
  checkTrue(!rstan:::is_legal_stan_vname('model'))
  checkTrue(!rstan:::is_legal_stan_vname('private'))
  checkTrue(!rstan:::is_legal_stan_vname('hello__'))
  checkTrue(rstan:::is_legal_stan_vname('y'))
} 

test_is_named_list <- function() {
  checkTrue(!rstan:::is_named_list(c(2, 3)))
  checkTrue(!rstan:::is_named_list(list(3, 4)))
  checkTrue(!rstan:::is_named_list(list(a = 3, 4)))
  checkTrue(rstan:::is_named_list(list(a = 3, b = 4)))
} 

test_data_preprocess <- function() {
  lst <- list(z = c(1L, 2L, 4L), 
              a = 1:100, 
              b = matrix(1:9 / 9, ncol = 3), 
              c = structure(1:100, .Dim = c(5, 20)),
              g = array(c(3, 3, 9, 3, 3, 4, 5, 6, 9, 8, 0, 2), dim = c(2, 2, 3)), 
              d = 1:100 + .1) 
  lst <- rstan:::data_preprocess(lst) 
  lst2 <- lst  
  lst2$f <- matrix(c(3, NA, NA, NA, 3, 4), ncol = 3) 
  lst3 <- lst
  lst3$h <- gl(3, 4)
  lst4 <- rstan:::data_preprocess(lst3)

  checkEquals(dim(lst$g), c(2, 2, 3), "Keep the dimension information")
  checkTrue(is.integer(lst$z), "Do as.integer when appropriate") 
  checkTrue(is.double(lst$b), msg = "Not do as.integer when it is not appropriate") 
  checkException(rstan:::data_preprocess(lst2), 
                 msg = "Stop if data have NA") 
  checkEquals(names(lst4), c("z", "a", "b", "c", "g", "d")) # check if h is removed

} 

test_data_preprocess2 <- function() {
  # a list of array as an element of the data list 
  I <- 3; J <- 4; K <- 5
  a <- lapply(1:I, function(i) rnorm(J))
  b <- lapply(1:I, function(i) matrix(rnorm(J * K), ncol = K))
  d <- lapply(1:I, function(i) rnorm(1, i))
  e <- lapply(1:I, function(i) rpois(J, 1) + 1.0) 
  lst2 <- rstan:::data_preprocess(list(a = a, b = b, d = d, e = e))
  checkEquals(dim(lst2$a), c(I, J))
  checkEquals(dim(lst2$b), c(I, J, K))
  checkEquals(dim(lst2$d), c(I, 1))
  checkTrue(is.integer(lst2$e[1, 1])) 
  checkTrue(!is.integer(e[[1]][1])) 

  a1 <- lapply(1:I, function(i) rnorm(J))
  a2 <- lapply(1:I, function(i) rnorm(J))
  checkException(rstan:::data_preprocess(list(a = list(a1 = a1, a2 = a2))))
} 


test_read_rdump <- function() {
  l <- rstan:::read_rdump("dumpabc.Rdump")
  checkEquals(l$a, c(1, 3, 5)) 
  checkEquals(l$b, matrix(1:10, ncol = 2))
  checkEquals(l$c, array(1:18, dim = c(2, 3, 3))) 
} 

test_stan_rdump <- function() {
  l <- rstan:::read_rdump("standumpabc.Rdump")
  checkEquals(l$a, c(1, 3, 5)) 
  checkEquals(l$b, matrix(1:10, ncol = 2))
  checkEquals(l$c, array(1:18, dim = c(2, 3, 3))) 
  checkEquals(l$d, c(1, 2, 2, 1, 1))
  checkEquals(l$e, c(1, 2, 2, 1, 1))
  checkEquals(l$f, c(1, 1, 0))
  text <- paste(readLines('standumpabc.Rdump'), collapse = "|")
  checkTrue(grepl("1000000000", text))
  checkEquals(l$h, 1.1e10)
} 

test_seq_array_ind <- function() {
  a <- rstan:::seq_array_ind(numeric(0))
  checkEquals(length(a), 0) 
  # by default, col_major is FALSE
  b <- rstan:::seq_array_ind(2:5, col_major = TRUE) 
  c <- arrayInd(1:prod(2:5), .dim = 2:5) 
  checkEquals(b, c) 
  d <- rstan:::seq_array_ind(2:3, col_major = FALSE)
  e <- matrix(c(1, 1, 1, 2, 1, 3, 2, 1, 2, 2, 2, 3), 
              nrow = 6, byrow = TRUE)
  checkEquals(d, as.array(e)) 
  f <- rstan:::seq_array_ind(1)
  checkEquals(f, array(1, dim = c(1, 1)))
} 

test_flatnames <- function() {
  names <- c("alpha", "beta", "gamma", "delta") 
  dims <- list(alpha = integer(0), beta = c(2, 3), gamma = c(2, 3, 4), delta = c(5))
  fnames <- rstan:::flatnames(names, dims)  
  checkEquals(fnames, 
             c('alpha', "beta[1,1]", "beta[1,2]", "beta[1,3]", 
                        "beta[2,1]", "beta[2,2]", "beta[2,3]", 
                        "gamma[1,1,1]", "gamma[1,1,2]", "gamma[1,1,3]", "gamma[1,1,4]", 
                        "gamma[1,2,1]", "gamma[1,2,2]", "gamma[1,2,3]", "gamma[1,2,4]", 
                        "gamma[1,3,1]", "gamma[1,3,2]", "gamma[1,3,3]", "gamma[1,3,4]", 
                        "gamma[2,1,1]", "gamma[2,1,2]", "gamma[2,1,3]", "gamma[2,1,4]", 
                        "gamma[2,2,1]", "gamma[2,2,2]", "gamma[2,2,3]", "gamma[2,2,4]", 
                        "gamma[2,3,1]", "gamma[2,3,2]", "gamma[2,3,3]", "gamma[2,3,4]", 
                        "delta[1]", "delta[2]", "delta[3]", "delta[4]", "delta[5]")) 
  names2 <- c('alpha') 
  dims2 <- list(alpha = integer(0))
  fnames2 <- rstan:::flatnames(names2, dims2)  
  checkEquals(fnames2, "alpha")
} 

test_idx_col2rowm <- function() {
  d <- integer(0) 
  idx <- rstan:::idx_col2rowm(d) 
  checkEquals(idx, 1) 
  d2 <- 8 
  idx2 <- rstan:::idx_col2rowm(d2) 
  checkEquals(idx2, 1:8) 
  d3 <- c(3, 4, 5) 
  idx3 <- rstan:::idx_col2rowm(d3) 
  yidx3 <- c(1, 13, 25, 37, 49, 4, 16, 28, 40, 52, 7, 19, 31, 43, 55,
             10, 22, 34, 46, 58, 2, 14, 26, 38, 50, 5, 17, 29, 41, 53, 8, 20, 32, 44, 56,
             11, 23, 35, 47, 59, 3, 15, 27, 39, 51, 6, 18, 30, 42, 54, 9, 21, 33, 45, 57,
             12, 24, 36, 48, 60) 
  checkEquals(idx3, yidx3)
} 

test_idx_row2colm <- function() {
  d <- integer(0) 
  idx <- rstan:::idx_row2colm(d) 
  checkEquals(idx, 1) 
  d2 <- 8 
  idx2 <- rstan:::idx_row2colm(d2) 
  checkEquals(idx2, 1:8) 
  d3 <- c(3, 4, 5) 
  idx3 <- rstan:::idx_row2colm(d3) 
  yidx3 <- c(1, 21, 41, 6, 26, 46, 11, 31, 51, 16, 36, 56, 2, 22, 42, 7, 27, 47, 12, 32, 52,
             17, 37, 57, 3, 23, 43, 8, 28, 48, 13, 33, 53, 18, 38, 58, 4, 24, 44, 9, 29, 49,
             14, 34, 54, 19, 39, 59, 5, 25, 45, 10, 30, 50, 15, 35, 55, 20, 40, 60) 
  checkEquals(idx3, yidx3)
} 

test_pars_total_indexes <- function() {
  names <- "alpha0" 
  dims <- list(alpha0 = c(2, 3)) 
  fnames <- rstan:::flatnames(names, dims)  
  tidx <- rstan:::pars_total_indexes(names, dims, fnames, "alpha0") 
  tidx.attr1 <- attr(tidx[[1]], "row_major_idx") 
  attributes(tidx[[1]]) <- NULL 
  checkEquals(unname(tidx[[1]]), 1:6) 
  checkEquals(unname(tidx.attr1), c(1, 3, 5, 2, 4, 6)) 
  names2 <- c(names, "alpha") 
  dims2 <- c(dims, list(alpha = 8)) 
  fnames2 <- rstan:::flatnames(names2, dims2)  
  tidx2 <- rstan:::pars_total_indexes(names2, dims2, fnames2, "alpha") 
  tidx2.attr1 <- attr(tidx2[[1]], "row_major_idx")
  attributes(tidx2[[1]]) <- NULL
  checkEquals(unname(tidx2[[1]]), 6 + 1:8)  
  checkEquals(unname(tidx2.attr1), 6 + 1:8)
  names3 <- c(names2, "p") 
  dims3 <- c(dims2, list(p = integer(0)))
  fnames3 <- rstan:::flatnames(names3, dims3)  
  tidx3 <- rstan:::pars_total_indexes(names3, dims3, fnames3, "p") 
  tidx3.attr1 <- attr(tidx3[[1]], "row_major_idx")
  attributes(tidx3[[1]]) <- NULL
  checkEquals(unname(tidx3[[1]]), 15)  
  checkEquals(unname(tidx3.attr1), 15)
} 

test_multi_idx_row2colm <- function() {
  checkEquals(rstan:::multi_idx_row2colm(list(integer(0))), 1)
  dims <- list(c(3), c(2,3), integer(0), c(2))
  col_idx <- rstan:::multi_idx_row2colm(dims)
  target <- c(1, 2, 3, 4, 7, 5, 8, 6, 9, 10, 11, 12)
  checkEquals(col_idx, target)

  fnames <- c("alpha[1]", "alpha[2]", "alpha[3]", 
              "alpha2[1,1]", "alpha2[1,2]", "alpha2[1,3]", 
              "alpha2[2,1]", "alpha2[2,2]", "alpha2[2,3]", 
              "p", "theta[1]", "theta[2]")
  fnames_colm <- c("alpha[1]", "alpha[2]", "alpha[3]", 
                   "alpha2[1,1]", "alpha2[2,1]", "alpha2[1,2]", "alpha2[2,2]", 
                   "alpha2[1,3]", "alpha2[2,3]", "p", "theta[1]", "theta[2]")
  checkEquals(fnames[col_idx], fnames_colm)
} 

test_mklist <- function() {
  x <- 3:5 
  y <- array(1:9, dim = c(3, 3))  
  z <- list(p = 3)
  f <- function() { TRUE }
  a <- list(x = x, y = y) 
  b <- rstan:::mklist(c("x", "y")) 
  checkTrue(identical(a, b)) 
  c <- list(x = x, y = y, z = z) 
  d <- rstan:::mklist(c("x", "y", "z"))
  checkTrue(identical(c, d))
  checkException(rstan:::mklist(c("x", "f")))

  p <- 4
  fun <- function() {
    p <- 3
    rstan:::mklist('p')
  } 
  checkEquals(fun()$p, 3, checkNames = FALSE )
  checkEquals(rstan:::mklist('p')$p, 4, checkNames = FALSE)
} 

#test_makeconf_path <- function() {
#  p <- makeconf_path()  
#  checkTrue(file.exists(makeconf_path()))
#} 

test_config_argss <- function() {
  # (chains, iter, warmup, thin, init, seed, sample_file, ...)
  a <- rstan:::config_argss(3, 100, 10, 3, 0, 0, "a.csv", algorithm = "NUTS",
                            control = NULL,
                            chain_id = 4)
  checkEquals(length(a), 3)
  checkEquals(a[[1]]$init, "0") 
  checkEquals(a[[1]]$chain_id, 4) 
  checkEquals(a[[3]]$chain_id, 6) 
  b <- rstan:::config_argss(3, 100, 10, 3, "0", 10, "a.csv", algorithm = "NUTS",
                            control = NULL)
  checkEquals(b[[3]]$chain_id, 3) 
  checkEquals(b[[1]]$init, "0") 
  c <- rstan:::config_argss(3, 100, 10, 3, "random", 10, "a.csv", algorithm = "HMC", 
                            control = list(adapt_engaged=FALSE)) 
  checkEquals(c[[1]]$init, "random") 
  d <- rstan:::config_argss(4, 100, 10, 3, "random", 10, "a.csv", 
                            chain_id = c(3, 2, 1), algorithm = "Metropolis", 
                            control = list(adapt_engaged=FALSE)) 
  checkEquals(d[[3]]$chain_id, 1) 
  checkEquals(d[[4]]$chain_id, 4) 
  checkException(rstan:::config_argss(3, 100, 10, 3, "random", 10, NA,
                                      algorithm = "NUTS", control = NULL, chain_id = c(3, 3)))
  b <- rstan:::config_argss(3, 100, 10, 3, 0, "12345", "a.csv", 
                            algorithm = "NUTS", chain_id = 4, control = NULL)
  checkEquals(b[[1]]$seed, '12345')
  checkException(rstan:::config_argss(3, 100, 10, 3, 0, "a12345", "a.csv",
                                      algorithm = "NUTS", control = NULL, chain_id = 4))
  checkException(rstan:::config_argss(3, 100, 10, 3, 0, "1a2345", "a.csv", 
                                      algorithm = "NUTS", control = NULL,
                                      chain_id = 4))
} 

test_data_list2array <- function() {
  d <- list(y = rnorm(20))
  d2 <- rstan:::data_list2array(d)
  checkEquals(d2, array(d$y, dim = c(1, 20)))

  I <- 4; J <- 5; K <- 6
  b <- lapply(1:I, function(i) rnorm(J))
  b2 <- rstan:::data_list2array(b)
  b3 <- data.matrix(do.call(rbind, b))
  checkEquals(b2, b3)

  a <- lapply(1:I, function(i) array(rnorm(J * K), dim = c(J, K)))
  a2 <- rstan:::data_list2array(a)
  for (i in 1:I) checkEquals(a[[i]], a2[i,,])
  checkEquals(a[[4]][5, 6], a2[4, 5, 6]) 
  checkEquals(a[[3]][4, 2], a2[3, 4, 2]) 
  checkEquals(a[[2]][1, 6], a2[2, 1, 6]) 
}

test_read_comments <- function() {
  a1 <- rstan:::read_comments('cc.csv', 5L)
  checkEquals(a1[1], "# comment line 1")
  checkEquals(a1[5], "# comment line 5")
  a2 <- rstan:::read_comments('cc.csv', 3L)
  checkEquals(length(a2), 3L)
  a3 <- rstan:::read_comments('cc.csv', -1)
  checkEquals(length(a3), 7L)
} 

test_read_csv_header <- function() {
  h1 <- rstan:::read_csv_header('cc2.csv')
  checkEquals(attr(h1, 'lineno'), 5)
  checkEquals(h1, 'c,a,b', check.attributes = FALSE)
}

test_get_dims_from_fnames <- function() {
  names <- c('alpha', 'beta2', 'g2amma', 'theta0')
  dims <- list(c(2L, 3L), integer(0L), c(4L), c(3L, 5L, 4L))
  fnames <- rstan:::flatnames(names, dims)
  fnames_d <- rstan:::sqrfnames_to_dotfnames(fnames)
  unames <- rstan:::unique_par(fnames_d)
  dims2 <- lapply(unames, 
                  function(n) { 
                    fnames_d2 <- fnames_d[sapply(fnames_d, function(i) grepl(n, i))]
                    # the above line works here since all parameters are not nested. 
                    # it would be probematic if say we have another parameter `p`,
                    # since p is also part of `alpha`
                    rstan:::get_dims_from_fnames(fnames_d2)
                  })
  checkEquals(dims, dims2)
} 


test_dotfnames_fromto_sqrfnames <- function() {
  dn <- c("alpha", "beta.1", "beta.2", "gamma.1.2", "gamma.1.4")
  sn <- rstan:::dotfnames_to_sqrfnames(dn)
  dn2 <- rstan:::sqrfnames_to_dotfnames(sn)
  checkEquals(dn, dn2)
}

test_par_vector2list <- function() {
  v <- c(2.3, 3.4, 4.5, (1:8)/9, 3.1415); 
  pars <- c('alpha', 'beta', 'gamma', 'delta')
  dims <- list(integer(0), c(2), c(2, 4), 1)
  vl <- rstan:::rstan_relist(v, rstan:::create_skeleton(pars, dims))
  alpha <- 2.3  
  beta <- array(v[2:3], dim = 2)
  gamma <- array(v[4:11], dim = c(2, 4))
  delta <- array(v[12], dim = 1)
  checkEquals(length(vl), 4)
  checkEquals(vl[[1]], alpha)
  checkEquals(vl[[2]], beta)
  checkEquals(vl[[3]], gamma)
  checkEquals(vl[[4]], delta)
} 

test_remove_empty_pars <- function() {
  pars <- c('alpha', 'beta', 'gamma', 'eta', 'xi')
  dims <- list(integer(0), c(2), c(2, 4), 0, c(2,0))
  names(dims) <- pars 
  checkEquals(rstan:::remove_empty_pars(pars[1:2], dims), pars[1:2])
  checkEquals(rstan:::remove_empty_pars(pars[1:4], dims), pars[1:3])
  checkEquals(rstan:::remove_empty_pars('beta[1]', dims), "beta[1]")
  checkEquals(rstan:::remove_empty_pars('eta', dims), character(0))
}
 
.tearDown <- function() {
  unlink('tmp.stan') 
  unlink('dumpabc.Rdump') 
  unlink('standumpabc.Rdump') 
  unlink('cc.csv')
  unlink('cc2.csv')
}

model_code <- "model { \n y ~ normal(0, 1); \n}"
cat(model_code, file = "tmp.stan")

a <- c(1, 3, 5)
b <- matrix(1:10, ncol = 2)
c <- array(1:18, dim = c(2, 3, 3))
dump(c("a", "b", "c"), file = "dumpabc.Rdump")
stan_rdump(c("a", "b", "c"), file = "standumpabc.Rdump")
d <- factor(c("a", "b", "b", "a", "a"))
e <- factor(c(1, 3, 3, 1, 1))
f <- c(TRUE, TRUE, FALSE)
g <- 1000000000
h <- 1.1e+10
stan_rdump(c("d", "e", "f"), file = "standumpabc.Rdump", append = TRUE)
stan_rdump(c("g", "h"), file = "standumpabc.Rdump", append = TRUE)

cc <- c(
  "# comment line 1",
  " no comments line 1",
  "# comment line 2",
  "# comment line 3",
  "# comment line 4",
  "# comment line 5",
  " not comments line 2",
  "# comment line 6",
  "not comments #comment line 7",
  "not comments at the end of file"
)
cat(file = "cc.csv", paste(cc, collapse = "\n"), "\n")
cc2 <- c("# line 1", "#line 2", "# line 3", "#line4", "c,a,b", "1,2,3")
cat(file = "cc2.csv", paste(cc2, collapse = "\n"), "\n")

test_that("get_model_strcode works", {
  model_code <- "model { \n y ~ normal(0, 1); \n}"
  code <- "parameters { real y; } model{y ~ normal(0,1);}"
  # FIXME: Warning - "incomplete final line found on 'tmp.stan'"
  str1 <- suppressWarnings(rstan:::get_model_strcode("tmp.stan"))
  str2 <- rstan:::get_model_strcode(model_code = code)
  str3 <- rstan:::get_model_strcode(model_code = "code")
  str4 <- rstan:::get_model_strcode(model_code = "parameters {real y;} model {y ~ normal(0,1); }")

  mname1 <- attr(str1, "model_name2")
  mname2 <- attr(str2, "model_name2")
  mname3 <- attr(str3, "model_name2")
  mname4 <- attr(str4, "model_name2")
  expect_equal(mname1, "tmp")
  expect_equal(mname2, "code")
  expect_equal(mname3, "code")
  expect_equal(mname4, "anon_model")

  attributes(str1) <- NULL
  attributes(str2) <- NULL
  attributes(str3) <- NULL
  expect_equal(str1, model_code)
  expect_equal(str2, code)
  expect_equal(str3, code)

  model_code <- "model { \n y ~ normal(0, 1); \n}"
  # cat(model_code, file = 'tmp.stan')
  # FIXME: Warning - "incomplete final line found on 'tmp.stan'"
  expect_equal(
    model_code,
    suppressWarnings(rstan:::read_model_from_con("tmp.stan"))
  )
  attr(model_code, "model_name2") <- "tmp"
  # FIXME: Warning - "incomplete final line found on 'tmp.stan'"
  expect_equal(
    model_code,
    suppressWarnings(rstan:::get_model_strcode("tmp.stan"))
  )
  attr(model_code, "model_name2") <- "model_code"
  expect_equal(model_code, rstan:::get_model_strcode(model_code = model_code))
  expect_error(rstan:::get_model_strcode())
})

test_that("is_legal_stan_name works", {
  expect_false(rstan:::is_legal_stan_vname("7dd"))
  expect_false(rstan:::is_legal_stan_vname("model"))
  expect_false(rstan:::is_legal_stan_vname("private"))
  expect_false(rstan:::is_legal_stan_vname("hello__"))
  expect_true(rstan:::is_legal_stan_vname("y"))
})

test_that("is_named_list works", {
  expect_false(rstan:::is_named_list(c(2, 3)))
  expect_false(rstan:::is_named_list(list(3, 4)))
  expect_false(rstan:::is_named_list(list(a = 3, 4)))
  expect_true(rstan:::is_named_list(list(a = 3, b = 4)))
})

test_that("data_preprocess works", {
  lst <- list(
    z = c(1L, 2L, 4L),
    a = 1:100,
    b = matrix(1:9 / 9, ncol = 3),
    c = structure(1:100, .Dim = c(5, 20)),
    g = array(c(3, 3, 9, 3, 3, 4, 5, 6, 9, 8, 0, 2), dim = c(2, 2, 3)),
    d = 1:100 + .1
  )
  lst <- rstan:::data_preprocess(lst)
  lst2 <- lst
  lst2$f <- matrix(c(3, NA, NA, NA, 3, 4), ncol = 3)
  lst3 <- lst
  lst3$h <- gl(3, 4)
  # Warns about `h` being not numeric.
  # FIXME: Should the warning be tested explicitly?
  lst4 <- suppressWarnings(rstan:::data_preprocess(lst3))

  # Keep the dimension information.
  expect_equal(dim(lst$g), c(2, 2, 3))
  # Do as.integer when appropriate.
  expect_true(is.integer(lst$z))
  # Don't do as.integer when it is not appropriate.
  expect_true(is.double(lst$b))
  # Stop if data have NA.
  expect_error(rstan:::data_preprocess(lst2))
  # Check if h is removed.
  expect_named(lst4, c("z", "a", "b", "c", "g", "d"))
})

test_that("data_preprocess works take 2", {
  # A list of array as an element of the data list.
  I <- 3
  J <- 4
  K <- 5
  a <- lapply(1:I, function(i) rnorm(J))
  b <- lapply(1:I, function(i) matrix(rnorm(J * K), ncol = K))
  d <- lapply(1:I, function(i) rnorm(1, i))
  e <- lapply(1:I, function(i) rpois(J, 1) + 1.0)
  lst2 <- rstan:::data_preprocess(list(a = a, b = b, d = d, e = e))
  expect_equal(dim(lst2$a), c(I, J))
  expect_equal(dim(lst2$b), c(I, J, K))
  expect_equal(dim(lst2$d), c(I, 1))
  expect_true(is.integer(lst2$e[1, 1]))
  expect_false(is.integer(e[[1]][1]))

  a1 <- lapply(1:I, function(i) rnorm(J))
  a2 <- lapply(1:I, function(i) rnorm(J))
  expect_error(rstan:::data_preprocess(list(a = list(a1 = a1, a2 = a2))))
})

test_that("read_rdump works", {
  l <- rstan:::read_rdump("dumpabc.Rdump")
  expect_equal(l$a, c(1, 3, 5))
  expect_equal(l$b, matrix(1:10, ncol = 2))
  expect_equal(l$c, array(1:18, dim = c(2, 3, 3)))
})

test_that("stan_rdump works", {
  l <- rstan:::read_rdump("standumpabc.Rdump")
  expect_equal(l$a, c(1, 3, 5))
  expect_equal(l$b, matrix(1:10, ncol = 2))
  expect_equal(l$c, array(1:18, dim = c(2, 3, 3)))
  expect_equal(l$d, c(1, 2, 2, 1, 1))
  expect_equal(l$e, c(1, 2, 2, 1, 1))
  expect_equal(l$f, c(1, 1, 0))
  text <- paste(readLines("standumpabc.Rdump"), collapse = "|")
  expect_true(grepl("1000000000", text))
  expect_equal(l$h, 1.1e10)
})

test_that("seq_array_ind works", {
  a <- rstan:::seq_array_ind(numeric(0))
  expect_length(a, 0)
  # By default, col_major is FALSE.
  b <- rstan:::seq_array_ind(2:5, col_major = TRUE)
  c <- arrayInd(1:prod(2:5), .dim = 2:5)
  expect_equal(b, c)
  d <- rstan:::seq_array_ind(2:3, col_major = FALSE)
  e <- matrix(c(1, 1, 1, 2, 1, 3, 2, 1, 2, 2, 2, 3),
    nrow = 6, byrow = TRUE
  )
  expect_equal(d, as.array(e))
  f <- rstan:::seq_array_ind(1)
  expect_equal(f, array(1, dim = c(1, 1)))
})

test_that("flatnames works", {
  names <- c("alpha", "beta", "gamma", "delta")
  dims <- list(alpha = integer(0), beta = c(2, 3), gamma = c(2, 3, 4),
               delta = c(5))
  fnames <- rstan:::flatnames(names, dims)
  expect_equal(
    fnames,
    c(
      "alpha", "beta[1,1]", "beta[1,2]", "beta[1,3]",
      "beta[2,1]", "beta[2,2]", "beta[2,3]",
      "gamma[1,1,1]", "gamma[1,1,2]", "gamma[1,1,3]", "gamma[1,1,4]",
      "gamma[1,2,1]", "gamma[1,2,2]", "gamma[1,2,3]", "gamma[1,2,4]",
      "gamma[1,3,1]", "gamma[1,3,2]", "gamma[1,3,3]", "gamma[1,3,4]",
      "gamma[2,1,1]", "gamma[2,1,2]", "gamma[2,1,3]", "gamma[2,1,4]",
      "gamma[2,2,1]", "gamma[2,2,2]", "gamma[2,2,3]", "gamma[2,2,4]",
      "gamma[2,3,1]", "gamma[2,3,2]", "gamma[2,3,3]", "gamma[2,3,4]",
      "delta[1]", "delta[2]", "delta[3]", "delta[4]", "delta[5]"
    )
  )
  names2 <- c("alpha")
  dims2 <- list(alpha = integer(0))
  fnames2 <- rstan:::flatnames(names2, dims2)
  expect_equal(fnames2, "alpha")
})

test_that("idx_col2rowm works", {
  d <- integer(0)
  idx <- rstan:::idx_col2rowm(d)
  expect_equal(idx, 1)
  d2 <- 8
  idx2 <- rstan:::idx_col2rowm(d2)
  expect_equal(idx2, 1:8)
  d3 <- c(3, 4, 5)
  idx3 <- rstan:::idx_col2rowm(d3)
  yidx3 <- c(
    1, 13, 25, 37, 49, 4, 16, 28, 40, 52, 7, 19, 31, 43, 55,
    10, 22, 34, 46, 58, 2, 14, 26, 38, 50, 5, 17, 29, 41, 53, 8, 20, 32, 44, 56,
    11, 23, 35, 47, 59, 3, 15, 27, 39, 51, 6, 18, 30, 42, 54, 9, 21, 33, 45, 57,
    12, 24, 36, 48, 60
  )
  expect_equal(idx3, yidx3)
})

test_that("idx_row2colm works", {
  d <- integer(0)
  idx <- rstan:::idx_row2colm(d)
  expect_equal(idx, 1)
  d2 <- 8
  idx2 <- rstan:::idx_row2colm(d2)
  expect_equal(idx2, 1:8)
  d3 <- c(3, 4, 5)
  idx3 <- rstan:::idx_row2colm(d3)
  yidx3 <- c(
    1, 21, 41, 6, 26, 46, 11, 31, 51, 16, 36, 56, 2, 22, 42, 7, 27, 47, 12, 32, 52,
    17, 37, 57, 3, 23, 43, 8, 28, 48, 13, 33, 53, 18, 38, 58, 4, 24, 44, 9, 29, 49,
    14, 34, 54, 19, 39, 59, 5, 25, 45, 10, 30, 50, 15, 35, 55, 20, 40, 60
  )
  expect_equal(idx3, yidx3)
})

test_that("pars_total_indexes works", {
  names <- "alpha0"
  dims <- list(alpha0 = c(2, 3))
  fnames <- rstan:::flatnames(names, dims)
  tidx <- rstan:::pars_total_indexes(names, dims, fnames, "alpha0")
  tidx_attr1 <- attr(tidx[[1]], "row_major_idx")
  attributes(tidx[[1]]) <- NULL
  expect_equal(unname(tidx[[1]]), 1:6)
  expect_equal(unname(tidx_attr1), c(1, 3, 5, 2, 4, 6))
  names2 <- c(names, "alpha")
  dims2 <- c(dims, list(alpha = 8))
  fnames2 <- rstan:::flatnames(names2, dims2)
  tidx2 <- rstan:::pars_total_indexes(names2, dims2, fnames2, "alpha")
  tidx2_attr1 <- attr(tidx2[[1]], "row_major_idx")
  attributes(tidx2[[1]]) <- NULL
  expect_equal(unname(tidx2[[1]]), 6 + 1:8)
  expect_equal(unname(tidx2_attr1), 6 + 1:8)
  names3 <- c(names2, "p")
  dims3 <- c(dims2, list(p = integer(0)))
  fnames3 <- rstan:::flatnames(names3, dims3)
  tidx3 <- rstan:::pars_total_indexes(names3, dims3, fnames3, "p")
  tidx3_attr1 <- attr(tidx3[[1]], "row_major_idx")
  attributes(tidx3[[1]]) <- NULL
  expect_equal(unname(tidx3[[1]]), 15)
  expect_equal(unname(tidx3_attr1), 15)
})

test_that("multi_idx_row2colm works", {
  expect_equal(rstan:::multi_idx_row2colm(list(integer(0))), 1)
  dims <- list(c(3), c(2, 3), integer(0), c(2))
  col_idx <- rstan:::multi_idx_row2colm(dims)
  target <- c(1, 2, 3, 4, 7, 5, 8, 6, 9, 10, 11, 12)
  expect_equal(col_idx, target)

  fnames <- c(
    "alpha[1]", "alpha[2]", "alpha[3]",
    "alpha2[1,1]", "alpha2[1,2]", "alpha2[1,3]",
    "alpha2[2,1]", "alpha2[2,2]", "alpha2[2,3]",
    "p", "theta[1]", "theta[2]"
  )
  fnames_colm <- c(
    "alpha[1]", "alpha[2]", "alpha[3]",
    "alpha2[1,1]", "alpha2[2,1]", "alpha2[1,2]", "alpha2[2,2]",
    "alpha2[1,3]", "alpha2[2,3]", "p", "theta[1]", "theta[2]"
  )
  expect_equal(fnames[col_idx], fnames_colm)
})

test_that("mklist works", {
  x <- 3:5
  y <- array(1:9, dim = c(3, 3))
  z <- list(p = 3)
  f <- function() {
    TRUE
  }
  a <- list(x = x, y = y)
  b <- rstan:::mklist(c("x", "y"))
  expect_identical(a, b)
  c <- list(x = x, y = y, z = z)
  d <- rstan:::mklist(c("x", "y", "z"))
  expect_identical(c, d)
  expect_error(rstan:::mklist(c("x", "f")))

  p <- 4
  fun <- function() {
    p <- 3
    rstan:::mklist("p")
  }
  expect_equal(fun()$p, 3, ignore_attr = "names")
  expect_equal(rstan:::mklist("p")$p, 4, ignore_attr = "names")
})

# test_makeconf_path <- function() {
#  p <- makeconf_path()
#  checkTrue(file.exists(makeconf_path()))
# }

test_that("config_argss works", {
  # (chains, iter, warmup, thin, init, seed, sample_file, ...)
  a <- rstan:::config_argss(3, 100, 10, 3, 0, 0, "a.csv",
    algorithm = "NUTS",
    control = NULL,
    chain_id = 4
  )
  expect_length(a, 3)
  expect_equal(a[[1]]$init, "0")
  expect_equal(a[[1]]$chain_id, 4)
  expect_equal(a[[3]]$chain_id, 6)
  b <- rstan:::config_argss(3, 100, 10, 3, "0", 10, "a.csv",
    algorithm = "NUTS",
    control = NULL
  )
  expect_equal(b[[3]]$chain_id, 3)
  expect_equal(b[[1]]$init, "0")
  c <- rstan:::config_argss(3, 100, 10, 3, "random", 10, "a.csv",
    algorithm = "HMC",
    control = list(adapt_engaged = FALSE)
  )
  expect_equal(c[[1]]$init, "random")
  d <- rstan:::config_argss(4, 100, 10, 3, "random", 10, "a.csv",
    chain_id = c(3, 2, 1), algorithm = "Metropolis",
    control = list(adapt_engaged = FALSE)
  )
  expect_equal(d[[3]]$chain_id, 1)
  expect_equal(d[[4]]$chain_id, 4)
  expect_error(rstan:::config_argss(3, 100, 10, 3, "random", 10, NA,
    algorithm = "NUTS", control = NULL, chain_id = c(3, 3)
  ))
  b <- rstan:::config_argss(3, 100, 10, 3, 0, "12345", "a.csv",
    algorithm = "NUTS", chain_id = 4, control = NULL
  )
  expect_equal(b[[1]]$seed, "12345")
  expect_error(rstan:::config_argss(3, 100, 10, 3, 0, "a12345", "a.csv",
    algorithm = "NUTS", control = NULL, chain_id = 4
  ))
  expect_error(rstan:::config_argss(3, 100, 10, 3, 0, "1a2345", "a.csv",
    algorithm = "NUTS", control = NULL,
    chain_id = 4
  ))
})

test_that("data_list2array works", {
  d <- list(y = rnorm(20))
  d2 <- rstan:::data_list2array(d)
  expect_equal(d2, array(d$y, dim = c(1, 20)))

  I <- 4
  J <- 5
  K <- 6
  b <- lapply(1:I, function(i) rnorm(J))
  b2 <- rstan:::data_list2array(b)
  b3 <- data.matrix(do.call(rbind, b))
  expect_equal(b2, b3)

  a <- lapply(1:I, function(i) array(rnorm(J * K), dim = c(J, K)))
  a2 <- rstan:::data_list2array(a)
  for (i in 1:I) {
    expect_equal(a[[i]], a2[i, , ])
  }
  expect_equal(a[[4]][5, 6], a2[4, 5, 6])
  expect_equal(a[[3]][4, 2], a2[3, 4, 2])
  expect_equal(a[[2]][1, 6], a2[2, 1, 6])
})

test_that("read_comments works", {
  a1 <- rstan:::read_comments("cc.csv", 5L)
  expect_equal(a1[1], "# comment line 1")
  expect_equal(a1[5], "# comment line 5")
  a2 <- rstan:::read_comments("cc.csv", 3L)
  expect_length(a2, 3L)
  a3 <- rstan:::read_comments("cc.csv", -1)
  expect_length(a3, 7L)
})

test_that("read_csv_header", {
  h1 <- rstan:::read_csv_header("cc2.csv")
  expect_equal(attr(h1, "lineno"), 5)
  expect_equal(h1, "c,a,b", ignore_attr = TRUE)
})

test_that("get_dims_from_fnames works", {
  names <- c("alpha", "beta2", "g2amma", "theta0")
  dims <- list(c(2L, 3L), integer(0L), c(4L), c(3L, 5L, 4L))
  fnames <- rstan:::flatnames(names, dims)
  fnames_d <- rstan:::sqrfnames_to_dotfnames(fnames)
  unames <- rstan:::unique_par(fnames_d)
  dims2 <- lapply(
    unames,
    function(n) {
      fnames_d2 <- fnames_d[sapply(fnames_d, function(i) grepl(n, i))]
      # The above line works here since all parameters are not nested.
      # it would be problematic if say we have another parameter `p`,
      # since p is also part of `alpha`.
      rstan:::get_dims_from_fnames(fnames_d2)
    }
  )
  expect_equal(dims, dims2)
})

test_that("Converting from/to dotfnames/sqrfnames works", {
  dn <- c("alpha", "beta.1", "beta.2", "gamma.1.2", "gamma.1.4")
  sn <- rstan:::dotfnames_to_sqrfnames(dn)
  dn2 <- rstan:::sqrfnames_to_dotfnames(sn)
  expect_equal(dn, dn2)
})

test_that("Can convert parameter from vector to list", {
  v <- c(2.3, 3.4, 4.5, (1:8) / 9, 3.1415)
  pars <- c("alpha", "beta", "gamma", "delta")
  dims <- list(integer(0), c(2), c(2, 4), 1)
  vl <- rstan:::rstan_relist(v, rstan:::create_skeleton(pars, dims))
  alpha <- 2.3
  beta <- array(v[2:3], dim = 2)
  gamma <- array(v[4:11], dim = c(2, 4))
  delta <- array(v[12], dim = 1)
  expect_length(vl, 4)
  expect_equal(vl[[1]], alpha)
  expect_equal(vl[[2]], beta)
  expect_equal(vl[[3]], gamma)
  expect_equal(vl[[4]], delta)
})

test_that("remove_empty_pars works", {
  pars <- c("alpha", "beta", "gamma", "eta", "xi")
  dims <- list(integer(0), c(2), c(2, 4), 0, c(2, 0))
  names(dims) <- pars
  expect_equal(rstan:::remove_empty_pars(pars[1:2], dims), pars[1:2])
  expect_equal(rstan:::remove_empty_pars(pars[1:4], dims), pars[1:3])
  expect_equal(rstan:::remove_empty_pars("beta[1]", dims), "beta[1]")
  expect_equal(rstan:::remove_empty_pars("eta", dims), character(0))
})

unlink("tmp.stan")
unlink("dumpabc.Rdump")
unlink("standumpabc.Rdump")
unlink("cc.csv")
unlink("cc2.csv")

test_that("mklist works", {
  skip("Backwards compatibility")

  c <- 4
  z <- matrix(0, ncol = 2, nrow = 3)
  x <- list()
  x[[1]] <- 1:2
  x[[2]] <- 3:4
  y <- 3
  fun1 <- function(n) {
    a <- 3
    mklist(n)[[n]]
  }

  expect_equal(fun1("a"), 3)
  expect_equal(fun1("c"), 4)

  # No list.
  L <- mklist(c("y", "z"))
  expect_equal(L$y, y)
  expect_equal(L$z, z)

  L <- mklist(c("x", "y", "z"))
  expect_equal(L$x, x)
  expect_equal(L$y, y)
  expect_equal(L$z, z)

  # Only a list.
  L <- mklist("x")
  expect_equal(L$x, x)
})

test_that("get_time_from_csv works", {
  skip("Backwards compatibility")

  expect_true(all(is.na(get_time_from_csv(""))))

  t_lines <- c("aa", "aa")
  expect_true(all(is.na(get_time_from_csv(t_lines))))

  t_lines <- c(
    "# Elapsed Time: 0.005308 seconds (Warm-up)",
    "#               0.003964 seconds (Sampling)"
  )
  t <- rstan:::get_time_from_csv(t_lines)

  expect_equal(unname(t), c(0.005308, 0.003964))
  expect_named(t, c("warmup", "sample"))
})

test_that("stan_rdump works", {
  skip("Backwards compatibility")

  a <- 1000000000
  b <- rnorm(3)
  c <- rexp(10)
  h <- 1.1e10
  i <- 5.0
  dumpf <- "standumpabc.Rdump"
  stan_rdump(c("a", "b", "c", "h", "i"), file = dumpf)
  text <- paste(readLines(dumpf), collapse = "|")
  expect_true(grepl("a.*<-.*1000000000\\|", text, perl = TRUE))
  expect_true(grepl("h.*<-.*1.1[Ee]\\+*10\\|", text, perl = TRUE))
  on.exit(unlink(dumpf), add = TRUE)
})

test_that("stan_rdump works take 2", {
  skip("Backwards compatibility")

  dat_list <- list(a = array(0, dim = c(3, 2, 1, 0)), d = double(),
                   i = integer(), m = matrix(0, nrow = 3, ncol = 0))
  dumpf <- 'standumpabc.Rdump'
  stan_rdump(names(dat_list), file = dumpf,
                    envir = as.environment(dat_list))
  x <- read_rdump(dumpf)
  on.exit(unlink(dumpf), add = TRUE)

  expect_true(!is.null(x$a))
  expect_true(!is.null(x$d))
  expect_true(!is.null(x$i))
  expect_true(!is.null(x$m))

  expect_true(is.integer(x$a))
  expect_true(is.integer(x$d))
  expect_true(is.integer(x$i))
  expect_true(is.integer(x$m))

  expect_true(length(x$a) == 0)
  expect_true(length(x$d) == 0)
  expect_true(length(x$i) == 0)
  expect_true(length(x$m) == 0)

  expect_equal(dim(x$a), c(3L, 2L, 1L, 0L))
  expect_equal(dim(x$m), c(3L, 0L))
})

## unit tests will not be done if RUnit is not available
if(!require("RUnit", quietly = TRUE)) {
  warning("cannot run unit tests -- package RUnit is not available")
  q('no');
} 

 
pkg <- "rstan" 

path <- system.file(package = pkg, "unitTests")
source(file.path(path, "runRunitTests.R"), echo = TRUE)
 

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

paridx_fun <- function(names) {
  # Args:
  #   names: names (character vector) such as lp__, treedepth__, stepsize__,
  #          alpha, beta.1, 
  # Returns: 
  #   The indexes in the names that are parameters other than lp__,
  #   treedepth__, or stepsize__. The vector has attribute meta
  #   with the indexes of 'treedepth__', 'lp__', and 'stepsize__'
  #   if available. 
  
  sampler_param_names <- c('lp__', 'accept_stat__', 'treedepth__', 'stepsize__', 
                           'divergent__', 'n_leapfrog__', "energy__")
  metaidx <- match(sampler_param_names, names)
  names(metaidx) <- sampler_param_names
  paridx <- setdiff(seq_along(names), metaidx)
  attr(paridx, "meta") <- metaidx[!sapply(metaidx, is.na)]
  paridx
}

parse_stancsv_comments <- function(comments) {
  # Parse the comments in Stan CSV files to get information such as
  # iter, thin, seed, etc. This is specific to the CSV files
  # generated from Stan

  adapt_term_lineno <- which(grepl("Adaptation terminated", comments))[1]
  if (is.na(adapt_term_lineno)) adapt_term_lineno <- length(comments)
  time_lineno <- which(grepl("Elapsed Time", comments))
  has_time <- length(time_lineno) > 0
  len <- length(comments)
  if (length(adapt_term_lineno) < 1) 
    adapt_term_lineno <- len
  if (length(time_lineno) < 1)
    warning("line with \"Elapsed Time\" not found")

  if (adapt_term_lineno == len)
    adaptation_info <- ''
  else {
    if (has_time)
      adaptation_info <- paste(comments[(adapt_term_lineno+1):(time_lineno-1)], collapse = '\n')
    else
      adaptation_info <- paste(comments[(adapt_term_lineno+1):len], collapse = '\n')
  }
  if (has_time)
    time_info <- comments[time_lineno:len]
  else
    time_info <- ''
  if (adapt_term_lineno > 0) comments <- head(comments, adapt_term_lineno - 1L)

  has_eq <- sapply(comments, function(i) grepl('=', i))
  comments <- comments[has_eq] 
  comments <- gsub('^#+\\s*|\\s*|\\(Default\\)', '', comments)
  eq_pos <- regexpr("=", comments, fixed = TRUE)
  names0 <- substr(comments, 0, eq_pos - 1)
  values <- as.list(substring(comments, eq_pos + 1))
  
  id_idx <- which("id" == names0)
  if (length(id_idx) > 0) 
  names0[id_idx] <- "chain_id"
  
  compute_iter <- FALSE
  id_warmup <- which("num_warmup" == names0)
  if (length(id_warmup) > 0) {
    names0[id_warmup] <- "warmup"
    compute_iter <- TRUE
  }   
  
  id_numsamples <- which("num_samples" == names0)
  if (length(id_numsamples) > 0) {
    names0[id_numsamples] <- "iter"
  }
  names(values) <- names0;

  add_lst <- list(adaptation_info = adaptation_info,
                  has_time = has_time,
                  time_info = time_info)

  sampler_t <- NULL
  if (!is.null(values$algorithm) && is.null(values$sampler_t)) {
    if (values$algorithm == 'rwm' || values$algorithm == 'Metropolis')  
      sampler_t <- "Metropolis"
    else if (values$algorithm == 'hmc') {
       if (values$engine == 'static')  sampler_t <- "HMC"
       else {
         if (values$metric == 'unit_e') sampler_t <- "NUTS(unit_e)"
         else if (values$metric == 'diag_e') sampler_t <- "NUTS(diag_e)"
         else if (values$metric == 'dense_e') sampler_t <- "NUTS(dense_e)"
       } 
    } 
    add_lst <- c(add_lst, sampler_t = sampler_t)
  } 
  names1 <- intersect(c("thin", "iter", "warmup", "chain_id", "max_depth", 
                        "num_samples", "num_warmup", "id",
                        "max_treedepth", "save_warmup"), names0)
  names2 <- intersect(c("stepsize", "stepsize_jitter", "adapt_gamma", "adapt_kappa", 
                        "adapt_delta", "gamma", "kappa", "delta", "t0",
                        "adapt_t0"), names0) 
  for (z in names1) values[[z]] <- as.integer(values[[z]])
  for (z in names2) values[[z]] <- as.numeric(values[[z]])
  if (compute_iter) values[["iter"]] <- values[["iter"]] + values[["warmup"]]
  if ("output_samples" %in% names(values)){ ## fix missing values for variational 
    values[["iter"]] <- as.integer(values[["output_samples"]])
    values[["warmup"]] <- 0L
    values[["thin"]] <- 1L
    values[["save_warmup"]] <- 1L
  }
  c(values, add_lst)  
}


read_stan_csv <- function(csvfiles, col_major = TRUE) {
  # Read the csv files saved from Stan (or RStan) to a stanfit object
  # Args:
  #   csvfiles: csv files fitted for the same model; each file contains 
  #     the sample of one chain 
  #   col_major: the order for array parameters. 
  # 

  if (length(csvfiles) < 1) 
    stop("csvfiles does not contain any CSV file name")

  g_skip <- 10
  
  ss_lst <- vector("list", length(csvfiles))
  cs_lst2 <- vector("list", length(csvfiles))

  for (i in seq_along(csvfiles)) {
    f = csvfiles[i]    
    header <- read_csv_header(f)
    lineno <- attr(header, 'lineno')
    vnames <- strsplit(header, ",")[[1]]
    iter.count <- attr(header,"iter.count")
    variable.count <- length(vnames)

    lines = readLines(f)
    comment_lines = grep("^#", lines)
    comments = lines[comment_lines]
    con = textConnection(lines[-comment_lines])
    on.exit(close(con))
    df = read.csv(con, colClasses = "numeric")
    cs_lst2[[i]] <- parse_stancsv_comments(comments)
    if("output_samples" %in% names(cs_lst2[[i]])) 
      df <- df[-1,] # remove the means 
    ss_lst[[i]] <- df
  } 

  # use the first CSV file name as model name
  m_name <- sub("(_\\d+)*$", '', filename_rm_ext(basename(csvfiles[1])))

  sdate <- do.call(max, lapply(csvfiles, function(csv) file.info(csv)$mtime))
  sdate <- format(sdate, "%a %b %d %X %Y") # same format as date() 

  chains <- length(ss_lst)
  fnames <- names(ss_lst[[1]])
  n_save <- nrow(ss_lst[[1]])
  paridx <- paridx_fun(fnames)
  lp__idx <- attr(paridx, 'meta')["lp__"]
  par_fnames <- c(fnames[paridx], "lp__")
  pars_oi <- unique_par(par_fnames)
  dims_oi <- lapply(pars_oi, 
                    function(i) {
                      pat <- paste('^', i, '(\\.\\d+)*$', sep = '')
                      i_fnames <- par_fnames[grepl(pat, par_fnames)]
                      get_dims_from_fnames(i_fnames, i) 
                    })
  names(dims_oi) <- pars_oi
  midx <- if (!col_major) multi_idx_row2colm(dims_oi) else 1:length(par_fnames)
  if (chains > 1) {
    if (!all(sapply(ss_lst[-1], function(i) identical(names(i), fnames))))
      stop('the CSV files do not have same parameters')
    if (!all(sapply(ss_lst[-1], function(i) identical(length(i[[1]]), n_save)))) 
      stop('the number of iterations are not the same in all CSV files')
  } 
  mode <- 0L
  
  samples <- lapply(ss_lst, 
                    function(df) {
                      ss <- df[c(paridx, lp__idx)[midx]]
                      attr(ss, "sampler_params") <- df[setdiff(attr(paridx, 'meta'), lp__idx)] 
                      ss
                    })
  par_fnames <- par_fnames[midx]
  for (i in seq_along(samples)) {
    attr(samples[[i]], "adaptation_info") <- cs_lst2[[i]]$adaptation_info 
    attr(samples[[i]], "args") <- 
      list(sampler_t = cs_lst2[[i]]$sampler_t,
           chain_id = cs_lst2[[i]]$chain_id)
    if (cs_lst2[[i]]$has_time)
      attr(samples[[i]], "elapsed_time") <- get_time_from_csv(cs_lst2[[i]]$time_info)
  } 

  save_warmup <- sapply(cs_lst2, function(i) i$save_warmup)
  warmup <- sapply(cs_lst2, function(i) i$warmup)
  thin <- sapply(cs_lst2, function(i) i$thin)
  iter <- sapply(cs_lst2, function(i) i$iter)

  if (!all_int_eq(warmup) || !all_int_eq(thin) || !all_int_eq(iter)) 
    stop("not all iter/warmups/thin are the same in all CSV files")
  

  n_kept0 <- 1 + (iter - warmup - 1) %/% thin
  warmup2 <- 0
  if (max(save_warmup) == 0L) { # all equal to 0L
    n_kept <- n_save
  } else if (min(save_warmup) == 1L) { # all equals to 1L 
    warmup2 <- 1 + (warmup[1] - 1) %/% thin[1]
    n_kept <- n_save - warmup2 
  } 
  if (n_kept0[1] != n_kept) {
    warning("the number of iterations after warmup found (", n_kept, 
            ") does not match iter/warmup/thin from CSV comments (",
            paste(n_kept0, collapse = ','), ")")

    if (n_kept < 0) {
      warmup <- warmup + n_kept
      n_kept <- 0
      mode <- 2L
    }
    n_kept0 <- n_save
    iter <- n_save

    for (i in 1:length(cs_lst2)) {
      cs_lst2[[i]]$warmup <- warmup
      cs_lst2[[i]]$iter <- iter
    }
  }
  idx_kept <- if (warmup2 == 0) 1:n_kept else -(1:warmup2)
  for (i in seq_along(samples)) {
    m <- vapply(samples[[i]], function(x) mean(x[idx_kept]), numeric(1))
    attr(samples[[i]], "mean_pars") <- m[-length(m)]
    attr(samples[[i]], "mean_lp__") <- m["lp__"]
  }

  perm_lst <- lapply(1:chains, function(id) sample.int(n_kept))

  sim = list(samples = samples, 
             iter = iter[1], 
             thin = thin[1], 
             warmup = warmup[1], 
             chains = chains, 
             n_save = rep(n_save, chains),
             warmup2 = rep(warmup2, chains),
             permutation = perm_lst,
             pars_oi = pars_oi, 
             dims_oi = dims_oi,
             fnames_oi = dotfnames_to_sqrfnames(par_fnames), 
             n_flatnames = length(par_fnames))
  null_dso <- new("cxxdso", sig = list(character(0)), dso_saved = FALSE, dso_filename = character(0), 
                  modulename = character(0), system = R.version$system, cxxflags = character(0), 
                 .CXXDSOMISC = new.env(parent = emptyenv()))
  null_sm <- new("stanmodel", model_name = m_name, model_code = character(0), 
                 model_cpp = list(), dso = null_dso)

  nfit <- new("stanfit", 
              model_name = m_name,
              model_pars = pars_oi,
              par_dims = dims_oi, 
              mode = mode,
              sim = sim,
              inits = list(), 
              stan_args = cs_lst2,
              stanmodel = null_sm,
              date = sdate, # not the time of sampling
              .MISC = new.env(parent = emptyenv()))
  return(nfit)
}

read_one_stan_csv <- function(csvfile) {
  if (length(csvfile) != 1) 
    stop("'csvfile' must be of length 1")
  if (!file.exists(csvfile))
    stop("'csvfile' does not exist on the disk")
  
  mark <- 0L
  fields <- character()
  while(length(fields) == 0) {
    mark <- mark + 1L
    fields <- scan(csvfile, what = character(), sep = ",", 
                   comment.char = "#", nlines = mark, quiet = TRUE)
  }
  
  comments <- scan(csvfile, what = character(), sep = "\n", comment.char = "", 
                   nlines = mark + 2L, quiet = TRUE)
  comments <- gsub("#", "", comments, fixed = TRUE)
  comments <- gsub("(Default)", "", comments, fixed = TRUE)
  comments <- grep("=", comments, fixed = TRUE, value = TRUE)
  comments <- strsplit(comments, split = "=", fixed = TRUE)
  comments <- lapply(comments, FUN = trimws)
  comments <- sapply(comments, FUN = function(x) {
    y <- x[2]
    names(y) <- x[1]
    return(y)
  })
  
  method <- comments["algorithm"]
  if (method %in% c("meanfield", "fullrank")) {
    draws <- scan(csvfile, what = double(), sep = ",", comment.char = "", 
                  quiet = TRUE, skip = mark + 2L, 
                  nlines = mark + as.integer(comments["output_samples"]) + 3L)
    timings <- NULL
  }
  else { # sampling
    iter <- as.integer(comments["iter"])
    draws <- scan(csvfile, what = double(), sep = ",", comment.char = "", 
                  quiet = TRUE, skip = mark, nlines = mark + iter)
    timings <- scan(csvfile, what = character(), sep = "\n", comment.char = "", 
                    quiet = TRUE, skip = mark + iter)
  }
  draws <- matrix(draws, ncol = length(fields), byrow = TRUE)
  colnames(draws) <- fields
  draws <- as.data.frame(draws)
  
  attributes(draws)$comments <- comments
  attributes(draws)$timings <- timings
  return(draws)
}

test_that("read_stan_csv works", {
  exfit <- read_stan_csv(dir(system.file("misc", package = "rstan"),
    pattern = "rstan_doc_ex_[[:digit:]].csv",
    full.names = TRUE
  ))
  expect_equal(dim(as.array(exfit)), c(100, 4, 10))
  expect_equal(exfit@model_pars, c("mu", "sigma", "z", "alpha", "lp__"))
  t <- matrix(c(0.005308, 0.003964, 0.004776, 0.003744, 0.010337, 0.002867, 0.004711, 0.004117),
    byrow = TRUE, ncol = 2
  )
  colnames(t) <- c("warmup", "sample")
  rownames(t) <- paste0("chain:", 1:nrow(t))
  expect_equal(get_elapsed_time(exfit), t)
})

test_that("read_stan_csv works with an incomplete csv (0 samples)", {
  skip("Backwards compatibility")

  exfit <- read_stan_csv(dir(system.file("misc", package = "rstan"),
    pattern = "rstan_doc_ex_incomplete_1.csv",
    full.names = TRUE
  ))
  expect_equal(exfit@model_pars, c("mu", "sigma", "z", "alpha", "lp__"))
  expect_true(is.null(get_elapsed_time(exfit)))
})

test_that("read_stan_csv works with an incomplete csv (few samples)", {
  skip("Backwards compatibility")

  exfit <- read_stan_csv(dir(system.file("misc", package = "rstan"),
    pattern = "rstan_doc_ex_incomplete_1.csv",
    full.names = TRUE
  ))
  expect_equal(exfit@model_pars, c("mu", "sigma", "z", "alpha", "lp__"))
})

test_that("paridx_fun works", {
  names <- c("alpha", "beta.1", "beta.2", "lp__", "treedepth__", "stepsize__")
  paridx <- rstan:::paridx_fun(names)
  meta <- 4:6
  names(meta) <- c("lp__", "treedepth__", "stepsize__")
  exp <- c(1, 2, 3); attr(exp, 'meta') <- meta
  expect_equal(paridx, exp)
  paridx2 <- rstan:::paridx_fun(names[-5])
  meta <- 4:5; names(meta) <- c("lp__", "stepsize__")
  attr(exp, 'meta') <- meta
  expect_equal(paridx2, exp)
  paridx3 <- rstan:::paridx_fun(names[-(4:6)])
  meta <- integer(0); names(meta) <- character(0)
  attr(exp, 'meta') <- meta
  expect_equal(paridx3, exp)
})

test_that("parse_stancsv works", {
  comments <- c("# stan_version_major = 1",
                "# stan_version_minor = 3",
                "# stan_version_patch = 0",
                "# model = dogs",
                "# method = sample (Default)",
                "#   sample",
                "#     num_samples = 1000 (Default)",
                "#     num_warmup = 1000 (Default)",
                "#     save_warmup = 0 (Default)",
                "#     thin = 1 (Default)",
                "#     adapt",
                "#       engaged = 1 (Default)",
                "#       gamma = 0.050000000000000003 (Default)",
                "#       delta = 0.65000000000000002 (Default)",
                "#       kappa = 0.75 (Default)",
                "#       t0 = 10 (Default)",
                "#     algorithm = hmc (Default)",
                "#       hmc",
                "#         engine = nuts (Default)",
                "#           nuts",
                "#             max_depth = 10 (Default)",
                "#         metric = diag_e (Default)",
                "#         stepsize = 1 (Default)",
                "#         stepsize_jitter = 0 (Default)",
                "# id = 0 (Default)",
                "# data = dogs.data.R",
                "# init = 2 (Default)",
                "# random",
                "#   seed = 3086139456",
                "# output",
                "#   file = samples.csv (Default)",
                "#   append_sample = 0 (Default)",
                "#   diagnostic_file =  (Default)",
                "#   append_diagnostic = 0 (Default)",
                "#   refresh = 100 (Default)",
                "# Adaptation terminated",
                "# Step size = 0.876835",
                "# Diagonal elements of inverse mass matrix:",
                "# 0.0110854, 0.0230723",
                "#  Elapsed Time: 5.92159 seconds (Warm-up)",
                "#                5.68831 seconds (Sampling)",
                "#                11.6099 seconds (Total)")
  lst <- rstan:::parse_stancsv_comments(comments)
  expect_equal(lst$chain_id, 0)
  existence <- c("seed", "chain_id", "iter", "warmup", "thin",
                 "save_warmup", "stepsize", "time_info", "has_time",
                 "adaptation_info") %in% names(lst)
  expect_true(all(existence))
  expect_equal(lst$seed, "3086139456")
  expect_equal(lst$stepsize, 1)
  expect_equal(lst$sampler_t, "NUTS(diag_e)")
  expect_equal(lst$has_time, TRUE)
})

# This file is part of RStan
# Copyright (C) 2018 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

throw_sampler_warnings <- function(object) {
  if (!is(object, "stanfit"))
    stop("'object' must be of class 'stanfit'")
  
  if (isTRUE(object@stan_args[[1]][["algorithm"]] == "Fixed_param")) {
    return(invisible(NULL))
  }
  
  sp <- get_sampler_params(object, inc_warmup = FALSE)
  n_d <- sum(sapply(sp, FUN = function(x) {
    if ("divergent__" %in% colnames(x)) return(sum(x[,"divergent__"]))
    else return(0)
  }))
  if (n_d > 0) {
    ad <- object@stan_args[[1]]$control
    if (is.null(ad)) ad <- 0.8
    else {
      ad <- ad$adapt_delta
      if (is.null(ad)) ad <- 0.8
    }
    warning("There were ", n_d, " divergent transitions after warmup. See\n",
            "https://mc-stan.org/misc/warnings.html#divergent-transitions-after-warmup\n",
            "to find out why this is a problem and how to eliminate them.", call. = FALSE)
  }
  max_td <- object@stan_args[[1]]$control
  if (is.null(max_td)) max_td <- 10
  else {
    max_td <- max_td$max_treedepth
    if (is.null(max_td)) max_td <- 10
  }
  n_m <- sum(sapply(sp, FUN = function(x) {
    if ("treedepth__" %in% colnames(x)) return(sum(x[,"treedepth__"] >= max_td))
    else return(0)
  }))
  if (n_m > 0)
    warning("There were ", n_m,
            " transitions after warmup that exceeded the maximum treedepth.",
            " Increase max_treedepth above ", max_td, ". See\n",
            "https://mc-stan.org/misc/warnings.html#maximum-treedepth-exceeded", call. = FALSE)
  n_e <- 0L
  if (is_sfinstance_valid(object) && all(sapply(sp, function(x) "energy__" %in% colnames(x)))) {
    E <- as.matrix(sapply(sp, FUN = function(x) x[,"energy__"]))
    threshold <- 0.2
    if (nrow(E) > 1) {
      EBFMI <- get_num_upars(object) / apply(E, 2, var)
      n_e <- sum(EBFMI < threshold, na.rm = TRUE)
    }
    else n_e <- 0L
    if (n_e > 0)
      warning("There were ", n_e, 
              " chains where the estimated Bayesian Fraction of Missing Information",
              " was low. See\n", 
              "https://mc-stan.org/misc/warnings.html#bfmi-low", call. = FALSE)
  }
  if (n_d > 0 || n_m > 0 || n_e > 0) 
    warning("Examine the pairs() plot to diagnose sampling problems\n",
            call. = FALSE, noBreaks. = TRUE)
  
  sims <- as.array(object)
  rhat <- apply(sims, MARGIN = 3, FUN = Rhat)
  if (any(rhat > 1.05, na.rm = TRUE))
      warning("The largest R-hat is ", round(max(rhat), digits = 2),
            ", indicating chains have not mixed.\n",
            "Running the chains for more iterations may help. See\n",
            "https://mc-stan.org/misc/warnings.html#r-hat", call. = FALSE)
  bulk_ess <- apply(sims, MARGIN = 3, FUN = ess_bulk)
  if (any(bulk_ess < 100 * ncol(sims), na.rm = TRUE))
    warning("Bulk Effective Samples Size (ESS) is too low, ",
            "indicating posterior means and medians may be unreliable.\n",
            "Running the chains for more iterations may help. See\n",
            "https://mc-stan.org/misc/warnings.html#bulk-ess", call. = FALSE)
  tail_ess <- apply(sims, MARGIN = 3, FUN = ess_tail)
  if (any(tail_ess < 100 * ncol(sims), na.rm = TRUE))
    warning("Tail Effective Samples Size (ESS) is too low, indicating ",
            "posterior variances and tail quantiles may be unreliable.\n",
            "Running the chains for more iterations may help. See\n",
            "https://mc-stan.org/misc/warnings.html#tail-ess", call. = FALSE)
  
  return(invisible(NULL))
}


# Check divergences, treedepth, and energy diagnostics
#
# @param object A stanfit object.
# @return Nothing, just prints the output from the functions it calls internally.
#
check_hmc_diagnostics <- function(object) {
  stopifnot(is.stanfit(object))
  cat("\nDivergences:\n")
  check_divergences(object)
  cat("\nTree depth:\n")
  check_treedepth(object)
  cat("\nEnergy:\n")
  check_energy(object)
}


# Get a logical vector indicating whict transitions ended with a divergence
#
# @param object A stanfit object.
# @return a logical vector
#
get_divergent_iterations <- function(object) {
  stopifnot(is.stanfit(object))
  sampler_param_vector(object, "divergent__") > 0
}

# Get the number of transitions that ended with a divergence
#
# @param object A stanfit object.
# @return the number of divergent transitions
#
get_num_divergent <- function(object) {
  stopifnot(is.stanfit(object))
  divergent <- get_divergent_iterations(object)
  sum(divergent)
}

# Check transitions that ended with a divergence
#
# @param object A stanfit object.
# @return nothing, just prints the number (and percentage) of iterations that 
#   ended with a divergence and, if any, suggests increasing adapt_delta.
#
check_divergences <- function(object) {
  stopifnot(is.stanfit(object))
  divergent <- get_divergent_iterations(object)
  n <- sum(divergent)
  N <- length(divergent)
  
  if (n == 0) {
    message(
      sprintf("%s of %s iterations ended with a divergence.", n, N)
    )
  } else {
    message(
      sprintf("%s of %s iterations ended with a divergence (%s%%).",
              n, N, 100 * n / N), 
      "\nTry increasing 'adapt_delta' to remove the divergences."
    )
  }
}

get_treedepth_threshold <- function(object) {
  stopifnot(is.stanfit(object))
  max_depth <- object@stan_args[[1]]$control$max_treedepth
  if (is.null(max_depth)) {
    max_depth <- 10
  }
  max_depth
}

# Get a logical vector indicating transitions that ended prematurely 
# due to maximum tree depth limit
#
# @param object A stanfit object.
# @return a logical vector
#
get_max_treedepth_iterations <- function(object) {
  stopifnot(is.stanfit(object))
  max_depth <- get_treedepth_threshold(object)
  treedepths <- sampler_param_vector(object, "treedepth__") >= max_depth
  
  treedepths
}

# Get the number of transitions that ended prematurely due to maximum tree depth limit
#
# @param object A stanfit object.
# @return the number of affected transitions
#
get_num_max_treedepth <- function(object) {
  stopifnot(is.stanfit(object))
  sum(get_max_treedepth_iterations(object))
}

# Check transitions that ended prematurely due to maximum tree depth limit
#
# @param object A stanfit object.
# @return Nothing, just prints the number (and percentage) of iterations that
#   saturated the max treedepth and, if any, suggests increasing max_treedepth.
#
check_treedepth <- function(object) {
  stopifnot(is.stanfit(object))
  max_depth <- get_treedepth_threshold(object)
  treedepths <- get_max_treedepth_iterations(object)
  n <- sum(treedepths)
  N <- length(treedepths)
  
  if (n == 0) {
    message(
      sprintf("%s of %s iterations saturated the maximum tree depth of %s.", 
              n, N, max_depth)
    )
  } else {
    message(
      sprintf('%s of %s iterations saturated the maximum tree depth of %s (%s%%).',
              n, N, max_depth, 100 * n / N), 
      "\nTry increasing 'max_treedepth' to avoid saturation."
    )
  }
}

# Get the Bayesian fraction of missing information (E-BFMI)
#
# @param object A stanfit object.
# @return vector, one element per chain containing the E-BFMI
#
get_bfmi <- function(object) {
  stopifnot(is.stanfit(object))
  energies_by_chain <- sampler_param_matrix(object, "energy__")
  EBFMIs <- apply(energies_by_chain, 2, function(x) {
    numer <- sum(diff(x) ^ 2) / length(x)
    denom <- var(x)
    numer / denom
  })
  
  EBFMIs
}

# Get the chains with low Bayesian fraction of missing information (E-BFMI)
#
# @param object A stanfit object.
# @return a vector of IDs of chains with low E-BFMI 
#
get_low_bfmi_chains <- function(object) {
  stopifnot(is.stanfit(object))
  which(get_bfmi(object) < 0.2)
}

# Check the energy Bayesian fraction of missing information (E-BFMI)
#
# @param object A stanfit object.
# @return Nothing, just prints E-BFMI for chains with low E-BFMI and suggests 
#   reparameterizing.
#
check_energy <- function(object) {
  stopifnot(is.stanfit(object))
  EBFMIs <- get_bfmi(object)
  
  bad_chains <- which(EBFMIs < 0.2)
  if (!length(bad_chains)) {
    message("E-BFMI indicated no pathological behavior.")
  } else {
    message(
      "E-BFMI indicated possible pathological behavior:\n", 
      sprintf("  Chain %s: E-BFMI = %.3f\n", bad_chains, EBFMIs[bad_chains]),
      "E-BFMI below 0.2 indicates you may need to reparameterize your model."
    )
  }
}


# Get the number of actual leapfrog evaluations for each iteration
#
# @param object A stanfit object.
# @return an integer vector with the number of evaluations for each iteration
#
get_num_leapfrog_per_iteration <- function(object) {
  stopifnot(is.stanfit(object))
  sampler_param_vector(object,"n_leapfrog__")
}

# internal ----------------------------------------------------------------

# Extract single sampler parameters in conventient form
#
# @param object A stanfit object.
# @param param The name of a single sampler parameter (e.g. "divergent__").
# @return sampler_param_vector returns a single vector of length chains * iters
#   (post-warmup). sampler_param_matrix returns an iters (post-warmup) by chains
#   matrix.
#
sampler_param_vector <- function(object, param) {
  stopifnot(length(param) == 1, is.character(param))
  sampler_params <- get_sampler_params(object, inc_warmup = FALSE)
  do.call(rbind, sampler_params)[, param]
}
sampler_param_matrix <- function(object, param) {
  stopifnot(length(param) == 1, is.character(param))
  sampler_params <- get_sampler_params(object, inc_warmup=FALSE)
  sapply(sampler_params, function(x) x[, param])
}

test_that("getting diagnostics works", {
  skip("Backwards compatibility")

  ex_fit <- read_stan_csv("test_fit_diagnostics.csv")
  expect_equal(get_bfmi(ex_fit), 1.17558932146879)
  expect_equal(get_low_bfmi_chains(ex_fit), integer(0))

  expected_divergent <- logical(10)
  expected_divergent[c(2, 3)] <- TRUE
  expect_equal(get_divergent_iterations(ex_fit), expected_divergent)
  expect_equal(get_num_divergent(ex_fit), 2)

  expected_max_treedepth <- logical(10)
  expected_max_treedepth[c(1, 5)] <- TRUE
  expect_equal(get_max_treedepth_iterations(ex_fit), expected_max_treedepth)
  expect_equal(get_num_max_treedepth(ex_fit), 2)

  expect_equal(
    get_num_leapfrog_per_iteration(ex_fit),
    c(1, 3, 5, 5, 7, 5, 3, 1, 1, 7)
  )
})

#  This file is part of RStan
#  Copyright (C) 2015 The R Core Team
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.R-project.org/Licenses/

  dynGet <- function (x, 
                      ifnotfound = stop(gettextf("%s not found", sQuote(x)), domain = NA), 
                      minframe = 1L, inherits = FALSE) 
  {
    n <- sys.nframe()
    while (n > minframe) {
      n <- n - 1L
      env <- sys.frame(n)
      if (exists(x, envir = env, inherits = inherits, mode = "numeric")) 
        return(get(x, envir = env, inherits = inherits, mode = "numeric"))
      else if (exists(x, envir = env, inherits = inherits, mode = "logical"))
        return(get(x, envir = env, inherits = inherits, mode = "logical"))
      else if (exists(x, envir = env, inherits = inherits, mode = "list"))
        return(get(x, envir = env, inherits = inherits, mode = "list"))
    }
    return(ifnotfound)
  }

# Leave-one-out cross-validation
# 
# The \code{loo} method for stanfit objects ---a wrapper around the
# \code{loo.array} method from the \pkg{loo} package--- computes approximate
# leave-one-out cross-validation using Pareto smoothed importance sampling
# (PSIS-LOO CV).
# 
# @param x stanfit object
# @param pars Name of parameter, transformed parameter, or generated quantity in
#   the Stan program corresponding to the pointwise log-likelihood. If not
#   specified the default behavior is to look for \code{"log_lik"}.
# @param save_psis Should the intermediate results from \code{\link[loo]{psis}}
#   be saved in the returned object? The default is \code{FALSE}. This can be
#   useful to avoid repeated computation when using other functions in the
#   \pkg{loo} and \pkg{bayesplot} packages.
# @param cores Number of cores to use for parallelization. Passed to
#   \code{\link[loo]{loo.array}}.
# @param moment_match Logical; Whether to use the moment matching algorithm for
#   observations with high Pareto k values to improve accuracy.
# @param k_threshold Threshold value for Pareto k values above which
#   the moment matching algorithm is used. If \code{moment_match} is \code{FALSE},
#   this is ignored.
# @param ... Ignored.
# 
# @details Stan does not automatically compute and store the log-likelihood. It
#   is up to the user to incorporate it into the Stan program if it is to be
#   extracted after fitting the model. In a Stan model, the pointwise log
#   likelihood can be coded as a vector in the transformed parameters block
#   (and then summed up in the model block) or it can be coded entirely in the
#   generated quantities block. We recommend using the generated quantities
#   block so that the computations are carried out only once per iteration
#   rather than once per HMC leapfrog step.
#
#   For example, the following is the \code{generated quantities} block for
#   computing and saving the log-likelihood for a linear regression model with
#   \code{N} data points, outcome \code{y}, predictor matrix \code{X},
#   coefficients \code{beta}, and standard deviation \code{sigma}:
#
#  \code{vector[N] log_lik;}
#
#  \code{for (n in 1:N) log_lik[n] = normal_lpdf(y[n] | X[n, ] * beta, sigma);}
#
loo.stanfit <-
  function(x,
           pars = "log_lik",
           save_psis = FALSE,
           cores = getOption("mc.cores", 1),
           moment_match = FALSE,
           k_threshold = 0.7,
           ...) {
    stopifnot(length(pars) == 1L)
    stopifnot(is.logical(save_psis))
    stopifnot(is.logical(moment_match))
    stopifnot(is.numeric(k_threshold))
    
    LLarray <- loo::extract_log_lik(stanfit = x,
                                    parameter_name = pars,
                                    merge_chains = FALSE)
    r_eff <- loo::relative_eff(x = exp(LLarray), cores = cores)
    
    if (moment_match) {
      loo <- suppressWarnings(loo::loo.array(LLarray,
                                             r_eff = r_eff,
                                             cores = cores,
                                             save_psis = save_psis))
      
      x_array <- as.array(x)
      chain_id <- rep(seq(dim(x_array)[2]),each = dim(x_array)[1])
      loo <- loo_moment_match.stanfit(
        x, loo = loo, chain_id = chain_id, k_threshold = k_threshold,
        cores = cores, parameter_name = pars, ...
      )
    }
    else {
      loo <- loo::loo.array(LLarray,
                            r_eff = r_eff,
                            cores = cores,
                            save_psis = save_psis)
    }

    loo
  }

setMethod("loo", "stanfit", loo.stanfit)



# INTERNAL -----------------------------


# wrapper around loo_moment_match.default from loo package
loo_moment_match.stanfit <- function(x, loo = loo, ...) {
  loo::loo_moment_match.default(
    x = x, loo = loo,
    post_draws = post_draws_stanfit,
    log_lik_i = log_lik_i_stanfit,
    unconstrain_pars = unconstrain_pars_stanfit,
    log_prob_upars = log_prob_upars_stanfit,
    log_lik_i_upars = log_lik_i_upars_stanfit,
    ...)
}

# create a named list of draws for use with rstan methods
.rstan_relist <- function (x, skeleton) {
  out <- utils::relist(x, skeleton)
  for (i in seq_along(skeleton)) {
    dim(out[[i]]) <- dim(skeleton[[i]])
  }
  out
}

# rstan helper function to get dims of parameters right
.create_skeleton <- function (pars, dims) {
  out <- lapply(seq_along(pars), function(i) {
    len_dims <- length(dims[[i]])
    if (len_dims < 1) return(0)
    return(array(0, dim = dims[[i]]))
  })
  names(out) <- pars
  out
}

# extract original posterior draws
post_draws_stanfit <- function(x, ...) {
  as.matrix(x)
}

# compute a matrix of log-likelihood values for the ith observation
# matrix contains information about the number of MCMC chains
log_lik_i_stanfit <- function(x, i, parameter_name = "log_lik", ...) {
  loo::extract_log_lik(x, parameter_name, merge_chains = FALSE)[, , i]
}

# transform parameters to the unconstraint space
unconstrain_pars_stanfit <- function(x, pars, ...) {
  skeleton <- .create_skeleton(x@sim$pars_oi, x@par_dims[x@sim$pars_oi])
  upars <- apply(pars, 1, FUN = function(theta) {
    rstan::unconstrain_pars(x, .rstan_relist(theta, skeleton))
  })
  # for one parameter models
  if (is.null(dim(upars))) {
    dim(upars) <- c(1, length(upars))
  }
  t(upars)
}

# compute log_prob for each posterior draws on the unconstrained space
log_prob_upars_stanfit <- function(x, upars, ...) {
  apply(upars, 1, rstan::log_prob, object = x,
        adjust_transform = TRUE, gradient = FALSE)
}

# compute log_lik values based on the unconstrained parameters
log_lik_i_upars_stanfit <- function(x, upars, i, parameter_name = "log_lik",
                                  ...) {
  S <- nrow(upars)
  out <- numeric(S)
  for (s in seq_len(S)) {
    out[s] <- rstan::constrain_pars(x, upars = upars[s, ])[[parameter_name]][i]
  }
  out
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

setGeneric(name = "grab_cxxfun",
           def = function(object, ...) { standardGeneric("grab_cxxfun")})

setGeneric(name = "is_dso_loaded",
           def = function(object, ...) { standardGeneric("is_dso_loaded")})
setGeneric(name = "reload_dso", 
           def = function(object, ...) { standardGeneric("reload_dso")})

setMethod("show", "cxxdso", 
          function(object) {
            cat("S4 class cxxdso: dso_saved = ", object@dso_saved, 
                ", dso_filename = ", object@dso_filename, 
                ", size = ", obj_size_str(object.size(object@.CXXDSOMISC$dso_bin)), ".\n", sep = '')  
            cat("And dso_last_path = '", object@.CXXDSOMISC$dso_last_path, "'.\n", sep = '')
            cat("Created on: '", object@system, "' with '", object@cxxflags, "'.\n", sep = '')
            cat("Loaded now: ", if (is_dso_loaded(object)) 'YES' else 'NO', ".\n", sep = '')
            cat("The signatures is/are as follows: \n")
            print(object@sig); 
          })

setMethod('is_dso_loaded', signature(object = 'cxxdso'), 
          function(object) {
            if (identical(object@modulename, character(0))) return(FALSE) # null dso
            f2 <- sub("\\.[^.]*$", "", basename(object@.CXXDSOMISC$dso_last_path)) 
            dlls <- getLoadedDLLs()
            f2 %in% names(dlls)
          }) 

setMethod('grab_cxxfun', signature(object = "cxxdso"), 
          function(object) {
            if (length(object@dso_saved) == 0)
              return(function(...) stop("this function should not be called"))
            if (!is_null_cxxfun(object@.CXXDSOMISC$cxxfun)) 
              return(object@.CXXDSOMISC$cxxfun)
            if (!object@dso_saved) 
              stop("the cxx fun is NULL now and this cxxdso is not saved")

            # If the file is still loaded  
            # from the help of function dyn.load 
            #   The function dyn.unload unlinks the DLL.  Note that unloading a
            #   DLL and then re-loading a DLL of the same name may or may not
            #   work: on Solaris it uses the first version loaded.
            f <- sub("\\.[^.]*$", "", basename(object@dso_filename)) 
            f2 <- sub("\\.[^.]*$", "", basename(object@.CXXDSOMISC$dso_last_path)) 
            dlls <- getLoadedDLLs()
            if (f2 %in% names(dlls)) { # still loaded 
              DLL <- dlls[[f2]] 
              fx <- cxxfun_from_dll(object@sig, object@.CXXDSOMISC$cxxfun@code, DLL, check_dll = FALSE) 
              assign('cxxfun', fx, envir = object@.CXXDSOMISC) 
              if (!is.null(object@modulename) && object@modulename != '') 
                assign("module", Module(object@modulename, getDynLib(fx)), envir = object@.CXXDSOMISC)
              return(fx) 
            }
            
            # not loaded  
            if (!identical(object@system, R.version$system)) 
              stop(paste("this cxxdso object was created on system '", object@system, "'", sep = ''))
            fx <- cxxfun_from_dso_bin(object) 
            assign('cxxfun', fx, envir = object@.CXXDSOMISC) 
            if (!is.null(object@modulename) && object@modulename != '') 
              assign("module", Module(object@modulename, getDynLib(fx)), envir = object@.CXXDSOMISC)
            return(fx) 
          }) 

setMethod("getDynLib", signature(x = "cxxdso"),
          function(x) { 
            fx <- grab_cxxfun(x) 
            env <- environment(fx@.Data)
            f <- get("f", env)
            dlls <- getLoadedDLLs()
            if (!f  %in% names(dlls)) 
              stop(paste('dso ', f, ' is not loaded', sep = ''))
            dlls[[f]]
          })

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

get_Rcpp_module_def_code <- function(model_name) {
  RCPP_MODULE <-
'
RCPP_MODULE(stan_fit4%model_name%_mod) {
  class_<rstan::stan_fit<stan_model, boost::random::ecuyer1988> >(
      "stan_fit4%model_name%")

      .constructor<SEXP, SEXP, SEXP>()

      .method(
          "call_sampler",
          &rstan::stan_fit<stan_model, boost::random::ecuyer1988>::call_sampler)
      .method(
          "param_names",
          &rstan::stan_fit<stan_model, boost::random::ecuyer1988>::param_names)
      .method("param_names_oi",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::param_names_oi)
      .method("param_fnames_oi",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::param_fnames_oi)
      .method(
          "param_dims",
          &rstan::stan_fit<stan_model, boost::random::ecuyer1988>::param_dims)
      .method("param_dims_oi",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::param_dims_oi)
      .method("update_param_oi",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::update_param_oi)
      .method("param_oi_tidx",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::param_oi_tidx)
      .method("grad_log_prob",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::grad_log_prob)
      .method("log_prob",
              &rstan::stan_fit<stan_model, boost::random::ecuyer1988>::log_prob)
      .method("unconstrain_pars",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::unconstrain_pars)
      .method("constrain_pars",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::constrain_pars)
      .method(
          "num_pars_unconstrained",
          &rstan::stan_fit<stan_model,
                           boost::random::ecuyer1988>::num_pars_unconstrained)
      .method(
          "unconstrained_param_names",
          &rstan::stan_fit<
              stan_model, boost::random::ecuyer1988>::unconstrained_param_names)
      .method(
          "constrained_param_names",
          &rstan::stan_fit<stan_model,
                           boost::random::ecuyer1988>::constrained_param_names)
      .method("standalone_gqs",
              &rstan::stan_fit<stan_model,
                               boost::random::ecuyer1988>::standalone_gqs);
}
'
gsub("%model_name%", model_name, RCPP_MODULE)
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

##
## Define rstan plugin for inline package.
## (original name: inline.R)
##

inc_path_fun <- function(package) {
  system.file('include', package = package)
}

# Using RcppEigen
eigen_path_fun <- function() {
  rstan_options("eigen_lib")
}

boost_path_fun <- function() {
  rstan_options("boost_lib")
}

boost_path_fun2 <- function() {
  rstan_options("boost_lib2")
}

PKG_CPPFLAGS_env_fun <- function() {
   Eigen <- dir(system.file("include", "stan", "math", "prim",
                            package = "StanHeaders", mustWork = TRUE),
                pattern = "Eigen.hpp$", full.names = TRUE, recursive = TRUE)[1]
   paste(' -I"', file.path(inc_path_fun("Rcpp"), '" '),
         ' -I"', file.path(eigen_path_fun(), '" '),
         ' -I"', file.path(eigen_path_fun(), 'unsupported" '),
         # ' -I"', boost_path_fun2(), '"', # boost_not_in_BH should come
         ' -I"', boost_path_fun(), '"',  # before BH/include
         ' -I"', file.path(inc_path_fun("StanHeaders"), "src", '" '),
         ' -I"', file.path(inc_path_fun("StanHeaders"), '" '),
         ' -I"', file.path(inc_path_fun("RcppParallel"), '" '),
         utils::capture.output(RcppParallel::CxxFlags()),
         ' -I"', inc_path_fun("rstan"), '"',
         ' -DEIGEN_NO_DEBUG ',
         ' -DBOOST_DISABLE_ASSERTS ',
         ' -DBOOST_PENDING_INTEGER_LOG2_HPP ',
         ' -DSTAN_THREADS ',
         ' -DUSE_STANC3',
         ' -DSTRICT_R_HEADERS ',
         ' -DBOOST_PHOENIX_NO_VARIADIC_EXPRESSION ',
         ' -D_HAS_AUTO_PTR_ETC=0 ',
         ' -include ', shQuote(Eigen), ' ',
         ifelse (.Platform$OS.type == "windows", ' -std=c++1y',
                 ' -D_REENTRANT -DRCPP_PARALLEL_USE_TBB=1 '),
         sep = '')
}

legitimate_space_in_path <- function(path) {
  # For windows, use the short path name (8.3 format)
  #
  if (.Platform$OS.type == "windows") {
    path <- normalizePath(path)
    if (grepl(" ", path, fixed = TRUE))
      path <- utils::shortPathName(path)
    # it is weird that the '\\' in the path name will be gone
    # when passed to cxxfunction, so change it to '/'
    path <- gsub('\\\\', '/', path, perl = TRUE)
  }
  path
}

rstanplugin <- function() {
  Rcpp_plugin <- getPlugin("Rcpp")
  rcpp_pkg_libs <- Rcpp_plugin$env$PKG_LIBS
  rcpp_pkg_path <- system.file(package = 'Rcpp')
  rcpp_pkg_path2 <- legitimate_space_in_path(rcpp_pkg_path)

  if (.Platform$OS.type == "windows") {
    StanHeaders_pkg_libs <- system.file("libs", .Platform$r_arch,
                                        package = "StanHeaders", mustWork = TRUE)
    RcppParallel_pkg_libs <- system.file("lib", .Platform$r_arch,
                                         package = "RcppParallel", mustWork = TRUE)
    rstan_StanServices <- system.file("lib", .Platform$r_arch, "libStanServices.a",
                                      package = "rstan", mustWork = TRUE)
  }
  else {
    StanHeaders_pkg_libs <- system.file("lib", .Platform$r_arch,
                                        package = "StanHeaders", mustWork = TRUE)
    RcppParallel_pkg_libs <- system.file("lib", .Platform$r_arch,
                                         package = "RcppParallel", mustWork = TRUE)
    rstan_StanServices <- system.file("lib", .Platform$r_arch, "libStanServices.a",
                                      package = "rstan", mustWork = TRUE)
  }

  # In case  we have space (typical on windows though not necessarily)
  # in the file path of Rcpp's library.

  # If rcpp_PKG_LIBS contains space without preceding '\\', add `\\';
  # otherwise keept it intact
  if (grepl('[^\\\\]\\s', rcpp_pkg_libs, perl = TRUE))
      rcpp_pkg_libs <- gsub(rcpp_pkg_path, rcpp_pkg_path2, rcpp_pkg_libs, fixed = TRUE)

  # cat("INFO: rcpp_pkg_libs = ", rcpp_pkg_libs, "\n")

  tbb_libs <- "-ltbb"

  PL <- paste(rcpp_pkg_libs,
              shQuote(rstan_StanServices),
              paste0("-L", shQuote(StanHeaders_pkg_libs)),
              "-lStanHeaders",
              paste0("-L", shQuote(RcppParallel_pkg_libs)),
              tbb_libs,
              utils::capture.output(RcppParallel::RcppParallelLibs()))
  if (.Platform$OS.type == "windows") {
    list(includes = '// [[Rcpp::plugins(cpp14)]]\n',
         body = function(x) x,
         env = list(PKG_LIBS = PL,
                    PKG_CPPFLAGS = paste(Rcpp_plugin$env$PKG_CPPFLAGS,
                                         PKG_CPPFLAGS_env_fun(), collapse = " ")))
  } else {
    list(includes = '// [[Rcpp::plugins(cpp14)]]\n',
         body = function(x) x,
         env = list(PKG_LIBS = PL,
                    PKG_CPPFLAGS = paste(Rcpp_plugin$env$PKG_CPPFLAGS,
                                         PKG_CPPFLAGS_env_fun(), collapse = " ")))
  }
}


# inlineCxxPlugin would automatically get registered in inline's plugin list.
# Note that everytime rstan plugin is used, inlineCxxPlugin
# gets called so we can change some settings on the fly
# for example now by setting rstan_options(boost_lib=xxx)
inlineCxxPlugin <- function(...) {
  settings <- rstanplugin()
  settings
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
# Copyright (C) 1995-2012 The R Core Team
# Some parts  Copyright (C) 1999 Dr. Jens Oehlschlaegel-Akiyoshi
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

pairs.stanfit <-
  function (x, labels = NULL, panel = NULL, ...,
            lower.panel = NULL, upper.panel = NULL, diag.panel = NULL, 
            text.panel = NULL, label.pos = 0.5 + 1/3, 
            cex.labels = NULL, font.labels = 1, 
            row1attop = TRUE, gap = 1, log = "",
            pars = NULL, include = TRUE,
            condition = "accept_stat__") {
    
    gsp <- get_sampler_params(x, inc_warmup = FALSE)
    if(is.null(pars)) {
      pars <- dimnames(x)[[3]]
      arr <- round(extract(x, pars = pars, permuted = FALSE), digits = 12)
      if ("energy__" %in% colnames(gsp[[1]])) {
        dims <- dim(arr)
        dims[3] <- dims[3] + 1L
        nms <- dimnames(arr)
        nms$parameters <- c(nms$parameters, "energy__")
        E <- sapply(gsp, FUN = function(y) y[,"energy__"])
        arr <- array(c(c(arr), c(E)), dim = dims, dimnames = nms)
      }
    }
    else if (!include) {
      incl_energy <- !any(pars == "energy__")
      pars <- setdiff(x@sim$pars_oi, pars)
      arr <- round(extract(x, pars = pars, permuted = FALSE), digits = 12)
      if (incl_energy && "energy__" %in% colnames(gsp[[1]])) {
        dims <- dim(arr)
        dims[3] <- dims[3] + 1L
        nms <- dimnames(arr)
        nms$parameters <- c(nms$parameters, "energy__")
        E <- sapply(gsp, FUN = function(y) y[,"energy__"])
        arr <- array(c(c(arr), c(E)), dim = dims, dimnames = nms)
      }
    }
    else arr <- round(extract(x, pars = pars, permuted = FALSE), digits = 12)
    sims <- nrow(arr)
    chains <- ncol(arr)
    varying <- apply(arr, 3, FUN = function(y) length(unique(c(y))) > 1)
    if (any(!varying)) {
      message("the following parameters were dropped because they are constant\n",
              paste(names(varying)[!varying], collapse = " "))
      arr <- arr[,,varying,drop = FALSE]
    }
    dupes <- duplicated(arr, MARGIN = 3)
    if (any(dupes)) {
      message("the following parameters were dropped because they are duplicative\n",
              paste(dimnames(arr)[[3]][dupes], collapse = " "))
      arr <- arr[,,!dupes,drop = FALSE]
    }
    divergent__ <- matrix(c(sapply(gsp, FUN = function(y) y[,"divergent__"])), 
                            nrow = sims * chains, ncol = dim(arr)[3])
    max_td <- x@stan_args[[1]]$control
    if (is.null(max_td)) max_td <- 10
    else {
      max_td <- max_td$max_treedepth
      if (is.null(max_td)) max_td <- 10
    }
    hit <- matrix(c(sapply(gsp, FUN = function(y) y[,"treedepth__"] >= max_td)), 
                    nrow = sims * chains, ncol = dim(arr)[3])
    
    if(is.list(condition)) {
      if(length(condition) != 2) stop("if a list, 'condition' must be of length 2")
      arr <- arr[,c(condition[[1]], condition[[2]]),,drop = FALSE]
      k <- length(condition[[1]])
      mark <- c(rep(TRUE, sims * k), rep(FALSE, sims * length(condition[[2]])))
    }
    else if(is.logical(condition)) {
      stopifnot(length(condition) == (sims * chains))
      mark <- !condition
    }
    else if(is.character(condition)) {
      condition <- match.arg(condition, several.ok = FALSE,
                             choices = c("accept_stat__", "stepsize__", "treedepth__", 
                                         "n_leapfrog__", "divergent__", "energy__",
                                         "lp__"))
      if (condition == "lp__") 
        mark <- simplify2array(get_logposterior(x, inc_warmup = FALSE))
      else mark <- sapply(gsp, FUN = function(y) y[,condition])
      
      if(condition == "divergent__") mark <- as.logical(mark)
      else mark <- c(mark) >= median(mark)
      if (length(unique(mark)) == 1) 
        stop(paste(condition, "is constant so it cannot be used as a condition"))
    }
    else if(!is.null(condition)) {
      if(all(condition == as.integer(condition))) {
        arr <- arr[,condition,,drop = FALSE]
        k <- ncol(arr) %/% 2
        mark <- c(rep(FALSE, sims * k), rep(TRUE, sims * (chains - k)))
      }
      else if(condition > 0 && condition < 1) {
        mark <- rep(1:sims > (condition * sims), times = chains)
      }
      else stop("'condition' must be an integer (vector) or a number between 0 and 1 exclusive")
    }
    else {
      k <- ncol(arr) %/% 2
      mark <- c(rep(FALSE, sims * k), rep(TRUE, sims * (chains - k)))
    }
    
    x <- apply(arr, MARGIN = "parameters", FUN = function(y) y)
    nc <- ncol(x)
    
    if (isTRUE(log)) {
      xl <- apply(x >= 0, 2, FUN = all)
      if("lp__" %in% names(xl)) xl["lp__"] <- FALSE
    }
    else if (is.numeric(log)) xl <- log
    else xl <- grepl("x", log)

    if (is.numeric(xl) || any(xl)) {
      x[,xl] <- log(x[,xl])
      colnames(x)[xl] <- paste("log", colnames(x)[xl], sep = "-")
    }
    if(is.null(lower.panel)) {
      if(!is.null(panel)) lower.panel <- panel
      else lower.panel <- function(x,y, ...) {
        dots <- list(...)
        x_tmp <- x[!mark]
        y_tmp <- y[!mark]
        div_tmp <- divergent__[!mark]
        hit_tmp <- hit[!mark]
        dots$x <- x_tmp
        dots$y <- y_tmp
        dots$nrpoints <- 0
        dots$postPlotHook <- function() {
          if (!identical(condition, "divergent__")) {
            points(x_tmp[div_tmp==1], y_tmp[div_tmp == 1], col = "red", pch = ".")
            points(x_tmp[hit_tmp == 1], y_tmp[hit_tmp == 1], col = "yellow", pch = ".")
          }
        }
        dots$add <- TRUE
        do.call(smoothScatter, args = dots)
      }
    }
    if(is.null(upper.panel)) {
      if(!is.null(panel)) upper.panel <- panel
      else upper.panel <- function(x,y, ...) {
        dots <- list(...)
        x_tmp <- x[mark]
        y_tmp <- y[mark]
        div_tmp <- divergent__[mark]
        hit_tmp <- hit[mark]
        dots$x <- x_tmp
        dots$y <- y_tmp
        dots$nrpoints <- 0
        dots$postPlotHook <- function() {
          if (!identical(condition, "divergent__")) {
            points(x_tmp[div_tmp==1], y_tmp[div_tmp == 1], col = "red", pch = ".")
            points(x_tmp[hit_tmp == 1], y_tmp[hit_tmp == 1], col = "yellow", pch = ".")
          }
        }
        dots$add <- TRUE
        do.call(smoothScatter, args = dots)
      }
    }
    if(is.null(diag.panel)) diag.panel <- function(x, ...) {
        usr <- par("usr"); on.exit(par(usr))
        par(usr = c(usr[1:2], 0, 1.5) )
        h <- hist(x, plot = FALSE)
        breaks <- h$breaks
        y <- h$counts; y <- y/max(y)
        nB <- length(breaks)
        rect(breaks[-nB], 0, breaks[-1], y, col="cyan", ...)
    }
    if(is.null(panel)) panel <- points
    
    if(is.null(text.panel)) textPanel <- function(x = 0.5, y = 0.5, txt, cex, font) {
      text(x,y, txt, cex = cex, font = font)
    }
    else textPanel <- text.panel
    if(is.null(labels)) labels <- colnames(x)

    mc <- match.call(expand.dots = TRUE)
    mc[1] <- call("pairs")
    mc$x <- x
    mc$labels <- labels
    mc$panel <- panel
    mc$lower.panel <- lower.panel
    mc$upper.panel <- upper.panel
    mc$diag.panel <- diag.panel
    mc$text.panel <- textPanel
    mc$log <- ""
    mc$condition <- NULL
    mc$pars <- NULL
    mc$include <- NULL
    eval(mc)
  }

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

stan_version <- function() {
  .Call(CPP_stan_version)
}

rstudio_stanc <- function(filename) {
  output <- stanc(filename, allow_undefined = TRUE)
  if (!output$status) {
    msg <- output$errors[2]
    line <- as.integer(sub("^.*line ([[:digit:]]+),.*$", "\\1", msg))
    column <- as.integer(sub("^.*column ([[:digit:]]+),.*$", "\\1", msg))
    msg <- gsub("\n", "", msg, fixed = TRUE)
    msg <- sub("^.*:", "", msg)
    rstudioapi::sourceMarkers(name = "Stan",
                              markers = data.frame(type = "error",
                                                   file = filename,
                                                   line = line,
                                                   column = column,
                                                   message = msg,
                                                   stringsAsFactors = FALSE),
                              basePath = dirname(filename),
                              autoSelect = "none")
  } else {
    message(filename, " is syntactically correct.")
    return(invisible(output))
  }
}

stanc_process <- function(file, model_code = '', model_name = "anon_model",
                          auto_format = FALSE,
                          isystem = c(dirname(file), getwd())) {
  model_name2 <- deparse(substitute(model_code))
  if (is.null(attr(model_code, "model_name2")))
    attr(model_code, "model_name2") <- model_name2

  model_code <- get_model_strcode(file, model_code)
  if (missing(model_name) || is.null(model_name))
    model_name <- attr(model_code, "model_name2")

  model_attr <- attributes(model_code)
  model_code <- scan(text = model_code, what = character(), sep = "\n", quiet = TRUE)

  # Remove trailing whitespaces
  model_code <- trimws(model_code, "r")

  includes <- grep("^[[:blank:]]*#include ", model_code)
  while(length(includes) > 0) {
    for (i in rev(includes)) {
      header <- sub("^[[:blank:]]*#include[[:blank:]]+", "", model_code[i])
      header <- gsub('\\"', '', header)
      header <- gsub("\\'", '', header)
      header <- sub("<", "", header, fixed = TRUE)
      header <- sub(">", "", header, fixed = TRUE)
      header <- sub("//.*$", "", header)
      header <- sub("/\\*.*$", "", header)
      header <- sub("#.*$", "", header)
      header <- sub("[[:blank:]]*$", "", header)
      files <- file.path(isystem, header)
      existent <- file.exists(files)
      if (any(existent))
        model_code <- append(model_code, values = readLines(files[which(existent)[1]]),
                          after = i)
      else model_code <- append(model_code, values = readLines(header), after = 1)
      model_code[i] <- ""
    }
    includes <- grep("^[[:blank:]]*#include ", model_code)
  }

  model_code <- gsub('#include /(.*$)', '#include "\\1"', model_code)
  has_pound <- any(grepl("#", model_code, fixed = TRUE))

  if (FALSE && has_pound && isFALSE(auto_format)) {
    unprocessed <- tempfile(fileext = ".stan")
    processed <- tempfile(fileext = ".stan")
    on.exit(file.remove(unprocessed))
    writeLines(model_code, con = unprocessed)
    ARGS <- paste("-E -nostdinc -x c++ -P -C",
                  paste("-I", isystem, " ", collapse = ""),
                  "-o", processed, unprocessed, "-Wno-invalid-pp-token")
    CPP <- system2(file.path(R.home(component = "bin"), "R"),
                   args = "CMD config CC", stdout = TRUE)
    pkgbuild::with_build_tools(system(paste(CPP, ARGS),
                                      ignore.stdout = TRUE, ignore.stderr = TRUE),
                               required = rstan_options("required") &&
                                 identical(Sys.getenv("WINDOWS"), "TRUE") &&
                                !identical(Sys.getenv("R_PACKAGE_SOURCE"), "") )
    if (file.exists(processed)) {
      on.exit(file.remove(processed), add = TRUE)
      model_code <- paste(readLines(processed), collapse = "\n")
    }
  } else model_code <- paste(model_code, collapse = "\n")

  mostattributes(model_code) <- model_attr

  return(model_code)
}

stanc_builder <- function(file, isystem = c(dirname(file), getwd()),
                          verbose = FALSE, obfuscate_model_name = FALSE,
                          allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
                          allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
                          standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
                          use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
                          warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
                          warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE))) {
  stopifnot(is.character(file), length(file) == 1, file.exists(file))
  model_name <- sub("\\.[^.]*$", "", filename_rm_ext(basename(file)))

  model_cppname <- legitimate_model_name(model_name, obfuscate_name = obfuscate_model_name)

  auto_format <- isTRUE(getOption("stanc.auto_format", FALSE))
  if (isTRUE(auto_format)) {
    model_code <- stanc_process(file = file,
                                model_name = model_name,
                                auto_format = TRUE,
                                isystem = isystem)

    stopifnot(stanc_ctx$validate("stanc"))
    formatted_code <- try(stanc_ctx$call("stanc", model_cppname,
                          model_code, as.array("auto-format")),
                          silent = TRUE)
    if (!inherits(formatted_code, "try-error") && !is.null(formatted_code$result)) {
      model_code <- formatted_code$result
    }
  }

  model_code <- stanc_process(file = file,
                              model_name = model_name,
                              auto_format = FALSE,
                              isystem = isystem)

  out <- stanc(model_code = model_code,
               model_name = model_name, verbose = verbose,
               obfuscate_model_name = obfuscate_model_name,
               allow_undefined = allow_undefined,
               allow_optimizations = allow_optimizations,
               standalone_functions = standalone_functions,
               use_opencl = use_opencl,
               warn_pedantic = warn_pedantic,
               warn_uninitialized = warn_uninitialized)
  return(out)
}

stanc <- function(file, model_code = '', model_name = "anon_model",
                  verbose = FALSE, obfuscate_model_name = TRUE,
                  allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
                  allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
                  standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
                  use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
                  warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
                  warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE)),
                  isystem = c(if (!missing(file)) dirname(file), getwd())) {
  if (missing(file)) {
    file <- tempfile(fileext = ".stan")
    on.exit(file.remove(file))
    writeLines(model_code, con = file)
  } else if (isTRUE(!nzchar(file)) || is.null(file)) {
    stop("Empty or invalid filename!")
  }

  if (missing(model_name) && is.character(file) && length(file) == 1 && file.exists(file))
    model_name <- sub("\\.[^.]*$", "", filename_rm_ext(basename(file)))

  model_cppname <- legitimate_model_name(model_name, obfuscate_name = obfuscate_model_name)

  auto_format <- isTRUE(getOption("stanc.auto_format", FALSE))
  if (isTRUE(auto_format)) {
    model_code <- stanc_process(file = file,
                                model_code = model_code,
                                model_name = model_name,
                                auto_format = TRUE,
                                isystem = isystem)

    stopifnot(stanc_ctx$validate("stanc"))
    formatted_code <- try(stanc_ctx$call("stanc", model_cppname,
                          model_code, as.array("auto-format")),
                          silent = TRUE)
    if (!inherits(formatted_code, "try-error") && !is.null(formatted_code$result)) {
      model_code <- formatted_code$result
    }
  }

  model_code <- stanc_process(file = file,
                              model_code = model_code,
                              model_name = model_name,
                              auto_format = FALSE,
                              isystem = isystem)

  if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
    Sys.setenv("STAN_NUM_THREADS" = rstan_options("threads_per_chain"))
  }

  if (verbose)
    cat("\nTRANSLATING MODEL '", model_name, "' FROM Stan CODE TO C++ CODE NOW.\n", sep = '')

  stopifnot(stanc_ctx$validate("stanc"))
  stanc_flags <- c("allow-undefined",
                   "O1",
                   "standalone-functions",
                   "use-opencl",
                   "warn-pedantic",
                   "warn-uninitialized")
  istanc_flags <- c(allow_undefined,
                    allow_optimizations,
                    standalone_functions,
                    use_opencl,
                    warn_pedantic,
                    warn_uninitialized)
  if (sum(istanc_flags) >= 1) {
    stanc_flags <- as.array(stanc_flags[istanc_flags])
  } else {
    stanc_flags <- as.array("")
  }
  model_cppcode <- try(stanc_ctx$call("stanc", model_cppname, model_code, stanc_flags),
                       silent = TRUE)
  if (inherits(model_cppcode, "try-error")) {
    stop("parser failed badly; maybe try installing the V8 package")
  } else if (length(model_cppcode$errors)) {
    model_cppcode$status <- FALSE
    stop(paste(model_cppcode$errors, collapse = "\n"))
  } else {
    model_cppcode$status <- TRUE
  }

  # Make sure that the model name is not NULL
  if (is.null(model_name)) model_name <- "anon_model"

  # Use model_name in locations_array__
  cppcode <- model_cppcode$result
  cppcode <- gsub(paste0(" (in ", shQuote('string'), ", line "),
                  paste0(" (in ", shQuote(model_name), ", line "),
                  cppcode, fixed = TRUE)

  if (isTRUE(rstan_options("threads_per_chain") > 1L)) {
    # Initialize Stan/math TBB arena and global control
    cppcode <- paste("#ifdef STAN_THREADS",
                     "#ifndef RSTAN_THREADING",
                     "#define RSTAN_THREADING",
                     "#include <stan/math/prim/core/init_threadpool_tbb.hpp>",
                     "auto tbb_init = stan::math::init_threadpool_tbb();",
                     "#endif",
                     "#endif",
                     cppcode,
                     sep = "\n")
  }

  # Define USE_STANC3 for StanHeaders 2.26
  cppcode <- paste("#ifndef USE_STANC3",
                   "#define USE_STANC3",
                   "#endif",
                   cppcode,
                   sep = "\n")

  # Remove leading space from keywords
  cppcode <- scan(text = cppcode, what = character(), sep = "\n", quiet = TRUE)
  cppcode <- gsub("^ class ", "class ", cppcode)
  cppcode <- gsub("^ namespace ", "namespace ", cppcode)
  cppcode <- gsub("^ private:", "private:", cppcode)
  cppcode <- gsub("^ public:", "public:", cppcode)
  cppcode <- paste(cppcode, collapse = "\n")

  return(list(status = model_cppcode$status,
              model_cppname = model_cppname, cppcode = cppcode,
              model_name = model_name, model_code = model_code))
}

test_that("long error messages work", {
  skip("Backwards compatibility")

  code <- "parameters { cov_matrix[3] y; }
           model { y ~ normal(0, 1); }"
  expect_error({r <- stanc(model_code = code)})
  warning_length <- getOption("warning.length")
  emsg <- geterrmessage()
  expect_true(nchar(emsg) < 1000)
  expect_equal(warning_length, getOption("warning.length"))
})

rstan_gg_options <- function(...) {
  ops <- c("size", "linewidth", "fill", "color", "chain_colors")
  dots <- list(...)
  if (length(dots) == 0) {
    oplist <- list()
    for (op in ops) oplist[[op]] <- .rstanvis_defaults[[op]]
    print(oplist)
    return(invisible(NULL))
  }
  nms <- names(dots)
  for (x in nms) {
    .rstanvis_defaults[[x]] <- dots[[x]]
  }
}

rstan_ggtheme_options <- function(...) {
  if (length(list(...)) == 0) {
    print(.rstanvis_defaults$theme)
    return(invisible(NULL))
  }
  els <- theme(...)
  nms <- names(els)
  for (x in nms) {
    .rstanvis_defaults$theme[[x]] <- els[[x]]
    .rstanvis_defaults$hist_theme[[x]] <- els[[x]]
    .rstanvis_defaults$multiparam_theme[[x]] <- els[[x]]
  }
}



# internal ----------------------------------------------------------------

.rstanvis_defaults <- new.env(parent = emptyenv())

rstanvis_theme <- function() .rstanvis_defaults$theme
rstanvis_hist_theme <- function() .rstanvis_defaults$hist_theme
rstanvis_multiparam_theme <- function() .rstanvis_defaults$multiparam_theme
rstanvis_aes_ops <- function(nm = NULL) {
  if (is.null(nm)) {
    return(.rstanvis_defaults$aes_ops)
  } else {
    return(.rstanvis_defaults$aes_ops[[nm]])
  }
}

# called in .onLoad()
set_rstan_ggplot_defaults <- function() {
  .rstanvis_defaults$theme <- ggplot2::theme_classic(base_size = 11) +
    ggplot2::theme(
      axis.line = ggplot2::element_line(color = "#222222"),
      axis.line.y = ggplot2::element_line(linewidth = .5),
      axis.line.x = ggplot2::element_line(linewidth = 1),
      axis.title = ggplot2::element_text(face = "bold", size = 13),
      strip.background = ggplot2::element_blank(),
      strip.text = ggplot2::element_text(color = "black", face = "bold"),
      legend.title = ggplot2::element_text(size = 11),
      plot.title = ggplot2::element_text(size = 18)
    )

  for (j in seq_along(.rstanvis_defaults$theme)) {
    if ("element_text" %in% class(.rstanvis_defaults$theme[[j]])) {
      .rstanvis_defaults$theme[[j]][["debug"]] <- FALSE
    }
  }

  .rstanvis_defaults$hist_theme <-
    .rstanvis_defaults$theme +
    ggplot2::theme(
      axis.ticks.y = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      axis.line.y = ggplot2::element_blank()
    )

  .rstanvis_defaults$multiparam_theme <-
    .rstanvis_defaults$theme +
    theme(
      axis.title = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_text(face = "bold", size = 13, debug = FALSE),
      legend.position = "none",
      panel.grid.major = ggplot2::element_line(linewidth = 0.1, color = "darkgray")
    )

  .rstanvis_defaults$aes_ops <- list()
  .rstanvis_defaults$aes_ops$pt_color <- "#B2001D"
  .rstanvis_defaults$aes_ops$alpha <- 0.5
  .rstanvis_defaults$aes_ops$shape <- 19
  .rstanvis_defaults$aes_ops$fill <- "#B2001D"
  .rstanvis_defaults$aes_ops$color <- "black" # "#590815"
  .rstanvis_defaults$aes_ops$size <- 3
  .rstanvis_defaults$aes_ops$linewidth <- 0.5
  .rstanvis_defaults$aes_ops$pt_size <- 3
  .rstanvis_defaults$aes_ops$chain_colors <-
    rgb(
      matrix(
        c(
          230, 97, 1,
          153, 142, 195,
          84, 39, 136,
          241, 163, 64,
          216, 218, 235,
          254, 224, 182
        ),
        byrow = TRUE,
        ncol = 3
      ),
      names = paste(1:6),
      maxColorValue = 255
    )

  .rstanvis_defaults$aes_ops$grays <-
    rgb(
      matrix(
        c(
          247, 247, 247, 204, 204, 204,
          150, 150, 150, 82, 82, 82
        ),
        byrow = TRUE,
        ncol = 3
      ),
      alpha = 100,
      names = paste(1:4),
      maxColorValue = 255
    )
}

# suppress messages from ggplot2
quietgg <- function(gg) {
  suppressMessages(suppressWarnings(print(gg)))
  invisible(gg)
}

# traceplot ---------------------------------------------------------------
stan_trace <- function(object, pars, include = TRUE,
                       unconstrain = FALSE,
                       inc_warmup = FALSE,
                       nrow = NULL, ncol = NULL,
                       ..., window = NULL) {

  .check_object(object, unconstrain)
  plot_data <- .make_plot_data(object, pars, include, inc_warmup, unconstrain)

  thm <- rstanvis_theme()
  clrs <- rep_len(rstanvis_aes_ops("chain_colors"), plot_data$nchains)

  base <-
    ggplot2::ggplot(
      plot_data$samp,
      ggplot2::aes(x = .data$iteration, y = .data$value, color = .data$chain)
    )
  if (inc_warmup) base <- base +
    ggplot2::annotate("rect", xmin = -Inf, xmax = plot_data$warmup,
             ymin = -Inf, ymax = Inf, fill = rstanvis_aes_ops("grays")[2L])

  graph <-
    base +
    ggplot2::geom_path(...) +
    ggplot2::scale_color_manual(values = clrs) +
    ggplot2::labs(x="",y="") +
    thm

  if (plot_data$nparams == 1)
    graph <- graph + ggplot2::ylab(unique(plot_data$samp$parameter))
  else
    graph <- graph + ggplot2::facet_wrap(~parameter, nrow = nrow, ncol = ncol, scales = "free")

  if (!is.null(window)) {
    if (!is.numeric(window) || length(window) != 2)
      stop("'window' should be a numeric vector of length 2.")
    graph <- graph + ggplot2::coord_cartesian(xlim = window)
  }
  graph
}


# scatterplot -------------------------------------------------------------
stan_scat <- function(object, pars, unconstrain = FALSE, inc_warmup = FALSE,
                      nrow = NULL, ncol = NULL, ...) {

  .check_object(object, unconstrain)
  thm <- rstanvis_theme()
  dots <- .add_aesthetics(list(...), c("fill", "pt_color", "pt_size", "alpha", "shape"))
  if (missing(pars) || length(pars) != 2L)
    stop("'pars' must contain exactly two parameter names", call. = FALSE)
#   ndivergent <-
#     .sampler_params_post_warmup(object, "divergent__", as.df = TRUE)[, -1L]
#   treedepth <-
#     .sampler_params_post_warmup(object, "treedepth__", as.df = TRUE)[, -1L]
#   max_td <- .max_td(object)
#   div <- unname(rowSums(ndivergent) == 1)
#   hit_max_td <- sapply(1:nrow(treedepth), function(i) any(treedepth[i,] >= max_td))
  plot_data <- .make_plot_data(
    object,
    pars = pars,
    include = TRUE,
    inc_warmup = inc_warmup,
    unconstrain = unconstrain
  )
  p1 <- plot_data$samp$parameter == pars[1]
  val1 <- plot_data$samp[p1, "value"]
  val2 <- plot_data$samp[!p1, "value"]
  df <- data.frame(x = val1, y = val2)
#   nchains <- plot_data$nchains
#   sel <- seq_len(nrow(df) / nchains)
#   div <- df[div[sel], ]
#   td <- df[hit_max_td[sel], ]
  base <- ggplot2::ggplot(df, ggplot2::aes(x = .data$x, y = .data$y))
  graph <-
    base +
    do.call(ggplot2::geom_point, dots) +
#     geom_point(data = div, aes(x = .data$x, y = .data$y), color = "red") +
#     geom_point(data = td, aes(x = .data$x, y = .data$y), color = "yellow") +
    ggplot2::labs(x = pars[1], y = pars[2]) +
    thm

  graph
}


# histograms ---------------------------------------------------------------
stan_hist <- function(object, pars, include = TRUE,
                      unconstrain = FALSE,
                      inc_warmup = FALSE,
                      nrow = NULL, ncol = NULL,
                      ...) {

  .check_object(object, unconstrain)
  dots <- .add_aesthetics(list(...), c("fill", "color"))
  plot_data <- .make_plot_data(object, pars, include, inc_warmup, unconstrain)
  thm <- rstanvis_hist_theme()
  base <- ggplot2::ggplot(plot_data$samp, ggplot2::aes(x = .data$value, y = ggplot2::after_stat(.data$density)))
  graph <-
    base +
    do.call(ggplot2::geom_histogram, dots) +
    ggplot2::xlab("") +
    thm

  if (plot_data$nparams == 1)
    graph + ggplot2::xlab(unique(plot_data$samp$parameter))
  else
    graph + ggplot2::facet_wrap(~parameter, nrow = nrow, ncol = ncol, scales = "free")
}



# densities -----------------------------------------------------------
stan_dens <- function(object, pars, include = TRUE,
                      unconstrain = FALSE,
                      inc_warmup = FALSE,
                      nrow = NULL, ncol = NULL,
                      ...,
                      separate_chains = FALSE) {

  .check_object(object, unconstrain)
  plot_data <- .make_plot_data(object, pars, include, inc_warmup, unconstrain)
  clrs <- rep_len(rstanvis_aes_ops("chain_colors"), plot_data$nchains)
  thm <- rstanvis_hist_theme()
  base <- ggplot2::ggplot(plot_data$samp, ggplot2::aes(x = .data$value)) + ggplot2::xlab("")
  if (!separate_chains) {
    dots <- .add_aesthetics(list(...), c("fill", "color"))
    graph <-
      base +
      do.call(ggplot2::geom_density, dots) +
      thm
  } else {
    dots <- .add_aesthetics(list(...), c("color", "alpha"))
    dots$mapping <- ggplot2::aes(fill = .data$chain)
    graph <-
      base +
      do.call(ggplot2::geom_density, dots) +
      ggplot2::scale_fill_manual(values = clrs) +
      thm
  }
  if (plot_data$nparams == 1)
    graph + ggplot2::xlab(unique(plot_data$samp$parameter))
  else
    graph + ggplot2::facet_wrap(~parameter, nrow = nrow, ncol = ncol, scales = "free")
}



# autocorrelation ---------------------------------------------------------
stan_ac <- function(object, pars, include = TRUE,
                    unconstrain = FALSE,
                    inc_warmup = FALSE,
                    nrow = NULL, ncol = NULL,
                    ...,
                    separate_chains = FALSE,
                    lags = 25, partial = FALSE) {
  .check_object(object, unconstrain)
  plot_data <- .make_plot_data(object, pars, include, inc_warmup, unconstrain)
  clrs <- rep_len(rstanvis_aes_ops("chain_colors"), plot_data$nchains)
  thm <- rstanvis_theme()
  dots <- .add_aesthetics(list(...), c("size", "color", "fill"))

  dat_args <- list(dat = plot_data$samp, lags = lags, partial = partial)
  dat_fn <- ifelse(plot_data$nparams == 1, ".ac_plot_data",".ac_plot_data_multi")
  ac_dat <- do.call(dat_fn, dat_args)
  if (!separate_chains) {
    dots$position <- "dodge"
    dots$stat <- "summary"
    dots$fun <- "mean"
    dots$fun.data <- "mean_se"
    y_lab <- paste("Avg.", if (partial) "partial", "autocorrelation")
    ac_labs <- ggplot2::labs(x = "Lag", y = y_lab)
    y_scale <- ggplot2::scale_y_continuous(breaks = seq(0, 1, 0.25))
    base <- ggplot2::ggplot(ac_dat, ggplot2::aes(x = .data$lag, y = .data$ac))
    graph <- base +
      do.call(ggplot2::geom_bar, dots) +
      y_scale +
      ac_labs +
      thm

    if (plot_data$nparams == 1) {
      y_lab <- ggplot2::ylab(paste0(y_lab, " (", pars,")"))
      return(graph + y_lab)
    }
    else return(graph + ggplot2::facet_wrap(~parameters, nrow = nrow, ncol = ncol,
                                   scales = "free_x"))
  }
  dots$position <- "identity"
  dots$stat <- "identity"
  ac_labs <- ggplot2::labs(x = "Lag", y = if (partial)
    "Partial autocorrelation" else "Autocorrelation")
  y_scale <- ggplot2::scale_y_continuous(breaks = seq(0, 1, 0.25),
                                labels = c("0","","0.5","",""))

  base <- ggplot2::ggplot(ac_dat, ggplot2::aes(x = .data$lag, y = .data$ac))
  graph <- base +
    do.call(ggplot2::geom_bar, dots) +
    y_scale +
    ac_labs +
    thm

  if (plot_data$nparams == 1) {
    graph <- graph + ggplot2::facet_wrap(~chains, nrow = nrow, ncol = ncol)
    return(graph)
  } else { # nParams > 1
    graph <- graph + ggplot2::facet_grid(parameters ~ chains, scales = "free_x")
    return(graph)
  }
}



# parameter estimates -----------------------------------------------------
stan_plot <- function(object, pars, include = TRUE, unconstrain = FALSE,
                      ...) {

  inc_warmup <- FALSE
  .check_object(object, unconstrain)
  thm <- rstanvis_multiparam_theme()
  plot_data <- .make_plot_data(object, pars, include, inc_warmup, unconstrain)

  color_by_rhat <- FALSE # FIXME
  dots <- list(...)
  defs <- list(point_est = "median", show_density = FALSE,
               show_outer_line = TRUE, ci_level = 0.8, outer_level = 0.95,
               fill_color = rstanvis_aes_ops("fill"),
               outline_color = rstanvis_aes_ops("color"),
               est_color = rstanvis_aes_ops("color"))
  args <- names(defs)
  dotenv <- list()
  for (j in seq_along(args)) {
    if (args[j] %in% names(dots))
      dotenv[[args[j]]] <- dots[[args[j]]]
    else dotenv[[args[j]]] <- defs[[j]]
  }
  if (!(dotenv[["point_est"]] %in% c("mean", "median")))
    stop("Point estimate should be either 'mean' or 'median'", call. = FALSE)
  if (color_by_rhat)
    stop("'color_by_rhat' not yet available", call. = FALSE)
  if (dotenv[["ci_level"]] > dotenv[["outer_level"]])
    stop("'ci_level' should be less than 'outer_level'", call. = FALSE)
  ci_level <- dotenv[["ci_level"]]
  outer_level <- dotenv[["outer_level"]]
  message("ci_level: ", ci_level," (",100 * ci_level, "% intervals)")
  message("outer_level: ", outer_level," (",100 * outer_level, "% intervals)")
  outer_level <- dotenv[["outer_level"]]
  probs.use <- c(0.5 - outer_level / 2, 0.5 - ci_level / 2, 0.5,
                 0.5 + ci_level / 2, 0.5 + outer_level / 2)
  samp <- plot_data$samp
  nparams <- plot_data$nparams
  statmat <- as.matrix(aggregate(samp$value, by = list(parameter = samp$parameter),
                                 FUN = function(x,...) c(mean(x), quantile(x,...)),
                                 probs = probs.use))
  param_names <- rownames(statmat) <- statmat[, 1L]
  statmat <- apply(statmat[, -1L, drop=FALSE], 1:2, as.numeric)
  colnames(statmat) <- c("mean", "2.5%", "25%", "50%", "75%", "97.5%")
  y <- as.numeric(seq(plot_data$nparams, 1, by = -1))
  xlim.use <- c(min(statmat[,2L]), max(statmat[,6L]))
  xlim.use <- xlim.use + diff(xlim.use) * c(-.05, .05)
  xy.df <- data.frame(params = rownames(statmat), y, statmat)
  colnames(xy.df) <- c("params", "y", "mean", "ll", "l", "m", "h", "hh")
  if (dotenv[["point_est"]] == "mean") xy.df$m <- xy.df$mean

  p.base <- ggplot2::ggplot(xy.df)
  p.name <- ggplot2::scale_y_continuous(breaks = y, labels = param_names,
                               limits = c(0.5, nparams + 1))
  p.all <- p.base + ggplot2::xlim(xlim.use) + p.name + thm
  show_density <- dotenv[["show_density"]]
  outline_color <- dotenv[["outline_color"]] %ifNULL% rstanvis_aes_ops("color")
  fill_color <- dotenv[["fill_color"]]
  est_color <- dotenv[["est_color"]]
  if (dotenv[["show_outer_line"]] || show_density) {
    p.ci <-
      ggplot2::geom_segment(
        mapping = ggplot2::aes(
          x = .data$ll,
          xend = .data$hh,
          y = .data$y,
          yend = .data$y
        ),
        color = outline_color
      )
    p.all <- p.all + p.ci
  }
  if (show_density) { # plot kernel density estimate
    npoint.den <- 512
    y.den <- x.den <- matrix(0, nrow = npoint.den, ncol = nparams)
    for(i in 1:nparams){
      d.temp <- density(samp[samp$parameter == param_names[i], "value"],
                        from = statmat[i,2L],
                        to = statmat[i,6L],
                        n = npoint.den)
      x.den[,i] <- d.temp$x
      y.max <- max(d.temp$y)
      y.den[,i] <- d.temp$y / y.max * 0.8 + y[i]
    }
    df.den <- data.frame(x = as.vector(x.den), y = as.vector(y.den),
                         name = rep(param_names, each = npoint.den))
    p.den <-
      ggplot2::geom_line(
        data = df.den,
        mapping = ggplot2::aes(x = .data$x, y = .data$y, group = .data$name),
        color = outline_color
      )

    #shaded interval
    y.poly <- x.poly <- matrix(0, nrow = npoint.den + 2, ncol = nparams)
    for(i in 1:nparams){
      d.temp <- density(samp[samp$parameter == param_names[i], "value"],
                        from = statmat[i, 3L],
                        to = statmat[i, 5L],
                        n = npoint.den)
      x.poly[,i] <- c(d.temp$x[1L], as.vector(d.temp$x), d.temp$x[npoint.den])
      y.max <- max(d.temp$y)
      y.poly[,i] <- as.vector(c(0, as.vector(d.temp$y) / y.max * 0.8, 0) + y[i])
    }
    df.poly <- data.frame(x = as.vector(x.poly), y = as.vector(y.poly),
                          name = rep(param_names, each = npoint.den + 2))
    p.poly <- ggplot2::geom_polygon(data = df.poly, mapping=ggplot2::aes(x = .data$x, y = .data$y, group = .data$name, fill = .data$y))
    p.col <- ggplot2::scale_fill_gradient(low = fill_color, high = fill_color, guide = "none")

    #point estimate
    if (color_by_rhat) {
      rhat_colors <- dotenv[["rhat_colors"]]
      p.point <- ggplot2::geom_segment(ggplot2::aes(x = .data$m, xend = .data$m, y = .data$y, yend = .data$y + 0.25,
                                         color = .data$rhat_id), linewidth = 1.5)
      p.all + p.poly + p.den + p.col + p.point + rhat_colors #+ rhat_lgnd
    } else {
      p.point <- ggplot2::geom_segment(ggplot2::aes(x = .data$m, xend = .data$m, y = .data$y, yend = .data$y + 0.25),
                              colour = est_color, linewidth = 1.5)
      p.all + p.poly + p.den + p.col + p.point
    }
  } else {
    p.ci.2 <- ggplot2::geom_segment(ggplot2::aes(x = .data$l, xend = .data$h, y = .data$y, yend = .data$y),
                           colour = fill_color, linewidth = 2)
    if (color_by_rhat) {
      p.point <- ggplot2::geom_point(ggplot2::aes(x = .data$m, y = .data$y, fill = .data$rhat_id),
                            color = "black", shape = 21, size = 4)
      p.all + p.ci.2 + p.point + rhat_colors # + rhat_lgnd
    } else {
      p.point <- ggplot2::geom_point(ggplot2::aes(x = .data$m, y = .data$y), size = 4,
                            color = fill_color, fill = est_color, shape = 21)
      p.all + p.ci.2 + p.point
    }
  }
}



# rhat, ess, mcse ---------------------------------------------------------
stan_rhat <- function(object, pars, ...) {
  .check_object(object)
  .vb_check(object)
  if (missing(pars)) pars <- NULL
  .rhat_neff_mcse_hist(which = "rhat", object = object, pars = pars, ...)
}

stan_ess <- function(object, pars, ...) {
  .check_object(object)
  .vb_check(object)
  if (missing(pars)) pars <- NULL
  .rhat_neff_mcse_hist(which = "n_eff_ratio", object = object, pars = pars, ...)
}

stan_mcse <- function(object, pars, ...) {
  .check_object(object)
  .vb_check(object)
  if (missing(pars)) pars <- NULL
  .rhat_neff_mcse_hist(which = "mcse_ratio", object = object, pars = pars, ...)
}



# NUTS --------------------------------------------------------------------
stan_diag <- function(object,
                      information = c("sample","stepsize","treedepth","divergence"),
                      chain = 0, ...) {
  .vb_check(object)
  if ("pars" %in% names(list(...)))
    stop("'stan_diag' does not accept a 'pars' argument.")
  nchains <- if (is.stanreg(object))
    ncol(object$stanfit) else ncol(object)
  if (!isTRUE(nchains > 1))
    stop("'stan_diag' requires more than one chain.", call. = FALSE)
  info <- match.arg(information)
  fn <- paste0("stan_", info)
  do.call(fn, list(object, chain, ...))
}

stan_stepsize <- function(object, chain = 0, ...) {
  .nuts_args_check(...)
  thm <- rstanvis_theme()
  stepsize <- .sampler_params_post_warmup(object, "stepsize__", as.df = TRUE)
  lp <- extract(if (is.stanreg(object)) object$stanfit else object,
                       pars = "lp__", permuted = FALSE)[,,1L]

  graphs <- list()
  graphs$stepsize_vs_lp <- .sampler_param_vs_param(p = lp, sp = stepsize[,-1L],
                                                   p_lab = .LP_LAB,
                                                   sp_lab = .STEPSIZE_LAB,
                                                   chain = chain, violin = TRUE)

  metrop <- .sampler_params_post_warmup(object, "accept_stat__", as.df = TRUE)
  graphs$stepsize_vs_metrop <-
    .sampler_param_vs_sampler_param_violin(round(stepsize[,-1L], 4), metrop[,-1L],
                                           lab_x = .STEPSIZE_LAB,
                                           lab_y = .METROP_LAB,
                                           chain = chain)
  graphs <- lapply(graphs, function(x) x + thm)
  .nuts_return(graphs, ...)
}

stan_sample <- function(object, chain = 0, ...) {
  .nuts_args_check(...)
  thm <- rstanvis_theme()
  hist_thm <- rstanvis_hist_theme()
  lp <- extract(if (is.stanreg(object)) object$stanfit else object,
                       pars = "lp__", permuted = FALSE)[,,1L]
  lp_df <- as.data.frame(cbind(iterations = 1:nrow(lp), lp))
  metrop <- .sampler_params_post_warmup(object, "accept_stat__", as.df = TRUE)
  graphs <- list()
  graphs$lp_hist <-
    .p_hist(lp_df, lab = .LP_LAB, chain = chain)
  graphs$metrop_hist <-
    .p_hist(metrop, lab = .METROP_LAB, chain = chain) + xlim(0,1)
  graphs <- lapply(graphs, function(x) x + thm)
  graphs$metrop_vs_lp <-
    .sampler_param_vs_param(p = lp, sp = metrop[,-1L], p_lab = .LP_LAB,
                            sp_lab = .METROP_LAB, chain = chain) +
    thm

  .nuts_return(graphs, ...)
}


stan_treedepth <- function(object, chain = 0, ...) {
  .nuts_args_check(...)
  thm <- rstanvis_theme()
  hist_thm <- rstanvis_hist_theme()
  lp <- extract(if (is.stanreg(object)) object$stanfit else object,
                       pars = "lp__", permuted = FALSE)[,,1L]
  treedepth <- .sampler_params_post_warmup(object, "treedepth__", as.df = TRUE)
  ndivergent <- .sampler_params_post_warmup(object, "divergent__", as.df = TRUE)
  metrop <- .sampler_params_post_warmup(object, "accept_stat__", as.df = TRUE)

  graphs <- graphs_nd <- list()

  graphs$treedepth_vs_lp <-
    .sampler_param_vs_param(p = lp, sp = treedepth[, -1L],
                            p_lab = .LP_LAB,
                            sp_lab = .TREEDEPTH_LAB,
                            chain = chain, violin = TRUE)
  graphs$treedepth_vs_metrop <-
    .sampler_param_vs_sampler_param_violin(treedepth[,-1L], metrop[,-1L],
                                           lab_x = .TREEDEPTH_LAB,
                                           lab_y = .METROP_LAB,
                                           chain = chain)

  graphs_nd$treedepth_ndivergent <-
    .treedepth_ndivergent_hist(treedepth, ndivergent, chain = chain,
                               divergent = "All")

  any_nd <- any(ndivergent[,-1L] != 0)
  if (any_nd) {
    graphs_nd$treedepth_ndivergent0 <-
      .treedepth_ndivergent_hist(treedepth, ndivergent, chain = chain,
                                 divergent = 0)
    graphs_nd$treedepth_ndivergent1 <-
      .treedepth_ndivergent_hist(treedepth, ndivergent, chain = chain,
                                 divergent = 1)
  }
  graphs <- lapply(graphs, function(x) x + thm)
  graphs_nd <- lapply(graphs_nd, function(x) x + hist_thm)
  graphs <- c(graphs, graphs_nd)
  .nuts_return(graphs, ...)
}


stan_divergence <- function(object, chain = 0, ...) {
  .nuts_args_check(...)
  thm <- rstanvis_theme()
  lp <- extract(if (is.stanreg(object)) object$stanfit else object,
                       pars = "lp__", permuted = FALSE)[,,1L]
  ndivergent <- .sampler_params_post_warmup(object, "divergent__", as.df = TRUE)
  metrop <- .sampler_params_post_warmup(object, "accept_stat__", as.df = TRUE)
  graphs <- list()
  graphs$ndivergent_vs_lp <-
    .sampler_param_vs_param(p = lp, sp = ndivergent[, -1L],
                            p_lab = .LP_LAB,
                            sp_lab = .NDIVERGENT_LAB,
                            chain = chain, violin = TRUE)
  graphs$ndivergent_vs_metrop <-
    .sampler_param_vs_sampler_param_violin(ndivergent[,-1L], metrop[,-1L],
                                           lab_x = .NDIVERGENT_LAB,
                                           lab_y = .METROP_LAB,
                                           chain = chain)
  graphs <- lapply(graphs, function(x) x + thm)
  .nuts_return(graphs, ...)
}


stan_par <- function(object, par, chain = 0, ...) {
  if (missing(par))
    stop("'par' must be specified", call. = FALSE)
  if (is.stanreg(object))
    object <- object$stanfit
  if (!isTRUE(ncol(object) > 1))
    stop("'stan_par' requires more than one chain.", call. = FALSE)
  thm <- rstanvis_theme()
  samp <- extract(object, pars = c("lp__", par), permuted = FALSE)
  par_sel <- which(dimnames(samp)$parameters == par)

  cntrl <- object@stan_args[[1L]]$control
  if (is.null(cntrl))
    max_td <- 11
  else {
    max_td <- cntrl$max_treedepth
    if (is.null(max_td))
      max_td <- 10
  }
  max_td <- .max_td(object)
  metrop <- .sampler_params_post_warmup(object, "accept_stat__", as.df = TRUE)[,-1L]
  stepsize <- .sampler_params_post_warmup(object, "stepsize__", as.df = TRUE)[,-1L]
  ndivergent <- .sampler_params_post_warmup(object, "divergent__", as.df = TRUE)[,-1L]
  treedepth <- .sampler_params_post_warmup(object, "treedepth__", as.df = TRUE)[,-1L]
  hit_max_td <- apply(treedepth, 2L, function(y) as.numeric(y >= max_td))
  graphs <- list()
  par_samp <- samp[,, par_sel]
  lp <- samp[,, -par_sel]
  graphs[[paste0(par,"_vs_lp")]] <-
    .sampler_param_vs_param(sp = as.data.frame(lp),
                            p = par_samp,
                            divergent = ndivergent,
                            hit_max_td = as.data.frame(hit_max_td),
                            sp_lab = .LP_LAB, p_lab = par,
                            chain = chain)
  graphs[[paste0(par,"_vs_metrop")]] <-
    .sampler_param_vs_param(p = par_samp, sp = metrop,
                            divergent = ndivergent,
                            hit_max_td = as.data.frame(hit_max_td),
                            p_lab = par, sp_lab = .METROP_LAB,
                            chain = chain)
  graphs[[paste0(par,"_vs_stepsize")]] <-
    .sampler_param_vs_param(p = par_samp, sp = stepsize,
                            p_lab = par, sp_lab = .STEPSIZE_LAB,
                            chain = chain, violin = TRUE)

  graphs <- lapply(graphs, function(x) x + thm)
  .nuts_return(graphs, ...)
}

exposeStanClass <- function(stanc_ret,
                            fields = character(),
                            field_access = c("none", "read_only", "read_write"),
                            where = tempdir(),
                            compile = interactive(), 
                            ...) {
  field_access <- match.arg(field_access)
  if (field_access != "none" && length(fields) == 0L) {
    public <- grep("^public:", stanc_ret$cppcode)
    if (length(public) == 1L) public <- c(grep("^private:", stanc_ret$cppcode), public)
    if (diff(public) > 1L) {
      public[1] <- public[1] + 1L
      public[2] <- public[2] - 1L
      fields <- sub("^.* (.*);$", "\\1", stanc_ret$cppcode[public[1]:public[2]])
    } else fields <- NULL
  }
  if (identical("src", where)) {
    tf <- paste0(stanc_ret$model_cppname, "Module.cc")
    header <- paste0(stanc_ret$model_cppname, ".hpp")
  } else {
    tf <- tempfile(tmpdir = where, fileext = "Module.cc")
    header <- sub("Module\\.cc$", ".hpp", tf)
  }
  
  if (any(grepl("ctor_body", stanc_ret$cppcode))) {
    ctor <- list("rstan::io::rlist_ref_var_context")
  } else ctor <- list(c("SEXP", "SEXP", "SEXP"))
  
  writeLines(stanc_ret$cppcode, con = header)
  meth <- names(stanc_ret$methods)
  Rmeth <- meth
  names(Rmeth) <- sub("_$", "", names(stanc_ret$methods))
  
  FQ <- options()$useFancyQuotes
  options(useFancyQuotes = FALSE)
  on.exit(options(useFancyQuotes = FQ))
  
  Rcpp::exposeClass(class = stanc_ret$model_name,
                    constructors = ctor,
                    fields = fields,
                    methods = meth,
                    file = tf,
                    header = c("// [[Rcpp::depends(rstan)]]",
                               "// [[Rcpp::plugins(cpp14)]]",
                               "#include <RcppEigen.h>",
                               paste0('#include "', header, '"')),
                    CppClass = "stan_model",
                    readOnly = if (field_access == "read_only") fields else character(),
                    rename = Rmeth,
                    Rfile = FALSE)
  if (compile) {
    Rcpp::sourceCpp(file = tf, ...)
    # Rcpp::loadRcppClass(Class = stanc_ret$model_name,
    #                     module = paste0("class_", stanc_ret$model_name),
    #                     where = if (interactive()) .GlobalEnv)
    g <- get(stanc_ret$model_name, envir = .GlobalEnv)@generator
    return(g)
  }
  return(invisible(NULL))
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2023 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(dplyr)
library(stringr)

# created via: ./stanc --dump-stan-math-signatures
signatures <- file.path("tools", "signatures.txt")
functions <- file.path("tools", "stan-functions.txt")

system2("sed", args = paste("'s/((.*=> vector,/(vector_function,/g'", signatures),
        stdout = functions, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i 's/((.*=> real,/(real_function,/g'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i 's/((.*=> array\\[\\] real,/(real_array_function,/g'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i 's/=> tuple(/=> tuple{/g'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i 's/ => /;/g'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i 's/(/;(/g'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i '1iStanFunction;Arguments;ReturnType'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)
system2("sed", args = paste("-i '1i# This file is semicolon delimited'", functions),
        stdout = NULL, stderr = NULL)
stopifnot(.Last.value == 0)

rosetta <- cbind(RFunction = NA_character_,
                 read.table(functions, header = TRUE, sep = ";", 
                            quote = NULL, stringsAsFactors = FALSE, strip.white = TRUE)) %>% 
  mutate(RFunction = if_else(StanFunction %in% unlist(sapply(search(), ls)),
                             StanFunction, NA_character_),
         templated = str_detect(Arguments, fixed(ReturnType)),
         Arguments = case_when(templated & paste0("(", ReturnType, ")") == Arguments ~ "(T)",
                               templated ~ str_replace(Arguments, 
                                                       fixed(paste0(", ", ReturnType, ")")),
                                                       ", T)"),
                               TRUE ~ Arguments),
         ReturnType = if_else(str_ends(Arguments, fixed("T)")), "T", ReturnType)) %>% 
  group_by(StanFunction, templated) %>% 
  slice_head(n = 1) %>% 
  ungroup %>% 
  arrange(str_to_lower(StanFunction), desc(templated)) %>% 
  group_by(StanFunction, RFunction) %>% 
  summarize(Arguments = paste(Arguments, collapse = ";"), 
            ReturnType = paste(ReturnType, collapse = ";"),
            .groups = "drop") %>% 
  arrange(str_to_lower(StanFunction)) %>% 
  as.data.frame

rosetta <- within(rosetta, RFunction[StanFunction == "add"] <- "+")  
rosetta <- within(rosetta, RFunction[grepl("algebra_solve", StanFunction)] <- "uniroot")
rosetta <- within(rosetta, RFunction[StanFunction == "append_array"] <- "c")
rosetta <- within(rosetta, RFunction[StanFunction == "append_col"] <- "cbind")  
rosetta <- within(rosetta, RFunction[StanFunction == "append_row"] <- "rbind")
rosetta <- within(rosetta, RFunction[StanFunction == "arg"] <- "Arg")  
rosetta <- within(rosetta, RFunction[grepl("^bernoulli_.*cdf", StanFunction)] <- "pbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "bernoulli"] <- "dbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "bernoulli_logit"] <- "dbinom")
rosetta <- within(rosetta, RFunction[grepl("^bernoulli_.*pmf", StanFunction)] <- "dbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "bernoulli_rng"] <- "rbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "bessel_first_kind"] <- "besselJ")
rosetta <- within(rosetta, RFunction[StanFunction == "bessel_second_kind"] <- "besselY")
rosetta <- within(rosetta, RFunction[StanFunction == "beta"] <- "dbeta")
rosetta <- within(rosetta, RFunction[grepl("^beta_[lc]*cdf", StanFunction)] <- "pbeta")
rosetta <- within(rosetta, RFunction[StanFunction == "beta_lpdf"] <- "dbeta")
rosetta <- within(rosetta, RFunction[StanFunction == "beta_rng"] <- "rbeta")
rosetta <- within(rosetta, RFunction[grepl("^binomial_[lc]*cdf", StanFunction)] <- "pbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "binomial_coefficient_log"] <- "lchoose")
rosetta <- within(rosetta, RFunction[grepl("^binomial_logit", StanFunction)] <- "binomial")
rosetta <- within(rosetta, RFunction[StanFunction == "binomial"] <- "dbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "binomial_lpmf"] <- "dbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "binomial_rng"] <- "rbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "block"] <- "subset")
rosetta <- within(rosetta, RFunction[grepl("^categorical", StanFunction)] <- "dmultinom")
rosetta <- within(rosetta, RFunction[StanFunction == "categorical_rng"] <- "rmultinom")
rosetta <- within(rosetta, RFunction[grepl("^cauchy[_lpdf]*", StanFunction)] <- "dcauchy")
rosetta <- within(rosetta, RFunction[grepl("^cauchy_[lc]*cdf", StanFunction)] <- "pcauchy")
rosetta <- within(rosetta, RFunction[StanFunction == "cauchy_rng"] <- "rcauchy")
rosetta <- within(rosetta, RFunction[StanFunction == "ceil"] <- "ceiling")
rosetta <- within(rosetta, RFunction[grepl("^chi_square[_lpdf]*", StanFunction)] <- "dchisq")
rosetta <- within(rosetta, RFunction[grepl("^chi_square_[lc]*cdf", StanFunction)] <- "pchisq")
rosetta <- within(rosetta, RFunction[StanFunction == "chi_square_rng"] <- "rchisq")
rosetta <- within(rosetta, RFunction[StanFunction == "cholesky_decompose"] <- "chol")
rosetta <- within(rosetta, RFunction[StanFunction == "choose"] <- "choose")
rosetta <- within(rosetta, RFunction[StanFunction == "col"] <- "subset")
rosetta <- within(rosetta, RFunction[StanFunction == "cols"] <- "NCOL")
rosetta <- within(rosetta, RFunction[grepl("^columns_", StanFunction)] <- "apply")
rosetta <- within(rosetta, RFunction[StanFunction == "conj"] <- "Conj")
rosetta <- within(rosetta, RFunction[StanFunction == "cumulative_sum"] <- "cumsum")
rosetta <- within(rosetta, RFunction[StanFunction == "determinant"] <- "det")
rosetta <- within(rosetta, RFunction[StanFunction == "diag_matrix"] <- "diag")
rosetta <- within(rosetta, RFunction[StanFunction == "diagonal"] <- "diag")
rosetta <- within(rosetta, RFunction[StanFunction == "dims"] <- "dim")
rosetta <- within(rosetta, RFunction[StanFunction == "distance"] <- "dist")
rosetta <- within(rosetta, RFunction[StanFunction == "divide"] <- "/")
rosetta <- within(rosetta, RFunction[StanFunction == "dot_self"] <- "crossprod")
rosetta <- within(rosetta, RFunction[StanFunction == "dot_product"] <- "%*%")
rosetta <- within(rosetta, RFunction[StanFunction == "e"] <- "exp")
rosetta <- within(rosetta, RFunction[grepl("^eigen", StanFunction)] <- "eigen")
rosetta <- within(rosetta, RFunction[grepl("^erf", StanFunction)] <- "pnorm")
rosetta <- within(rosetta, RFunction[grepl("^exponential[_lpdf]*", StanFunction)] <- "dexp")
rosetta <- within(rosetta, RFunction[grepl("^exponential_[lc]*cdf", StanFunction)] <- "pexp")
rosetta <- within(rosetta, RFunction[StanFunction == "exponential_rng"] <- "rexp")
rosetta <- within(rosetta, RFunction[StanFunction == "fabs"] <- "abs")
rosetta <- within(rosetta, RFunction[StanFunction == "fmax"] <- "max")
rosetta <- within(rosetta, RFunction[StanFunction == "fmin"] <- "min")
rosetta <- within(rosetta, RFunction[StanFunction == "fmod"] <- "%%")
rosetta <- within(rosetta, RFunction[grepl("^gamma[_lpdf]*", StanFunction)] <- "dgamma")
rosetta <- within(rosetta, RFunction[grepl("^gamma_[lc]*cdf", StanFunction)] <- "pgamma")
rosetta <- within(rosetta, RFunction[StanFunction == "gamma_rng"] <- "rgamma")
rosetta <- within(rosetta, RFunction[grepl("^gamma_[pq]$", StanFunction)] <- "pgamma")
rosetta <- within(rosetta, RFunction[StanFunction == "generalized_inverse"] <- "MASS::ginv")
rosetta <- within(rosetta, RFunction[grepl("^hypergeometric[_lpmf]*", StanFunction)] <- "dhyper")
rosetta <- within(rosetta, RFunction[grepl("^hypergeometric_[lc]*cdf", StanFunction)] <- "phyper")
rosetta <- within(rosetta, RFunction[StanFunction == "hypergeometric_rng"] <- "rhyper")
rosetta <- within(rosetta, RFunction[StanFunction == "if_else"] <- "ifelse")
rosetta <- within(rosetta, RFunction[StanFunction == "integrate_1d"] <- "integrate")
rosetta <- within(rosetta, RFunction[StanFunction == "inverse"] <- "solve")
rosetta <- within(rosetta, RFunction[StanFunction == "inverse_spd"] <- "solve")
rosetta <- within(rosetta, RFunction[StanFunction == "inv_logit"] <- "plogis")
rosetta <- within(rosetta, RFunction[StanFunction == "inv_Phi"] <- "qnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "is_inf"] <- "is.finite")
rosetta <- within(rosetta, RFunction[StanFunction == "is_nan"] <- "is.nan")
rosetta <- within(rosetta, RFunction[StanFunction == "lgamma"] <- "lgamma")
rosetta <- within(rosetta, RFunction[StanFunction == "log1m"] <- "log1p")
rosetta <- within(rosetta, RFunction[StanFunction == "log_determinant"] <- "determinant")
rosetta <- within(rosetta, RFunction[StanFunction == "log_inv_logit"] <- "plogis")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_and"] <- "&&")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_eq"] <- "==")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_gt"] <- ">")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_gte"] <- ">=")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_lt"] <- "<")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_lte"] <- "<=")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_negation"] <- "!")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_neq"] <- "!=")
rosetta <- within(rosetta, RFunction[StanFunction == "logical_or"] <- "||")
rosetta <- within(rosetta, RFunction[grepl("^logistic[_lpdf]*", StanFunction)] <- "dlogis")
rosetta <- within(rosetta, RFunction[grepl("^logistic_[lc]*cdf", StanFunction)] <- "plogis")
rosetta <- within(rosetta, RFunction[StanFunction == "logistic_rng"] <- "rlogis")
rosetta <- within(rosetta, RFunction[StanFunction == "logit"] <- "qlogis")
rosetta <- within(rosetta, RFunction[grepl("^lognormal[_lpdf]*", StanFunction)] <- "dlnorm")
rosetta <- within(rosetta, RFunction[grepl("^lognormal_[lc]*cdf", StanFunction)] <- "plnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "lognormal_rng"] <- "rlnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "machine_precision"] <- ".Machine")
rosetta <- within(rosetta, RFunction[StanFunction == "map_rect"] <- "vapply")
rosetta <- within(rosetta, RFunction[StanFunction == "mdivide_left"] <- "solve")
rosetta <- within(rosetta, RFunction[StanFunction == "mdivide_left_spd"] <- "solve")
rosetta <- within(rosetta, RFunction[grepl("^mdivide_left_tri_low", StanFunction)] <- "forwardsolve")
rosetta <- within(rosetta, RFunction[StanFunction == "minus"] <- "-")
rosetta <- within(rosetta, RFunction[StanFunction == "modified_bessel_first_kind"] <- "besselI")
rosetta <- within(rosetta, RFunction[StanFunction == "modified_bessel_second_kind"] <- "besselK")
rosetta <- within(rosetta, RFunction[StanFunction == "modulus"] <- "Mod")
rosetta <- within(rosetta, RFunction[grepl("^multinomial[_lpmf]*", StanFunction)] <- "dmultinom")
rosetta <- within(rosetta, RFunction[grepl("^multinomial_[lc]*cdf", StanFunction)] <- "pmultinom")
rosetta <- within(rosetta, RFunction[StanFunction == "multinomial_rng"] <- "rmultinom")
rosetta <- within(rosetta, RFunction[grepl("^multi_normal", StanFunction)] <- "mvtnorm::dmvnorm")
rosetta <- within(rosetta, RFunction[grepl("^multi_normal.*_rng$", StanFunction)] <- "mvtnorm::rmvnorm")
rosetta <- within(rosetta, RFunction[grepl("^multi_student", StanFunction)] <- "mvtnorm::dmvt")
rosetta <- within(rosetta, RFunction[grepl("^multi_student.*_rng$", StanFunction)] <- "mvtnorm::rmvt")
rosetta <- within(rosetta, RFunction[StanFunction == "multiply_lower_tri_self_transpose"] <- "crossprod")
rosetta <- within(rosetta, RFunction[StanFunction == "negative_infinity"] <- "Inf")
rosetta <- within(rosetta, RFunction[grepl("^neg_binomial[_lpmf]*", StanFunction)] <- "dnbinom")
rosetta <- within(rosetta, RFunction[grepl("^neg_binomial_[lc]*cdf", StanFunction)] <- "pnbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "neg_binomial_rng"] <- "rnbinom")
rosetta <- within(rosetta, RFunction[grepl("^neg_binomial_2[_lpmf]*", StanFunction)] <- "dnbinom")
rosetta <- within(rosetta, RFunction[grepl("^neg_binomial_2_[lc]*cdf", StanFunction)] <- "pnbinom")
rosetta <- within(rosetta, RFunction[StanFunction == "neg_binomial_2_rng"] <- "rnbinom")
rosetta <- within(rosetta, RFunction[grepl("^normal[_lpdf]*", StanFunction)] <- "dnorm")
rosetta <- within(rosetta, RFunction[grepl("^normal_[lc]*cdf", StanFunction)] <- "pnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "normal_rng"] <- "rnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "not_a_number"] <- "NaN")
rosetta <- within(rosetta, RFunction[StanFunction == "num_elements"] <- "length")
rosetta <- within(rosetta, RFunction[StanFunction == "Phi"] <- "pnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "Phi_approx"] <- "pnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "plus"] <- "+")
rosetta <- within(rosetta, RFunction[grepl("^poisson[_lpmf]*", StanFunction)] <- "dpois")
rosetta <- within(rosetta, RFunction[grepl("^poisson_[lc]*cdf", StanFunction)] <- "ppois")
rosetta <- within(rosetta, RFunction[StanFunction == "poisson_rng"] <- "rpois")
rosetta <- within(rosetta, RFunction[StanFunction == "positive_infinity"] <- "Inf")
rosetta <- within(rosetta, RFunction[StanFunction == "pow"] <- "^")
rosetta <- within(rosetta, RFunction[StanFunction == "qr_thin_Q"] <- "qr.Q")
rosetta <- within(rosetta, RFunction[StanFunction == "qr_thin_R"] <- "qr.R")
rosetta <- within(rosetta, RFunction[StanFunction == "rep_array"] <- "array")
rosetta <- within(rosetta, RFunction[StanFunction == "rep_matrix"] <- "matrix")
rosetta <- within(rosetta, RFunction[grepl("^rep_.*vector", StanFunction)] <- "rep")
rosetta <- within(rosetta, RFunction[StanFunction == "reverse"] <- "rev")
rosetta <- within(rosetta, RFunction[StanFunction == "row"] <- "subset")
rosetta <- within(rosetta, RFunction[grepl("^rows_", StanFunction)] <- "apply")
rosetta <- within(rosetta, RFunction[StanFunction == "rows"] <- "NROW")
rosetta <- within(rosetta, RFunction[StanFunction == "segment"] <- "subset")
rosetta <- within(rosetta, RFunction[StanFunction == "singular_values"] <- "svd")
rosetta <- within(rosetta, RFunction[StanFunction == "size"] <- "dim")
rosetta <- within(rosetta, RFunction[grepl("^sort_", StanFunction)] <- "sort")
rosetta <- within(rosetta, RFunction[StanFunction == "square"] <- "^")
rosetta <- within(rosetta, RFunction[StanFunction == "squared_distance"] <- "dist")
rosetta <- within(rosetta, RFunction[grepl("^std_normal_*cdf", StanFunction)] <- "pnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "std_normal_lpdf"] <- "dnorm")
rosetta <- within(rosetta, RFunction[grepl("^std_normal_*qf", StanFunction)] <- "qnorm")
rosetta <- within(rosetta, RFunction[StanFunction == "step"] <- NA_character_)
rosetta <- within(rosetta, RFunction[grepl("^student_t[_lpdf]*", StanFunction)] <- "dt")
rosetta <- within(rosetta, RFunction[grepl("^student_t_[lc]*cdf", StanFunction)] <- "pt")
rosetta <- within(rosetta, RFunction[StanFunction == "student_t_rng"] <- "rt")
rosetta <- within(rosetta, RFunction[grepl("^std_normal_*cdf", StanFunction)] <- "pnorm")
rosetta <- within(rosetta, RFunction[grepl("^svd_", StanFunction)] <- "svd")
rosetta <- within(rosetta, RFunction[StanFunction == "tgamma"] <- "gamma")
rosetta <- within(rosetta, RFunction[StanFunction == "to_array_1d"] <- "as.vector")
rosetta <- within(rosetta, RFunction[StanFunction == "to_complex"] <- "as.complex")
rosetta <- within(rosetta, RFunction[StanFunction == "to_int"] <- "as.integer")
rosetta <- within(rosetta, RFunction[StanFunction == "to_matrix"] <- "as.matrix")
rosetta <- within(rosetta, RFunction[StanFunction == "to_row_vector"] <- "as.vector")
rosetta <- within(rosetta, RFunction[StanFunction == "to_vector"] <- "as.vector")
rosetta <- within(rosetta, RFunction[StanFunction == "trace"] <- NA_character_)
rosetta <- within(rosetta, RFunction[StanFunction == "transpose"] <- "t")
rosetta <- within(rosetta, RFunction[grepl("^uniform[_lpdf]*", StanFunction)] <- "dunif")
rosetta <- within(rosetta, RFunction[grepl("^uniform_[lc]*cdf", StanFunction)] <- "punif")
rosetta <- within(rosetta, RFunction[StanFunction == "uniform_rng"] <- "runif")
rosetta <- within(rosetta, RFunction[StanFunction == "variance"] <- "var")
rosetta <- within(rosetta, RFunction[grepl("^weibull[_lpdf]*", StanFunction)] <- "dweibull")
rosetta <- within(rosetta, RFunction[grepl("^weibull_[lc]*cdf", StanFunction)] <- "pweibull")
rosetta <- within(rosetta, RFunction[StanFunction == "weibull_rng"] <- "rweibull")
rosetta <- within(rosetta, RFunction[StanFunction == "weibull_rng"] <- "rweibull")
rosetta <- within(rosetta, RFunction[StanFunction == "wishart_rng"] <- "rWishart")

save(rosetta, file = "R/sysdata.rda")
tools::resaveRdaFiles("R/sysdata.rda")

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

setClass(Class = "cxxdso",
         representation = representation(
           sig = "list", # A list of function signature that would be returned by cxxfuncion 
           dso_saved = "logical", # flag for if the dso is saved or not
           # dso_last_path = 'character', # where the dso is saved last time 
           dso_filename = "character", # the dso file name when it is created the first time
           modulename = 'character', # in rstan, we always compile the c++ code with an Rcpp module
           system = "character", # what is the OS (R.version$system)?  
           cxxflags = "character", # the CXXFLAGS used to compile the DSO 
           .CXXDSOMISC = "environment" # an envir to save 
                                 #  1. the returned object by cxxfuncion using name cxxfun 
                                 #  2. the file path used last time using name dso_last_path 
                                 #  3. The binary dso with name dso_bin, which is a raw vector.  
                                 #     We put it here since the environment is not copied 
                                 #     when assigned to another 
                                 #     https://cran.r-project.org/doc/manuals/R-lang.html#Environment-objects
         ),
         validity = function(object) {
           length(object@sig) > 0 && identical(object@system, R.version$system)
         })

setClass(Class = "stanmodel",
         representation = representation(
           model_name = "character",
           model_code = "character",
           model_cpp = "list", 
             # model_cppname (used to define Rcpp module)  & 
             # model_cppcode (just the C++ code for the model) 
           mk_cppmodule = "function", # use a function to create the cpp module
           dso = 'cxxdso'), 
         validity = function(object) {
           return(TRUE)
         })


setClass(Class = "stanfit",
         representation = representation(
           model_name = "character", 
           model_pars = "character", 
           par_dims = "list", # the order of par in the list should match those in model_pars
           mode = "integer", # 0: samples; 1: test_grad (no samples); 2: other error (no samples) 
           sim = "list", # samples with other helper information: pars_oi, dims_oi, fnames_oi, n_flatnames,
           inits = "list", 
           stan_args = "list", 
           stanmodel = "stanmodel", # the instance of S4 class stanmodel 
           date = "character", # the date samples were drawn 
           .MISC = "environment"
         ),  
         validity = function(object) {
           if(length(object@sim) > 0 && !is.null(object@sim$samples)) {
             NAs <- rapply(object@sim$samples, f = function(x) anyNA(x))
             if(any(NAs)) 
               warning(paste("The following variables have undefined values: ",
                              unique(names(NAs[NAs])), collapse = ","),
                              ". Many subsequent functions will not work correctly.")
           }
           return(TRUE) 
         })

# list all methods for stanfit 
# showMethods(class = 'stanfit', where = getNamespace("rstan"))

sbcFitFile <- function(save_progress, stanmodel, S) {
  file.path(save_progress, paste0(stanmodel@model_name, '-', S, '.rda'))
}

sbc <- function(stanmodel, data, M, ..., save_progress, load_incomplete=FALSE) {
  stopifnot(is(stanmodel, "stanmodel"))
  
  doSave <- !missing(save_progress)
  if (doSave && !dir.exists(save_progress)) {
    stop(paste0("Argument save_progress='", save_progress,
      "' provided but directory does not exist or is not a directory"))
  }

  # parameter names
  stan_code <- get_stancode(stanmodel)
  stan_code <- scan(what = character(), sep = "\n", quiet = TRUE, text = stan_code)
  pars_lines <- grep("[[:space:]]*(pars_)|(pars_\\[.*\\])[[:space:]]*=", stan_code, value = TRUE)
  if(length(pars_lines)==1){
    # if the parameter was defined and assigned in one line:
    pars_lines <- gsub(".*?=.*?(\\{|\\[)(.*?)(\\]|\\}).*","\\2",pars_lines)
    pars_names <- trimws(strsplit(pars_lines, split = ",")[[1]])
    pars_names <- unique(sub("^([a-z,A-Z,0-9,_]*)_.*", "\\1", 
        pars_names))
     } else {
    pars_lines <- pars_lines[!grepl("^[[:space:]]*vector", pars_lines) & 
        !grepl("^[[:space:]]*real", pars_lines)]
    pars_names <- trimws(sapply(strsplit(pars_lines, split = "=", 
        fixed = TRUE), tail, n = 1))
    pars_names <- unique(sub("^([a-z,A-Z,0-9,_]*)_.*;", "\\1", 
        pars_names))
     }
  noUnderscore <- grepl(";", pars_names, fixed=TRUE)
  if (any(noUnderscore)) {
    warning(paste("The following parameters were added to pars_ but did not",
      "have the expected underscore postfix:",
      paste(pars_names[noUnderscore], collapse = ' ')))
    return()
  }
  has_log_lik <- any(grepl("log_lik[[:space:]]*;[[:space:]]*", stan_code))
  
  if (!load_incomplete) {
    todo <- as.integer(seq(from = 0, to = .Machine$integer.max, length.out = M))
  } else {
    mn <- stanmodel@model_name
    runs <- dir(save_progress)
    runs <- runs[grepl(paste0("^", mn,'-(\\d+).rda$'), runs)]
    if (length(runs) == 0) {
      stop(paste("No completed runs found in", dir,
                 "matching regular expression", paste0("^", mn,'-(\\d+).rda$'),
                 "\nDid you use sbc(..., save_progress='/path/to/results')?"))
    }
    todo <- as.integer(sub(paste0(mn,'-(\\d+).rda'), "\\1", runs))
  }
  post <- parallel::mclapply(todo, FUN = function(S) {
    if (doSave) {
      file <- sbcFitFile(save_progress, stanmodel, S)
      if (file.exists(file)) return(TRUE)
    }
    out <- sampling(stanmodel, data, pars = c("ranks_", if (has_log_lik) "log_lik"), include = TRUE,
      chains = 1L, cores = 1L, seed = S, save_warmup = FALSE, thin = 1L, ...)
    out@stanmodel <- new("stanmodel")
    if (doSave) {
      save(out, file=file)
      return(TRUE)
    }
    out
  })
  if (doSave) {
    bad <- c()
    for (m in 1:length(todo)) {
      file <- sbcFitFile(save_progress, stanmodel, todo[m])
      got <- try(load(file), silent=TRUE)
      if (is(got, "try-error")) {
        bad <- c(bad, file)
        next
      }
      post[[m]] <- out
    }
    if (length(bad)) stop(paste("Remove corrupt files:",
      paste(bad, collapse=' '), "\nThen try again"))
  }
  bad <- sapply(post, FUN = function(x) x@mode != 0)
  if (any(bad)) {
    warning(sum(bad), " out of ", length(todo), " runs failed. Try decreasing 'init_r'")
    if(all(bad)) stop("cannot continue")
    post <- post[!bad]
  }

  noTwin <- is.na(match(pars_names, names(post[[1]]@par_dims)))
  if (any(noTwin)) {
    warning(paste("The following underscored priors did not have a matching",
      "parameter without the underscore:", paste(pars_names[noTwin], collapse = ' ')))
    pars_names <- NULL
  }
  pars_names <- try(flatnames(pars_names, post[[1]]@par_dims[pars_names]), silent = TRUE)
  if (!is.character(pars_names)) {
    warning("parameter names could not be calculated due to non-compliance with conventions; see help(sbc)")
    pars_names <- NULL
  }
  
  # divergences, etc.
  sampler_params <- simplify2array(sapply(post, FUN = get_sampler_params, inc_warmup = FALSE))
  
  # prior predictive distribution
  Y <- sapply(post, FUN = function(p) {
    means <- get_posterior_mean(p)[, 1]
    # will be just 'y_' if length 1, otherwise will have brackets
    means[grepl("y_$|y_\\[.*\\]", names(means))] 
  })
  
  # realizations of true parameters from transformed data
  pars <- sapply(post, FUN = function(p) {
    means <- get_posterior_mean(p)[, 1]
    mark <- grepl("pars_\\[[[:digit:]]+\\]", names(means))
    return(means[mark])
  })
  if (length(pars) > 0L) {
    if (is.null(dim(pars))) pars <- matrix(pars, nrow=1)
    if (dim(pars)[1] != length(pars_names)) {
      warning("parameter names miscalculated due to non-compliance with conventions; see help(sbc)")
      pars_names <- NULL
    }
    if (is.null(dim(pars))) {
      pars <- t(as.matrix(pars))
    }
    rownames(pars) <- pars_names
  }
  
  # not actually ranks but rather unthinned binary values for draw > true
  ranks <- lapply(post, FUN = function(p) {
    r <- extract(p, pars = "ranks_", permuted = FALSE)[, 1, ]
    if (is.null(dim(r))) {
      r <- as.matrix(r)
    }
    colnames(r) <- pars_names
    r[] <- r > 0
    return(r)
  })
    
  if (has_log_lik) # high Pareto k values will be shown by the print method
    pareto_k <- sapply(post, FUN = function(x) suppressWarnings(loo(x))$diagnostics$pareto_k)

  out <- list(ranks = ranks, Y = Y, pars = pars, sampler_params = sampler_params, 
              pareto_k = if (has_log_lik) pareto_k)
  class(out) <- "sbc"
  return(out)
}

  plot.sbc <- function(x, thin = 3, ...) {
    thinner <- seq(from = 1, to = nrow(x$ranks[[1]]), by = thin)
    u <- t(sapply(x$ranks, FUN = function(r) 1L + colSums(r[thinner, , drop = FALSE])))
    parameter <- as.factor(rep(colnames(u), each = nrow(u)))
    d <- data.frame(u = c(u), parameter)
    suppressWarnings(ggplot2::ggplot(d) + 
      ggplot2::geom_freqpoly(ggplot2::aes(x = u), ...) + 
      ggplot2::facet_wrap("parameter"))
  }

print.sbc <- function(x, ...) {
  divergences <- apply(x$sampler_params, MARGIN = 3, FUN = function(y) sum(y[,"divergent__"]))
  bad <- sum(divergences > 0L)
  cat(paste(bad, "chains had divergent transitions after warmup\n"))
  if (bad > 0L) cat(paste("there were a total of", sum(divergences), 
                            "divergent transitions across all chains\n"))
  if (length(x$pareto_k)) {
    cut_pareto_k <- cut(c(x$pareto_k), breaks = c(-Inf, 0.5, 0.7, 1, Inf))
    cat("Aggregate Pareto k estimates:\n")
    print(prop.table(table(cut_pareto_k)))
  }
  return(invisible(NULL))
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

git_head <- function() "57643021a05d101485c55f7b331ea59d91a3cd75"
git_head <- function() "04cfa06a318f7947c8610d2a857c9cb64c16b8fe"
git_head <- function() "04cfa06a318f7947c8610d2a857c9cb64c16b8fe"
git_head <- function() "edb702c04ad7a29cabc0f192679700574fee6ab9"
git_head <- function() "bf61c9a85c6425f0553aad93ee9052d03c448dd2"
git_head <- function() "2ecf3f718c301f60df6a2f81a27dc5c473c2f43c"
git_head <- function() "05c3d0058b6a42c7caf6fe6e04f17bb7a4a0258b"
git_head <- function() "85f7a56811da8e3c73bf92d6978111eea45d0c6c"
git_head <- function() "5fa1e80eb8173ff81c98a00c96d39dd942950ce5"
git_head <- function() "2e1f913d3ca3678128f159b3d17d3d1f9b82704e"
git_head <- function() "2e1f913d3ca3678128f159b3d17d3d1f9b82704e"
git_head <- function() "2e1f913d3ca3678128f159b3d17d3d1f9b82704e"
git_head <- function() "2e1f913d3ca3678128f159b3d17d3d1f9b82704e"
git_head <- function() "2e1f913d3ca3678128f159b3d17d3d1f9b82704e"

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

rstan_ess <- function(sim, n) {
  # Args:
  #   n: Chain index starting from 1.
  ess <- .Call(effective_sample_size, sim, n - 1)
  if (!is.na(ess)) {
    S <- floor((sim$iter - sim$warmup) * sim$chains / sim$thin)
    if (length(S) == 0) return(ess)
    max_ess <- ifelse(S <= 0, 0, S * log10(S))
    if (ess < 0 || ess > max_ess) ess <- max_ess
  }
  ess
}

rstan_splitrhat <- function(sim, n) {
  # Args:
  #   n: Chain index starting from 1.
  if (sim$n_save[1] - sim$warmup2[1] < 2) return(NaN)
  rhat <- .Call(split_potential_scale_reduction, sim, n - 1)
  rhat
}

rstan_splitrhat2_cpp <- function(sims) {
  # Args:
  #   sim: samples of several chains _without_ warmup
  rhat <- .Call(split_potential_scale_reduction2, sims)
  rhat
}

rstan_ess2_cpp <- function(sims) {
  # Args:
  #   sim: samples of several chains _without_ warmup
  ess <- .Call(effective_sample_size2, sims)
  if (!is.na(ess)) {
    S <- floor((sims$iter - sims$warmup) * sims$chains / sims$thin)
    if (length(S) == 0) return(ess)
    max_ess <- ifelse(S <= 0, 0, S * log10(S))
    if (ess < 0 || ess > max_ess) ess <- max_ess
  }
  ess
}

rstan_seq_perm <- function(n, chains, seed, chain_id = 1) {
  # Args:
  #   n: length of sequence to be generated 
  #   chains: the number of chains, for which the permuations are applied
  #   seed: the seed for RNG 
  #   chain_id: the chain id, for which the returned permuation is applied 
  # 
  conf <- list(n = n, chains = chains, seed = seed, chain_id = chain_id) 
  perm <- .Call(seq_permutation, conf)
  perm + 1L # start from 1 
} 

# This file is part of RStan
# Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

rstan.package.skeleton <- function(name = "anRpackage", list = character(),
                                   environment = .GlobalEnv,
                                   path = ".", force = FALSE,
                                   code_files = character(),
                                   stan_files = character()) {
  stop(
    "'rstan.package.skeleton' is now 'rstan_package_skeleton' ",
    "and can be found in the 'rstantools' package.",
    call. = FALSE
  )
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017, 2018 Trustees of Columbia University
# Copyright (C) 2018, 2019 Aki Vehtari, Paul Bürkner
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

fft_next_good_size <- function(N) {
  # Find the optimal next size for the FFT so that
  # a minimum number of zeros are padded.
  if (N <= 2)
    return(2)
  while (TRUE) {
    m <- N
    while ((m %% 2) == 0) m <- m / 2
    while ((m %% 3) == 0) m <- m / 3
    while ((m %% 5) == 0) m <- m / 5
    if (m <= 1)
      return(N)
    N <- N + 1
  }
}

#' Autocovariance estimates
#'
#' Compute autocovariance estimates for every lag for the specified
#' input sequence using a fast Fourier transform approach. Estimate
#' for lag t is scaled by N-t.
#'
#' @param y A numeric vector forming a sequence of values.
#'
#' @return A numeric vector of autocovariances at every lag (scaled by N-lag).
autocovariance <- function(y) {
  N <- length(y)
  M <- fft_next_good_size(N)
  Mt2 <- 2 * M
  yc <- y - mean(y)
  yc <- c(yc, rep.int(0, Mt2 - N))
  transform <- fft(yc)
  ac <- fft(Conj(transform) * transform, inverse = TRUE)
  # use "biased" estimate as recommended by Geyer (1992)
  ac <- Re(ac)[1:N] / (N^2 * 2)
  ac
}

#' Autocorrelation estimates
#'
#' Compute autocorrelation estimates for every lag for the specified
#' input sequence using a fast Fourier transform approach. Estimate
#' for lag t is scaled by N-t.
#'
#' @param y A numeric vector forming a sequence of values.
#'
#' @return A numeric vector of autocorrelations at every lag (scaled by N-lag).
autocorrelation <- function(y) {
  ac <- autocovariance(y)
  ac <- ac / ac[1]
}

#' Rank normalization
#'
#' Compute rank normalization for a numeric array. First replace each
#' value by its rank. Average rank for ties are used to conserve the
#' number of unique values of discrete quantities. Second, normalize
#' ranks via the inverse normal transformation.
#'
#' @param x A numeric array of values.
#'
#' @return A numeric array of rank normalized values with the same
#'     size as input.
z_scale <- function(x) {
  S <- length(x)
  r <- rank(x, ties.method = 'average')
  z <- qnorm((r - 1 / 2) / S)
  z[is.na(x)] <- NA
  if (!is.null(dim(x))) {
    # output should have the input dimension
    z <- array(z, dim = dim(x), dimnames = dimnames(x))
  }
  z
}

#' Rank uniformization
#'
#' Compute rank uniformization for a numeric array. First replace each
#' value by its rank. Average rank for ties are used to conserve the
#' number of unique values of discrete quantities. Second, uniformize
#' ranks to scale \code{[1/(2S), 1-1/(2S)]}, where \code{S} is the the number 
#' of values.
#'
#' @param x A numeric array of values.
#'
#' @return A numeric array of rank uniformized values with the same
#'     size as input.
#'     
u_scale <- function(x) {
  S <- length(x)
  r <- rank(x, ties.method = 'average')
  u <- (r - 1 / 2) / S
  u[is.na(x)] <- NA
  if (!is.null(dim(x))) {
    # output should have the input dimension
    u <- array(u, dim = dim(x), dimnames = dimnames(x))
  }
  u
}

#' Rank values
#'
#' Compute ranks for a numeric array. First replace each
#' value by its rank. Average rank for ties are used to conserve the
#' number of unique values of discrete quantities. Second, normalize
#' ranks via the inverse normal transformation.
#'
#' @param x A numeric array of values.
#'
#' @return A numeric array of ranked values with the same
#'     size as input.
#'     
r_scale <- function(x) {
  S <- length(x)
  r <- rank(x, ties.method = 'average')
  r[is.na(x)] <- NA
  if (!is.null(dim(x))) {
    # output should have the input dimension
    r <- array(r, dim = dim(x), dimnames = dimnames(x))
  }
  r
}

split_chains <- function(sims) {
  # split Markov chains
  # Args:
  #   sims: a 2D array of samples (# iter * # chains)
  if (is.vector(sims)) {
    dim(sims) <- c(length(sims), 1)
  }
  niter <- dim(sims)[1]
  if (niter == 1L) return(sims)
  half <- niter / 2
  cbind(sims[1:floor(half), ], sims[ceiling(half + 1):niter, ])
}

is_constant <- function(x, tol = .Machine$double.eps) {
  abs(max(x) - min(x)) < tol
}

#' Traditional Rhat convergence diagnostic
#'
#' Compute the Rhat convergence diagnostic for a single parameter
#' For split-Rhat, call this with split chains.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for Rhat.
#' 
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
rhat_rfun <- function(sims) {
  if (anyNA(sims)) {
    return(NA)
  }
  if (any(!is.finite(sims))) {
    return(NaN)
  }
  if (is_constant(sims)) {
    return(NA)
  }
  if (is.vector(sims)) {
    dim(sims) <- c(length(sims), 1)
  }
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  chain_mean <- numeric(chains)
  chain_var <- numeric(chains)
  for (i in seq_len(chains)) {
    chain_mean[i] <- mean(sims[, i])
    chain_var[i] <- var(sims[, i])
  }
  var_between <- n_samples * var(chain_mean)
  var_within <- mean(chain_var)
  sqrt((var_between / var_within + n_samples - 1) / n_samples)
}

#' Effective sample size
#'
#' Compute the effective sample size estimate for a sample of several chains
#' for one parameter. For split-ESS, call this with split chains.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the effective sample size.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
ess_rfun <- function(sims) {
  if (is.vector(sims)) {
    dim(sims) <- c(length(sims), 1)
  }
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  if (n_samples < 3L || should_return_NA(sims)) {
    return(NA_real_)
  }
  acov <- lapply(seq_len(chains), function(i) autocovariance(sims[, i]))
  acov <- do.call(cbind, acov)
  chain_mean <- apply(sims, 2, mean)
  mean_var <- mean(acov[1, ]) * n_samples / (n_samples - 1)
  var_plus <- mean_var * (n_samples - 1) / n_samples
  if (chains > 1)
    var_plus <- var_plus + var(chain_mean)

  # Geyer's initial positive sequence
  rho_hat_t <- rep.int(0, n_samples)
  t <- 0
  rho_hat_even <- 1
  rho_hat_t[t + 1] <- rho_hat_even
  rho_hat_odd <- 1 - (mean_var - mean(acov[t + 2, ])) / var_plus
  rho_hat_t[t + 2] <- rho_hat_odd
  while (t < nrow(acov) - 5 && !is.nan(rho_hat_even + rho_hat_odd) &&
         (rho_hat_even + rho_hat_odd > 0)) {
    t <- t + 2
    rho_hat_even = 1 - (mean_var - mean(acov[t + 1, ])) / var_plus
    rho_hat_odd = 1 - (mean_var - mean(acov[t + 2, ])) / var_plus
    if ((rho_hat_even + rho_hat_odd) >= 0) {
      rho_hat_t[t + 1] <- rho_hat_even
      rho_hat_t[t + 2] <- rho_hat_odd
    }
  }
  max_t <- t
  # this is used in the improved estimate
  if (rho_hat_even>0)
      rho_hat_t[max_t + 1] <- rho_hat_even
  
  # Geyer's initial monotone sequence
  t <- 0
  while (t <= max_t - 4) {
    t <- t + 2
    if (rho_hat_t[t + 1] + rho_hat_t[t + 2] >
        rho_hat_t[t - 1] + rho_hat_t[t]) {
      rho_hat_t[t + 1] = (rho_hat_t[t - 1] + rho_hat_t[t]) / 2;
      rho_hat_t[t + 2] = rho_hat_t[t + 1];
    }
  }
  ess <- chains * n_samples
  # Geyer's truncated estimate
  # tau_hat <- -1 + 2 * sum(rho_hat_t[1:max_t])
  # Improved estimate reduces variance in antithetic case
  tau_hat <- -1 + 2 * sum(rho_hat_t[1:max_t]) + rho_hat_t[max_t+1]
  # Safety check for negative values and with max ess equal to ess*log10(ess)
  tau_hat <- max(tau_hat, 1/log10(ess))
  ess <- ess / tau_hat
  ess
}

#' Rhat convergence diagnostic
#'
#' Compute Rhat convergence diagnostic as the maximum of rank normalized
#' split-Rhat and rank normalized folded-split-Rhat for one parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the effective sample size.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
Rhat <- function(sims) {
  bulk_rhat <- rhat_rfun(z_scale(split_chains(sims)))
  sims_folded <- abs(sims - median(sims))
  tail_rhat <- rhat_rfun(z_scale(split_chains(sims_folded)))
  max(bulk_rhat, tail_rhat)
}

#' Bulk effective sample size (bulk-ESS)
#'
#' Compute bulk effective sample size estimate (bulk-ESS) for one parameter.
#' Bulk-ESS is useful as a generic diagnostic for the sampling
#' efficiency in the bulk of the posterior. It is defined as the
#' effective sample size for rank normalized values using split chains.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the bulk effective sample size.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
ess_bulk <- function(sims) {
  ess_rfun(z_scale(split_chains(sims)))
}

#' Tail effective sample size (tail-ESS)
#'
#' Compute tail effective sample size estimate (tail-ESS) for one parameter.
#' Tail-ESS is useful for generic diagnostic for the sampling
#' efficiency in the tails of the posterior. It is defined as
#' the minimum of the effective sample sizes for 5% and 95% quantiles.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the tail effective sample size.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
ess_tail <- function(sims) {
  I05 <- sims <= quantile(sims, 0.05, na.rm = TRUE)
  q05_ess <- ess_rfun(split_chains(I05))
  I95 <- sims <= quantile(sims, 0.95, na.rm = TRUE)
  q95_ess <- ess_rfun(split_chains(I95))
  min(q05_ess, q95_ess)
}

#' Quantile effective sample size
#'
#' Compute effective sample size estimate for a quantile estimate of
#' one parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#' @param prob A single numeric value of probability.
#'
#' @return A single numeric value for the effective sample size for a
#'     quantile estimate corresponding to the probability.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
ess_quantile <- function(sims, prob) {
  if (should_return_NA(sims)) {
    return(NA_real_)
  }
  I <- sims <= quantile(sims, prob, na.rm = TRUE)
  ess_rfun(split_chains(I))
}

#' Effective sample size
#'
#' Compute effective sample size estimate for a mean (expectation)
#' estimate of one parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the effective sample size
#'     estimate for mean estimate.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
ess_mean <- function(sims) {
  ess_rfun(split_chains(sims))
}

#' Effective sample size
#'
#' Compute effective sample size estimate for standard deviation (s)
#' estimate of one parameter. This is defined as minimum of effective
#' sample size estimate for mean and mean of squared value.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for the effective sample size
#'     estimate for standard deviation estimate.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
ess_sd <- function(sims) {
  min(ess_rfun(split_chains(sims)), ess_rfun(split_chains(sims^2)))
}

#' Monte Carlo diagnostics for a quantile
#'
#' Compute Monte Carlo standard error, 5%-quantile, 95%-quantile, and
#' effective sample size estimate for a quantile estimate of a single
#' parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#' @param prob A single numeric value of probability.
#'
#' @return A data frame with Monte Carlo standard error (mcse),
#'     5%-quantile (Q05), 95%-quantile (Q95), and effective sample
#'     size estimate (ess).
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
conv_quantile <- function(sims, prob) {
  if (is.vector(sims)) {
    dim(sims) <- c(length(sims), 1)
  }
  ess <- ess_quantile(sims, prob)
  p <- c(0.1586553, 0.8413447, 0.05, 0.95)
  a <- qbeta(p, ess * prob + 1, ess * (1 - prob) + 1)
  ssims <- sort(sims)
  S <- length(ssims)
  th1 <- ssims[max(round(a[1] * S), 1)]
  th2 <- ssims[min(round(a[2] * S), S)]
  mcse <- (th2 - th1) / 2
  th1 <- ssims[max(round(a[3] * S), 1)]
  th2 <- ssims[min(round(a[4] * S), S)]
  data.frame(mcse = mcse, Q05 = th1, Q95 = th2, ess = ess)
}

#' Monte Carlo standard error for a quantile
#'
#' Compute Monte Carlo standard error for a quantile estimate of a
#' single parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#' @param prob A single numeric value of probability.
#'
#' @return A single numeric value for Monte Carlo standard error for a
#'     quantile estimate corresponding to the probability.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
mcse_quantile <- function(sims, prob) {
  conv_quantile(sims, prob)$mcse
}

#' Monte Carlo standard error for mean
#'
#' Compute Monte Carlo standard error for mean (expectation) of a
#' single parameter.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for Monte Carlo standard error
#'     for mean estimate.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
mcse_mean <- function(sims) {
  sd(sims) / sqrt(ess_mean(sims))
}

#' Monte Carlo standard error for standard error
#'
#' Compute Monte Carlo standard error for standard deviation (sd) of a
#' single parameter using Stirling's approximation and assuming
#' approximate normality.
#'
#' @param sims A 2D array _without_ warmup samples (# iter * # chains).
#'
#' @return A single numeric value for Monte Carlo standard error
#'     for standard deviation estimate.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
mcse_sd <- function(sims) {
  # assumes normality of sims and uses Stirling's approximation
  ess_sd <- ess_sd(sims)
  sd(sims) * sqrt(exp(1) * (1 - 1 / ess_sd)^(ess_sd - 1) - 1)
}

#' Summary of General Simulation Results
#'
#' Create a summary for general simulation results. Computed
#' quantities are specified quantiles, mean, standard deviation,
#' corresponding Monte Carlo standard errors, Rhat, Bulk-ESS and
#' Tail-ESS.
#'
#' @param sims A 3-dimensional array of simulation results. The first
#'     dimension is the number of iterations per chain, the second its
#'     the number of chains and the third is the number of
#'     parameters. Alternatively, \code{sims} can be a \code{stanfit}
#'     object from which the simulation results will be extracted.
#' @param warmup The number of iterations used for warmup. These will
#'     be removed before computing summary. Default is 0.
#' @param probs A vector of probabilities defining summarizing
#'     quantiles. Default is c(0.05, 0.50, 0.95).
#' @param se Logical, defaulting to FALSE. If TRUE print also MCSEs.
#' @param print Logical, defaulting to PRINT. If TRUE print summary table.
#' @param digits Positive integer, a number of digits printedm
#'     defaulting to 1.
#'
#' @return A summary matrix.
#'
#' @references
#' Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
#' Paul-Christian Bürkner (2019). Rank-normalization, folding, and
#' localization: An improved R-hat for assessing convergence of
#' MCMC. \emph{arXiv preprint} \code{arXiv:1903.08008}.
#' 
#' @export
monitor <- function(sims, warmup = floor(dim(sims)[1] / 2), 
                    probs = c(0.025, 0.25, 0.50, 0.75, 0.975), 
                    digits_summary = 1, print = TRUE, ...) { 
  if (inherits(sims, "stanfit")) {
    chains <- sims@sim$chains
    iter <- sims@sim$iter
    warmup <- 0L
    parnames <- names(sims)
    sims <- as.array(sims)
  } else {
    dim_sims <- dim(sims)
    if (is.null(dim_sims)) {
      dim(sims) <- c(length(sims), 1, 1)
    } else if (length(dim_sims) == 2) {
      dim(sims) <- c(dim_sims, 1)
    } else if (length(dim_sims) > 3) {
      stop("'sims' has more than 3 dimensions")
    }
    parnames <- dimnames(sims)[[3]]
    if (is.null(parnames)) {
      parnames <- paste0("V", seq_len(dim(sims)[3]))
    }
    iter <- dim(sims)[1]
    chains <- dim(sims)[2]
    if (warmup > dim(sims)[1]) {
      stop("warmup is larger than the total number of iterations")
    }
    if (warmup >= 1) {
      sims <- sims[-seq_len(warmup), , , drop = FALSE]
    }
  }

  out <- vector("list", length(parnames))
  out <- setNames(out, parnames)
  # loop over parameters
  for (i in seq_along(out)) {
    sims_i <- sims[, , i]
    valid <- all(is.finite(sims_i))
    quan <- unname(quantile(sims_i, probs = probs, na.rm = TRUE))
    quan2 <- quantile(sims_i, probs = c(0.05, 0.5, 0.95), na.rm = TRUE)
    mean <- mean(sims_i)
    sd <- sd(sims_i)
    mcse_quan <- sapply(probs, mcse_quantile, sims = sims_i)
    mcse_mean <- mcse_mean(sims_i)
    mcse_sd <- mcse_sd(sims_i)
    rhat <- Rhat(sims_i)
    ess_bulk <- round(ess_bulk(sims_i))
    ess_tail <- round(ess_tail(sims_i))
    ess <- round(ess_rfun(sims_i))
    out[[i]] <- c(
      mean, mcse_mean, sd, quan, ess, rhat,
      valid, quan2, mcse_quan, mcse_sd, ess_bulk, ess_tail
    )
  }
  
  out <- as.data.frame(do.call(rbind, out))
  probs_str <- names(quantile(sims_i, probs = probs, na.rm = TRUE))
  str_quan <- paste0("Q", probs * 100)
  str_quan2 <- paste0("Q", c(0.05, 0.5, 0.95) * 100)
  str_mcse_quan <- paste0("MCSE_", str_quan)
  colnames(out) <- c("mean", "se_mean", "sd", probs_str, "n_eff", "Rhat",
                     "valid", str_quan2, str_mcse_quan, "MCSE_SD", "Bulk_ESS", "Tail_ESS")
  rownames(out) <- parnames

  # replace NAs with appropriate values if draws are valid
  S <- prod(dim(sims)[1:2])
  valid <- out[, "valid"]
  out[valid & !is.finite(out[, "Rhat"]), "Rhat"] <- 1
  out[valid & !is.finite(out[, "Bulk_ESS"]), "Bulk_ESS"] <- S
  out[valid & !is.finite(out[, "Tail_ESS"]), "Tail_ESS"] <- S
  SE_vars <- colnames(out)[grepl("^SE_", colnames(out), ignore.case = TRUE)]
  for (v in SE_vars) {
  	out[valid & !is.finite(out[, v]), v] <- 0
  }
  
  out <- structure(
    out,
    chains = chains,
    iter = iter,
    warmup = warmup,
    class = c("simsummary", "data.frame")
  )
  if (print) print.simsummary(out, digits = digits_summary, ...)
  return(invisible(out))
}

print.simsummary <- function(x, digits = 3, se = FALSE, ...) {
  atts <- attributes(x)
  px <- x
  class(px) <- "data.frame"
  quan2 <- grepl("^Q", colnames(px))
  if (se) {
    px <- cbind(px[ , quan2], Mean = px$mean, SD = px$sd,
                px[ , grepl("^MCSE", colnames(px))], " Rhat" = px$Rhat,
                Bulk_ESS = px$Bulk_ESS, Tail_ESS = px$Tail_ESS)
  } else {
    px <- cbind(px[ , quan2], Mean = px$mean, SD = px$sd, " Rhat" = px$Rhat,
                Bulk_ESS = px$Bulk_ESS, Tail_ESS = px$Tail_ESS)
  }
  
  decimal_places <- max(1, digits - 1)
  px$` Rhat` <- round(px$` Rhat`, digits = max(2, decimal_places))
  estimates <- setdiff(names(px), c(" Rhat", "Bulk_ESS", "Tail_ESS"))
  px[, estimates] <- round(px[, estimates], digits = decimal_places)
  cat(
    "Inference for the input samples (", atts$chains,
    " chains: each with iter = ", atts$iter,
    "; warmup = ", atts$warmup, "):\n\n", sep = ""
  )
  print(px, ...)
  if (!isTRUE(atts$extra)) {
  	cat(
  		"\nFor each parameter, Bulk_ESS and Tail_ESS are crude measures of \n",
  		"effective sample size for bulk and tail quantities respectively (an ESS > 100 \n",
  		"per chain is considered good), and Rhat is the potential scale reduction \n",
  		"factor on rank normalized split chains (at convergence, Rhat <= 1.05).\n", sep = ""
  	)
  }
  invisible(x)
}

# this is needed to make DeLorean's vignette build
`[.simsummary` <- function (x, i, j, drop = if (missing(i)) TRUE else length(j) == 1) {
  out <- `[.data.frame`(x, i, j, drop)
  nms <- rownames(x)[i]
  if (drop) names(out) <- nms
  else rownames(out) <- nms
  return(out)
}

# should NA be returned by a convergence diagnostic?
should_return_NA <- function(x) {
  anyNA(x) || any(!is.finite(x)) || is_constant(x)
}

# This file is part of RStan
# Copyright (C) 2012, 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
#
# RStan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# RStan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###  deal with the optimization level for c++ compilation

get_all_makefile_paths <- function() {
  out <- system2(file.path(Sys.getenv("R_HOME"), "bin", "R"),
                 args = "CMD SHLIB --dry-run", stdout = TRUE)
  out <- grep("SHLIB", out, value = TRUE)[1]
  makefiles <- strsplit(sub("SHLIB.*$", "", out), split = "-f ")[[1]][-1]
  makefiles <- gsub("'", "", makefiles)
  makefiles <- gsub("[[:space:]]*$", "", makefiles)
  makefiles <- gsub('\"', '', makefiles)
  makefiles <- makefiles[file.exists(makefiles)]
  return(makefiles)
}

get_makefile_txt <- function() {
  # get the all makefile content used for R CMD SHLIB
  # The order or files to look for are following the code
  # of function .SHLIB in packages tools/R/install.R
  #
  # Return:
  #   a character vector in which each element is a line of
  #   the makefile
  makefiles <- get_all_makefile_paths()
  do.call(c, lapply(makefiles, function(f) readLines(f, warn = FALSE)))
}

get_makefile_flags <- function(FLAGNAME, makefile_txt, headtotail = FALSE) {
  # get current CXXFLAGS used for in R CMD SHLIB, which controls
  # how the generated C++ code for stan model are compiled
  # at which optimization level.
  # Args:
  #   FLAGNAME: the name of the flag of interest such as CXXFLAGS
  #   makefile_txt: a character vector each element of which is a line
  #     of the make file, typically obtained by get_makefile_txt()
  # Return:
  #   the whole line that defines CXXFLAGS

  if (missing(makefile_txt))
    makefile_txt <- get_makefile_txt()
  if (length(makefile_txt) == 0) return(as.character(NULL))

  lineno <- -1
  iseq <- if (headtotail) 1:length(makefile_txt) else length(makefile_txt):1
  for (i in iseq) {
    pattern <- paste("^\\s*", FLAGNAME, "\\s*=", sep = '')
    if (!is.null(makefile_txt[i]) && grepl(pattern, makefile_txt[i])) {
      lineno <- i
      break;
    }
  }

  if (-1 != lineno) return(makefile_txt[lineno])
  paste(FLAGNAME, " = ", sep = '')
}

last_makefile <- function() {
  # Return the last makefile for 'R CMD SHLIB'.
  # In essential, R CMD SHLIB uses GNU make.
  # R CMD SHLIB uses a series of files as the whole makefile.
  # This function return the last one, where we can set flags
  # to overwrite what is set before.
  #
  makefiles <- get_all_makefile_paths()
  return(tail(makefiles, n = 1L))
}

set_makefile_flags <- function(flags) {
  # Set user-defined CXXFLAGS for R CMD SHLIB by
  # set CXXFLAGS or others in the last makefile obtained by last_makefile
  # Args:
  #   flags: a named list with names CXXFLAGS and R_XTRA_CPPFLAGS
  #
  # Return:
  #   TRUE if it is successful, otherwise, the function
  #   stops and reports an error.
  #
  lmf <- last_makefile()
  homedotR <- dirname(lmf)
  if (!file.exists(homedotR)) {
    if (!dir.create(homedotR, showWarnings = FALSE, recursive = TRUE))
      stop(paste("failed to create directory ", homedotR, sep = ''))
  }

  flagnames <- names(flags)
  paste_bn <- function(...)  paste(..., sep = '\n')
  flags <- lapply(flags,
                  function(x) {
                    if (grepl('#set_by_rstan', x)) return(x)
                    paste(x, " #set_by_rstan")
                  })

  # the Makevars file does not exist
  if (!file.exists(lmf)) {
    if (file.access(homedotR, mode = 2) < 0 ||
        file.access(homedotR, mode = 1) < 0 )
      stop(paste("directory ", homedotR, " is not writable", sep = ''))
    cat(paste("# created by rstan at ", date(), sep = ''), '\n', file = lmf)
    cat(do.call(paste_bn, flags), file = lmf, append = TRUE)
    return(invisible(NULL))
  }

  if (file.access(lmf, mode = 2) < 0)
    stop(paste(lmf, " is not writable", sep = ''))

  lmf_txt <- readLines(lmf, warn = FALSE)
  if (length(lmf_txt) < 1) {  # empty file
    cat(do.call(paste_bn, flags), file = lmf, append = TRUE)
    return(invisible(NULL))
  }

  # change the existing file
  makefile_exta <- ""
  for (fname in flagnames) {
    found <- FALSE
    for (i in length(lmf_txt):1) {
      pattern <- paste("^\\s*", fname, "\\s*=", sep = '')
      if (grepl(pattern, lmf_txt[i])) {
        lmf_txt[i] <- flags[[fname]]
        found <- TRUE
        break
      }
    }
    if (!found)
      makefile_exta <- paste(makefile_exta, flags[[fname]], sep = '\n')
  }

  cat(paste(lmf_txt, collapse = '\n'), file = lmf)
  cat(makefile_exta, file = lmf, append = TRUE)
  invisible(NULL)
}

rm_last_makefile <- function(force = TRUE) {
  # remove the file given by last_makefile()
  # return:
  #  0 for success, 1 for failure, invisibly.
  #  -1 file does not exist
  lmf <- last_makefile()
  msgs <- c(paste(lmf, " does not exist", sep = ''),
            paste("removed file ", lmf, sep = ''),
            paste("failed to remove file ", lmf, sep = ''))
  ret <- -1
  if (file.exists(lmf)) ret <- unlink(lmf, force = force)
  cat(msgs[ret + 2], '\n')
  invisible(ret)
}

get_cxxo_level <- function(str) {
  # obtain the optimization level from a string like -O3
  # Return:
  #   the optimization level after -O as a character
  #
  p <- regexpr('-O.', str, perl = TRUE) + 2
  if (p == 1) return("")  # not found
  substr(str, p, p)
}

if_debug_defined <- function(str) {
  grepl("DDEBUG", str)
}

if_ndebug_defined <- function(str) {
  grepl("DNDEBUG", str)
}


rm_rstan_makefile_flags <- function() {
  # remove flags in $HOME/.R/Makevars with #rstan
  lmf <- last_makefile()
  if (!file.exists(lmf)) {
    message("no user Makevars file found; nothing need be done.")
    return(invisible(NULL))
  }
  if (file.access(lmf, mode = 2) < 0)
    stop(paste(lmf, " is not writable", sep = ''))
  lmf_txt <- readLines(lmf, warn = FALSE)
  if (length(lmf_txt) < 1) return(invisible(NULL))
  for (i in length(lmf_txt):1) {
    if (grepl("#set_by_rstan", lmf_txt[i]))
      lmf_txt[i] <- NA
  }
  lmf_txt <- lmf_txt[!is.na(lmf_txt)]
  cat(paste(lmf_txt, collapse = '\n'), file = lmf)
  message("compiler flags set by rstan are removed.")
  invisible(NULL)
}

#' Shim for tools::makevars_user()
#' @keywords internal
#' @export
makevars_user <- function() {
  if (getRversion() >= "3.3") {
    return(tools::makevars_user())
  }
  # Below is tools::makevars_user() from R 3.6.2
  m <- character()
  if (.Platform$OS.type == "windows") {
    if (!is.na(f <- Sys.getenv("R_MAKEVARS_USER", NA_character_))) {
      if (file.exists(f))
        m <- f
    }
    else if ((Sys.getenv("R_ARCH") == "/x64") && file.exists(f <- path.expand("~/.R/Makevars.win64")))
      m <- f
    else if (file.exists(f <- path.expand("~/.R/Makevars.win")))
      m <- f
    else if (file.exists(f <- path.expand("~/.R/Makevars")))
      m <- f
  }
  else {
    if (!is.na(f <- Sys.getenv("R_MAKEVARS_USER", NA_character_))) {
      if (file.exists(f))
        m <- f
    }
    else if (file.exists(f <- path.expand(paste0("~/.R/Makevars-",
            Sys.getenv("R_PLATFORM")))))
      m <- f
    else if (file.exists(f <- path.expand("~/.R/Makevars")))
      m <- f
  }
  m
}


#' Remove march flag from Makevars
.warn_march_makevars <- function() {
  makevar_files <- makevars_user()
  if (length(makevar_files)) {
    cxx_flags <- grep("^CXX.*FLAGS", readLines(file.path(makevar_files)), value = TRUE)
  } else {
    cxx_flags <- character(0)
  }
  if (length(cxx_flags) != 0) {
    has_march_native <- any(grepl("-march[[:space:]]*=[[:space:]]*native",
                                substr(cxx_flags, regexpr("-", cxx_flags), nchar(cxx_flags))))
    if (has_march_native && !rstan_options("disable_march_warning")) {
      warning(paste0("Detected -march=native in the Makevars file at '", makevar_files,
                     "'. Compiling with the -march=native flag on windows with Rtools",
                     " can cause crashes because of the compiler implementation. rstan",
                     " will ignore the Makevars file until -march=native is removed.",
                     " You can disable this by setting ",
                     'rstan_options(disable_march_warning = TRUE)'))
      return(TRUE)
    }
  }
  return(FALSE)
}

# FIXME: This should be uncommented (and possibly refactored) when the tests in
# this file are unskipped.

# Needed to test part of stan_args.hpp.
# src <- "
#   BEGIN_RCPP
#   Rcpp::List lst(x);
#   rstan::stan_args args(lst);
#   return args.stan_args_to_rlist();
#   END_RCPP
# "
#
# fx <- inline::cxxfunction(signature(x = "list"),
#   body = src,
#   includes = "#include <rstan/stan_args.hpp>",
#   plugin = "rstan", verbose = TRUE
# )

test_that("stan_args_hppb works", {
  skip("Backwards compatibility")

  b1 <- fx(list(iter = 100, seed = 12354, method = "optim"))
  expect_equal(b1$random_seed, "12354")
  expect_equal(b1$algorithm, "LBFGS")
  b2 <- fx(list(iter = 100, seed = 12354, method = "optim", algorithm = "BFGS"))
  b3 <- fx(list(iter = 100, seed = 12354, method = "optim", algorithm = "LBFGS"))
  b4 <- fx(list(iter = 100, seed = 12354, method = "optim", algorithm = "LBFGS", history_size = 6))
  expect_equal(b1$random_seed, "12354")
  expect_true(is.null(b2$history_size))
  expect_equal(b3$history_size, 5)
  expect_equal(b4$history_size, 6)
})

test_that("stan_args_hppvb works", {
  skip("Backwards compatibility")

  b1 <- fx(list(iter = 100, seed = 12354, method = "variational"))
  expect_equal(b1$random_seed, "12354")
  expect_equal(b1$algorithm, "meanfield")
  expect_true(b1$adapt_engaged)
  b2 <- fx(list(iter = 100, seed = 12354, method = "variational", algorithm = "fullrank"))
  expect_equal(b2$algorithm, "fullrank")
  expect_equal(b2$iter, 100)
  expect_equal(b2$grad_samples, 1)
  expect_equal(b2$eta, 1)
  b3 <- fx(list(
    iter = 101, seed = 12354, method = "variational",
    algorithm = "fullrank", grad_samples = 2,
    adapt_iter = 102,
    elbo_samples = 50, eval_elbo = 48, output_samples = 500,
    eta = .5, tol_rel_obj = 0.001
  ))
  expect_equal(b3$iter, 101)
  expect_equal(b3$adapt_iter, 102)
  expect_equal(b3$random_seed, "12354")
  expect_equal(b3$grad_samples, 2)
  expect_equal(b3$eval_elbo, 48)
  expect_equal(b3$tol_rel_obj, 0.001)
  expect_equal(b3$eta, 0.5)
  expect_equal(b3$elbo_samples, 50)
  expect_equal(b3$output_samples, 500)
})

test_that("stan_args_hpp works", {
  skip("Backwards compatibility")

  a1 <- fx(list(iter = 100, thin = 100))
  a2 <- fx(list(iter = 100, thin = 3))
  a3 <- fx(list(iter = 5, thin = 3, refresh = -1, seed = "12345"))
  a4 <- fx(list(iter = 5, thin = 3, refresh = -1, method = "test_grad"))
  a4b <- fx(list(
    iter = 5, thin = 3, refresh = -1, method = "test_grad",
    control = list(epsilon = 1.3, error = 0.1)
  ))
  a4c <- fx(list(
    iter = 5, thin = 3, refresh = -1, method = "test_grad",
    control = list(epsilon = 1.3)
  ))
  a4d <- fx(list(
    iter = 5, thin = 3, refresh = -1, method = "test_grad",
    control = list(error = 1.3)
  ))
  a5 <- fx(list(
    iter = 5, thin = 3, algorithm = "HMC",
    control = list(stepsize = .1)
  ))
  a6 <- fx(list(
    iter = 5, thin = 3, algorithm = "NUTS",
    control = list(stepsize = .1, metric = "unit_e")
  ))
  a7 <- fx(list(
    iter = 5, thin = 3, algorithm = "NUTS",
    control = list(stepsize = .1, metric = "diag_e")
  ))
  a7b <- fx(list(
    iter = 5, thin = 3, algorithm = "NUTS",
    control = list(
      stepsize = .1, metric = "diag_e",
      adapt_term_buffer = 4,
      adapt_window = 30,
      adapt_init_buffer = 40
    )
  ))
  a8 <- fx(list(
    iter = 5, thin = 3, algorithm = "NUTS",
    control = list(stepsize = .1, metric = "dense_e")
  ))
  a9 <- fx(list(
    iter = 5, thin = 3, algorithm = "HMC",
    control = list(stepsize = .1, metric = "unit_e")
  ))
  a10 <- fx(list(
    iter = 5, thin = 3, algorithm = "HMC",
    control = list(stepsize = .1, metric = "diag_e")
  ))
  a11 <- fx(list(
    iter = 5, thin = 3, algorithm = "HMC",
    control = list(stepsize = .1, metric = "dense_e")
  ))

  a12 <- fx(list(
    iter = 5, thin = 3, algorithm = "Fixed_param",
    control = list(adapt_engaged = TRUE)
  ))

  expect_equal(a1$iter, 100)
  expect_equal(a1$thin, 100)
  expect_equal(a2$iter, 100)
  expect_equal(a2$thin, 3)
  expect_equal(a2$control$adapt_init_buffer, 75)
  expect_equal(a2$control$adapt_term_buffer, 50)
  expect_equal(a2$control$adapt_window, 25)
  expect_equal(a3$iter, 5)
  expect_equal(a3$random_seed, "12345")
  expect_equal(a3$test_grad, FALSE)
  expect_equal(a3$refresh, -1)
  expect_equal(a3$control$adapt_init_buffer, 75)
  expect_equal(a3$control$adapt_term_buffer, 50)
  expect_equal(a3$control$adapt_window, 25)
  expect_equal(a4$test_grad, TRUE)
  expect_equal(a4$method, "test_grad")
  expect_equal(a4$control$epsilon, 1e-6)
  expect_equal(a4$control$error, 1e-6)
  expect_equal(a4b$control$epsilon, 1.3)
  expect_equal(a4b$control$error, 1e-1)
  expect_equal(a4c$control$epsilon, 1.3)
  expect_equal(a4c$control$error, 1e-6)
  expect_equal(a4c$control$error, 1e-6)
  expect_equal(a4c$control$epsilon, 1.3)

  expect_error(fx(list(iter = 5, thin = 3, refresh = -1, test_grad = TRUE, seed = "111111111111111111111")))

  expect_equal(a6$control$metric, "unit_e")
  expect_equal(a7$control$metric, "diag_e")
  expect_equal(a7b$control$adapt_init_buffer, 40)
  expect_equal(a7b$control$adapt_term_buffer, 4)
  expect_equal(a7b$control$adapt_window, 30)
  expect_equal(a8$control$metric, "dense_e")
  expect_equal(a9$control$metric, "unit_e")
  expect_equal(a10$control$metric, "diag_e")
  expect_equal(a11$control$metric, "dense_e")
  expect_equal(a11$control$adapt_window, 25)
})

---
title: "Simulation Based Calibration"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulation Based Calibration}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
library(rstan)
knitr::opts_chunk$set(eval = .Platform$OS.type != "windows")
```

Here is a Stan program for a beta-binomial model
```{stan output.var="beta_binomial", eval = FALSE}
data {
  int<lower = 1> N;
  real<lower = 0> a;
  real<lower = 0> b;
}
transformed data { // these adhere to the conventions above
  real pi_ = beta_rng(a, b);
  int y = binomial_rng(N, pi_);
}
parameters {
  real<lower = 0, upper = 1> pi;
}
model {
  target += beta_lpdf(pi | a, b);
  target += binomial_lpmf(y | N, pi);
}
generated quantities { // these adhere to the conventions above
  int y_ = y;
  vector[1] pars_;
  int ranks_[1] = {pi > pi_};
  vector[N] log_lik;
  pars_[1] = pi_;
  for (n in 1:y) log_lik[n] = bernoulli_lpmf(1 | pi);
  for (n in (y + 1):N) log_lik[n] = bernoulli_lpmf(0 | pi);
}
```
Notice that it adheres to the following conventions:

* Realizations of the unknown parameters are drawn in the `transformed data`
  block are postfixed with an underscore, such as `pi_`.
  These are considered the "true" parameters being estimated by
  the corresponding symbol declared in the `parameters` block, which
  have the same names except for the trailing underscore, such as `pi`.
* The realizations of the unknown parameters are then conditioned on when drawing from
  the prior predictive distribution in `transformed data` block, which in this
  case is `int y = binomial_rng(N, pi_);`. To avoid confusion, `y` does not have a 
  training underscore.
* The realizations of the unknown parameters are copied into a `vector`
  in the `generated quantities` block named `pars_`
* The realizations from the prior predictive distribution are copied 
  into an object (of the same type) in the `generated quantities` block
  named `y_. This is optional.
* The `generated quantities` block contains an integer array named 
  `ranks_` whose only values are zero or one, depending on whether the realization of a 
  parameter from the posterior distribution exceeds the corresponding "true"
  realization, which in this case is `ranks_[1] = {pi > pi_};`. These are not actually "ranks"
  but can be used afterwards to reconstruct (thinned) ranks.
* The `generated quantities` block contains a vector named `log_lik` whose values
  are the contribution to the log-likelihood by each observation. In this case, 
  the "observations" are the implicit successes and failures that are aggregated
  into a binomial likelihood. This is optional but facilitates calculating the
  Pareto k shape parameter estimates that indicate whether the posterior distribution
  is sensitive to particular observations.

Assuming the above is compile to a code `stanmodel` named `beta_binomial`, we can
then call the `sbc` function
```{r, eval = FALSE}
output <- sbc(beta_binomial, data = list(N = 10, a = 1, b = 1), M = 500, refresh = 0)
```
```{r, include = FALSE}
# This fakes what would happen if we actually took the time to run Stan.
N <- 10
M <- 500
pars_ <- rbeta(M, 1, 1)
y_ <- matrix(rbinom(pars_, size = N, prob = pars_), ncol = M)
post_ <- matrix(rbeta(M * 1000L, 1 + y_, 1 + N - y_), ncol = M)
ranks_ <- lapply(1:M, FUN = function(m) {
  matrix(post_[ , m] > pars_[m], ncol = 1, 
         dimnames = list(NULL, "pi"))
})
log_lik <- t(sapply(1:M, FUN = function(m) {
  c(dbinom(rep(1, y_[m]), size = 1, prob = pars_[m], log = TRUE),
    dbinom(rep(1, N - y_[m]), size = 1, prob = pars_[m], log = TRUE))
}))
sampler_params <- array(0, dim = c(1000, 6, M))
colnames(sampler_params) <- c("accept_stat__", "stepsize__", "treedepth__",   
                              "n_leapfrog__",  "divergent__",   "energy__")
output <- list(ranks = ranks_, Y = y_, pars = pars_, 
               log_lik = log_lik, sampler_params = sampler_params)
class(output) <- "sbc"
```
At which point, we can then call
```{r}
print(output)
plot(output, bins = 10) # it is best to specify the bins argument yourself
```

# References

  Talts, S., Betancourt, M., Simpson, D., Vehtari, A., and Gelman, A. (2018).
  Validating Bayesian Inference Algorithms with Simulation-Based Calibration.
  [arXiv preprint arXiv:1804.06788](https://arxiv.org/abs/1804.06788)

---
title: "Accessing the contents of a stanfit object"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{stanfit objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE, error = TRUE,
  comment = NA,
  fig.align = "center",
  fig.height = 5,
  fig.width = 7
  )
```

This vignette demonstrates how to access most of data stored in a stanfit 
object. A stanfit object (an object of class `"stanfit"`) contains the output 
derived from fitting a Stan model using Markov chain Monte Carlo or one of
Stan's variational approximations (meanfield or full-rank). Throughout the
document we'll use the stanfit object obtained from fitting the Eight Schools
example model:

```{r, example-model, eval=FALSE}
library(rstan)
fit <- stan_demo("eight_schools", refresh = 0)
```
```{r, fit, echo=FALSE, cache=FALSE, results="hide"}
J <- 8
y <- c(28,  8, -3,  7, -1,  1, 18, 12)
sigma <- c(15, 10, 16, 11,  9, 11, 10, 18)
fit <- stan(
  file= "schools.stan", 
  model_name = "eight_schools",
  data = c("y", "J", "sigma"), 
  refresh = 0
  )
```
```{r, stanfit-class}
class(fit)
```

## Posterior draws

There are several functions that can be used to access the draws from the 
posterior distribution stored in a stanfit object. These are `extract`, 
`as.matrix`, `as.data.frame`, and `as.array`, each of which returns the draws in
a different format.

<br>

#### extract()

The `extract` function (with its default arguments) returns a list with named
components corresponding to the model parameters.

```{r, extract-1}
list_of_draws <- extract(fit)
print(names(list_of_draws))
```

In this model the parameters `mu` and `tau` are scalars and `theta` is a vector
with eight elements. This means that the draws for `mu` and `tau` will be 
vectors (with length equal to the number of post-warmup iterations times the 
number of chains) and the draws for `theta` will be a matrix, with each column 
corresponding to one of the eight components:

```{r, extract-2}
head(list_of_draws$mu)
head(list_of_draws$tau)
head(list_of_draws$theta)
```

<br>

#### as.matrix(), as.data.frame(), as.array()

The `as.matrix`, `as.data.frame`, and `as.array` functions can also be used
to retrieve the posterior draws from a stanfit object:

```{r, as.matrix-1}
matrix_of_draws <- as.matrix(fit)
print(colnames(matrix_of_draws))

df_of_draws <- as.data.frame(fit)
print(colnames(df_of_draws))

array_of_draws <- as.array(fit)
print(dimnames(array_of_draws))
```

The `as.matrix` and `as.data.frame` methods essentially return the same 
thing except in matrix and data frame form, respectively. The `as.array`
method returns the draws from each chain separately and so has an additional
dimension:

```{r, as.matrix-2, results="hold"}
print(dim(matrix_of_draws))
print(dim(df_of_draws))
print(dim(array_of_draws))
```

By default all of the functions for retrieving the posterior draws return the
draws for _all_ parameters (and generated quantities). The optional argument 
`pars` (a character vector) can be used if only a subset of the parameters is
desired, for example:

```{r, as.matrix-3}
mu_and_theta1 <- as.matrix(fit, pars = c("mu", "theta[1]"))
head(mu_and_theta1)
```

<br>

## Posterior summary statistics and convergence diagnostics

Summary statistics are obtained using the `summary` function. The object 
returned is a list with two components:

```{r, summary-1}
fit_summary <- summary(fit)
print(names(fit_summary))
```

In `fit_summary$summary` all chains are merged whereas `fit_summary$c_summary`
contains summaries for each chain individually. Typically we want the 
summary for all chains merged, which is what we'll focus on here.

The summary is a matrix with rows corresponding to parameters and columns to the
various summary quantities. These include the posterior mean, the posterior
standard deviation, and various quantiles computed from the draws. The `probs`
argument can be used to specify which quantiles to compute and `pars` can be
used to specify a subset of parameters to include in the summary.

For models fit using MCMC, also included in the summary are the Monte Carlo
standard error (`se_mean`), the effective sample size (`n_eff`), and the R-hat
statistic (`Rhat`).

```{r, summary-2}
print(fit_summary$summary)
```

If, for example, we wanted the only quantiles included to be 10% and 90%, and  for only the parameters included to be `mu` and `tau`, we would specify that like this:

```{r, summary-3}
mu_tau_summary <- summary(fit, pars = c("mu", "tau"), probs = c(0.1, 0.9))$summary
print(mu_tau_summary)
```

Since `mu_tau_summary` is a matrix we can pull out columns using their names:

```{r, summary-4}
mu_tau_80pct <- mu_tau_summary[, c("10%", "90%")]
print(mu_tau_80pct)
```


<br>

## Sampler diagnostics

For models fit using MCMC the  stanfit object will also contain the values of
parameters used for the sampler. The `get_sampler_params` function can 
be used to access this information.

The object returned by `get_sampler_params` is a list with one component (a
matrix) per chain. Each of the matrices has number of columns corresponding to
the number of sampler parameters and the column names provide the parameter
names. The optional argument inc_warmup (defaulting to `TRUE`) indicates whether
to include the warmup period.

```{r, get_sampler_params-1}
sampler_params <- get_sampler_params(fit, inc_warmup = FALSE)
sampler_params_chain1 <- sampler_params[[1]]
colnames(sampler_params_chain1)
```

To do things like calculate the average value of `accept_stat__` for each chain 
(or the maximum value of `treedepth__` for each chain if using the NUTS
algorithm, etc.) the `sapply` function is useful as it will apply the 
same function to each component of `sampler_params`:

```{r, get_sampler_params-2}
mean_accept_stat_by_chain <- sapply(sampler_params, function(x) mean(x[, "accept_stat__"]))
print(mean_accept_stat_by_chain)

max_treedepth_by_chain <- sapply(sampler_params, function(x) max(x[, "treedepth__"]))
print(max_treedepth_by_chain)
```

<br>

## Model code

The Stan program itself is also stored in the stanfit object and can be 
accessed using `get_stancode`:

```{r, get_stan_code-1}
code <- get_stancode(fit)
```

The object `code` is a single string and is not very intelligible when printed:

```{r, get_stan_code-2}
print(code)
```

A readable version can be printed using `cat`:
```{r, get_stan_code-3}
cat(code)
```

<br>

## Initial values

The `get_inits` function returns initial values as a list with one component per
chain. Each component is itself a (named) list containing the initial values for each parameter for the corresponding chain:

```{r, get_inits}
inits <- get_inits(fit)
inits_chain1 <- inits[[1]]
print(inits_chain1)
```

<br>

## (P)RNG seed

The `get_seed` function returns the (P)RNG seed as an integer:

```{r, get_seed}
print(get_seed(fit))
```

<br>

## Warmup and sampling times

The `get_elapsed_time` function returns a matrix with the warmup and sampling
times for each chain: 

```{r, get_elapsed_time}
print(get_elapsed_time(fit))
```

---
title: "Interfacing with External C++ Code"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Interfacing with External C++ Code}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE, eval = FALSE, error = TRUE
)
```

Starting with the 2.13 release, it is much easier to use external C++ code in a 
Stan program. This vignette briefly illustrates how to do so. 

Suppose that you have (part of) a Stan program that involves Fibonacci numbers, 
such as
```{stan eval = FALSE, output.var = "fib"}
functions {
  int fib(int n);
  int fib(int n) {
    if (n <= 0) reject("n must be positive");
    return n <= 2 ? 1 : fib(n - 1) + fib(n - 2);
  }
}
model {} // use the fib() function somehow
```
On the second line, we have _declared_ the `fib` function before it is _defined_
in order to call it recursively. 

For functions that are not recursive, it is not necessary to declare them before 
defining them but it may be advantageous. For example, I often like to hide the
definitions of complicated utility functions that are just a distraction using
the `#include "file"` mechanism
```{stan eval = FALSE, output.var="includes"}
functions {
  real complicated(real a, real b, real c, real d, real e, real f, real g);
#include "complicated.stan"
}
model {} // use the complicated() function somehow
```
This Stan program would have to be parsed using the `stanc_builder` function in
the __rstan__ package rather than the default `stanc` function (which is called
by `sampling` and `stan` internally).

Returning to the Fibonacci example, it is not necessary to define the `fib`
function using the Stan language because Stan programs with functions that are 
_declared_ but not _defined_ can use the standard capabilities of the C++ 
toolchain to provide the function definitions in C++. For example, this program
produces a parser error by default
```{r, eval = TRUE}
mc <- 
'
functions { int fib(int n); }
model {} // use the fib() function somehow
'
try(stan_model(model_code = mc, model_name = "parser_error"), silent = TRUE)
```
However, if we specify the `allow_undefined` and `includes` arguments to the 
`stan_model` function, and define a `fib` function in the named C++ header file, 
then it will parse and compile
```{r}
stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
           includes = paste0('\n#include "', 
                             file.path(getwd(), 'fib.hpp'), '"\n'))
```
Specifying the `includes` argument is a bit awkward because the C++ 
representation of a Stan program is written and compiled in a temporary
directory. Thus, the `includes` argument must specify a _full_ path to the
fib.hpp file, which in this case is in the working directory. Also, the
path must be enclosed in double-quotes, which is why single quotes are used
in the separate arguments to the `paste0` function so that double-quotes are
interpreted literally. Finally, the `includes` argument should include newline
characters (`"\n"`) at the start and end. It is possible to specify multiple
paths using additional newline characters or include a "meta-header" file
that contains `#include` directives to other C++ header files.

The result of the `includes` argument is inserted into the C++ file directly
at the end of the lines (as opposed to CmdStan where it is inserted directly
_before_ the following lines)
```{Rcpp, eval = FALSE}
#include <stan/model/model_header.hpp>

namespace some_namespace {

using std::istream;
using std::string;
using std::stringstream;
using std::vector;
using stan::io::dump;
using stan::math::lgamma;
using stan::model::prob_grad;
using namespace stan::math;

typedef Eigen::Matrix<double,Eigen::Dynamic,1> vector_d;
typedef Eigen::Matrix<double,1,Eigen::Dynamic> row_vector_d;
typedef Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic> matrix_d;

static int current_statement_begin__;

stan::io::program_reader prog_reader__() {
    stan::io::program_reader reader;
    reader.add_event(0, 0, "start", "model181f4ea8e0c2_mc");
    reader.add_event(3, 3, "end", "model181f4ea8e0c2_mc");
    return reader;
}
// various function declarations and / or definitions
#include "/full/path/to/fib.hpp"
```
Thus, the definition of the `fib` function in the fib.hpp file need not be
enclosed in any particular namespace (which is a random string by default. The 
"meta-include" stan/model/model_header.hpp file reads as
```{r, echo = FALSE, eval = TRUE, comment = NA}
cat(readLines(system.file("include", "src", "stan", "model", "model_header.hpp", 
                          package = "StanHeaders")), sep = "\n")
```
so the definition of the `fib` function in the fib.hpp file could utilize any
function in the Stan Math Library (without having to prefix function calls with
`stan::math::`), some typedefs to classes in the Eigen matrix algebra library, 
plus streams, exceptions, etc. without having to worry about the corresponding
header files. Nevertheless, an external C++ file _may_ contain additional
include directives that bring in class definitions, for example.

Now let's examine the fib.hpp file, which contains the C++ lines
```{Rcpp, eval = FALSE}
int fib(const int&n, std::ostream* pstream__) {
  if (n <= 0) {
    stringstream errmsg;
    errmsg << "n must be positive";
    throw std::domain_error(errmsg.str());
  }
  return n <= 1 ? 1 : fib(n - 1, 0) + fib(n - 2, 0);
}
```
This C++ function is essentially what the preceding user-defined function in the 
Stan language
```{stan eval = FALSE, output.var="def"}
int fib(int n) {
  if (n <= 0) reject("n must be positive");
  return n <= 2 ? 1 : fib(n - 1) + fib(n - 2);
}
```
parses to. Thus, there is no _speed_ advantage to defining the `fib` function
in the external fib.hpp file. However, it is possible to use an external C++
file to handle the gradient of a function analytically as opposed to using
Stan's autodifferentiation capabilities, which are slower and more memory
intensive but fully general. In this case, the `fib` function only deals with
integers so there is nothing to take the derivative of. The primary advantage
of using an external C++ file is flexibility to do things that cannot be done
directly in the Stan language. It is also useful for R packages like
__rstanarm__ that may want to define some C++ functions in the package's src
directory and rely on the linker to make them available in its Stan programs,
which are compiled at (or before) installation time.

In the C++ version, we check if `n` is non-positive, in which case we throw an
exception. The way the MCMC samplers are configured, if there is an exception 
thrown and it is `std::domain_error`, it will treat that iteration as a rejection 
but as a recoverable error: set that iteration's log probability value to negative
infinity and move to the next iteration. If there is a `std::invalid_argument` 
thrown, MCMC terminates. We treat these as non-recoverable, user programming errors. 
It is unnecessary to prefix `stringstream` with `std::` because of
the `using std::stringstream;` line in the _generated_ C++ file. However, there
is no corresponding `using std::domain_error;` line, so it has to be qualified
appropriately when the exception is thrown.

The only confusing part of the C++ version of the `fib` function is that it has
an additional argument (with no default value) named `pstream__` that is added
to the _declaration_ of the `fib` function by Stan. Thus, your _definition_ of
the `fib` function needs to match with this signature. This additional argument 
is a pointer to a `std::ostream` and is only used if your function prints
something to the screen, which is rare. Thus, when we call the `fib` function 
recursively in the last line, we specify `fib(n - 1, 0) + fib(n - 2, 0);` so 
that the output (if any, and in this case there is none) is directed to the null
pointer.

This vignette has employed a toy example with the Fibonacci function, which has
little apparent use in a Stan program and if it were useful, would more easily 
be implemented as a user-defined function in the `functions` block as 
illustrated at the outset. The ability to use external C++ code only becomes
useful with more complicated C++ functions. It goes without saying that this
mechanism ordinarily cannot call functions in C, Fortran, R, or other languages
because Stan needs the derivatives with respect to unknown parameters in order
to perform estimation. These derivatives are handled with custom C++ types that
cannot be processed by functions in other languages that only handle primitive
types such as `double`, `float`, etc. 

That said, it is possible to accomplish a great deal in C++, particularly when 
utilizing the Stan Math Library. For more details, see
[The Stan Math Library: Reverse-Mode Automatic Differentiation in C++](https://arxiv.org/abs/1509.07164)
and its GitHub [repository](https://github.com/stan-dev/math/). The functions
that you _declare_ in the `functions` block of a Stan program will typically
involve templating and type promotion in their signatures when parsed to C++
(the only exceptions are functions whose only arguments are integers, as in
the `fib` function above). Suppose you wanted to define a function whose
arguments are real numbers (or at least one of the arguments is). For example,
```{r, eval = TRUE}
mc <- 
'
functions { real sinc(real x); }
transformed data { real sinc_pi = sinc(pi()); }
'
stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
           includes = paste0('\n#include "', 
                             file.path(getwd(), 'sinc.hpp'), '"\n'))
```
The sinc.hpp file reads as
```{r, echo = FALSE, eval = TRUE, comment=""}
cat(readLines("sinc.hpp"), sep = "\n")
```
The body of the first `sinc` function is simply its mathematical definition in the 
form of a ternary operator, which is sufficient when the input is a `double`. 

The last lines of sinc.hpp are a specialization for when the input is an
unknown real parameter, which is represented in the Stan Math Library as a
`stan::math::var`. Since the derivative of the `sinc` function is easy to
compute analytically, we extract the underlying double-precision value from
the inputted `stan::math::var` and use that to calculate the function value
and its first derivative. Then, we return the result of `precomputed_gradients`,
whose arguments are the function value (`f`), the derivative of `x` with
respect to any other parameters (`x.vi_`), and the first derivative of `f`
(`dfdx_`). The latter two are actually `std::vector`s but only have one
element each because there is only one unknown.

An easy way to see what the generated function signature will be is to call the
`stanc` function in the __rstan__ package with `allow_undefined = TRUE` and
inspect the resuling C++ code. In this case, I first did
```{r, eval = TRUE}
try(readLines(stanc(model_code = mc, allow_undefined = TRUE)$cppcode))
```
to see what function signatures needed to be written for sinc.hpp.

Once you go to the trouble of writing a numerically stable C++ function, we 
would welcome a pull request on GitHub to include your C++ function in the 
Stan Math Library for everyone to benefit from, provided that it can be licensed 
under the 3-clause BSD license and its use is not overly-specific to your 
particular Stan program.

The Stan Math Library is compliant with the C++14 standard but not all compilers
fully support the C++14 standard. In particular, the compiler that comes with
RTools does not support all C++14 features. But you can use many C++14 features,
such as the `auto` keyword.

---
title: "Simulation Based Calibration"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulation Based Calibration}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
# Nothing
```

Here is a Stan program for a beta-binomial model
```{stan output.var="beta_binomial"}
data {
  int<lower = 1> N;
  real<lower = 0> a;
  real<lower = 0> b;
}
transformed data { // these adhere to the conventions above
  real pi_ = beta_rng(a, b);
  int y = binomial_rng(N, pi_);
}
parameters {
  real<lower = 0, upper = 1> pi;
}
model {
  target += beta_lpdf(pi | a, b);
  target += binomial_lpmf(y | N, pi);
}
generated quantities { // these adhere to the conventions above
  int y_ = y;
  vector[1] pars_;
  int ranks_[1] = {pi > pi_};
  vector[N] log_lik;
  pars_[1] = pi_;
  for (n in 1:y) log_lik[n] = bernoulli_lpmf(1 | pi);
  for (n in (y + 1):N) log_lik[n] = bernoulli_lpmf(0 | pi);
}
```
Notice that it adheres to the following conventions:
* Realizations of the unknown parameters are drawn in the `transformed data`
  block are postfixed with an underscore, such as `pi_`.
  These are considered the "true" parameters being estimated by
  the corresponding symbol declared in the `parameters` block, which
  have the same names except for the trailing underscore, such as `pi`.
* The realizations of the unknown parameters are then conditioned on when drawing from
  the prior predictive distribution in `transformed data` block, which in this
  case is `int y = binomial_rng(N, pi_);`. To avoid confusion, `y` does not have a 
  training underscore.
* The realizations of the unknown parameters are copied into a `vector`
  in the `generated quantities` block named `pars_`
* The realizations from the prior predictive distribution are copied 
  into an object (of the same type) in the `generated quantities` block
  named `y_. This is optional.
* The `generated quantities` block contains an integer array named 
  `ranks_` whose only values are zero or one, depending on whether the realization of a 
  parameter from the posterior distribution exceeds the corresponding "true"
  realization, which in this case is `ranks_[1] = {pi > pi_};`. These are not actually "ranks"
  but can be used afterwards to reconstruct (thinned) ranks.
* The `generated quantities` block contains a vector named `log_lik` whose values
  are the contribution to the log-likelihood by each observation. In this case, 
  the "observations" are the implicit successes and failures that are aggregated
  into a binomial likelihood. This is optional but facilitates calculating the
  Pareto k shape parameter estimates that indicate whether the posterior distribution
  is sensitive to particular observations.

Assuming the above is compile to a code `stanmodel` named `beta_binomial`, we can
then call the `sbc` function
```{r, eval = FALSE}
output <- sbc(beta_binomial, data = list(N = 10, a = 1, b = 1), M = 500, refresh = 0)
```
```{r, include = FALSE}
# This fakes what would happen if we actually took the time to run Stan.
N <- 10
M <- 500
pars_ <- rbeta(M, 1, 1)
y_ <- matrix(rbinom(pars_, size = N, prob = pars_), ncol = M)
post_ <- matrix(rbeta(M * 1000L, 1 + y_, 1 + N - y_), ncol = M)
ranks_ <- lapply(1:M, FUN = function(m) {
  matrix(post_[ , m] > pars_[m], ncol = 1, 
         dimnames = list(NULL, "pi"))
})
log_lik <- t(sapply(1:M, FUN = function(m) {
  c(dbinom(rep(1, y_[m]), size = 1, prob = pars_[m], log = TRUE),
    dbinom(rep(1, N - y_[m]), size = 1, prob = pars_[m], log = TRUE))
}))
sampler_params <- array(0, dim = c(1000, 6, M))
colnames(sampler_params) <- c("accept_stat__", "stepsize__", "treedepth__",   
                              "n_leapfrog__",  "divergent__",   "energy__")
output <- list(ranks = ranks_, Y = y_, pars = pars_, 
               log_lik = log_lik, sampler_params = sampler_params)
class(output) <- "sbc"
```
At which point, we can then call
```{r}
print(output)
plot(output, bins = 10) # it is best to specify the bins argument yourself
```

# References

  Talts, S., Betancourt, M., Simpson, D., Vehtari, A., and Gelman, A. (2018).
  Validating Bayesian Inference Algorithms with Simulation-Based Calibration.
  [arXiv preprint arXiv:1804.06788](https://arxiv.org/abs/1804.06788)

---
title: "Accessing the contents of a stanfit object"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{stanfit objects}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE,
  comment = NA,
  fig.align = "center",
  fig.height = 5,
  fig.width = 7
  )
```

This vignette demonstrates how to access most of data stored in a stanfit 
object. A stanfit object (an object of class `"stanfit"`) contains the output 
derived from fitting a Stan model using Markov chain Monte Carlo or one of
Stan's variational approximations (meanfield or full-rank). Throughout the
document we'll use the stanfit object obtained from fitting the Eight Schools
example model:

```{r, example-model, eval=FALSE}
library(rstan)
fit <- stan_demo("eight_schools", refresh = 0)
```
```{r, fit, echo=FALSE, cache=FALSE, results="hide"}
J <- 8
y <- c(28,  8, -3,  7, -1,  1, 18, 12)
sigma <- c(15, 10, 16, 11,  9, 11, 10, 18)
fit <- stan(
  file= "schools.stan", 
  model_name = "eight_schools",
  data = c("y", "J", "sigma"), 
  refresh = 0
  )
```
```{r, stanfit-class}
class(fit)
```

## Posterior draws

There are several functions that can be used to access the draws from the 
posterior distribution stored in a stanfit object. These are `extract`, 
`as.matrix`, `as.data.frame`, and `as.array`, each of which returns the draws in
a different format.

<br>

#### extract()

The `extract` function (with its default arguments) returns a list with named
components corresponding to the model parameters.

```{r, extract-1}
list_of_draws <- extract(fit)
print(names(list_of_draws))
```

In this model the parameters `mu` and `tau` are scalars and `theta` is a vector
with eight elements. This means that the draws for `mu` and `tau` will be 
vectors (with length equal to the number of post-warmup iterations times the 
number of chains) and the draws for `theta` will be a matrix, with each column 
corresponding to one of the eight components:

```{r, extract-2}
head(list_of_draws$mu)
head(list_of_draws$tau)
head(list_of_draws$theta)
```

<br>

#### as.matrix(), as.data.frame(), as.array()

The `as.matrix`, `as.data.frame`, and `as.array` functions can also be used
to retrieve the posterior draws from a stanfit object:

```{r, as.matrix-1}
matrix_of_draws <- as.matrix(fit)
print(colnames(matrix_of_draws))

df_of_draws <- as.data.frame(fit)
print(colnames(df_of_draws))

array_of_draws <- as.array(fit)
print(dimnames(array_of_draws))
```

The `as.matrix` and `as.data.frame` methods essentially return the same 
thing except in matrix and data frame form, respectively. The `as.array`
method returns the draws from each chain separately and so has an additional
dimension:

```{r, as.matrix-2, results="hold"}
print(dim(matrix_of_draws))
print(dim(df_of_draws))
print(dim(array_of_draws))
```

By default all of the functions for retrieving the posterior draws return the
draws for _all_ parameters (and generated quantities). The optional argument 
`pars` (a character vector) can be used if only a subset of the parameters is
desired, for example:

```{r, as.matrix-3}
mu_and_theta1 <- as.matrix(fit, pars = c("mu", "theta[1]"))
head(mu_and_theta1)
```

<br>

## Posterior summary statistics and convergence diagnostics

Summary statistics are obtained using the `summary` function. The object 
returned is a list with two components:

```{r, summary-1}
fit_summary <- summary(fit)
print(names(fit_summary))
```

In `fit_summary$summary` all chains are merged whereas `fit_summary$c_summary`
contains summaries for each chain individually. Typically we want the 
summary for all chains merged, which is what we'll focus on here.

The summary is a matrix with rows corresponding to parameters and columns to the
various summary quantities. These include the posterior mean, the posterior
standard deviation, and various quantiles computed from the draws. The `probs`
argument can be used to specify which quantiles to compute and `pars` can be
used to specify a subset of parameters to include in the summary.

For models fit using MCMC, also included in the summary are the Monte Carlo
standard error (`se_mean`), the effective sample size (`n_eff`), and the R-hat
statistic (`Rhat`).

```{r, summary-2}
print(fit_summary$summary)
```

If, for example, we wanted the only quantiles included to be 10% and 90%, and  for only the parameters included to be `mu` and `tau`, we would specify that like this:

```{r, summary-3}
mu_tau_summary <- summary(fit, pars = c("mu", "tau"), probs = c(0.1, 0.9))$summary
print(mu_tau_summary)
```

Since `mu_tau_summary` is a matrix we can pull out columns using their names:

```{r, summary-4}
mu_tau_80pct <- mu_tau_summary[, c("10%", "90%")]
print(mu_tau_80pct)
```


<br>

## Sampler diagnostics

For models fit using MCMC the  stanfit object will also contain the values of
parameters used for the sampler. The `get_sampler_params` function can 
be used to access this information.

The object returned by `get_sampler_params` is a list with one component (a
matrix) per chain. Each of the matrices has number of columns corresponding to
the number of sampler parameters and the column names provide the parameter
names. The optional argument inc_warmup (defaulting to `TRUE`) indicates whether
to include the warmup period.

```{r, get_sampler_params-1}
sampler_params <- get_sampler_params(fit, inc_warmup = FALSE)
sampler_params_chain1 <- sampler_params[[1]]
colnames(sampler_params_chain1)
```

To do things like calculate the average value of `accept_stat__` for each chain 
(or the maximum value of `treedepth__` for each chain if using the NUTS
algorithm, etc.) the `sapply` function is useful as it will apply the 
same function to each component of `sampler_params`:

```{r, get_sampler_params-2}
mean_accept_stat_by_chain <- sapply(sampler_params, function(x) mean(x[, "accept_stat__"]))
print(mean_accept_stat_by_chain)

max_treedepth_by_chain <- sapply(sampler_params, function(x) max(x[, "treedepth__"]))
print(max_treedepth_by_chain)
```

<br>

## Model code

The Stan program itself is also stored in the stanfit object and can be 
accessed using `get_stancode`:

```{r, get_stan_code-1}
code <- get_stancode(fit)
```

The object `code` is a single string and is not very intelligible when printed:

```{r, get_stan_code-2}
print(code)
```

A readable version can be printed using `cat`:
```{r, get_stan_code-3}
cat(code)
```

<br>

## Initial values

The `get_inits` function returns initial values as a list with one component per
chain. Each component is itself a (named) list containing the initial values for each parameter for the corresponding chain:

```{r, get_inits}
inits <- get_inits(fit)
inits_chain1 <- inits[[1]]
print(inits_chain1)
```

<br>

## (P)RNG seed

The `get_seed` function returns the (P)RNG seed as an integer:

```{r, get_seed}
print(get_seed(fit))
```

<br>

## Warmup and sampling times

The `get_elapsed_time` function returns a matrix with the warmup and sampling
times for each chain: 

```{r, get_elapsed_time}
print(get_elapsed_time(fit))
```

---
title: "RStan: the R interface to Stan"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
bibliography: rstan.bib
vignette: >
  %\VignetteIndexEntry{RStan}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE, error = TRUE,
  comment = NA,
  fig.align = "center",
  fig.height = 5,
  fig.width = 7
  )
```

In this vignette we present RStan, the R interface to Stan. Stan is a C++
library for Bayesian inference using the No-U-Turn sampler (a variant of
Hamiltonian Monte Carlo) or frequentist inference via optimization. We
illustrate the features of RStan through an example in
@GelmanCarlinSternRubin:2003.


## Introduction 

Stan is a C++ library for Bayesian modeling and inference that primarily uses
the No-U-Turn sampler (NUTS) [@hoffman-gelman:2012] to obtain posterior
simulations given a user-specified model and data. Alternatively, Stan can
utilize the LBFGS optimization algorithm to maximize an objective function, such
as a log-likelihood. The R package __rstan__ provides RStan, the R interface to
Stan. The __rstan__ package allows one to conveniently fit Stan models from R
[@rprj] and access the output, including posterior inferences and intermediate
quantities such as evaluations of the log posterior density and its gradients.

In this vignette we provide a concise introduction to the functionality included
in the __rstan__ package. Stan's website [mc-stan.org](https://mc-stan.org) has 
additional details and provides up-to-date information about how to operate both
Stan and its many interfaces including RStan. See, for example, _RStan Getting 
Started_ [@rstangettingstarted2012].


## Prerequisites

Stan has a modeling language, which is similar to but not identical to that of
the Bayesian graphical modeling package BUGS [@WinBUGS]. A parser translates a
model expressed in the Stan language to C++ code, whereupon it is compiled to an
executable program and loaded as a Dynamic Shared Object (DSO) in R which can
then be called by the user.

A C++ compiler, such as [`g++`](http://gcc.gnu.org) or 
[`clang++`](http://clang.llvm.org), is required for this process. For
instructions on installing a C++ compiler for use with RStan see 
[RStan-Getting-Started](https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started).

The __rstan__ package also depends heavily on several other R packages:

* __StanHeaders__  (Stan C++ headers)
* __BH__ (Boost C++ headers)
* __RcppEigen__ (Eigen C++ headers)
* __Rcpp__ (facilitates using C++ from R)
* __inline__ (compiles C++ for use with R)

These dependencies should be automatically installed if you install the 
__rstan__ package via one of the conventional mechanisms.

## Typical Workflow

The following is a typical workflow for using Stan via RStan for Bayesian
inference.

1. Represent a statistical model by writing its log posterior density (up to an 
normalizing constant that does not depend on the unknown parameters in the 
model) using the Stan modeling language. We recommend using a separate file with
a `.stan` extension, although it can also be done using a character string
within R.
2. Translate the Stan program to C++ code using the `stanc` function.
3. Compile the C++ code to create a DSO (also called a dynamic link library
(DLL)) that can be loaded by R.
4. Run the DSO to sample from the posterior distribution.
5. Diagnose non-convergence of the MCMC chains.
6. Conduct inference based on the posterior sample (the MCMC draws from the
posterior distribution).

Conveniently, steps 2, 3, and 4, above, are all performed implicitly by a single
call to the `stan` function.


## Example

Throughout the rest of the vignette we'll use a hierarchical meta-analysis model
described in section 5.5 of @GelmanCarlinSternRubin:2003 as a running example. A
hierarchical model is used to model the effect of coaching programs on college
admissions tests. The data, shown in the table below, summarize the results of
experiments conducted in eight high schools, with an estimated standard error
for each. These data and model are of historical interest as an example of full
Bayesian inference [@Rubin1981]. For short, we call this the _Eight
Schools_ examples.

School | Estimate ($y_j$) | Standard Error ($\sigma_j$)
------ | -------- | --------------
A      | 28       | 15
B      | 8        | 10
C      | -3       | 16
D      | 7        | 11
E      | -1       | 9
F      | 1        | 11
G      | 18       | 10
H      | 12       | 18

We use the Eight Schools example here because it is simple but also represents a
nontrivial Markov chain simulation problem in that there is dependence between
the parameters of original interest in the study --- the effects of coaching in
each of the eight schools --- and the hyperparameter representing the variation
of these effects in the modeled population.  Certain implementations of a Gibbs
sampler or a Hamiltonian Monte Carlo sampler can be slow to converge in this
example.

The statistical model of interest is specified as

$$
\begin{aligned} 
y_j &\sim \mathsf{Normal}(\theta_j, \sigma_j), \quad j=1,\ldots,8 \\
\theta_j &\sim \mathsf{Normal}(\mu, \tau), \quad j=1,\ldots,8 \\
p(\mu, \tau) &\propto 1,
\end{aligned}
$$

where each $\sigma_j$ is assumed known.


### Write a Stan Program

RStan allows a Stan program to be coded in a text file
(typically with suffix `.stan`) or in a R character vector (of length one). We
put the following code for the Eight Schools model into the file `schools.stan`:

```{r, echo=FALSE, comment=""}
cat(readLines("schools.stan"), sep = "\n")
```

The first section of the Stan program above, the `data` block, specifies the 
data that is conditioned upon in Bayes Rule: the number of schools, $J$, the 
vector of estimates, $(y_1, \ldots, y_J)$, and the vector of standard errors of
the estimates $(\sigma_{1}, \ldots, \sigma_{J})$. Data are declared as integer
or real and can be vectors (or, more generally, arrays) if dimensions are 
specified. Data can also be constrained; for example, in the above model $J$ has
been restricted to be at least $1$ and the components of $\sigma_y$ must all be 
positive.

The `parameters` block declares the parameters whose posterior distribution is
sought. These are the the mean, $\mu$, and standard deviation, $\tau$, of the
school effects, plus the _standardized_ school-level effects $\eta$. In this
model, we let the unstandardized school-level effects, $\theta$, be a
transformed parameter constructed by scaling the standardized effects by
$\tau$ and shifting them by $\mu$ rather than directly declaring $\theta$ as a
parameter. By parameterizing the model this way, the sampler runs more 
efficiently because the resulting multivariate geometry is more amendable to 
Hamiltonian Monte Carlo [@Neal:2011].

Finally, the `model` block looks similar to standard statistical notation.
(Just be careful:  the second argument to Stan's normal$(\cdot,\cdot)$
distribution is the standard deviation, not the variance as is usual in
statistical notation). We have written the model in vector notation, which
allows Stan to make use of more efficient algorithmic differentiation (AD). It
would also be possible --- but less efficient --- to write the model by
replacing `normal_lpdf(y | theta,sigma)` with a loop over the $J$ schools,

```
for (j in 1:J) 
  target += normal_lpdf(y[j] | theta[j],sigma[j]);
```

Stan has versions of many of the most useful R functions for statistical 
modeling, including probability distributions, matrix operations, and various
special functions. However, the names of the Stan functions may differ from
their R counterparts and, more subtly, the parameterizations of probability
distributions in Stan may differ from those in R for the same distribution. To
mitigate this problem, the `lookup` function can be passed an R function or
character string naming an R function, and RStan will attempt to look up the
corresponding Stan function, display its arguments, and give the page number in
@StanManual where the function is discussed.

```{r, lookup}
lookup("dnorm")
lookup(dwilcox)   # no corresponding Stan function
```

If the `lookup` function fails to find an R function that corresponds to a 
Stan function, it will treat its argument as a regular expression and attempt to
find matches with the names of Stan functions.

### Preparing the Data

The `stan` function accepts data as a named list, a character vector of object
names, or an `environment`. Alternatively, the `data` argument can be omitted
and R will search for objects that have the same names as those declared in the
`data` block of the Stan program. Here is the data for the Eight Schools
example:

```{r, schools-data}
schools_data <- list(
  J = 8,
  y = c(28,  8, -3,  7, -1,  1, 18, 12),
  sigma = c(15, 10, 16, 11,  9, 11, 10, 18)
)
```

It would also be possible (indeed, encouraged) to read in the data from a file
rather than to directly enter the numbers in the R script.


### Sample from the Posterior Distribution

Next, we can call the `stan` function to draw posterior samples:

```{r, callstan, results="hide"}
library(rstan)
fit1 <- stan(
  file = "schools.stan",  # Stan program
  data = schools_data,    # named list of data
  chains = 4,             # number of Markov chains
  warmup = 1000,          # number of warmup iterations per chain
  iter = 2000,            # total number of iterations per chain
  cores = 1,              # number of cores (could use one per chain)
  refresh = 0             # no progress shown
  )
```

The `stan` function wraps the following three steps: 

* Translate a model in Stan code to C++ code 
* Compile the C++ code to a dynamic shared object (DSO) and load the DSO
* Sample given some user-specified data and other settings

A single call to `stan` performs all three steps, but they can also be executed 
one by one (see the help pages for `stanc`, `stan_model`, and `sampling`), which
can be useful for debugging. In addition, Stan saves the DSO so that when the 
same model is fit again (possibly with new data and settings) we can avoid
recompilation. If an error happens after the model is compiled but before
sampling (e.g., problems with inputs like data and initial values), we can still
reuse the compiled model.

The `stan` function returns a stanfit object, which is an S4 object of class 
`"stanfit"`. For those who are not familiar with the concept of class and S4 
class in R, refer to @chambers2010software. An S4 class consists of some
attributes (data) to model an object and some methods to model the behavior of
the object. From a user's perspective, once a stanfit object is created, we are
mainly concerned about what methods are defined.

If no error occurs, the returned stanfit object includes the sample drawn from 
the posterior distribution for the model parameters and other quantities defined
in the model. If there is an error (e.g. a syntax error in the Stan program),
`stan` will either quit or return a stanfit object that contains no posterior
draws.

For class `"stanfit"`, many methods such as `print` and `plot` are defined for
working with the MCMC sample. For example, the following shows a summary of the
parameters from the Eight Schools model using the `print` method:

```{r, print}
print(fit1, pars=c("theta", "mu", "tau", "lp__"), probs=c(.1,.5,.9))
```

The last line of this output, `lp__`, is the logarithm of the (unnormalized)
posterior density as calculated by Stan.  This log density can be used in
various ways for model evaluation and comparison (see, e.g., @Vehtari2012).

#### Arguments to the `stan` Function

The primary arguments for sampling (in functions `stan` and `sampling`) include 
data, initial values, and the options of the sampler such as `chains`, `iter`, 
and `warmup`. In particular, `warmup` specifies the number of iterations that 
are used by the NUTS sampler for the adaptation phase before sampling begins. 
After the warmup, the sampler turns off adaptation and continues until a total 
of `iter` iterations (including `warmup`) have been completed. There is no 
theoretical guarantee that the draws obtained during warmup are from the
posterior distribution, so the warmup draws should only be used for diagnosis
and not inference. The summaries for the parameters shown by the `print` method
are calculated using only post-warmup draws.

The optional `init` argument can be used to specify initial values for the
Markov chains. There are several ways to specify initial values, and the details
can be found in the documentation of the `stan` function. The vast majority of
the time it is adequate to allow Stan to generate its own initial values
randomly. However, sometimes it is better to specify the initial values for at
least a subset of the objects declared in the `parameters` block of a Stan
program.

Stan uses a random number generator (RNG) that supports parallelism. The 
initialization of the RNG is determined by the arguments `seed` and `chain_id`. 
Even if we are running multiple chains from one call to the `stan` function we 
only need to specify one seed, which is randomly generated by R if not 
specified.

#### Data Preprocessing and Passing

The data passed to `stan` will go through a preprocessing procedure. The details
of this preprocessing are documented in the documentation for the `stan` 
function. Here we stress a few important steps. First, RStan allows the user to
pass more objects as data than what is declared in the `data` block (silently 
omitting any unnecessary objects). In general, an element in the list of data passed
to Stan from R should be numeric and its dimension should match the declaration
in the `data` block of the model. So for example, the `factor` type in R is not
supported as a data element for RStan and must be converted to integer codes via
`as.integer`. The Stan modeling language distinguishes between integers and
doubles (type `int` and `real` in Stan modeling language, respectively). The
`stan` function will convert some R data (which is double-precision usually) to
integers if possible.

The Stan language has scalars and other types that are sets of scalars, e.g. 
vectors, matrices, and arrays. As R does not have true scalars, RStan treats
vectors of length one as scalars. However, consider a model with a `data` block
defined as

```
data {                
  int<lower=1> N;      
  real y[N];
} 
```

in which `N` can be $1$ as a special case. So if we know that `N` is always 
larger than $1$, we can use a vector of length `N` in R as the data input for 
`y` (for example, a vector created by `y <- rnorm(N)`). If we want to prevent 
RStan from treating the input data for `y` as a scalar when $N`$ is $1$, we
need to explicitly make it an array as the following R code shows:

```
y <- as.array(y)
```

Stan cannot handle missing values in data automatically, so no element of the 
data can contain `NA` values. An important step in RStan's data preprocessing is
to check missing values and issue an error if any are found. There are, however,
various ways of writing Stan programs that account for missing data (see
@StanManual).


### Methods for the `"stanfit"` Class

The other vignette included with the __rstan__ package discusses stanfit objects
in greater detail and gives examples of accessing the most important content 
contained in the objects (e.g., posterior draws, diagnostic summaries). Also, a
full list of available methods can be found in the documentation for the 
`"stanfit"` class at `help("stanfit", "rstan")`. Here we give only a few 
examples.

The `plot` method for stanfit objects provides various graphical overviews of 
the output. The default plot shows posterior uncertainty intervals (by default 
80% (inner) and 95% (outer)) and the posterior median for all the parameters as 
well as `lp__` (the log of posterior density function up to an additive 
constant):

```{r, stanfit-plot}
plot(fit1)
```

The optional `plotfun` argument can be used to select among the various 
available plots. See `help("plot,stanfit-method")`.

The `traceplot` method is used to plot the time series of the posterior draws. 
If we include the warmup draws by setting `inc_warmup=TRUE`, the background
color of the warmup area is different from the post-warmup phase:

```{r, stanfit-traceplot}
traceplot(fit1, pars = c("mu", "tau"), inc_warmup = TRUE, nrow = 2)
```

To assess the convergence of the Markov chains, in addition to visually 
inspecting traceplots we can calculate the split $\hat{R}$ statistic. Split 
$\hat{R}$ is an updated version of the $\hat{R}$ statistic proposed in
@GelmanRubin:1992 that is based on splitting each chain into two halves. See the
Stan manual for more details. The estimated $\hat{R}$ for each parameter is 
included as one of the columns in the output from the `summary` and `print`
methods.

```{r, stanfit-print}
print(fit1, pars = c("mu", "tau"))
```

Again, see the additional vignette on stanfit objects for more details.


### Sampling Difficulties

The best way to visualize the output of a model is through the ShinyStan 
interface, which can be accessed via the 
[__shinystan__](https://cran.r-project.org/package=shinystan) R package. 
ShinyStan facilitates both the visualization of parameter distributions and
diagnosing problems with the sampler. The documentation for the __shinystan__
package provides instructions for using the interface with stanfit objects.

In addition to using ShinyStan, it is also possible to diagnose some 
sampling problems using functions in the __rstan__ package. The
`get_sampler_params` function returns information on parameters
related the performance of the sampler:

```{r, get_sampler_params}
# all chains combined
sampler_params <- get_sampler_params(fit1, inc_warmup = TRUE)
summary(do.call(rbind, sampler_params), digits = 2)

# each chain separately
lapply(sampler_params, summary, digits = 2)
```

Here we see that there are a small number of divergent transitions, which are 
identified by `divergent__` being $1$. Ideally, there should be no divergent 
transitions after the warmup phase. The best way to try to eliminate divergent 
transitions is by increasing the target acceptance probability, which by default
is $0.8$. In this case the mean of `accept_stat__` is close to $0.8$ for all 
chains, but has a very skewed distribution because the median is near $0.95$. We
could go back and call `stan` again and specify the optional argument 
`control=list(adapt_delta=0.9)` to try to eliminate the divergent transitions. 
However, sometimes when the target acceptance rate is high, the stepsize is very
small and the sampler hits its limit on the number of leapfrog steps it can take
per iteration. In this case, it is a non-issue because each chain has a 
`treedepth__` of at most $7$ and the default is $10$. But if any `treedepth__` 
were $11$, then it would be wise to increase the limit by passing 
`control=list(max_treedepth=12)` (for example) to `stan`. See the vignette on
stanfit objects for more on the structure of the object returned by
`get_sampler_params`.

We can also make a graphical representation of (much of the) the same 
information using `pairs`. The "pairs"" plot can be used to get a sense of
whether any sampling difficulties are occurring in the tails or near the mode:

```{r, pairs-plot}
pairs(fit1, pars = c("mu", "tau", "lp__"), las = 1)
```

In the plot above, the marginal distribution of each selected parameter is 
included as a histogram along the diagonal. By default, draws with below-median 
`accept_stat__` (MCMC proposal acceptance rate) are plotted below the diagonal 
and those with above-median `accept_stat__` are plotted above the diagonal (this
can be changed using the `condition` argument). Each off-diagonal square 
represents a bivariate distribution of the draws for the intersection of the 
row-variable and the column-variable. Ideally, the below-diagonal intersection 
and the above-diagonal intersection of the same two variables should have 
distributions that are mirror images of each other. Any yellow points would 
indicate transitions where the maximum `treedepth__` was hit, and red points 
indicate a divergent transition.


## Additional Topics

### User-defined Stan Functions

Stan also permits users to define their own functions that can be used 
throughout a Stan program. These functions are defined in the `functions` block.
The `functions` block is optional but, if it exists, it must come before any
other block. This mechanism allows users to implement statistical distributions
or other functionality that is not currently available in Stan. However, even if
the user's function merely wraps calls to existing Stan functions, the code in
the `model` block can be much more readible if several lines of Stan code that
accomplish one (or perhaps two) task(s) are replaced by a call to a user-defined
function.

Another reason to utilize user-defined functions is that RStan provides the 
`expose_stan_functions` function for exporting such functions to the R global
environment so that they can be tested in R to ensure they are working
properly. For example,

```{r, expose_stan_functions}
model_code <-
'
functions {
  real standard_normal_rng() {
    return normal_rng(0,1);
  }
}
model {}
'
expose_stan_functions(stanc(model_code = model_code))
standard_normal_rng()
```


### The Log-Posterior (function and gradient)

Stan defines the log of the probability density function of a posterior 
distribution up to an unknown additive constant. We use `lp__` to represent the 
realizations of this log kernel at each iteration (and `lp__` is treated as an 
unknown in the summary and the calculation of split $\hat{R}$ and effective 
sample size).

A nice feature of the __rstan__ package is that it exposes functions for 
calculating both `lp__` and its gradients for a given stanfit object. These two 
functions are `log_prob` and `grad_log_prob`, respectively. Both take parameters
on the _unconstrained_ space, even if the support of a parameter is not the 
whole real line. The Stan manual [@StanManual] has full details on the 
particular transformations Stan uses to map from the entire real line to some 
subspace of it (and vice-versa).

It maybe the case that the number of unconstrained parameters might be less than
the total number of parameters. For example, for a simplex parameter of length 
$K$, there are actually only $K-1$ unconstrained parameters because of the 
constraint that all elements of a simplex must be nonnegative and sum to one. 
The `get_num_upars` method is provided to get the number of unconstrained 
parameters, while the `unconstrain_pars` and `constrain_pars` methods can be 
used to compute unconstrained and constrained values of parameters respectively.
The former takes a list of parameters as input and transforms it to an 
unconstrained vector, and the latter does the opposite. Using these functions, 
we can implement other algorithms such as maximum a posteriori estimation of 
Bayesian models.

### Optimization in Stan

RStan also provides an interface to Stan's optimizers, which can be used to
obtain a point estimate by maximizing the (perhaps penalized) likelihood 
function defined by a Stan program. We illustrate this feature using a very
simple example: estimating the mean from samples assumed to be drawn from a
normal distribution with known standard deviation. That is, we assume

$$y_n \sim \mathsf{Normal}(\mu,1), \quad n = 1, \ldots, N. $$

By specifying a prior $p(\mu) \propto 1$, the maximum a posteriori estimator for
$\mu$ is just the sample mean. We don't need to explicitly code this prior for
$\mu$, as $p(\mu) \propto 1$ is the default if no prior is specified.

We first create an object of class `"stanmodel"` and then use the `optimizing`
method, to which data and other arguments can be fed.

```{r, optimizer, results="hide"}
ocode <- "
  data {
    int<lower=1> N;
    real y[N];
  } 
  parameters {
    real mu;
  } 
  model {
    target += normal_lpdf(y | mu, 1);
  } 
"

sm <- stan_model(model_code = ocode)
y2 <- rnorm(20)
```

```{r}
mean(y2)
optimizing(sm, data = list(y = y2, N = length(y2)), hessian = TRUE)
```


### Model Compilation

As mentioned earlier in the vignette, Stan programs are written in the Stan
modeling language, translated to C++ code, and then compiled to a dynamic shared
object (DSO). The DSO is then loaded by R and executed to draw the posterior
sample. The process of compiling C++ code to DSO sometimes takes a while. When
the model is the same, we can reuse the DSO from a previous run. The `stan`
function accepts the optional argument `fit`, which can be used to pass an
existing fitted model object so that the compiled model is reused. When reusing
a previous fitted model, we can still specify different values for the other
arguments to `stan`, including passing different data to the `data` argument.

In addition, if fitted models are saved using functions like `save` and 
`save.image`, RStan is able to save DSOs, so that they can be used across R 
sessions. To avoid saving the DSO, specify `save_dso=FALSE` when calling the 
`stan` function.

If the user executes `rstan_options(auto_write = TRUE)`, then a serialized 
version of the compiled model will be automatically saved to the hard disk in 
the same directory as the `.stan` file or in R's temporary directory if the Stan
program is expressed as a character string. Although this option is not enabled 
by default due to CRAN policy, it should ordinarily be specified by users in 
order to eliminate redundant compilation.

Stan runs much faster when the code is compiled at the maximum level of 
optimization, which is `-O3` on most C++ compilers. However, the default value 
is `-O2` in R, which is appropriate for most R packages but entails a slight 
slowdown for Stan. You can change this default locally by following the 
instructions at 
[CRAN - Customizing-package-compilation](https://cran.r-project.org/doc/manuals/r-release/R-admin.html#Customizing-package-compilation).
However, you should be advised that setting `CXXFLAGS = -O3` may cause adverse 
side effects for other R packages.

See the documentation for the `stanc` and `stan_model` functions for more
details on the parsing and compilation of Stan programs.

### Running Multiple Chains in Parallel

The number of Markov chains to run can be specified using the `chains` argument 
to the `stan` or `sampling` functions. By default, the chains are executed 
serially (i.e., one at a time) using the parent R process. There is also an 
optional `cores` argument that can be set to the number of chains (if the 
hardware has sufficient processors and RAM), which is appropriate on most 
laptops. We typically recommend first calling 
`options(mc.cores=parallel::detectCores())` once per R session so that all
available cores can be used without needing to manually specify the `cores`
argument.

For users working with a different parallelization scheme (perhaps with a remote
cluster), the __rstan__ package provides a function called `sflist2stanfit` for
consolidating a list of multiple stanfit objects (created from the same Stan
program and using the same number of warmup and sampling iterations) into a
single stanfit object. It is important to specify the same seed for all the
chains and equally important to use a different chain ID (argument `chain_id`),
the combination of which ensures that the random numbers generated in Stan for
all chains are essentially independent. This is handled automatically
(internally) when $`cores` > 1$.


## Working with CmdStan

The __rstan__ package provides some functions for creating data for and reading 
output from CmdStan, the command line interface to Stan.

First, when Stan reads data or initial values, it supports a subset of the 
syntax of R dump data formats. So if we use the `dump` function in base R to 
prepare data, Stan might not be able to read the contents. The `stan_rdump` 
function in __rstan__ is designed to dump the data from R to a format that is 
supported by Stan, with semantics that are very similar to the `dump` function.

Second, the `read_stan_csv` function creates a stanfit object from reading the 
CSV files generated by CmdStan. The resulting stanfit object is compatible with
the various methods for diagnostics and posterior analysis.


## See Also

* The [Stan Forums](https://discourse.mc-stan.org/) on Discourse
* The [other vignettes](https://mc-stan.org/rstan/articles/)
for the __rstan__ package, which show how to access the contents of stanfit
objects and use external C++ in a Stan program.
* The very thorough [Stan manual](https://mc-stan.org/users/documentation/) [@StanManual].
* The `stan_demo` function, which can be used to fit many of the example models
in the manual.
* The [__bayesplot__](https://mc-stan.org/bayesplot/) package for visual MCMC diagnostics, posterior predictive checking, and other 
plotting (ggplot based).
* The [__shinystan__](https://mc-stan.org/shinystan/) R
package, which provides a GUI for exploring MCMC output.
* The [__loo__](http://mc-stan.org/loo/) R
package, which is very useful for model comparison using stanfit objects.
* The [__rstanarm__](https://mc-stan.org/rstanarm/) R
package, which provides a `glmer`-style interface to Stan.

---------

---
title: "Interfacing with External C++ Code"
author: "Stan Development Team"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Interfacing with External C++ Code}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(rstan)
knitr::opts_chunk$set(
  echo = TRUE, eval = FALSE, error = TRUE
)
```

Starting with the 2.13 release, it is much easier to use external C++ code in a 
Stan program. This vignette briefly illustrates how to do so. 

Suppose that you have (part of) a Stan program that involves Fibonacci numbers, 
such as
```{stan eval = FALSE, output.var = "fib"}
functions {
  int fib(int n);
  int fib(int n) {
    if (n <= 0) reject("n must be positive");
    return n <= 2 ? 1 : fib(n - 1) + fib(n - 2);
  }
}
model {} // use the fib() function somehow
```
On the second line, we have _declared_ the `fib` function before it is _defined_
in order to call it recursively. 

For functions that are not recursive, it is not necessary to declare them before 
defining them but it may be advantageous. For example, I often like to hide the
definitions of complicated utility functions that are just a distraction using
the `#include "file"` mechanism
```{stan eval = FALSE, output.var="includes"}
functions {
  real complicated(real a, real b, real c, real d, real e, real f, real g);
#include "complicated.stan"
}
model {} // use the complicated() function somehow
```
This Stan program would have to be parsed using the `stanc_builder` function in
the __rstan__ package rather than the default `stanc` function (which is called
by `sampling` and `stan` internally).

Returning to the Fibonacci example, it is not necessary to define the `fib`
function using the Stan language because Stan programs with functions that are 
_declared_ but not _defined_ can use the standard capabilities of the C++ 
toolchain to provide the function definitions in C++. For example, this program
produces a parser error by default
```{r, eval = TRUE}
mc <- 
'
functions { int fib(int n); }
model {} // use the fib() function somehow
'
try(stan_model(model_code = mc, model_name = "parser_error"), silent = TRUE)
```
However, if we specify the `allow_undefined` and `includes` arguments to the 
`stan_model` function, and define a `fib` function in the named C++ header file, 
then it will parse and compile
```{r}
stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
           includes = paste0('\n#include "', 
                             file.path(getwd(), 'fib.hpp'), '"\n'))
```
Specifying the `includes` argument is a bit awkward because the C++ 
representation of a Stan program is written and compiled in a temporary
directory. Thus, the `includes` argument must specify a _full_ path to the
fib.hpp file, which in this case is in the working directory. Also, the
path must be enclosed in double-quotes, which is why single quotes are used
in the separate arguments to the `paste0` function so that double-quotes are
interpreted literally. Finally, the `includes` argument should include newline
characters (`"\n"`) at the start and end. It is possible to specify multiple
paths using additional newline characters or include a "meta-header" file
that contains `#include` directives to other C++ header files.

The result of the `includes` argument is inserted into the C++ file directly
at the end of the lines (as opposed to CmdStan where it is inserted directly
_before_ the following lines)
```{Rcpp, eval = FALSE}
#include <stan/model/model_header.hpp>

namespace some_namespace {

using std::istream;
using std::string;
using std::stringstream;
using std::vector;
using stan::io::dump;
using stan::math::lgamma;
using stan::model::prob_grad;
using namespace stan::math;

typedef Eigen::Matrix<double,Eigen::Dynamic,1> vector_d;
typedef Eigen::Matrix<double,1,Eigen::Dynamic> row_vector_d;
typedef Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic> matrix_d;

static int current_statement_begin__;

// various function declarations and / or definitions
#include "/full/path/to/fib.hpp"
```
Thus, the definition of the `fib` function in the fib.hpp file need not be
enclosed in any particular namespace (which is a random string by default. The 
"meta-include" stan/model/model_header.hpp file reads as
```{r, echo = FALSE, eval = TRUE, comment = NA}
cat(readLines(system.file("include", "src", "stan", "model", "model_header.hpp", 
                          package = "StanHeaders")), sep = "\n")
```
so the definition of the `fib` function in the fib.hpp file could utilize any
function in the Stan Math Library (without having to prefix function calls with
`stan::math::`), some typedefs to classes in the Eigen matrix algebra library, 
plus streams, exceptions, etc. without having to worry about the corresponding
header files. Nevertheless, an external C++ file _may_ contain additional
include directives that bring in class definitions, for example.

Now let's examine the fib.hpp file, which contains the C++ lines
```{Rcpp, eval = FALSE}
int fib(const int&n, std::ostream* pstream__) {
  if (n <= 0) {
    stringstream errmsg;
    errmsg << "n must be positive";
    throw std::domain_error(errmsg.str());
  }
  return n <= 1 ? 1 : fib(n - 1, 0) + fib(n - 2, 0);
}
```
This C++ function is essentially what the preceding user-defined function in the 
Stan language
```{stan eval = FALSE, output.var="def"}
int fib(int n) {
  if (n <= 0) reject("n must be positive");
  return n <= 2 ? 1 : fib(n - 1) + fib(n - 2);
}
```
parses to. Thus, there is no _speed_ advantage to defining the `fib` function
in the external fib.hpp file. However, it is possible to use an external C++
file to handle the gradient of a function analytically as opposed to using
Stan's autodifferentiation capabilities, which are slower and more memory
intensive but fully general. In this case, the `fib` function only deals with
integers so there is nothing to take the derivative of. The primary advantage
of using an external C++ file is flexibility to do things that cannot be done
directly in the Stan language. It is also useful for R packages like
__rstanarm__ that may want to define some C++ functions in the package's src
directory and rely on the linker to make them available in its Stan programs,
which are compiled at (or before) installation time.

In the C++ version, we check if `n` is non-positive, in which case we throw an
exception. The way the MCMC samplers are configured, if there is an exception 
thrown and it is `std::domain_error`, it will treat that iteration as a rejection 
but as a recoverable error: set that iteration's log probability value to negative
infinity and move to the next iteration. If there is a `std::invalid_argument` 
thrown, MCMC terminates. We treat these as non-recoverable, user programming errors. 
It is unnecessary to prefix `stringstream` with `std::` because of
the `using std::stringstream;` line in the _generated_ C++ file. However, there
is no corresponding `using std::domain_error;` line, so it has to be qualified
appropriately when the exception is thrown.

The only confusing part of the C++ version of the `fib` function is that it has
an additional argument (with no default value) named `pstream__` that is added
to the _declaration_ of the `fib` function by Stan. Thus, your _definition_ of
the `fib` function needs to match with this signature. This additional argument 
is a pointer to a `std::ostream` and is only used if your function prints
something to the screen, which is rare. Thus, when we call the `fib` function 
recursively in the last line, we specify `fib(n - 1, 0) + fib(n - 2, 0);` so 
that the output (if any, and in this case there is none) is directed to the null
pointer.

This vignette has employed a toy example with the Fibonacci function, which has
little apparent use in a Stan program and if it were useful, would more easily 
be implemented as a user-defined function in the `functions` block as 
illustrated at the outset. The ability to use external C++ code only becomes
useful with more complicated C++ functions. It goes without saying that this
mechanism ordinarily cannot call functions in C, Fortran, R, or other languages
because Stan needs the derivatives with respect to unknown parameters in order
to perform estimation. These derivatives are handled with custom C++ types that
cannot be processed by functions in other languages that only handle primitive
types such as `double`, `float`, etc. 

That said, it is possible to accomplish a great deal in C++, particularly when 
utilizing the Stan Math Library. For more details, see
[The Stan Math Library: Reverse-Mode Automatic Differentiation in C++](https://arxiv.org/abs/1509.07164)
and its GitHub [repository](https://github.com/stan-dev/math/). The functions
that you _declare_ in the `functions` block of a Stan program will typically
involve templating and type promotion in their signatures when parsed to C++
(the only exceptions are functions whose only arguments are integers, as in
the `fib` function above). Suppose you wanted to define a function whose
arguments are real numbers (or at least one of the arguments is). For example,
```{r, eval = TRUE}
mc <- 
'
functions { real sinc(real x); }
transformed data { real sinc_pi = sinc(pi()); }
'
stan_model(model_code = mc, model_name = "external", allow_undefined = TRUE,
           includes = paste0('\n#include "', 
                             file.path(getwd(), 'sinc.hpp'), '"\n'))
```
The sinc.hpp file reads as
```{r, echo = FALSE, eval = TRUE, comment=""}
cat(readLines("sinc.hpp"), sep = "\n")
```
The body of the first `sinc` function is simply its mathematical definition in the 
form of a ternary operator, which is sufficient when the input is a `double`. 

The last lines of sinc.hpp are a specialization for when the input is an
unknown real parameter, which is represented in the Stan Math Library as a
`stan::math::var`. Since the derivative of the `sinc` function is easy to
compute analytically, we extract the underlying double-precision value from
the inputted `stan::math::var` and use that to calculate the function value
and its first derivative. Then, we return the result of `precomputed_gradients`,
whose arguments are the function value (`f`), the derivative of `x` with
respect to any other parameters (`x.vi_`), and the first derivative of `f`
(`dfdx_`). The latter two are actually `std::vector`s but only have one
element each because there is only one unknown.

An easy way to see what the generated function signature will be is to call the
`stanc` function in the __rstan__ package with `allow_undefined = TRUE` and
inspect the resuling C++ code. In this case, I first did
```{r, eval = TRUE}
try(readLines(stanc(model_code = mc, allow_undefined = TRUE)$cppcode))
```
to see what function signatures needed to be written for sinc.hpp.

Once you go to the trouble of writing a numerically stable C++ function, we 
would welcome a pull request on GitHub to include your C++ function in the 
Stan Math Library for everyone to benefit from, provided that it can be licensed 
under the 3-clause BSD license and its use is not overly-specific to your 
particular Stan program.

The Stan Math Library is compliant with the C++14 standard but not all compilers
fully support the C++14 standard. In particular, the compiler that comes with
RTools does not support all C++14 features. But you can use many C++14 features,
such as the `auto` keyword.

\name{Plots}
\alias{stan_plot}
\alias{stan_trace}
\alias{stan_dens}
\alias{stan_hist}
\alias{stan_scat}
\alias{stan_ac}
\alias{quietgg}

\title{ggplot2 for RStan}
\description{
  Visual posterior analysis using ggplot2.
}
\usage{
  stan_plot(object, pars, include = TRUE, unconstrain = FALSE, ...)
  stan_trace(object, pars, include = TRUE, unconstrain = FALSE,
            inc_warmup = FALSE, nrow = NULL, ncol = NULL, ..., 
            window = NULL)
  stan_scat(object, pars, unconstrain = FALSE,
            inc_warmup = FALSE, nrow = NULL, ncol = NULL, ...)
  stan_hist(object, pars, include = TRUE, unconstrain = FALSE,
            inc_warmup = FALSE, nrow = NULL, ncol = NULL, ...)
  stan_dens(object, pars, include = TRUE, unconstrain = FALSE,
            inc_warmup = FALSE, nrow = NULL, ncol = NULL, ..., 
            separate_chains = FALSE)
  stan_ac(object, pars, include = TRUE, unconstrain = FALSE,
            inc_warmup = FALSE, nrow = NULL, ncol = NULL, ..., 
            separate_chains = FALSE, lags = 25, partial = FALSE)
  quietgg(gg)
}
\arguments{
  \item{object}{A stanfit or stanreg object.}
  
  \item{pars}{Optional character vector of parameter names. 
    If \code{object} is a stanfit object, the default is to show
    all user-defined parameters or the first 10 (if there are more 
    than 10). If \code{object} is a stanreg object, the default is 
    to show all (or the first 10) regression coefficients 
    (including the intercept). For \code{stan_scat} only, 
    \code{pars} should not be missing and should contain exactly 
    two parameter names.}
  
  \item{include}{Should the parameters given by the \code{pars} argument be
    included (the default) or excluded from the plot?}
  
  \item{unconstrain}{Should parameters be plotted on the unconstrained space?
    Defaults to \code{FALSE}. Only available if \code{object} is a 
    stanfit object.}
  
  \item{inc_warmup}{Should warmup iterations be included? Defaults to
    \code{FALSE}.}

  \item{nrow,ncol}{Passed to \code{\link[ggplot2]{facet_wrap}}.}
  
  \item{...}{Optional additional named arguments passed to geoms 
  (e.g. for \code{stan_trace} the geom is \code{\link[ggplot2]{geom_path}} 
  and we could specify \code{linetype}, \code{size}, \code{alpha}, etc.). 
  For \code{stan_plot} there are also additional arguments that can be specified 
  in \code{...} (see Details).}
  
  \item{window}{For \code{stan_trace} \code{window} is used to control
  which iterations are shown in the plot. See \code{\link{traceplot}}.}
  
  \item{separate_chains}{For \code{stan_dens}, should the density for each 
  chain be plotted? The default is \code{FALSE}, which means that for each 
  parameter the draws from all chains are combined. For \code{stan_ac}, 
  if \code{separate_chains=FALSE} (the default), the autocorrelation is
  averaged over the chains. If \code{TRUE} each chain is plotted separately.}
  
  \item{lags}{For \code{stan_ac}, the maximum number of lags to show.}
  
  \item{partial}{For \code{stan_ac}, should partial autocorrelations be 
  plotted instead? Defaults to \code{FALSE}.}
  
  \item{gg}{A ggplot object or an expression that creates one.}
}
\value{
  A \code{\link[ggplot2]{ggplot}} object that can be further customized
  using the \pkg{ggplot2} package.
}
\note{
Because the \pkg{rstan} plotting functions use \pkg{ggplot2} (and thus the 
resulting plots behave like \code{ggplot} objects), when calling a plotting 
function within a loop or when assigning a plot to a name 
(e.g., \code{graph <- plot(fit, plotfun = "rhat")}),
if you also want the side effect of the plot being displayed you 
must explicity print it (e.g., \code{(graph <- plot(fit, plotfun = "rhat"))}, 
\code{print(graph <- plot(fit, plotfun = "rhat"))}).
}
\details{
For \code{stan_plot}, there are additional arguments that can be specified in 
\code{...}. The optional arguments and their default values are:
\describe{
  \item{\code{point_est = "median"}}{The point estimate to show. 
    Either "median" or "mean".
    }
  \item{\code{show_density = FALSE}}{Should kernel density estimates be plotted 
    above the intervals?
    }
  \item{\code{ci_level = 0.8}}{The posterior uncertainty interval to highlight.
    Central \code{100*ci_level}\% intervals are computed from the quantiles of 
    the posterior draws.}
  \item{\code{outer_level = 0.95}}{An outer interval to also draw as a line 
    (if \code{show_outer_line} is \code{TRUE}) but not highlight.
    }
  \item{\code{show_outer_line = TRUE}}{Should the \code{outer_level} interval 
    be shown or hidden? Defaults to = \code{TRUE} (to plot it).
    }
  \item{\code{fill_color}, \code{outline_color}, \code{est_color}}{Colors to 
    override the defaults for the highlighted interval, the outer interval 
    (and density outline), and the point estimate.
    }
  }
}
\seealso{
\code{\link[=rstan-plotting-functions]{List of RStan plotting functions}},
\code{\link[=rstan_gg_options]{Plot options}}
}
\examples{
\dontrun{
example("read_stan_csv")
stan_plot(fit)
stan_trace(fit)

library(gridExtra)
fit <- stan_demo("eight_schools")

stan_plot(fit)
stan_plot(fit, point_est = "mean", show_density = TRUE, fill_color = "maroon")


# histograms
stan_hist(fit)
# suppress ggplot2 messages about default bindwidth
quietgg(stan_hist(fit))
quietgg(h <- stan_hist(fit, pars = "theta", binwidth = 5)) 

# juxtapose histograms of tau and unconstrained tau 
tau <- stan_hist(fit, pars = "tau")
tau_unc <- stan_hist(fit, pars = "tau", unconstrain = TRUE) + 
            xlab("tau unconstrained")
grid.arrange(tau, tau_unc)

# kernel density estimates
stan_dens(fit)
(dens <- stan_dens(fit, fill = "skyblue", ))
dens <- dens + ggtitle("Kernel Density Estimates\\n") + xlab("")
dens

(dens_sep <- stan_dens(fit, separate_chains = TRUE, alpha = 0.3))
dens_sep + scale_fill_manual(values = c("red", "blue", "green", "black"))
(dens_sep_stack <- stan_dens(fit, pars = "theta", alpha = 0.5,
                             separate_chains = TRUE, position = "stack"))

# traceplot
trace <- stan_trace(fit)
trace +
  scale_color_manual(values = c("red", "blue", "green", "black"))
trace +
  scale_color_brewer(type = "div") +
  theme(legend.position = "none")

facet_style <- theme(strip.background = ggplot2::element_rect(fill = "white"),
                     strip.text = ggplot2::element_text(size = 13, color = "black"))
(trace <- trace + facet_style)

# scatterplot
(mu_vs_tau <- stan_scat(fit, pars = c("mu", "tau"), color = "blue", size = 4))
mu_vs_tau + 
  ggplot2::coord_flip() + 
  theme(panel.background = ggplot2::element_rect(fill = "black"))
  
}
}

\docType{methods}
\name{summary-methods}
\alias{summary,stanfit-method}

\title{Summary method for stanfit objects}
\description{
Summarize the distributions of estimated parameters and derived quantities 
using the posterior draws.
}

\usage{
  \S4method{summary}{stanfit}(object, pars, probs = c(0.025, 0.25, 0.50, 0.75, 0.975),
  use_cache = TRUE, ...) 
}

\arguments{
  \item{object}{An instance of class \code{\linkS4class{stanfit}}.}
  \item{pars}{A character vector of parameter names. Defaults to all parameters
  as well as the log-posterior (\code{lp__}).}
  \item{probs}{A numeric vector of \code{\link[stats]{quantile}}s of interest. 
    The default is \code{c(0.025,0.25,0.5,0.75,0.975)}.}
  \item{use_cache}{Logical, defaulting to \code{TRUE}. When \code{use_cache=TRUE}
    the summary quantities for all parameters are computed and cached for future
    use. Setting \code{use_cache=FALSE} can be used to avoid performing the 
    summary computations for all parameters if \code{pars} is given as some 
    specific parameters.}
  \item{\dots}{Currently unused.}
}
\value{
The \code{summary} method returns a named list with elements \code{summary} 
and \code{c_summary}, which contain summaries for for all chains merged and 
individual chains, respectively.
Included in the summaries are quantiles, means, standard deviations (sd),
effective sample sizes (n_eff), and split Rhats (the potential scale
reduction derived from all chains after splitting each chain in half and
treating the halves as chains). For the summary of all chains merged, 
Monte Carlo standard errors (se_mean) are also reported.
}

\seealso{
\itemize{
\item \code{\link{monitor}}, which computes similar summaries but accepts an 
array of MCMC draws as its input rather than a \code{stanfit} object.
\item The RStan vignettes for more example usage.
}
}

\examples{\dontrun{
ecode <- '
  parameters {
    array[2] real<lower=0> y;
  } 
  model {
    y ~ exponential(1);
  }
'
fit <- stan(model_code = ecode)
s <- summary(fit, probs = c(0.1, 0.9))
s$summary  # all chaines merged
s$c_summary  # individual chains
}
}
\keyword{methods}

\name{rstan-plotting-functions}
\alias{rstan-plotting-functions}

\title{RStan Plotting Functions}
\description{
  List of RStan plotting functions that return ggplot objects
}
\section{RStan plotting functions}{
\describe{
  \item{Posterior intervals and point estimates}{\code{\link{stan_plot}}}
  \item{Traceplots}{\code{\link{stan_trace}}}
  \item{Histograms}{\code{\link{stan_hist}}}
  \item{Kernel density estimates}{\code{\link{stan_dens}}}
  \item{Scatterplots}{\code{\link{stan_scat}}}
  \item{Diagnostics for Hamiltonian Monte Carlo and the No-U-Turn Sampler}{\code{\link{stan_diag}}}
  \item{Rhat}{\code{\link{stan_rhat}}}
  \item{Ratio of effective sample size to total posterior sample size}{\code{\link{stan_ess}}}
  \item{Ratio of Monte Carlo standard error to posterior standard deviation}{\code{\link{stan_mcse}}}
  \item{Autocorrelation}{\code{\link[rstan]{stan_plot}}}
}
}
\seealso{
\code{\link[=rstan_gg_options]{Plot options}}
}

\name{stanfit-class} 
\Rdversion{1.1}

\docType{class}
\alias{stanfit-class}
\alias{stanfit}
\alias{show,stanfit-method}
\alias{get_cppo_mode} 
\alias{get_cppo_mode,stanfit-method} 
\alias{get_stancode}
\alias{get_stancode,stanfit-method}
\alias{get_stanmodel}
\alias{get_stanmodel,stanfit-method}
\alias{get_seed}
\alias{get_seed,stanfit-method}
\alias{get_seeds}
\alias{get_seeds,stanfit-method}
\alias{get_inits}
\alias{get_inits,stanfit-method}
\alias{get_posterior_mean}
\alias{get_posterior_mean,stanfit-method}
\alias{get_elapsed_time}
\alias{get_elapsed_time,stanfit-method}
\alias{get_logposterior} 
\alias{get_logposterior,stanfit-method}
\alias{get_adaptation_info} 
\alias{get_adaptation_info,stanfit-method}
\alias{get_sampler_params}
\alias{get_sampler_params,stanfit,logical-method}

\title{Class \code{stanfit}: fitted Stan model}

\description{
The components (slots) of a \code{stanfit} object and the various available
methods are described below. When methods have their own more detailed 
documentation pages links are provided. 
} 

\section{Objects from the Class}{
An object of class \code{stanfit} contains the
output derived from fitting a Stan model as returned by the top-level function
\code{\link{stan}} or the lower-level methods \code{\link{sampling}} and
\code{\link{vb}} (which are defined on class \code{\linkS4class{stanmodel}}).
Many methods (e.g., \code{print}, \code{plot}, \code{summary}) are provided for 
summarizing results and various access methods also allow the underlying data 
(e.g., simulations, diagnostics) contained in the object to be retrieved.
}

\section{Slots}{
  \describe{
    \item{\code{model_name}:}{The model name as a string.} 
    \item{\code{model_pars}:}{A character vector of names of parameters
      (including transformed parameters and derived quantities).}
    \item{\code{par_dims}:}{A named list giving the dimensions for all 
      parameters. The dimension for a scalar parameter is given as 
      \code{numeric(0)}.}
    \item{\code{mode}:}{An integer indicating the mode of the fitted model.
       \code{0} indicates sampling mode, \code{1} indicates test gradient mode 
       (no sampling is done), and \code{2} indicates error mode (an error occurred
       before sampling). Most methods for \code{stanfit} objects are useful only 
       if \code{mode=0}.}
    \item{\code{sim}:}{A list containing simulation results including the 
      posterior draws as well as various pieces of metadata used by many of the 
      methods for \code{stanfit} objects.} 
    \item{\code{inits}:}{The initial values (either user-specified or generated
      randomly) for all chains. This is a list with one component per chain. Each
      component is a named list containing the initial values for each parameter
      for the corresponding chain.}
    \item{\code{stan_args}:}{A list with one component per chain containing the 
      arguments used for sampling (e.g. \code{iter}, \code{seed}, etc.).}
    \item{\code{stanmodel}:}{The instance of S4 class \code{\linkS4class{stanmodel}}.} 
    \item{\code{date}:}{A string containing the date and time the object was created.}
    \item{\code{.MISC}:}{Miscellaneous helper information used for the fitted model. 
      This is an object of type \code{environment}. Users rarely (if ever)
      need to access the contents of \code{.MISC}.}
  }
}

\section{Methods}{
  \strong{Printing, plotting, and summarizing:}
  \describe{
    \item{\code{show}}{Print the default summary for the model.}
    \item{\code{print}}{Print a customizable summary for the model. 
      See \code{\link{print.stanfit}}.}
    \item{\code{plot}}{Create various plots summarizing the fitted model. 
      See \code{\link{plot,stanfit-method}}.}
    \item{\code{summary}}{Summarize the distributions of estimated 
      parameters and derived quantities using the posterior draws. 
      See \code{\link{summary,stanfit-method}}.}
    \item{\code{get_posterior_mean}}{
      Get the posterior mean for parameters of interest (using \code{pars}
      to specify a subset of parameters). Returned is a matrix with 
      one column per chain and an additional column for all chains combined.}
  }
  \strong{Extracting posterior draws:}
  \describe{
    \item{\code{extract}}{Extract the draws for all chains for all 
      (or specified) parameters. See \code{\link{extract}}.}
    \item{\code{as.array}, \code{as.matrix}, \code{as.data.frame}}{
      Coerce the draws (without warmup) to an array, 
      matrix or data frame. See \code{\link{as.array.stanfit}}.}
    \item{\code{As.mcmc.list}}{Convert a \code{stanfit} object to an
      \code{\link[coda]{mcmc.list}} as in package \pkg{coda}. 
      See \code{\link{As.mcmc.list}}.}
    \item{\code{get_logposterior}}{
      Get the log-posterior at each iteration.
      Each element of the returned \code{list} is the vector of log-posterior
      values (up to an additive constant, i.e. up to a multiplicative constant 
      on the linear scale) for a single chain.
      The optional argument \code{inc_warmup} (defaulting to \code{TRUE}) 
      indicates whether to include the warmup period.}
  }
  \strong{Diagnostics, log probability, and gradients:}
  \describe{
    \item{\code{get_sampler_params}}{
      Obtain the parameters used for the sampler such as 
      \code{stepsize} and \code{treedepth}. The results are returned
      as a list with one component (an array) per chain.
      The array has number of columns corresponding to the number
      of parameters used in the sampler and its column names 
      provide the parameter names. Optional argument \code{inc_warmup} 
      (defaulting to \code{TRUE}) indicates whether to include the warmup period.}
    \item{\code{get_adaptation_info}}{
      Obtain the adaptation information for the sampler if NUTS was used. 
      The results are returned as a list, each element of which is a character 
      string with the info for a single chain.} 
    \item{\code{log_prob}}{
      Compute the log probability density (\code{lp__}) for a set of parameter 
      values (on the \emph{unconstrained} space) up to an additive constant. 
      The unconstrained parameters are specified using a numeric vector.
      The number of parameters on the unconstrained space can be obtained 
      using method \code{get_num_upars}. A numeric value is returned. See also
      the documentation in \code{\link{log_prob}}.} 
    \item{\code{grad_log_prob}}{
      Compute the gradient of log probability density function for a set of parameter 
      values (on the \emph{unconstrained} space) up to an additive constant.
      The unconstrained parameters are specified using a numeric vector
      with the length being the number of unconstrained parameters.
      A numeric vector is returned with the length of the number of 
      unconstrained parameters and an attribute named \code{log_prob} being
      the \code{lp__}. See also the documentation in \code{\link{grad_log_prob}}.}
    \item{\code{get_num_upars}}{
      Get the number of unconstrained parameters of the model. The number of parameters 
      for a model is not necessarily equal to this number of unconstrained 
      parameters. For example, when a parameter is specified as a simplex of 
      length K, the number of unconstrained parameters is K-1.} 
    \item{\code{unconstrain_pars}}{
      Transform the parameters to unconstrained space. The input is a named list 
      as for specifying initial values for each parameter. A numeric vector is
      returned. See also the documentation in \code{\link{unconstrain_pars}}.}
    \item{\code{constrain_pars}}{
      Get the parameter values from their unconstrained space. The input is a 
      numeric vector. A list is returned.  This function is contrary to
      \code{unconstrain_pars}. See also the documentation in 
      \code{\link{constrain_pars}}.}
  }
  \strong{Metadata and miscellaneous:}
  \describe{
    \item{\code{get_stancode}}{
      Get the Stan code for the fitted model as a string. The result can 
      be printed in a readable format using \code{\link{cat}}.}
    \item{\code{get_stanmodel}}{
      Get the object of S4 class \code{\linkS4class{stanmodel}} of the fitted 
      model.}
    \item{\code{get_elapsed_time}}{
      Get the warmup time and sample time in seconds.
      A matrix of two columns is returned with each row containing the warmup
      and sample times for one chain.}
    \item{\code{get_inits, iter = NULL}}{
      Get the initial values for parameters used in sampling all chains. The 
      returned object is a list with the same structure as the \code{inits} 
      slot described above. If \code{object@mode=2} (error mode) an empty list 
      is returned. If \code{iter} is not \code{NULL}, then the draw from that
      iteration is returned for each chain rather than the initial state.}
    \item{\code{get_cppo_mode}}{
      Get the optimization mode used for compilation. The returned string is 
      one of \code{"fast"}, \code{"presentation2"}, \code{"presentation1"},
      and \code{"debug"}.}
    \item{\code{get_seed}}{
      Get the (P)RNG seed used. When the fitted object 
      is empty (\code{mode=2}), \code{NULL} might be returned.
      In the case that the seeds for all chains are different, use 
      \code{get_seeds}.} 
    \item{\code{get_seeds}}{
      Get the seeds used for all chains. When the fitted object 
      is empty (\code{mode=2}), \code{NULL} might be returned.} 
  }
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org}.
}
\seealso{
  \code{\link{stan}} and \code{\linkS4class{stanmodel}} 
}
\examples{\dontrun{
showClass("stanfit")
ecode <- '
  parameters {
    array[2] real<lower=0> y;
  } 
  model {
    y ~ exponential(1);
  }
'
fit <- stan(model_code = ecode, iter = 10, chains = 1)
fit2 <- stan(fit = fit)
print(fit2)
plot(fit2)
traceplot(fit2)
ainfo <- get_adaptation_info(fit2)
cat(ainfo[[1]])
seed <- get_seed(fit2)
sp <- get_sampler_params(fit2)
sp2 <- get_sampler_params(fit2, inc_warmup = FALSE)
head(sp[[1]])

lp <- log_prob(fit, c(1, 2))
grad <- grad_log_prob(fit, c(1, 2))
lp2 <- attr(grad, "log_prob") # should be the same as "lp"

# get the number of parameters on the unconstrained space
n <- get_num_upars(fit)

# parameters on the positive real line (constrained space) 
y1 <- list(y = rep(1, 2)) 

uy <- unconstrain_pars(fit, y1) 
## uy should be c(0, 0) since here the log transformation is used
y1star <- constrain_pars(fit, uy)

print(y1)
print(y1star) # y1start should equal to y1 
}

# Create a stanfit object from reading CSV files of samples (saved in rstan
# package) generated by funtion stan for demonstration purpose from model as follows. 
# 
excode <- '
  transformed data {
    array[20] real y;
    y[1] <- 0.5796;  y[2]  <- 0.2276;   y[3] <- -0.2959; 
    y[4] <- -0.3742; y[5]  <- 0.3885;   y[6] <- -2.1585;
    y[7] <- 0.7111;  y[8]  <- 1.4424;   y[9] <- 2.5430; 
    y[10] <- 0.3746; y[11] <- 0.4773;   y[12] <- 0.1803; 
    y[13] <- 0.5215; y[14] <- -1.6044;  y[15] <- -0.6703; 
    y[16] <- 0.9459; y[17] <- -0.382;   y[18] <- 0.7619;
    y[19] <- 0.1006; y[20] <- -1.7461;
  }
  parameters {
    real mu;
    real<lower=0, upper=10> sigma;
    vector[2] z[3];
    real<lower=0> alpha;
  } 
  model {
    y ~ normal(mu, sigma);
    for (i in 1:3) 
      z[i] ~ normal(0, 1);
    alpha ~ exponential(2);
  } 
'

# exfit <- stan(model_code = excode, save_dso = FALSE, iter = 200, 
#               sample_file = "rstan_doc_ex.csv")
# 

exfit <- read_stan_csv(dir(system.file('misc', package = 'rstan'),
                       pattern='rstan_doc_ex_[[:digit:]].csv',
                       full.names = TRUE))

print(exfit)
\dontrun{
plot(exfit)
}

adaptinfo <- get_adaptation_info(exfit)
inits <- get_inits(exfit) # empty
inits <- get_inits(exfit, iter = 101)
seed <- get_seed(exfit)
sp <- get_sampler_params(exfit)
ml <- As.mcmc.list(exfit)
cat(get_stancode(exfit))
}

\keyword{classes}

\name{rstan-internal}
\alias{rstan-internal}
\alias{get_cppcode,ANY-method}
\alias{get_logposterior,ANY-method}
\alias{get_adaptation_info,ANY-method}
\alias{get_sampler_params,ANY-method}
\alias{get_posterior_mean,ANY-method}
\alias{get_elapsed_time,ANY-method}
\alias{get_num_upars,ANY-method}
\alias{log_prob,ANY-method}
\alias{grad_log_prob,ANY-method}
\alias{unconstrain_pars,ANY-method}
\alias{constrain_pars,ANY-method}
\alias{cpp_object_initializer}
\alias{cxxdso-class}
\alias{show,cxxdso-method}
\alias{grab_cxxfun,cxxdso-method}
\alias{is_dso_loaded,cxxdso-method}
\alias{RNG}
\alias{OUT}
\title{Internal Functions and Methods}
\description{
These functions are not intended to be called directly. In the case of
methods, they are documented elsewhere, either with the associated generic
function or with the class of the object for which the method is defined.
}
\keyword{internal}

\name{stanc}
\alias{stanc}
\alias{stanc_builder}
\docType{package}
\title{
Translate Stan model specification to C++ code
}
\description{
  Translate a model specification in Stan code to C++ code, which can
  then be compiled and loaded for sampling.
}
\usage{
  stanc(
    file, model_code = '', model_name = "anon_model", verbose = FALSE,
    obfuscate_model_name = TRUE,
    allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
    allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
    standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
    use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
    warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
    warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE)),
    isystem = c(if (!missing(file)) dirname(file), getwd()))
  stanc_builder(
    file, isystem = c(dirname(file), getwd()),
    verbose = FALSE, obfuscate_model_name = FALSE,
    allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
    allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
    standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
    use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
    warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
    warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE)))
}

\arguments{
  \item{file}{A character string or a connection that \R supports
    specifying the Stan model specification in Stan's modeling language.}
  \item{model_code}{Either a character string containing a Stan model
    specification or the name of a character string object in the workspace.
    This parameter is used only if parameter \code{file} is not specified,
    so it defaults to the empty string.}
  \item{model_name}{A character string naming the model. The
    default is \code{"anon_model"}. However, the model name will be derived
    from \code{file} or \code{model_code} (if \code{model_code} is the name of a
    character string object) if \code{model_name} is not specified.}
  \item{verbose}{Logical, defaulting to \code{FALSE}. If \code{TRUE}
    more intermediate information is printed during the translation procedure.}
  \item{obfuscate_model_name}{Logical, defaulting to \code{TRUE},
    indicating whether to use a randomly-generated character string for the
    name of the C++ class. This prevents name clashes when compiling multiple
    models in the same R session.}
  \item{isystem}{A character vector naming a path to look for
    file paths in \code{file} that are to be included within the Stan program
    named by \code{file}. See the Details section below.}
  \item{allow_undefined}{A logical scalar defaulting to \code{FALSE} indicating
    whether to allow Stan functions to be declared but not defined in \code{file}
    or \code{model_code}. If \code{TRUE}, then it is the caller's responsibility
    to provide a function definition in another header file or linked shared object.}
  \item{allow_optimizations}{A logical scalar defaulting to \code{FALSE} indicating
    whether to allow level-1 compiler optimization to the \pkg{Stan} code.}
  \item{standalone_functions}{A logical scalar defaulting to \code{FALSE} indicating
    whether to generate the standalone functions \pkg{C++} code.}
  \item{use_opencl}{A logical scalar defaulting to \code{FALSE} indicating
    whether to try to use \code{matrix_cl} signatures.}
  \item{warn_pedantic}{A logical scalar defaulting to \code{FALSE} indicating
    whether to emit warnings about common mistakes in \pkg{Stan} programs.}
  \item{warn_uninitialized}{A logical scalar defaulting to \code{FALSE} indicating
    whether emit warnings about uninitialized variables.}
}

\details{
  The \code{stanc_builder} function supports the standard C++ convention of
  specifying something like \code{#include "my_includes.txt"} on an entire line
  within the file named by the \code{file} argument. In other words,
  \code{stanc_builder} would look for \code{"my_includes.txt"} in (or under) the
  directories named by the \code{isystem} argument and --- if found --- insert its
  contents verbatim at that position before calling \code{stanc} on the resulting
  \code{model_code}. This mechanism reduces the need to copy common chunks of code
  across Stan programs. It is possible to include such files recursively.

  Note that line numbers referred to in parser warnings or errors refer to the
  postprocessed Stan program rather than \code{file}. In the case of a parser
  error, the postprocessed Stan program will be printed after the error message.
  Line numbers referred to in messages while Stan is executing also refer to
  the postprocessed Stan program which can be obtained by calling
  \code{\link{get_stancode}}.
}

\value{
  A list with named entries:
  \enumerate{
    \item \code{model_name} Character string for the model name.
    \item \code{model_code} Character string for the model's Stan specification.
    \item \code{cppcode}    Character string for the model's C++ code.
    \item \code{status}     Logical indicating success/failure (always \code{TRUE})
                            of translating the Stan code.
  }
}

\note{
  Unlike \R, in which variable identifiers may contain dots (e.g. \code{a.1}),
  Stan prohibits dots from occurring in variable identifiers. Furthermore,
  C++ reserved words and Stan reserved words may not be used for variable names;
  see the Stan User's Guide for a complete list.
}

\references{
  The Stan Development Team
  \emph{Stan Modeling Language User's Guide and Reference Manual}.
  \url{https://mc-stan.org/}.

  The Stan Development Team
  \emph{CmdStan Interface User's Guide}.
  \url{https://mc-stan.org}.
}

\seealso{
  \code{\link{stan_model}} and \code{\link{stan}}
}

\examples{
stanmodelcode <- "
data {
  int<lower=0> N;
  array[N] real y;
}

parameters {
  real mu;
}

model {
  mu ~ normal(0, 10);
  y ~ normal(mu, 1);
}
"

r <- stanc(model_code = stanmodelcode, model_name = "normal1")
str(r)
}

\name{stan_model}
\alias{stan_model}
\title{Construct a Stan model}
\description{
  Construct an instance of S4 class \code{stanmodel} from a model
  specified in Stan's modeling language. A \code{stanmodel} object
  can then be used to draw samples from the model. The Stan program
  (the model expressed in the Stan modeling language) is first translated to
  C++ code and then the C++ code for the model plus other auxiliary
  code is compiled into a dynamic shared object (DSO) and then loaded.
  The loaded DSO for the model can be executed to draw samples, allowing
  inference to be performed for the model and data.
}

\usage{
  stan_model(
    file, model_name = "anon_model",
    model_code = "", stanc_ret = NULL,
    boost_lib = NULL, eigen_lib = NULL,
    save_dso = TRUE, verbose = FALSE,
    auto_write = rstan_options("auto_write"),
    obfuscate_model_name = TRUE,
    allow_undefined = isTRUE(getOption("stanc.allow_undefined", FALSE)),
    allow_optimizations = isTRUE(getOption("stanc.allow_optimizations", FALSE)),
    standalone_functions = isTRUE(getOption("stanc.standalone_functions", FALSE)),
    use_opencl = isTRUE(getOption("stanc.use_opencl", FALSE)),
    warn_pedantic = isTRUE(getOption("stanc.warn_pedantic", FALSE)),
    warn_uninitialized = isTRUE(getOption("stanc.warn_uninitialized", FALSE)),
    includes = NULL,
    isystem = c(if (!missing(file)) dirname(file), getwd()))
}

\arguments{
  \item{file}{A character string or a connection that \R supports
    specifying the Stan model specification in Stan's modeling language.
  }

  \item{model_name}{A character string naming the model; defaults
    to \code{"anon_model"}. However, the model name will be derived from
    \code{file} or \code{model_code} (if \code{model_code} is the name of a
    character string object) if \code{model_name} is not specified.}

  \item{model_code}{Either a character string containing the model
    specification or the name of a character string object in the workspace.
    This is an alternative to specifying the model via the \code{file}
    or \code{stanc_ret} arguments.
  }
  \item{stanc_ret}{A named list returned from a previous call to
    the \code{\link{stanc}} function. The list can be used to specify the model
    instead of using the \code{file} or \code{model_code} arguments.
  }
  \item{boost_lib}{The path to a version of the Boost C++ library to
    use instead of the one in the \pkg{BH} package.}
  \item{eigen_lib}{The path to a version of the Eigen C++ library to
    use instead of the one in the \pkg{RcppEigen} package.}
  \item{save_dso}{Logical, defaulting to \code{TRUE}, indicating
    whether the  dynamic shared object (DSO) compiled from the C++ code for the
    model will be saved or not. If \code{TRUE}, we can draw samples from
    the same model in another \R session using the saved DSO (i.e.,
    without compiling the C++ code again).}
  \item{verbose}{Logical, defaulting to \code{FALSE}, indicating whether
    to report additional intermediate output to the console,
    which might be helpful for debugging.}
  \item{auto_write}{Logical, defaulting to the value of
    \code{rstan_options("auto_write")}, indicating whether to write the
    object to the hard disk using \code{\link{saveRDS}}. Although this argument
    is \code{FALSE} by default, we recommend calling
    \code{rstan_options("auto_write" = TRUE)} in order to avoid unnecessary
    recompilations. If \code{file} is supplied and its \code{\link{dirname}}
    is writable, then the object will be written to that same directory,
    substituting a \code{.rds} extension for the \code{.stan} extension.
    Otherwise, the object will be written to the \code{\link{tempdir}}.}
  \item{obfuscate_model_name}{A logical scalar that is \code{TRUE} by default and
    passed to \code{\link{stanc}}.}
  \item{allow_undefined}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{allow_optimizations}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{standalone_functions}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{use_opencl}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{warn_pedantic}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{warn_uninitialized}{A logical scalar that is \code{FALSE} by default and
    passed to \code{\link{stanc}}.}
  \item{includes}{If not \code{NULL} (the default), then a character vector of
    length one (possibly containing one or more \code{"\n"}) of the form
    \code{'#include "/full/path/to/my_header.hpp"'}, which will be inserted
    into the C++ code in the model's namespace and can be used to provide definitions
    of functions that are declared but not defined in \code{file} or
    \code{model_code} when \code{allow_undefined = TRUE}}
  \item{isystem}{A character vector naming a path to look for
    file paths in \code{file} that are to be included within the Stan program
    named by \code{file}. See the Details section below.}
}

\details{
  If a previously compiled \code{stanmodel} exists on the hard drive, its validity
  is checked and then returned without recompiling. The most common form of
  invalidity seems to be Stan code that ends with a \code{\}} rather than a blank
  line, which causes the hash checker to think that the current model is different
  than the one saved on the hard drive. To avoid reading previously
  compiled \code{stanmodel}s from the hard drive, supply the \code{stanc_ret}
  argument rather than the \code{file} or \code{model_code} arguments.

  There are three ways to specify the model's code for \code{stan_model}:
  \enumerate{
  \item parameter \code{model_code}: a character string containing the
        Stan model specification,
  \item parameter \code{file}: a file name (or a connection) from
       which to read the Stan model specification, or
  \item parameter \code{stanc_ret}: a list returned by \code{stanc}
        to be reused.
  }
}
\value{
  An instance of S4 class \code{\linkS4class{stanmodel}} that can be
  passed to the \code{\link{sampling}}, \code{\link{optimizing}}, and
  \code{\link{vb}} functions.
}
\references{
  The Stan Development Team
  \emph{Stan Modeling Language User's Guide and Reference Manual}.
  \url{https://mc-stan.org/}.
}
\seealso{
  \code{\linkS4class{stanmodel}} for details on the class.

  \code{\link{sampling}}, \code{\link{optimizing}}, and \code{\link{vb}},
  which take a \code{stanmodel} object as input, for estimating the model
  parameters.

  More details on Stan, including the full user's guide and reference
  manual, can be found at \url{https://mc-stan.org/}.
}
\examples{\dontrun{
stancode <- 'data {real y_mean;} parameters {real y;} model {y ~ normal(y_mean,1);}'
mod <- stan_model(model_code = stancode, verbose = TRUE)
fit <- sampling(mod, data = list(y_mean = 0))
fit2 <- sampling(mod, data = list(y_mean = 5))
}}
\keyword{rstan}

\docType{methods}
\name{traceplot}
\alias{traceplot} 
\alias{traceplot,stanfit-method}

\title{Markov chain traceplots}
\description{Draw the traceplot corresponding to one or more
 Markov chains, providing a visual way to inspect sampling behavior
 and assess mixing across chains and convergence.}

\usage{
  %% traceplot(object, \dots)  
  \S4method{traceplot}{stanfit}(object, pars, include = TRUE, unconstrain = FALSE, 
          inc_warmup = FALSE, window = NULL, nrow = NULL, ncol = NULL, ...) 
} 

\section{Methods}{
  \describe{
    \item{traceplot}{\code{signature(object = "stanfit")} Plot the sampling paths for all chains.}
  }
}

\arguments{
  \item{object}{An instance of class \code{\linkS4class{stanfit}}.}
  \item{pars}{A character vector of parameter names. Defaults to all parameters 
    or the first 10 parameters (if there are more than 10).}
  \item{include}{Should the parameters given by the \code{pars} argument be
    included (the default) or excluded from the plot? Only relevant if 
    \code{pars} is not missing.}
  \item{inc_warmup}{\code{TRUE} or \code{FALSE}, indicating whether 
    the warmup sample are included in the trace plot; defaults to \code{FALSE}.}
  \item{window}{A vector of length 2. Iterations between \code{window[1]}
    and \code{window[2]} will be shown in the plot. The default is to 
    show all iterations if \code{inc_warmup} is \code{TRUE} and all iterations
    from the sampling period only if \code{inc_warmup} is \code{FALSE}. If 
    \code{inc_warmup} is \code{FALSE} the iterations specified in \code{window}
    should not include iterations from the warmup period.}
  \item{unconstrain}{Should parameters be plotted on the unconstrained space?
    Defaults to \code{FALSE}.}  
  \item{nrow,ncol}{Passed to \code{\link[ggplot2]{facet_wrap}}.} 
  \item{\dots}{Optional arguments to pass to \code{\link[ggplot2]{geom_path}} 
    (e.g. \code{size}, \code{linetype}, \code{alpha}, etc.).} 
}
\value{A \code{\link[ggplot2]{ggplot}} object that can be further customized
  using the \pkg{ggplot2} package.} 

\seealso{
\code{\link[=rstan-plotting-functions]{List of RStan plotting functions}},
\code{\link[=rstan_gg_options]{Plot options}}
}

\examples{\dontrun{
# Create a stanfit object from reading CSV files of samples (saved in rstan
# package) generated by funtion stan for demonstration purpose from model as follows. 
# 
excode <- '
  transformed data {
    array[20] real y;
    y[1] <- 0.5796;  y[2]  <- 0.2276;   y[3] <- -0.2959; 
    y[4] <- -0.3742; y[5]  <- 0.3885;   y[6] <- -2.1585;
    y[7] <- 0.7111;  y[8]  <- 1.4424;   y[9] <- 2.5430; 
    y[10] <- 0.3746; y[11] <- 0.4773;   y[12] <- 0.1803; 
    y[13] <- 0.5215; y[14] <- -1.6044;  y[15] <- -0.6703; 
    y[16] <- 0.9459; y[17] <- -0.382;   y[18] <- 0.7619;
    y[19] <- 0.1006; y[20] <- -1.7461;
  }
  parameters {
    real mu;
    real<lower=0, upper=10> sigma;
    vector[2] z[3];
    real<lower=0> alpha;
  } 
  model {
    y ~ normal(mu, sigma);
    for (i in 1:3) 
      z[i] ~ normal(0, 1);
    alpha ~ exponential(2);
  } 
'
# exfit <- stan(model_code = excode, save_dso = FALSE, iter = 200, 
#               sample_file = "rstan_doc_ex.csv")
# 
exfit <- read_stan_csv(dir(system.file('misc', package = 'rstan'),
                       pattern='rstan_doc_ex_[[:digit:]].csv',
                       full.names = TRUE))

print(exfit)
traceplot(exfit)
traceplot(exfit, size = 0.25)
traceplot(exfit, pars = "sigma", inc_warmup = TRUE)

trace <- traceplot(exfit, pars = c("z[1,1]", "z[3,1]"))
trace + scale_color_discrete() + theme(legend.position = "top")
}
} 
\keyword{methods}

\name{rstan_options}
\alias{rstan_options}
\title{Set and read options used in RStan}
\description{
Set and read options used in RStan. Some settings as options can be 
controlled by the user.
}
\usage{
rstan_options(...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{\dots}{Arguments of the form \code{opt = val} set
  option \code{opt} to value \code{val}.  Arguments of the
  form \code{opt} set the function to return option \code{opt}'s value.  
  Each argument must be a character string.}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
  The available options are:
  \enumerate{
    \item \code{plot_rhat_breaks}: The cut off points for Rhat for which we
      would indicate using a different color. This is a numeric vector,
      defaulting to \code{c(1.1, 1.2, 1.5, 2)}.
      The value for this option will be sorted in ascending order,
      so for example \code{plot_rhat_breaks = c(1.2, 1.5)} is equivalent to
      \code{plot_rhat_breaks = c(1.5, 1.2)}. 
    \item \code{plot_rhat_cols}: A vector of the same length as   
          \code{plot_rhat_breaks} that indicates the colors for the
          breaks.
    \item \code{plot_rhat_nan_col}: The color for Rhat when it is \code{Inf} or \code{NaN}. 
    \item \code{plot_rhat_large_col}: The color for Rhat when it is larger than 
          the largest value of \code{plot_rhat_breaks}. 
    \item \code{rstan_alert_col}: The color used in method \code{plot}
          of S4 class \code{\linkS4class{stanfit}}  to show that the vector/array
          parameters are truncated. 
    \item \code{rstan_chain_cols}: The colors used in methods \code{plot}
          and \code{traceplot} of S4 class \code{\linkS4class{stanfit}}  
          for coloring different chains. 
    \item \code{rstan_warmup_bg_col}: The background color for  
          the warmup area in the traceplots. 
    \item \code{boost_lib}: The path for the Boost C++ library used
          to compile Stan models.  This option is valid 
          for the whole \R session if not changed again. 
    \item \code{eigen_lib}: The path for the Eigen C++ library used
          to compile Stan models.  This option is valid 
         for the whole \R session if not changed again. 
    \item \code{auto_write}: A logical scalar (defaulting to \code{FALSE}) that
         controls whether a compiled instance of a \code{\link{stanmodel-class}}
         is written to the hard disk in the same directory as the \code{.stan}
         program.
    \item \code{threads_per_chain}: A positive integer (defaulting to \code{1}).
         If the model was compiled with threading support, the number of
         threads to use in parallelized sections _within_ an MCMC chain (e.g., when
         using the Stan functions `reduce_sum()` or `map_rect()`). The actual number of CPU cores
         used is `chains * threads_per_chain` where `chains` is the number of parallel chains.
         For an example of using threading, see the Stan case study [Reduce Sum: A Minimal
         Example](https://mc-stan.org/users/documentation/case-studies/reduce_sum_tutorial.html).
  } 
}
\value{
The values as a \code{list} for existing options and \code{NA} for non-existent options.
When only one option is specified, its old value is returned. 
}
%\references{
%% ~put references to the literature/web site here ~
%}
%\author{
%%  ~~who you are~~
%}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
%\examples{}
\keyword{rstan} 

\docType{methods}
\name{gqs}
\alias{gqs} 
\alias{gqs,stanmodel-method}

\title{Draw samples of generated quantities from a Stan model}
\description{
  Draw samples from the generated quantities block of a 
  \code{\linkS4class{stanmodel}}.
}

\usage{
  \S4method{gqs}{stanmodel}(object, data = list(), draws, 
    seed = sample.int(.Machine$integer.max, size = 1L))
} 

\section{Methods}{
  \describe{
    \item{\code{object}}{\code{signature(object = "stanmodel")} 
      Evaluate the generated quantities block of a Stan program
      by supplying \code{data} and the \code{draws} output from a
      previous Stan program.
    }
  }
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanmodel}}.}

  \item{data}{A named \code{list} or \code{environment}
    providing the data for the model or a character vector 
    for all the names of objects used as data. 
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.} 

  \item{draws}{A matrix of posterior draws, typically created by
    calling \code{as.matrix} on a \code{\linkS4class{stanfit}}.}

  \item{seed}{The seed for random number generation. The default is generated 
    from 1 to the maximum integer supported by \R on the machine.
    When a seed is specified by a number, \code{as.integer} will be applied to it. 
    If \code{as.integer} produces \code{NA}, the seed is generated randomly. 
    The seed can also be specified as a character string of digits, such as
    \code{"12345"}, which is converted to integer.}
}

\value{
   An object of S4 class \code{\linkS4class{stanmodel}} representing
   the fitted results. 
} 

\seealso{
  \code{\linkS4class{stanmodel}}, \code{\linkS4class{stanfit}}, \code{\link{stan}}
}

\examples{\dontrun{
m <- stan_model(model_code = 'parameters {real y;} model {y ~ normal(0,1);}')
f <- sampling(m, iter = 300)
mc <-
'
parameters {real y;}
generated quantities {real y_rep = normal_rng(y, 1);}
'
m2 <- stan_model(model_code = mc)
f2 <- gqs(m2, draws = as.matrix(f))
f2
}}

\docType{methods}
\name{sampling}
\alias{sampling} 
\alias{sampling,stanmodel-method}

\title{Draw samples from a Stan model}
\description{
  Draw samples from the model defined by class \code{stanmodel}. 
}

\usage{
  %% sampling(object, \dots)  
  \S4method{sampling}{stanmodel}(object, data = list(), pars = NA, 
    chains = 4, iter = 2000, warmup = floor(iter/2), thin = 1,
    seed = sample.int(.Machine$integer.max, 1), 
    init = 'random', check_data = TRUE, 
    sample_file = NULL, diagnostic_file = NULL, verbose = FALSE, 
    algorithm = c("NUTS", "HMC", "Fixed_param"),
    control = NULL, include = TRUE, 
    cores = getOption("mc.cores", 1L),
    open_progress = interactive() && !isatty(stdout()) &&
                    !identical(Sys.getenv("RSTUDIO"), "1"),
    show_messages = TRUE, \dots)
} 

\section{Methods}{
  \describe{
    \item{\code{sampling}}{\code{signature(object = "stanmodel")}
      Call a sampler (NUTS, HMC, or Fixed_param depending on parameters) 
      to draw samples from the model defined by S4 class \code{stanmodel} 
      given the data, initial values, etc. 
    }
  }
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanmodel}}.}

  \item{data}{A named \code{list} or \code{environment}
    providing the data for the model or a character vector 
    for all the names of objects used as data. 
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.} 

  \item{pars}{A vector of character strings specifying parameters of interest. 
    The default is \code{NA} indicating all parameters in the model. 
    If \code{include = TRUE}, only samples for parameters named in \code{pars} 
    are stored in the fitted results. Conversely, if \code{include = FALSE}, 
    samples for all parameters \emph{except} those named in \code{pars} are 
    stored in the fitted results.}

  \item{chains}{A positive integer specifying the number of Markov chains. 
    The default is 4.} 
 
  \item{iter}{A positive integer specifying the number of iterations for each 
    chain (including warmup). The default is 2000.}

  \item{warmup}{A positive integer specifying the number of warmup (aka burnin)
    iterations per chain. If step-size adaptation is on (which it is by default), 
    this also controls the number of iterations for which adaptation is run (and
    hence these warmup samples should not be used for inference). The number of 
    warmup iterations should be smaller than \code{iter} and the default is
    \code{iter/2}.}

  \item{thin}{A positive integer specifying the period for saving samples. 
    The default is 1, which is usually the recommended value.}

  \item{seed}{The seed for random number generation. The default is generated 
    from 1 to the maximum integer supported by \R on the machine. Even if 
    multiple chains are used, only one seed is needed, with other chains having 
    seeds derived from that of the first chain to avoid dependent samples.
    When a seed is specified by a number, \code{as.integer} will be applied to it. 
    If \code{as.integer} produces \code{NA}, the seed is generated randomly. 
    The seed can also be specified as a character string of digits, such as
    \code{"12345"}, which is converted to integer.}

  \item{init}{Initial values specification. See the detailed documentation for 
    the init argument in \code{\link{stan}}.}

  \item{check_data}{Logical, defaulting to \code{TRUE}. If \code{TRUE} 
    the data will be preprocessed; otherwise not.
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.}

  \item{sample_file}{An optional character string providing the name of a file.
    If specified the draws for \emph{all} parameters and other saved quantities
    will be written to the file. If not provided, files are not created. 
    When the folder specified is not writable, \code{tempdir()} is used. 
    When there are multiple chains, an underscore and chain number are appended
    to the file name prior to the \code{.csv} extension.}

  \item{diagnostic_file}{An optional character string providing the name of a file.
    If specified the diagnostics data for \emph{all} parameters will be written
    to the file. If not provided, files are not created. When the folder specified 
    is not writable, \code{tempdir()} is used. When there are multiple chains, 
    an underscore and chain number are appended to the file name prior to the
    \code{.csv} extension.}
   
  \item{verbose}{\code{TRUE} or \code{FALSE}: flag indicating whether 
    to print intermediate output from Stan on the console, which might
    be helpful for model debugging.}

  \item{algorithm}{One of sampling algorithms that are implemented in Stan. 
    Current options are \code{"NUTS"} (No-U-Turn sampler, Hoffman and Gelman 2011, Betancourt 2017), 
    \code{"HMC"} (static HMC), or \code{"Fixed_param"}. The default and 
    preferred algorithm is \code{"NUTS"}.}

  \item{control}{A named \code{list} of parameters to control the sampler's
    behavior. See the details in the documentation for the \code{control} argument
    in \code{\link{stan}}.}

  \item{include}{Logical scalar defaulting to \code{TRUE} indicating
    whether to include or exclude the parameters given by the 
    \code{pars} argument. If \code{FALSE}, only entire multidimensional
    parameters can be excluded, rather than particular elements of them.}
  
  \item{cores}{Number of cores to use when executing the chains in parallel,
    which defaults to 1 but we recommend setting the \code{mc.cores} option 
    to be as many processors as the hardware and RAM allow (up to the 
    number of chains).}
    
  \item{open_progress}{Logical scalar that only takes effect if 
    \code{cores > 1} but is recommended to be \code{TRUE} in interactive
    use so that the progress of the chains will be redirected to a file
    that is automatically opened for inspection. For very short runs, the
    user might prefer \code{FALSE}.}
    
  \item{show_messages}{Either a logical scalar (defaulting to \code{TRUE})
    indicating whether to print the summary of Informational Messages to
    the screen after a chain is finished or a character string naming a path
    where the summary is stored. Setting to \code{FALSE} is not recommended
    unless you are very sure that the model is correct up to numerical 
    error.}

  \item{\dots}{Additional arguments can be \code{chain_id}, \code{init_r}, 
    \code{test_grad}, \code{append_samples}, \code{refresh},
    \code{enable_random_init}. See the documentation in \code{\link{stan}}.} 
}

\value{
   An object of S4 class \code{stanfit} representing
   the fitted results. Slot \code{mode} for this object
   indicates if the sampling is done or not. 
} 

\seealso{
  \code{\linkS4class{stanmodel}}, \code{\linkS4class{stanfit}}, \code{\link{stan}}
}

\examples{\dontrun{
m <- stan_model(model_code = 'parameters {real y;} model {y ~ normal(0,1);}')
f <- sampling(m, iter = 100)
}}

\name{read_rdump}
\alias{read_rdump}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Read data in an \R dump file to a list}
\description{Create an \R list from an \R dump file}
\usage{
read_rdump(f, keep.source = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{f}{A character string providing the dump file name.}
  \item{keep.source}{logical: should the source formatting be retained when
                     echoing expressions, if possible?}
  \item{...}{passed to \code{source}}
}
\details{
The \R dump file can be read directly by \R function \code{source}, which
by default would read the data into the user's workspace (the global environment). 
This function instead read the data to a list, making it convenient to 
prepare data for the \code{stan} model-fitting function.
}
\value{
A list containing all the data defined in the dump file with
keys corresponding to variable names.
}
\seealso{
\code{\link{stan_rdump}}; \code{\link{dump}} 
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
x <- 1; y <- 1:10; z <- array(1:10, dim = c(2,5)) 
stan_rdump(ls(pattern = '^[xyz]'), file.path(tempdir(), "xyz.Rdump"))
l <- read_rdump(file.path(tempdir(), 'xyz.Rdump'))
print(l)
unlink(file.path(tempdir(), "xyz.Rdump"))
}
\keyword{rstan} 

\name{As.mcmc.list}
\alias{As.mcmc.list}
\title{Create an mcmc.list from a stanfit object} 
\description{
  Create an \code{\link[coda]{mcmc.list}} (\pkg{coda}) 
  from a \code{stanfit} object.
}
\usage{
  As.mcmc.list(object, pars, include = TRUE, ...)
}
\arguments{
  \item{object}{object of class \code{"stanfit"}}
  \item{pars}{optional character vector of parameters to include}
  \item{include}{logical scalar indicating whether to include (the default) or 
    exclude the parameters named in \code{pars}}
  \item{\dots}{unused}
}
\value{
  An object of class \code{\link[coda]{mcmc.list}}.
}

\name{pairs.stanfit}
\alias{pairs.stanfit}

\title{Create a matrix of output plots from a \code{stanfit} object} 

\description{A \code{\link{pairs}} method that is customized for MCMC output} 

\usage{
  \method{pairs}{stanfit}(x, labels = NULL, panel = NULL, \dots, 
    lower.panel = NULL, 
    upper.panel = NULL, diag.panel = NULL, text.panel = NULL, 
    label.pos = 0.5 + 1/3, cex.labels = NULL, font.labels = 1, 
    row1attop = TRUE, gap = 1, log = "", pars = NULL, include = TRUE,
    condition = "accept_stat__") 
} 

\arguments{
  \item{x}{An object of S4 class \code{stanfit}}
  \item{labels, panel, \dots, lower.panel, upper.panel, diag.panel}{Same as in 
    \code{\link{pairs}} syntactically but see the Details section for different
    default arguments}
  \item{text.panel, label.pos, cex.labels, font.labels, row1attop, gap}{Same as in \code{\link{pairs.default}}}
  \item{log}{Same as in \code{\link{pairs.default}}, which makes it possible to
    utilize logarithmic axes and additionally accepts \code{log = TRUE}. See the
    Details section.}
  \item{pars}{If not \code{NULL}, a character vector indicating which quantities
    to include in the plots, which is passed to \code{\link{extract}}. Thus,
    by default, all unknown quantities are included, which may be far too many
    to visualize on a small computer screen. If \code{include = FALSE}, then
    the named parameters are excluded from the plot.}
  \item{condition}{If \code{NULL}, it will plot roughly half of the
    chains in the lower panel and the rest in the upper panel. An
    integer vector can be passed to select some subset of the chains, of which
    roughly half will be plotted in the lower panel and the rest in the upper
    panel. A list of two integer vectors can be passed, each specifying a subset
    of the chains to be plotted in the lower and upper panels respectively.
    
    A single number between zero and one exclusive can be passed, which is 
    interpreted as the proportion of realizations (among all chains) to plot
    in the lower panel starting with the first realization in each chain, 
    with the complement (from the end of each chain) plotted in the upper panel.
    
    A (possibly abbreviated) character vector of length one can be passed among 
    \code{"accept_stat__", "stepsize__"}, \code{"treedepth__", "n_leapfrog__"},
    \code{"divergent__"},, \code{"energy__"}, or \code{"lp__"}, which are the 
    variables produced by \code{\link{get_sampler_params}} and 
    \code{\link{get_logposterior}}. In that case the lower panel will plot 
    realizations that are below the median of the indicated variable (or are zero 
    in the case of \code{"divergent__"}) and the upper panel will plot realizations 
    that are greater than or equal to the median of the indicated variable (or are 
    one in the case of \code{"divergent__"}). 
    Finally, any logical vector whose length is equal to the product of the number 
    of iterations and the number of chains can be passed, in which case realizations
    corresponding to \code{FALSE} and \code{TRUE} will be plotted in the lower 
    and upper panel respectively. The default is \code{"accept_stat__"}.}
  \item{include}{Logical scalar indicating whether to include (the default) or
    exclude the parameters named in the \code{pars} argument from the plot.}
} 

\details{
  This method differs from the default \code{\link{pairs}} method in the following
  ways. If unspecified, the \code{\link{smoothScatter}} function is used for the
  off-diagonal plots, rather than \code{\link{points}}, since the former is more
  appropriate for visualizing thousands of draws from a posterior distribution.
  Also, if unspecified, histograms of the marginal distribution of each quantity
  are placed on the diagonal of the plot, after pooling all of the chains specified
  by the \code{chain\_id} argument.
  
  The draws from the warmup phase are always discarded before plotting.
  
  By default, the lower (upper) triangle of the plot contains draws with below
  (above) median acceptance probability. Also, if \code{condition} is not
  \code{"divergent__"}, red points will be superimposed onto the smoothed
  density plots indicating which (if any) iterations encountered a divergent 
  transition. Otherwise, yellow points indicate a transition that hit the
  maximum treedepth rather than terminated its evolution normally.
  
  You may very well want to specify the \code{log} argument for non-negative
  parameters. However, the \code{pairs} function will drop (with a message)
  parameters that are either constant or duplicative with previous parameters.
  For example, if a correlation matrix is included among \code{pars}, then 
  neither its diagonal elements (which are always 1) nor its upper triangular
  elements (which are the same as the corresponding lower triangular elements)
  will be included. Thus, if \code{log} is an integer vector, it needs to
  pertain to the parameters after constant and duplicative ones are dropped.
  It is perhaps easiest to specify \code{log = TRUE}, which will utilize
  logarithmic axes for all non-negative parameters, except \code{lp__} and
  any integer valued quantities.
}
  
\seealso{
  S4 class \code{\linkS4class{stanfit}} and its method \code{\link{extract}} as
  well as the \code{\link{pairs}} generic function. Also, see
  \code{\link{get_sampler_params}} and \code{\link{get_logposterior}}.
} 

\examples{
example(read_stan_csv)
pairs(fit, pars = c("mu", "sigma", "alpha", "lp__"), log = TRUE, las = 1)
# sigma and alpha will have logarithmic axes
}

\name{as.array}
\alias{is.array.stanfit}
\alias{as.array.stanfit}
\alias{as.matrix.stanfit}
\alias{as.data.frame.stanfit}
\alias{dim.stanfit}
\alias{dimnames.stanfit}
\alias{names.stanfit}
\alias{names<-.stanfit}

\title{Create array, matrix, or data.frame objects from samples in a \code{stanfit} object} 


\description{The samples (without warmup) included in a \code{\linkS4class{stanfit}} 
  object can be coerced to an \code{array}, \code{matrix}, or \code{data.frame}.
  Methods are also provided for checking and setting names and dimnames.} 

\usage{
  \method{as.array}{stanfit}(x, \dots) 
  \method{as.matrix}{stanfit}(x, \dots)
  \method{as.data.frame}{stanfit}(x, \dots)
  \method{is.array}{stanfit}(x) 
  \method{dim}{stanfit}(x)
  \method{dimnames}{stanfit}(x)
  \method{names}{stanfit}(x)
  \method{names}{stanfit}(x) <- value
} 

\arguments{
  \item{x}{An object of S4 class \code{\linkS4class{stanfit}}.}
  \item{\dots}{Additional parameters that can be passed to \code{\link{extract}} 
    for extracting samples from \code{x}. For now, \code{pars}
    is the only additional parameter supported.
  }
  \item{value}{For the \code{names} replacement method, a character vector to 
    set/replace the parameter names in \code{x}.}
} 

\details{
  \code{as.array} and \code{as.matrix} can be applied to a \code{stanfit}
  object to coerce the samples without warmup to \code{array} or \code{matrix}.
  The \code{as.data.frame} method first calls \code{as.matrix} and then coerces
  this matrix to a \code{data.frame}.

  The array has three named dimensions: iterations, chains, parameters. 
  For \code{as.matrix}, all chains are combined, leaving a matrix of iterations
  by parameters.
}
\value{
  \code{as.array}, \code{as.matrix}, and \code{as.data.frame} return an array,
  matrix, and data.frame, respectively.

  \code{dim} and \code{dimnames} return the dim and dimnames of the 
  array object that could be created, while \code{names} returns the third
  element of the \code{dimnames}, which are the names of the margins of the
  posterior distribution. The \code{names} assignment method allows for 
  assigning more interpretable names to them.

  \code{is.array} returns \code{TRUE} for \code{stanfit} objects that include
  samples; otherwise \code{FALSE}. 

  When the \code{stanfit} object does not contain samples, empty objects
  are returned from \code{as.array}, \code{as.matrix}, \code{as.data.frame},
  \code{dim}, \code{dimnames}, and \code{names}. 
}
  
\seealso{
  S4 class \code{\linkS4class{stanfit}} and its method \code{\link{extract}}
} 

\examples{\dontrun{
ex_model_code <- '
  parameters {
    array[2, 3] real alpha;
    array[2] real beta; 
  } 
  model {
    for (i in 1:2) for (j in 1:3) 
      alpha[i, j] ~ normal(0, 1); 
    for (i in 1:2) 
      beta[i] ~ normal(0, 2); 
    # beta ~ normal(0, 2) // vectorized version
  } 
'

## fit the model 
fit <- stan(model_code = ex_model_code, chains = 4) 

dim(fit)
dimnames(fit)
is.array(fit) 
a <- as.array(fit)
m <- as.matrix(fit)
d <- as.data.frame(fit)
}}  

\name{rstan-package}
\alias{rstan-package}
\alias{rstan}
\docType{package}
\title{RStan --- the \R interface to Stan}
\description{
\if{html}{
   \figure{stanlogo.png}{options: width="50" alt="mc-stan.org"}
   \emph{Stan Development Team}
}

  RStan is the \R interface to the \href{https://mc-stan.org/}{Stan} C++ package.
  The RStan interface (\pkg{rstan} \R package) provides:
  \itemize{
  \item Full Bayesian inference using the No-U-Turn sampler (NUTS),
    a variant of Hamiltonian Monte Carlo (HMC)
  \item Approximate Bayesian inference using automatic differentiation
    variational inference (ADVI)
  \item Penalized maximum likelihood estimation using L-BFGS optimization
  }

  For documentation on Stan itself, including the manual and user guide
  for the modeling language, case studies and worked examples, and other
  tutorial information visit the Users section of the Stan website:

  \itemize{
  \item \href{https://mc-stan.org/users/documentation/}{mc-stan.org/users/documentation}
  }

}
\section{Other \R packages from the Stan Development Team}{
  Various related \R packages are also available from the Stan Development Team including these and more:
  \tabular{llll}{
  \strong{Package} \tab \strong{Description} \tab \strong{Doc} \tab \strong{Website} \cr
  \pkg{bayesplot}
    \tab ggplot-based plotting of parameter estimates, diagnostics, and posterior predictive checks.
    \tab \link[bayesplot]{bayesplot-package}
    \tab \href{https://mc-stan.org/bayesplot/}{mc-stan.org/bayesplot}
    \cr
  \pkg{shinystan}
    \tab Interactive GUI for exploring MCMC output.
    \tab \link[shinystan]{shinystan-package}
    \tab \href{https://mc-stan.org/shinystan/}{mc-stan.org/shinystan}
    \cr
  \pkg{loo}
    \tab Out-of-sample predictive performance estimates and model comparison.
    \tab \link[loo]{loo-package}
    \tab \href{https://mc-stan.org/loo/}{mc-stan.org/loo}
    \cr
  \pkg{rstanarm}
    \tab \R formula interface for applied regression modeling.
    \tab \link[rstanarm]{rstanarm-package}
    \tab \href{https://mc-stan.org/rstanarm/}{mc-stan.org/rstanarm}
    \cr
  \pkg{rstantools}
    \tab Tools for developers of \R packages interfacing with Stan.
    \tab \link[rstantools]{rstantools-package}
    \tab \href{https://mc-stan.org/rstantools/}{mc-stan.org/rstantools}
    \cr
  }
}
\author{
  \tabular{ll}{
    Jonah Gabry (author)\tab <jonah.sol.gabry@columbia.edu> \cr
    Ben Goodrich (maintainer, author)\tab <benjamin.goodrich@columbia.edu> \cr
    Jiqiang Guo (author)\tab <guojq28@gmail.com> \cr
  }

  There are also many other important contributors to RStan
  (\href{https://github.com/stan-dev/rstan}{github.com/rstan}).
  Please use 'Stan Development Team' whenever citing the R interface to Stan.
  A BibTex entry is available from \url{https://mc-stan.org/rstan/authors}
  or \code{citation("rstan")}.
}

\seealso{
\itemize{
  \item The RStan vignettes: \url{https://mc-stan.org/rstan/articles/}.
  \item \code{\link{stan}} for details on fitting models and
  \code{\linkS4class{stanfit}} for information on the fitted model objects.
  \item The \code{\link{lookup}} for finding a function in the Stan language
  that corresponds to a \R function or name.
  \item \url{https://github.com/stan-dev/rstan/issues/} to submit a bug
  report or feature request.
  \item \url{https://discourse.mc-stan.org} to ask a
  question on the Stan Forums.
}
}

\examples{\dontrun{

stanmodelcode <- "
data {
  int<lower=0> N;
  array[N] real y;
}

parameters {
  real mu;
}

model {
  target += normal_lpdf(mu | 0, 10);
  target += normal_lpdf(y  | mu, 1);
}
"

y <- rnorm(20)
dat <- list(N = 20, y = y);
fit <- stan(model_code = stanmodelcode, model_name = "example",
            data = dat, iter = 2012, chains = 3, verbose = TRUE,
            sample_file = file.path(tempdir(), 'norm.csv'))
print(fit)

# extract samples
e <- extract(fit, permuted = FALSE) # return a list of arrays
str(e)

arr <- as.array(fit) # return an array
str(arr)

mat <- as.matrix(fit) # return a matrix
str(mat)
}}
\keyword{rstan}
\keyword{package}

\name{sbc}
\alias{sbc}
\alias{print.sbc}
\alias{plot.sbc}
\title{Simulation Based Calibration (sbc)}
\description{
\if{html}{\figure{stanlogo.png}{options: width="25" alt="https://mc-stan.org/about/logo/"}}
Check whether a model is well-calibrated with respect to the
prior distribution and hence possibly amenable to obtaining
a posterior distribution conditional on observed data.
}
\usage{
  sbc(stanmodel, data, M, ..., save_progress, load_incomplete=FALSE)
  \method{plot}{sbc}(x, thin = 3, ...)
  \method{print}{sbc}(x, ...)

}
\arguments{
  \item{stanmodel}{
    An object of \code{\link{stanmodel-class}} that is first created by
    calling the \code{\link{stan_model}} function}
  \item{data}{A named \code{list} or \code{environment} providing the data for
    the model, or a character vector for all the names of objects to use as data.
    This is the same format as in \code{\link{stan}} or \code{\link{sampling}}.}
  \item{M}{
    The number of times to condition on draws from the prior predictive
    distribution}
  \item{\dots}{Additional arguments that are passed to \code{\link{sampling}},
    such as \code{refresh = 0} when calling \code{sbc}. For the \code{plot}
    and \code{print} methods, the additional arguments are not used.}
  \item{x}{An object produced by \code{sbc}}
  \item{thin}{An integer vector of length one indicating the thinning interval
    when plotting, which defaults to 3}
  \item{save_progress}{If a directory is provided, stanfit objects
    are saved to disk making it easy to resume a partial \code{sbc}
    run after interruption.}
  \item{load_incomplete}{When \code{save_progress} is used, load
    whatever runs have been saved to disk and ignore argument \code{M}.}
}
\details{
  This function assumes adherence to the following conventions in the
  underlying Stan program:
  \enumerate{
  \item Realizations of the unknown parameters are drawn in the \code{transformed data}
    block of the Stan program and are postfixed with an underscore, such as
    \code{theta_}. These are considered the \dQuote{true} parameters being estimated by
    the corresponding symbol declared in the \code{parameters} block, which
    should have the same name except for the trailing underscore, such as \code{theta}.
  \item The realizations of the unknown parameters are then conditioned on when drawing from
    the prior predictive distribution, also in the \code{transformed data} block.
    There is no restriction on the symbol name that holds the realizations from
    the prior predictive distribution but for clarity, it should not end with
    a trailing underscore.
  \item The realizations of the unknown parameters should be copied into a \code{vector}
    in the \code{generated quantities} block named \code{pars_}.
  \item The realizations from the prior predictive distribution should be copied
    into an object (of the same type) in the \code{generated quantities} block
    named \code{y_}. Technically, this step is optional and could be omitted
    to conserve RAM, but inspecting the realizations from the prior predictive distribution
    is a good way to judge whether the priors are reasonable.
  \item The \code{generated quantities} block must contain an integer array named
    \code{ranks_} whose only values are zero or one, depending on whether the realization of a
    parameter from the posterior distribution exceeds the corresponding \dQuote{true}
    realization, such as \code{theta > theta_;}. These are not actually "ranks"
    but can be used afterwards to reconstruct (thinned) ranks.
  \item The \code{generated quantities} block may contain a vector named \code{log_lik}
    whose values are the contribution to the log-likelihood by each observation. This
    is optional but facilitates calculating Pareto k shape parameters to judge whether
    the posterior distribution is sensitive to particular observations.
  }

  Although the user can pass additional arguments to \code{\link{sampling}} through the \dots,
  the following arguments are hard-coded and should not be passed through the \dots:
  \enumerate{
  \item \code{pars = "ranks_"} because nothing else needs to be stored for each posterior draw
  \item \code{include = TRUE} to ensure that \code{"ranks_"} is included rather than excluded
  \item \code{chains = 1} because only one chain is run for each integer less than \code{M}
  \item \code{seed} because a sequence of seeds is used across the \code{M} runs to preserve
    independence across runs
  \item \code{save_warmup = FALSE} because the warmup realizations are not relevant
  \item \code{thin = 1} because thinning can and should be done after the Markov Chain is
    finished, as is done by the \code{thin} argument to the \code{plot} method in order to
    make the histograms consist of approximately independent realizations
  }
  Other arguments will take the default values used by \code{\link{sampling}} unless
  passed through the \dots. Specifying \code{refresh = 0} is recommended to avoid printing
  a lot of intermediate progress reports to the screen. It may be necessary to pass a
  list to the \code{control} argument of \code{\link{sampling}} with elements \code{adapt_delta}
  and / or \code{max_treedepth} in order to obtain adequate results.

  Ideally, users would want to see the absence of divergent transitions (which is shown
  by the \code{print} method) and other warnings, plus an approximately uniform histogram
  of the ranks for each parameter (which are shown by the \code{plot} method). See the
  vignette for more details.
}
\value{
  The \code{sbc} function outputs a list of S3 class \code{"sbc"}, which contains the
  following elements:
  \enumerate{
  \item \code{ranks} A list of \code{M} matrices, each with number of
    rows equal to the number of saved iterations and number of columns equal to
    the number of unknown parameters. These matrices contain the realizations
    of the \code{ranks_} object from the \code{generated quantities} block of the
    Stan program.
  \item \code{Y} If present, a matrix of realizations from the prior predictive
    distribution whose rows are equal to the number of observations and whose columns
    are equal to \code{M}, which are taken from the \code{y_} object in the
    \code{generated quantities} block of the Stan program.
  \item \code{pars} A matrix of realizations from the prior distribution whose rows
    are equal to the number of parameters and whose columns are equal to \code{M},
    which are taken from the \code{pars_} object in the \code{generated quantities}
    block of the Stan program.
  \item \code{pareto_k} A matrix of Pareto k shape parameter estimates or \code{NULL}
    if there is no \code{log_lik} symbol in the \code{generated quantities} block
    of the Stan program
  \item \code{sampler_params} A three-dimensional array that results from combining
    calls to \code{\link{get_sampler_params}} for each of
    the \code{M} runs. The resulting matrix has rows equal to the number of
    post-warmup iterations, columns equal to six, and \code{M} floors. The columns
    are named \code{"accept_stat__"}, \code{"stepsize__"}, \code{"treedepth__"},
    \code{"n_leapfrog__"}, \code{"divergent__"}, and \code{"energy__"}. The most
    important of which is \code{"divergent__"}, which should be all zeros and
    perhaps \code{"treedepth__"}, which should only rarely get up to the value
    of \code{max_treedepth} passed as an element of the \code{control} list
    to \code{\link{sampling}} or otherwise defaults to \eqn{10}.
  }
  The \code{print} method outputs the number of divergent transitions and
  returns \code{NULL} invisibly.
  The \code{plot} method returns a \code{\link[ggplot2]{ggplot}} object
  with histograms whose appearance can be further customized.
}


\references{
  The Stan Development Team
  \emph{Stan Modeling Language User's Guide and Reference Manual}.
  \url{https://mc-stan.org}.

  Talts, S., Betancourt, M., Simpson, D., Vehtari, A., and Gelman, A. (2018).
  Validating Bayesian Inference Algorithms with Simulation-Based Calibration.
  arXiv preprint arXiv:1804.06788. \url{https://arxiv.org/abs/1804.06788}
}
\seealso{
  \code{\link{stan_model}} and \code{\link{sampling}}
}
\examples{
scode <- "
data {
  int<lower = 1> N;
  real<lower = 0> a;
  real<lower = 0> b;
}
transformed data { // these adhere to the conventions above
  real pi_ = beta_rng(a, b);
  int y = binomial_rng(N, pi_);
}
parameters {
  real<lower = 0, upper = 1> pi;
}
model {
  target += beta_lpdf(pi | a, b);
  target += binomial_lpmf(y | N, pi);
}
generated quantities { // these adhere to the conventions above
  int y_ = y;
  vector[1] pars_;
  int ranks_[1] = {pi > pi_};
  vector[N] log_lik;
  pars_[1] = pi_;
  for (n in 1:y) log_lik[n] = bernoulli_lpmf(1 | pi);
  for (n in (y + 1):N) log_lik[n] = bernoulli_lpmf(0 | pi);
}
"
}

\name{extract_sparse_parts}
\alias{extract_sparse_parts}
\title{Extract the compressed representation of a sparse matrix}
\description{Create a list of vectors that represents a sparse matrix.} 

\usage{
extract_sparse_parts(A)
}

\arguments{
  \item{A}{A \code{\link{matrix}} or \code{\link[Matrix]{Matrix}}.}
}

\details{
  The Stan Math Library has a function called \code{csr_matrix_times_vector},
  which inputs a matrix in compressed row storage form and a dense vector and
  returns their product without fillin. To use the 
  \code{csr_matrix_times_vector} function with a large
  sparse matrix, it is optimal in terms of memory to simply pass the three vectors
  that characterize the compressed row storage form of the matrix to the 
  \code{data} block of the Stan program. 
  The \code{extract_sparse_parts} function provides a convenient means of 
  obtaining these vectors.
} 

\value{
  A named list with components 
  \enumerate{ 
    \item \code{w} A numeric vector containing the non-zero elements of \code{A}.
    \item \code{v} An integer vector containing the column indices of the non-zero
      elements of \code{A}.
    \item \code{u} An integer vector indicating where in \code{w} a given row's
      non-zero values start.
  } 
}
\examples{
  A <- rbind(
    c(19L, 27L,  0L,  0L),
    c( 0L,  0L,  0L,  0L),
    c( 0L,  0L,  0L, 52L),
    c(81L,  0L, 95L, 33L)
  )
  str(extract_sparse_parts(A))
}

\docType{methods}
\name{vb}
\alias{vb} 
\alias{vb,stanmodel-method}

\title{Run Stan's variational algorithm for approximate posterior sampling}
\description{
  Approximately draw from a posterior distribution using variational inference.
  
  This is still considered an experimental feature.
  We recommend calling \code{\link{stan}} or \code{\link{sampling}} for 
  final inferences and only using \code{vb} to get a rough idea of the parameter
  distributions.
}

\usage{
  %% vb(object, \dots)  
  \S4method{vb}{stanmodel}(object, data = list(), pars = NA, include = TRUE,
    seed = sample.int(.Machine$integer.max, 1), 
    init = 'random', check_data = TRUE, 
    sample_file = tempfile(fileext = '.csv'),
    algorithm = c("meanfield", "fullrank"), 
    importance_resampling = FALSE, keep_every = 1,
    \dots)
} 

\section{Methods}{
  \describe{
    \item{vb}{\code{signature(object = "stanmodel")}
      Call Stan's variational Bayes methods
      for the model defined by S4 class \code{stanmodel} 
      given the data, initial values, etc. 
    }
  }
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanmodel}}.}
  
  \item{data}{A named \code{list} or \code{environment}
    providing the data for the model or a character vector 
    for all the names of objects used as data. 
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.}
    
  \item{pars}{If not \code{NA}, then a character vector naming parameters,
    which are included in the output if \code{include = TRUE} and excluded
    if \code{include = FALSE}. By default, all parameters are included.}
    
  \item{include}{Logical scalar defaulting to \code{TRUE} indicating
    whether to include or exclude the parameters given by the 
    \code{pars} argument. If \code{FALSE}, only entire multidimensional
    parameters can be excluded, rather than particular elements of them.}
    
  \item{seed}{The seed for random number generation. The default is generated 
    from 1 to the maximum integer supported by \R on the machine. Even if 
    multiple chains are used, only one seed is needed, with other chains having 
    seeds derived from that of the first chain to avoid dependent samples.
    When a seed is specified by a number, \code{as.integer} will be applied to it. 
    If \code{as.integer} produces \code{NA}, the seed is generated randomly. 
    The seed can also be specified as a character string of digits, such as
    \code{"12345"}, which is converted to integer.}
    
  \item{init}{Initial values specification. See the detailed documentation for 
    the init argument in \code{\link{stan}}.}
      
  \item{check_data}{Logical, defaulting to \code{TRUE}. If \code{TRUE} 
    the data will be preprocessed; otherwise not.
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.}
    
  \item{sample_file}{A character string of file name for specifying where to 
    write samples for \emph{all} parameters and other saved quantities. 
    This defaults to a temporary file.}
    
  \item{algorithm}{Either \code{"meanfield"} (the default) or \code{"fullrank"}, 
    indicating which variational inference algorithm is used. The \code{"meanfield"} 
    option uses a fully factorized Gaussian for the approximation whereas the 
    \code{fullrank} option uses a Gaussian with a full-rank covariance matrix 
    for the approximation. Details and additional references are available in 
    the Stan manual.}
    
  \item{importance_resampling}{Logical scalar (defaulting to \code{FALSE}) indicating
    whether to do importance resampling to adjust the draws at the optimum
    to be more like draws from the posterior distribution}
  \item{keep_every}{Integer scalar (defaulting to 1) indicating the interval
    by which to thin the draws when \code{imporance_resampling = TRUE}}
    
  \item{\dots}{Other optional parameters:
    \itemize{
      \item \code{iter} (positive \code{integer}), the maximum number 
      of iterations, defaulting to 10000.
      \item \code{grad_samples} (positive \code{integer}), the number of samples
      for Monte Carlo estimate of gradients, defaulting to 1.
      \item \code{elbo_samples} (positive \code{integer}), the number of samples
      for Monte Carlo estimate of ELBO (objective function), defaulting to 100.
      (ELBO stands for "the evidence lower bound".)
      \item \code{eta} (\code{double}), positive stepsize weighting parameter
      for variational inference but is ignored if adaptation is engaged, which
      is the case by default.
      \item \code{adapt_engaged} (\code{logical}), a flag indicating whether to
      automatically adapt the stepsize, defaulting to \code{TRUE}.
      \item \code{tol_rel_obj} (positive \code{double}), the convergence tolerance 
      on the relative norm of the objective, defaulting to 0.01. 
      \item \code{eval_elbo} (positive \code{integer}), evaluate ELBO every Nth 
      iteration, defaulting to 100.
      \item \code{output_samples} (positive \code{integer}), number of posterior
      samples to draw and save, defaults to 1000.
      \item \code{adapt_iter} (positive \code{integer}), the maximum number of 
      iterations to adapt the stepsize, defaulting to 50. Ignored if 
      \code{adapt_engaged = FALSE}.
    }
    Refer to the manuals for both CmdStan and Stan for more details. 
  }
}
\value{
  An object of \code{\link{stanfit-class}}.
} 

\seealso{
  \code{\linkS4class{stanmodel}}
 
  The manuals of CmdStan and Stan. 
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org}.

  The Stan Development Team
  \emph{CmdStan Interface User's Guide}.
  \url{https://mc-stan.org}.
}

\examples{\dontrun{
m <- stan_model(model_code = 'parameters {real y;} model {y ~ normal(0,1);}')
f <- vb(m)
}} 

\docType{methods}
\name{log_prob-methods}
\alias{log_prob}
\alias{log_prob,stanfit-method}
\alias{grad_log_prob}
\alias{grad_log_prob,stanfit-method}
\alias{constrain_pars}
\alias{constrain_pars,stanfit-method}
\alias{unconstrain_pars}
\alias{unconstrain_pars,stanfit-method}
\alias{get_num_upars}
\alias{get_num_upars,stanfit-method}

\title{\code{log_prob} and \code{grad_log_prob} functions} 

\description{
  Using model's \code{log_prob} and \code{grad_log_prob} take values from the
  unconstrained space of model parameters and (by default) return values in 
  the same space. Sometimes we need to convert the values of parameters from 
  their support defined in the parameters block (which might be constrained, 
  and for simplicity, we call it the constrained space) to the unconstrained 
  space and vice versa. The \code{constrain_pars} and \code{unconstrain_pars} 
  functions are used for this purpose.
} 
\usage{
  %% log_prob(object, upars, adjust_transform = TRUE, gradient = FALSE) 
  \S4method{log_prob}{stanfit}(object, upars, adjust_transform = TRUE, gradient = FALSE)
  %% grad_log_prob(object, upars, adjust_transform = TRUE) 
  \S4method{grad_log_prob}{stanfit}(object, upars, adjust_transform = TRUE)
  %% get_num_upars(object)
  \S4method{get_num_upars}{stanfit}(object)
  %% constrain_pars(object, upars)
  \S4method{constrain_pars}{stanfit}(object, upars)
  %% unconstrain_pars(object, pars)
  \S4method{unconstrain_pars}{stanfit}(object, pars)
} 

\section{Methods}{
  \describe{
    \item{log_prob}{\code{signature(object = "stanfit")}
      Compute \code{lp__}, the log posterior (up to an additive constant)
      for the model represented by a \code{stanfit} object. Note that,
      by default, \code{log_prob} returns the log posterior in the 
      \emph{unconstrained} space Stan works in internally.
      set \code{adjust_transform = FALSE} to make the values match Stan's output.
    }
    \item{grad_log_prob}{\code{signature(object = "stanfit")} Compute the gradients
      for \code{log_prob} as well as the log posterior. The latter is returned as 
      an attribute. 
    }
    \item{get_num_upars}{\code{signature(object = "stanfit")} Get the number
      of unconstrained parameters.
    } 
    \item{constrain_pars}{\code{signature(object = "stanfit")} Convert values
      of the parameter from unconstrained space (given as a vector) to their
      constrained space (returned as a named list).}
    \item{unconstrain_pars}{\code{signature(object = "stanfit")} Contrary to
      \code{constrained}, conert values of the parameters from constrained
      to unconstrained space.
    }
  }
} 

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanfit}}.}
  \item{pars}{An list specifying the values for all parameters on the
    constrained space.} 
  \item{upars}{A numeric vector for specifying the values for all parameters 
    on the unconstrained space.}  
  \item{adjust_transform}{Logical to indicate whether to adjust
    the log density since Stan transforms parameters to unconstrained
    space if it is in constrained space. Set to \code{FALSE} to make the
    function return the same values as Stan's \code{lp__} output.
    
    } 
  \item{gradient}{Logical to indicate whether gradients are also 
    computed as well as the log density. 
  } 
} 

\details{
  Stan requires that parameters be defined along with their support.
  For example, for a variance parameter, we must define it 
  on the positive real line. But inside Stan's samplers all parameters
  defined on the constrained space are transformed to an unconstrained
  space amenable to Hamiltonian Monte Carlo. Because of this, Stan adjusts 
  the log density function by adding the log absolute value of the 
  Jacobian determinant. Once a new iteration is drawn, Stan transforms 
  the parameters back to the original constrained space without
  requiring interference from the user. However, when using the log 
  density function for a model exposed to R, we need to be careful.
  For example, if we are interested in finding the mode of parameters 
  on the constrained space, we then do not need the adjustment. 
  For this reason, the \code{log_prob} and \code{grad_log_prob} functions 
  accept an \code{adjust_transform} argument. 
} 

\value{
  \code{log_prob} returns a value (up to an additive constant) the log posterior. 
  If \code{gradient} is \code{TRUE},  the gradients are also returned as an
  attribute with name \code{gradient}.

  \code{grad_log_prob} returns a vector of the gradients.  Additionally, the vector
  has an attribute named \code{log_prob} being the value the same as \code{log_prob}
  is called for the input parameters. 

  \code{get_num_upars} returns the number of parameters on the unconstrained space. 

  \code{constrain_pars} returns a list and \code{unconstrain_pars} returns a vector. 
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org}.
}

\seealso{
  \code{\linkS4class{stanfit}} 
} 

\examples{\dontrun{
# see the examples in the help for stanfit as well
# do a simple optimization problem 
opcode <- "
parameters {
  real y;
}
model {
  target += log(square(y - 5) + 1);
}
"
opfit <- stan(model_code = opcode, chains = 0)
tfun <- function(y) log_prob(opfit, y)
tgrfun <- function(y) grad_log_prob(opfit, y)
or <- optim(1, tfun, tgrfun, method = 'BFGS')
print(or)

# return the gradient as an attribute
tfun2 <- function(y) { 
  g <- grad_log_prob(opfit, y) 
  lp <- attr(g, "log_prob")
  attr(lp, "gradient") <- g
  lp
} 

or2 <- nlm(tfun2, 10)
or2 
}} 

\name{expose_stan_functions}
\alias{expose_stan_functions}
\alias{get_stream}
\alias{get_rng}
\title{
Expose user-defined Stan functions to \R for testing and simulation
}
\description{
  The Stan modeling language allows users to define their own functions in a 
  \code{functions} block at the top of a Stan program. The 
  \code{expose_stan_functions} utility function uses
  \code{\link[Rcpp]{sourceCpp}} to export those user-defined functions
  to the specified environment for testing inside \R or for doing posterior
  predictive simulations in \R rather than in the \code{generated 
  quantities} block of a Stan program.
}
\usage{
  expose_stan_functions(stanmodel, includes = NULL, 
                        show_compiler_warnings = FALSE, ...)
  get_rng(seed = 0L)
  get_stream()
}

\arguments{
  \item{stanmodel}{
    A \code{\linkS4class{stanmodel}} object, a \code{\linkS4class{stanfit}} 
    object, a list produced by \code{\link{stanc}} or the path to a Stan 
    program (\code{.stan} file). In any of these cases, the underlying Stan 
    program should contain a non-empty \code{functions} block.
  }
  \item{includes}{If not \code{NULL} (the default), then a character vector of
    length one (possibly containing one or more \code{"\n"}) of the form 
    \code{'#include "/full/path/to/my_header.hpp"'}, which will be inserted
    into the C++ code in the model's namespace and can be used to provide definitions 
    of functions that are declared but not defined in \code{stanmodel}
  }
  \item{show_compiler_warnings}{Logical scalar defaulting to \code{FALSE} that
    controls whether compiler warnings, which can be numerous and have never
    been relevant, are shown}
  \item{seed}{An integer vector of length one indicating the state of Stan's
    pseudo-random number generator}
  \item{\dots}{
    Further arguments passed to \code{\link[Rcpp]{sourceCpp}}.
  }
}

\details{
  The \code{expose_stan_functions} function requires as much compliance with
  the C++14 standard as is implemented in the RTools toolchain for Windows.
  On Windows, you will likely need to specify \code{CXX14 = g++ -std=c++1y}
  in the file whose path is \code{\link{normalizePath}("~/.R/Makevars")} in
  order for \code{expose_stan_functions} to work. Outside of Windows, the
  necessary compiler flags are set programatically, which is likely to suffice.

  There are a few special types of user-defined Stan functions for which some 
  additional details are relevant:
  
  \subsection{(P)RNG functions}{
  If a user-defined Stan function ends in \code{_rng}, then it can
  use the Boost pseudo-random number generator used by Stan. When exposing
  such functions to \R, \code{base_rng__} and \code{pstream__} arguments will 
  be added to the \code{\link{formals}}. The \code{base_rng__} argument should
  be passed the result of a call to \code{get_rng} (perhaps specifying its
  \code{seed} argument for reproducibility) and the \code{pstream__} should
  be passed the result of a call to \code{get_stream}, which can be used to
  see the result of \code{print} and \code{reject} calls in the user-defined
  Stan functions. These arguments default to \code{get_stream()} and
  \code{get_rng()} respectively.
  }
  \subsection{LP functions}{
  If a user-defined Stan function ends in \code{_lp}, then it can
  modify the log-probability used by Stan to evaluate Metropolis
  proposals or as an objective function for optimization. When exposing
  such functions to \R, a \code{lp__} argument will be added to the 
  \code{\link{formals}}. This \code{lp__} argument defaults to zero, but a 
  \code{\link{double}} precision scalar may be  passed to this argument when the 
  function is called from \R. Such a user-defined Stan function can terminate 
  with \code{return target();} or can execute \code{print(target());} to verify that 
  the calculation is correct.
  }
}
\value{
  The names of the new functions in \code{env} are returned invisibly.
}
\seealso{
  \code{\link[Rcpp]{sourceCpp}} and the section in the Stan User Manual on
  user-defined functions
}
\examples{
\dontrun{
model_code <-
  '
  functions {
    real standard_normal_rng() {
      return normal_rng(0,1);
   }
  }
'
expose_stan_functions(stanc(model_code = model_code))
standard_normal_rng()
PRNG <- get_rng(seed = 3)
o <- get_stream()
standard_normal_rng(PRNG, o)
}
}

\docType{methods}
\name{optimizing}
\alias{optimizing} 
\alias{optimizing,stanmodel-method}

\title{Obtain a point estimate by maximizing the joint posterior}
\description{
  Obtain a point estimate by maximizing the joint posterior 
  from the model defined by class \code{stanmodel}. 
}

\usage{
  %% optimizing(object, \dots)  
  \S4method{optimizing}{stanmodel}(object, data = list(), 
    seed = sample.int(.Machine$integer.max, 1), init = 'random', 
    check_data = TRUE, sample_file = NULL, 
    algorithm = c("LBFGS", "BFGS", "Newton"),
    verbose = FALSE, hessian = FALSE, as_vector = TRUE, 
    draws = 0, constrained = TRUE, importance_resampling = FALSE, \dots)   
} 

\section{Methods}{
  \describe{
    \item{optimizing}{\code{signature(object = "stanmodel")}
      Call Stan's optimization methods to obtain a point estimate 
      for the model defined by S4 class \code{stanmodel} 
      given the data, initial values, etc. 
    }
  }
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanmodel}}.}
  
  \item{data}{A named \code{list} or \code{environment}
    providing the data for the model or a character vector 
    for all the names of objects used as data. 
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.}
    
  \item{seed}{The seed for random number generation. The default is generated 
    from 1 to the maximum integer supported by \R on the machine. Even if 
    multiple chains are used, only one seed is needed, with other chains having 
    seeds derived from that of the first chain to avoid dependent samples.
    When a seed is specified by a number, \code{as.integer} will be applied to it. 
    If \code{as.integer} produces \code{NA}, the seed is generated randomly. 
    The seed can also be specified as a character string of digits, such as
    \code{"12345"}, which is converted to integer.}
    
  \item{init}{Initial values specification. See the detailed documentation for 
    the \code{init} argument in \code{\link{stan}} with one exception. If specifying inits  
    using a list then only a single named list of values should be provided.
    For example, to initialize a parameter \code{alpha} to \code{value1} and 
    \code{beta} to \code{value2} you can specify \code{list(alpha = value1, beta = value2)}.}
    
  \item{check_data}{Logical, defaulting to \code{TRUE}. If \code{TRUE} 
    the data will be preprocessed; otherwise not.
    See the \strong{Passing data to Stan} section in \code{\link{stan}}.}
    
  \item{sample_file}{A character string of file name for specifying where to 
    write samples for \emph{all} parameters and other saved quantities. 
    If not provided, files are not created. When the folder specified 
    is not writable, \code{tempdir()} is used.}
    
  \item{algorithm}{One of \code{"Newton"}, \code{"BFGS"}, 
    and \code{"LBFGS"} (the default) indicating which optimization algorithm 
    to use.}
    
  \item{verbose}{\code{TRUE} or \code{FALSE} (the default): flag indicating 
    whether to print intermediate output from Stan on the console, which might
    be helpful for model debugging.}
     
  \item{hessian}{\code{TRUE} or \code{FALSE} (the default): flag indicating
     whether to calculate the Hessian (via numeric differentiation of the
     gradient function in the unconstrained parameter space).}
     
  \item{as_vector}{\code{TRUE} (the default) or \code{FALSE}: flag indicating
     whether a vector is used to store the point estimate found. A list can be 
     used instead by specifying it to be \code{FALSE}.}
     
  \item{draws}{A non-negative integer (that defaults to zero) indicating how
     many times to draw from a multivariate normal distribution whose parameters
     are the mean vector and the inverse negative Hessian in the unconstrained
     space. If \code{draws > 0} and \code{importance_resampling=TRUE} 
     then \code{log_p} and \code{log_g} will be computed and returned 
     (see description in the \strong{Value} section).}
     
  \item{constrained}{A logical scalar indicating, if \code{draws > 0}, whether
     the draws should be transformed to the constrained space defined in the
     parameters block of the Stan program. Defaults to \code{TRUE}.}
     
  \item{importance_resampling}{A logical scalar (defaulting to \code{FALSE}) 
     indicating whether to do importance resampling to compute diagnostics on the 
     draws from the normal approximation to the posterior distribution.
     If \code{TRUE} and \code{draws > 0} then \code{log_p} 
     and \code{log_g} will be computed and returned (see description in the 
     \strong{Value} section).}
     
  \item{\dots}{Other optional parameters:
    \itemize{
      \item \code{iter} (\code{integer}), the maximum number of iterations, 
      defaulting to 2000.
      \item \code{save_iterations} (logical), a flag indicating whether to save 
      the iterations, defaulting to \code{FALSE}.
      \item \code{refresh} (\code{integer}), the number of interations between 
      screen updates, defaulting to 100.
      \item \code{init_alpha} (\code{double}), for BFGS and LBFGS, 
      the line search step size for first iteration, defaulting to 0.001.
      \item \code{tol_obj} (\code{double}), for BFGS and LBFGS, 
      the convergence tolerance on changes in objective function value, 
      defaulting to 1e-12.
      \item \code{tol_rel_obj} (\code{double}), for BFGS and LBFGS, 
      the convergence tolerance on relative changes in objective function value,
      defaulting to 1e4.
      \item \code{tol_grad} (\code{double}), for BFGS and LBFGS, 
      the convergence tolerance on the norm of the gradient, defaulting to 1e-8.
      \item \code{tol_rel_grad} (\code{double}), for BFGS and LBFGS, 
      the convergence tolerance on the relative norm of the gradient, defaulting
      to 1e7.
      \item \code{tol_param} (\code{double}), for BFGS and LBFGS, 
      the convergence tolerance on changes in parameter value, defaulting to 1e-8.
      \item \code{history_size} (\code{integer}), for LBFGS, 
      the number of update vectors to use in Hessian approximations, 
      defaulting to 5.
    }
    Refer to the manuals for both CmdStan and Stan for more details.
  }
}
\value{
A list with components: 

   \item{par}{The point estimate found. Its form (vector or list) 
     is determined by the \code{as_vector} argument.}
   \item{value}{The value of the log-posterior (up to an additive constant, 
     the \code{"lp__"} in Stan) corresponding to \code{par}.}
   \item{return_code}{The value of the return code from the optimizer;
     anything that is not zero is problematic.}
   \item{hessian}{The Hessian matrix if \code{hessian} is \code{TRUE}}
   \item{theta_tilde}{If \code{draws > 0}, the matrix of parameter draws
    in the constrained or unconstrained space, depending on the value of 
    the \code{constrained} argument.}
   \item{log_p}{If \code{draws > 0} and \code{importance_resampling=TRUE}, 
     a vector of length \code{draws} that contains the value of the 
     log-posterior evaluated at each row of \code{theta_tilde}.}
   \item{log_g}{If \code{draws > 0}, a vector of length \code{draws} that 
     contains the value of the logarithm of the multivariate normal density 
     evaluated at each row of \code{theta_tilde}.}

  If the optimization is not completed for reasons such as feeding wrong data,
  it returns \code{NULL}. 
} 

\seealso{
  \code{\linkS4class{stanmodel}}
}

\examples{\dontrun{
m <- stan_model(model_code = 'parameters {real y;} model {y ~ normal(0,1);}')
f <- optimizing(m, hessian = TRUE)
}} 

\name{set_cppo} 
\alias{set_cppo} 

\title{Defunct function to set the compiler optimization level} 

\description{This function returns nothing and does nothing except throw a warning. See \url{https://cran.r-project.org/doc/manuals/r-release/R-admin.html#Customizing-package-compilation} for information on customizing the compiler options, but doing so should be unnecessary for normal useage.
} 
 
\usage{
  set_cppo(...)
} 
 
\arguments{
  \item{\dots}{Any input is ignored}
} 


\value{
  An invisible \code{NULL}
}

\name{makeconf_path}
\alias{makeconf_path}
\title{
Obtain the full path of file \code{Makeconf} 
}


\description{
  Obtain the full path of file \code{Makeconf}, in which, for example
  the flags for compiling C/C++ code are configured.
}

\usage{
makeconf_path() 
} 

\value{
  An character string for the full path of file \code{Makeconf}. 
}

\details{
  The configuration for compiling shared objects using \code{R CMD SHLIB} 
  are set in file \code{Makeconf}. To change how the C++ code is
  compiled, modify this file. For RStan, package \pkg{inline} 
  compiles the C++ code using \code{R CMD SHLIB}. To speed up compiled
  Stan models, increase the optimization level to \code{-O3} defined
  in property \code{CXXFLAGS} in the file \code{Makeconf}.
  This file may also be modified to specify alternative C++ compilers,
  such as clang++ or later versions of g++.

}

\seealso{
  \code{\link{stan}}  
}
\examples{
makeconf_path() 
}
\keyword{rstan} 

\name{lookup}
\alias{lookup} 
\title{
Look up the Stan function that corresponds to a \R function or name.
}
\description{
  This function helps to map between R functions and Stan functions.
}
\usage{
  lookup(FUN, ReturnType = character()) 
} 

\arguments{
  \item{FUN}{A character string naming a \R function or a \R function for
    which the (near) equivalent Stan function is sought. If no matching
    \R function is found, \code{FUN} is reinterpreted as a \code{\link{regexp}} 
    and matches are sought.}
  \item{ReturnType}{A character string of positive length naming a valid
    return type for a Stan function: \code{int}, \code{int[]}, \code{matrix},
    \code{real}, \code{real[,]}, \code{real[]}, \code{row_vector}, 
    \code{T[]}, \code{vector}, or \code{void}. If \code{"ANY"} is passed,
    then the entire \code{\link{data.frame}} is returned and can be 
    inspected with the \code{\link{View}} function, for example.}
}

\value{
  Ordinarily, a data.frame with rows equal to the number of partial matches and four columns:
  \enumerate{
    \item \code{StanFunction} Character string for the Stan function's name.
    \item \code{Arguments} Character string indicating the arguments to that Stan function. 
    \item \code{ReturnType} Character string indicating the return type of that Stan function.
    \item \code{Page} Integer indicating the page of the Stan reference manual where  
                      that Stan function is defined. 
  }
  If there are no matching Stan functions, a character string indicating so is returned.
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org/}.

  The Stan Development Team
  \emph{CmdStan Interface User's Guide}.
  \url{https://mc-stan.org}.
} 

\examples{
lookup(dnorm)      # Stan equivalents for the normal PDF (in log form)
lookup("foo")      # fails
lookup("Student")  # succeeds even though there is no such R function
lookup("^poisson") # every Stan function that starts with poisson
}

\name{stanmodel-class} 
\Rdversion{1.1}

\docType{class}
\alias{stanmodel-class}
%% \alias{sampling,stanmodel-method}
\alias{get_cppcode}
\alias{get_cxxflags}
\alias{get_cppcode,stanmodel-method}
\alias{get_cxxflags,stanmodel-method}
\alias{get_stancode,stanmodel-method}

\alias{show,stanmodel-method}

\title{Class representing model compiled from C++}

\description{
  A \code{stanmodel} object represents the model compiled from C++ code. 
  The \code{sampling} method defined in this class may be used to 
  draw samples from the model and \code{optimizing} method is for
  obtaining a point estimate by maximizing the log-posterior.
} 

\section{Objects from the Class}{
  Instances of \code{stanmodel} are usually created by calling 
  function \code{stan_model} or function \code{stan}.
}


\section{Slots}{
  \describe{
    \item{\code{model_name}:}{The model name, an object of type \code{character}.} 
    \item{\code{model_code}:}{The Stan model specification, an object of type \code{character}.} 
    \item{\code{model_cpp}:}{Object of type \code{list}
      that includes the C++ code for the model.} 
    \item{\code{mk_cppmodule}:}{A function to return a RCpp module. This function will be
      called in function \code{sampling} and \code{optimzing} with one
      argument (the instance of \code{stanmodel} itself).} 
    \item{\code{dso}:}{Object of S4 class \code{cxxdso}. The container for the dynamic 
      shared objects compiled from the C++ code of the model, returned from function
      \code{cxxfunction} in package \pkg{inline}.}
  }
}

\section{Methods}{
  \describe{
    \item{\code{show}}{\code{signature(object = "stanmodel")}: print the Stan model specification.}
    \item{\code{vb}}{\code{signature(object = "stanmodel")}: use the variational Bayes algorithms.}
    \item{\code{sampling}}{\code{signature(object = "stanmodel")}: draw samples for
      the model (see \code{\link{sampling}}).}
    \item{\code{optimizing}}{\code{signature(object = "stanmodel")}: obtain a point 
      estimate by maximizing the posterior (see \code{\link{optimizing}}).}
    \item{\code{get_cppcode}}{\code{signature(object = "stanmodel")}: returns the C++ code for the model
      as a character string. This is part of the C++ code that is compiled to the dynamic 
      shared object for the model.}
    \item{\code{get_stancode}}{\code{signature(object = "stanmodel")}: returns the Stan code for
      the model as a character string}  
    \item{\code{get_cxxflags}}{\code{signature(object = "stanmodel")}: return the \code{CXXFLAGS} 
      used for compiling the model. The returned string is like \code{CXXFLAGS = -O3}.}
  }
}

\note{
  Objects of class \code{stanmodel} can be saved for use across 
  \R sessions only if \code{save_dso = TRUE} is set during calling
  functions that create \code{stanmodel} objects (e.g., \code{stan} 
  and \code{stan_model}). 

  Even if \code{save_dso = TRUE}, the model cannot be loaded on
  a platform (operating system, 32 bits or 64 bits, etc.) that differs from
  the one on which it was compiled.
}

\seealso{
\code{\link{stan_model}}, \code{\link{stanc}}
\code{\link{sampling}}, \code{\link{optimizing}}, \code{\link{vb}}
}
\examples{
showClass("stanmodel")
}
\keyword{classes}

\name{Diagnostic plots}
\alias{stan_diag}
\alias{stan_par}
\alias{stan_rhat}
\alias{stan_ess}
\alias{stan_mcse}

\title{RStan Diagnostic plots}
\description{
  Diagnostic plots for HMC and NUTS using ggplot2. 
}
\usage{
  stan_diag(object, 
            information = c("sample","stepsize", "treedepth","divergence"), 
            chain = 0, ...)
  stan_par(object, par, chain = 0, ...)
  
  stan_rhat(object, pars, ...)
  stan_ess(object, pars, ...)
  stan_mcse(object, pars, ...)
}
\arguments{
  \item{object}{A stanfit or stanreg object.}
  
  \item{information}{The information to be contained in the diagnostic plot.}
  
  \item{par,pars}{The name of a single scalar parameter (\code{par}) or 
    one or more parameter names (\code{pars}).}
  
  \item{chain}{If \code{chain=0} (the default) all chains are combined.
    Otherwise the plot for \code{chain} is overlaid on the plot for all chains
    combined.}
  
  \item{...}{For \code{stan_diag} and \code{stan_par}, optional arguments to 
    \code{\link[gridExtra]{arrangeGrob}}. For \code{stan_rhat}, \code{stan_ess}, 
    and \code{stan_mcse}, optional arguments to \code{stat_bin} in the 
    \pkg{ggplot2} package.}
}
\value{
  For \code{stan_diag} and \code{stan_par}, a list containing the ggplot objects for 
  each of the displayed plots. For \code{stan_rhat}, \code{stan_ess}, 
  and \code{stan_mcse}, a single ggplot object.
}
\details{
 \describe{
  \item{\code{stan_rhat},\code{stan_ess},\code{stan_mcse}}{Respectively, 
  these plots show the 
  distribution of the Rhat statistic, the ratio of effective sample size to 
  total sample size, and the ratio of Monte Carlo standard error
  to posterior standard deviation for the estimated parameters. These 
  plots are not intended to identify individual parameters, but rather to allow
  for quickly identifying if the estimated values of these quantities are
  desireable for all parameters.}
  \item{\code{stan_par}}{Calling \code{stan_par} generates three plots: 
  (i) a scatterplot of \code{par} vs. the accumulated log-posterior (\code{lp__}), 
  (ii) a scatterplot of \code{par} vs. the average Metropolis acceptance rate 
  (\code{accept_stat}), and 
  (iii) a violin plot showing the distribution of \code{par} at each of the 
  sampled step sizes (one per chain).
  For the scatterplots, red points are superimposed to indicate which 
  (if any) iterations encountered a divergent transition. Yellow points indicate 
  a transition that hit the maximum treedepth rather than terminated its 
  evolution normally.}
  \item{\code{stan_diag}}{The \code{information} argument is used to specify which 
  plots \code{stan_diag} should generate:
  
  \itemize{
    \item \code{information='sample'} Histograms of \code{lp__} 
      and \code{accept_stat}, as well as a scatterplot showing their 
      joint distribution.
    \item \code{information='stepsize'} Violin plots showing the 
      distributions of \code{lp__} and \code{accept_stat} at each of the sampled
      step sizes (one per chain).
    \item \code{information='treedepth'} Histogram of \code{treedepth} and 
      violin plots showing the distributions of \code{lp__} and 
      \code{accept_stat} for each value of \code{treedepth}.
    \item \code{information='divergence'} Violin plots showing the 
      distributions of \code{lp__} and \code{accept_stat} for iterations that
      encountered divergent transitions (\code{divergent=1}) and those that
      did not (\code{divergent=0}).
    }
  }
}
}
\note{
  For details about the individual diagnostics and sampler parameters and their
  interpretations see the Stan Modeling Language User's Guide and Reference 
  Manual at \url{https://mc-stan.org/docs/}.
}
\seealso{
\code{\link[=rstan-plotting-functions]{List of RStan plotting functions}},
\code{\link[=rstan_gg_options]{Plot options}}
}
\examples{
\dontrun{
fit <- stan_demo("eight_schools")

stan_diag(fit, info = 'sample') # shows three plots together
samp_info <- stan_diag(fit, info = 'sample') # saves the three plots in a list
samp_info[[3]] # access just the third plot

stan_diag(fit, info = 'sample', chain = 1) # overlay chain 1

stan_par(fit, par = "mu")
}
}

\name{rstan_gg_options}
\alias{rstan_gg_options}
\alias{rstan_ggtheme_options}
\title{Set default appearance options}
\usage{
  rstan_gg_options(...)
  
  rstan_ggtheme_options(...)
}
\arguments{
  \item{...}{For \code{rstan_ggtheme_options}, see
    \code{\link[ggplot2]{theme}} for the theme elements that can be specified
    in \code{...}. For \code{rstan_gg_options}, \code{...} can be \code{fill},
    \code{color}, \code{chain_colors}, \code{size}, \code{pt_color}, or
    \code{pt_size}. See Examples.}
}
\description{
  Set default appearance options
}
\seealso{
\code{\link[=rstan-plotting-functions]{List of RStan plotting functions}}
}
\examples{
rstan_ggtheme_options(panel.background = ggplot2::element_rect(fill = "gray"),
                      legend.position = "top")
rstan_gg_options(fill = "skyblue", color = "skyblue4", pt_color = "red")
}

\name{print}
\alias{print.stanfit}

\title{Print a summary for a fitted model represented by a \code{stanfit} object}
\description{Print basic information regarding the fitted model and 
  a summary for the parameters of interest estimated by the samples included
  in a \code{stanfit} object.
} 

\usage{
  \method{print}{stanfit}(x, pars = x@sim$pars_oi, 
      probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
      digits_summary = 2, include = TRUE, ...)
}

\arguments{
  \item{x}{An object of S4 class \code{stanfit}.}
  \item{pars}{A character vector of parameter names. The default is all parameters
    for which samples are saved. If \code{include = FALSE}, then the specified parameters
    are excluded from the printed summary.}
  \item{probs}{A numeric vector of quantiles of interest. The default is
    \code{c(0.025,0.25,0.5,0.75,0.975)}.}
  \item{digits_summary}{The number of significant digits to use when printing
    the summary, defaulting to 2. Applies to the quantities other than 
    the effective sample size, which is always rounded to the nearest integer.}
  \item{include}{Logical scalar (defaulting to \code{TRUE}) indicating whether to
    include or exclude the parameters named by the \code{pars} argument.}
  \item{\dots}{Additional arguments passed to the \code{summary} method 
    for \code{stanfit} objects.}
}

\details{
  The information regarding the fitted model includes the number of iterations,
  the number of chains, the total number of saved iterations, 
  the estimation algorithm used, and the timestamp indicating when 
  sampling finished.

  The parameter summaries computed include means, standard deviations (sd), 
  quantiles, Monte Carlo standard errors (se_mean), split Rhats, and 
  effective sample sizes (n_eff). The summaries are computed after dropping 
  the warmup iterations and merging together the draws from all chains.

  In addition to the model parameters, summaries for the 
  log-posterior (\code{lp__}) are also reported. 
}

\seealso{
  S4 class \code{\linkS4class{stanfit}} and particularly its method
  \code{\link[=summary,stanfit-method]{summary}}, which is used to obtain the 
  values that are printed.
} 

\name{stan}
\alias{stan}
\title{Fit a model with Stan}
\description{
\if{html}{\figure{stanlogo.png}{options: width="25" alt="https://mc-stan.org/about/logo/"}}
Fit a model defined in the Stan modeling language and
return the fitted result as an instance of \code{stanfit}.
}
\usage{
stan(file, model_name = "anon_model", model_code = "", fit = NA,
  data = list(), pars = NA,
  chains = 4, iter = 2000, warmup = floor(iter/2), thin = 1,
  init = "random", seed = sample.int(.Machine$integer.max, 1),
  algorithm = c("NUTS", "HMC", "Fixed_param"), %, "Metropolis"),
  control = NULL, sample_file = NULL, diagnostic_file = NULL,
  save_dso = TRUE, verbose = FALSE, include = TRUE,
  cores = getOption("mc.cores", 1L),
  open_progress = interactive() && !isatty(stdout()) &&
                  !identical(Sys.getenv("RSTUDIO"), "1"),
  ...,
  boost_lib = NULL, eigen_lib = NULL
  )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file}{
    The path to the Stan program to use.
    \code{file} should be a character string file name or a connection that
    \R supports containing the text of a model specification in the Stan
    modeling language.

    A model may also be specified directly as a character string using the
    \code{model_code} argument, but we recommend always putting Stan programs
    in separate files with a \code{.stan} extension.

    The \code{stan} function can also use the Stan program from an existing
    \code{stanfit} object via the \code{fit} argument. When \code{fit} is
    specified, the \code{file} argument is ignored.
    }

  \item{model_code}{
    A character string either containing the model definition or the name of
    a character string object in the workspace. This argument is used only
    if arguments \code{file} and \code{fit} are not specified.
  }

  \item{fit}{
    An instance of S4 class \code{stanfit} derived from a previous fit;
    defaults to \code{NA}. If \code{fit} is not \code{NA}, the compiled model
    associated with the fitted result is re-used; thus the time that would
    otherwise be spent recompiling the C++ code for the model can be saved.
  }

  \item{model_name}{
    A string to use as the name of the model; defaults
    to \code{"anon_model"}. However, the model name will be derived from
    \code{file} or \code{model_code} (if \code{model_code} is the name
    of a character string object) if \code{model_name} is not specified.
    This is not a particularly important argument, although since it
    affects the name used in printed messages, developers of other packages
    that use \pkg{rstan} to fit models may want to use informative names.
  }

  \item{data}{A named \code{list} or \code{environment} providing the data for
    the model, or a character vector for all the names of objects to use as data.
    See the \strong{Passing data to Stan} section below.}

  \item{pars}{A character vector specifying parameters of interest to be saved.
    The default is to save all parameters from the model.
    If \code{include = TRUE}, only samples for parameters named in \code{pars}
    are stored in the fitted results. Conversely, if \code{include = FALSE},
    samples for all parameters \emph{except} those named in \code{pars} are
    stored in the fitted results.}

  \item{include}{Logical scalar defaulting to \code{TRUE} indicating
    whether to include or exclude the parameters given by the
    \code{pars} argument. If \code{FALSE}, only entire multidimensional
    parameters can be excluded, rather than particular elements of them.}

  \item{iter}{A positive integer specifying the number of iterations for each
    chain (including warmup). The default is 2000.}

  \item{warmup}{A positive integer specifying the number of warmup (aka burnin)
    iterations per chain. If step-size adaptation is on (which it is by default),
    this also controls the number of iterations for which adaptation is run (and
    hence these warmup samples should not be used for inference). The number of
    warmup iterations should be smaller than \code{iter} and the default is
    \code{iter/2}.}

  \item{chains}{A positive integer specifying the number of Markov chains.
    The default is 4.}

  \item{cores}{
    The number of cores to use when executing the Markov chains in parallel.
    The default is to use the value of the \code{"mc.cores"} option if it
    has been set and otherwise to default to 1 core. However, we recommend
    setting it to be as many processors as the hardware and RAM allow
    (up to the number of chains). See \code{\link[parallel]{detectCores}}
    if you don't know this number for your system.
  }

  \item{thin}{A positive integer specifying the period for saving samples.
    The default is 1, which is usually the recommended value. Unless your
    posterior distribution takes up too much memory we do \emph{not} recommend
    thinning as it throws away information. The tradition of thinning when
    running MCMC stems primarily from the use of samplers that require
    a large number of iterations to achieve the desired effective sample size.
    Because of the efficiency (effective samples per second) of
    Hamiltonian Monte Carlo, rarely should this be necessary when using Stan.}

  \item{init}{
    Specification of initial values for all or some parameters.
    Can be the digit \code{0}, the strings \code{"0"} or \code{"random"},
    a function that returns a named list, or a list of named lists:
    \describe{
    \item{\code{init="random"} (default):}{Let Stan generate random initial values
      for all parameters. The seed of the random number generator used by Stan
      can be specified via the \code{seed} argument. If the seed for Stan is
      fixed, the same initial values are used. The default is to randomly
      generate initial values between \code{-2} and \code{2} \emph{on the
      unconstrained support}. The optional additional parameter \code{init_r} can
      be set to some value other than \code{2} to change the range of the randomly
      generated inits.}
    \item{\code{init="0", init=0}:}{Initialize all parameters to zero on
      the unconstrained support.}
    \item{inits via list:}{Set inital values by providing a list
      equal in length to the number of chains. The elements of this list should
      themselves be named lists, where each of these named lists has the name
      of a parameter and is used to specify the initial values for that parameter
      for the corresponding chain.}
    \item{inits via function:}{Set initial values by providing a function that
      returns a list for specifying the initial values of parameters for a chain.
      The function can take an optional parameter \code{chain_id} through which the
      \code{chain_id} (if specified) or the integers from 1 to \code{chains} will be
      supplied to the function for generating initial values.
      See the \strong{Examples} section below for examples of defining
      such functions and using a list of lists for specifying initial values.}
    }
    When specifying initial values via a \code{list} or \code{function}, any
    parameters for which values are not specified will receive initial values
    generated as described in the \code{init="random"} description above.
  }

  \item{seed}{The seed for random number generation. The default is generated
    from 1 to the maximum integer supported by \R on the machine. Even if
    multiple chains are used, only one seed is needed, with other chains having
    seeds derived from that of the first chain to avoid dependent samples.
    When a seed is specified by a number, \code{as.integer} will be applied to it.
    If \code{as.integer} produces \code{NA}, the seed is generated randomly.
    The seed can also be specified as a character string of digits, such as
    \code{"12345"}, which is converted to integer.

    Using \R's \code{set.seed} function to set the seed for Stan will not work.
    }

  \item{algorithm}{
    One of the sampling algorithms that are implemented in Stan.
    The default and preferred algorithm is \code{"NUTS"}, which is
    the No-U-Turn sampler variant of Hamiltonian Monte Carlo
    (Hoffman and Gelman 2011, Betancourt 2017). Currently the other options
    are \code{"HMC"} (Hamiltonian Monte Carlo), and \code{"Fixed_param"}.
    When \code{"Fixed_param"} is used no MCMC sampling is performed
    (e.g., for simulating with in the generated quantities block).
  }

  \item{sample_file}{An optional character string providing the name of a file.
    If specified the draws for \emph{all} parameters and other saved quantities
    will be written to the file. If not provided, files are not created.
    When the folder specified is not writable, \code{tempdir()} is used.
    When there are multiple chains, an underscore and chain number are appended
    to the file name.}

  \item{diagnostic_file}{An optional character string providing the name of a file.
    If specified the diagnostics data for \emph{all} parameters will be written
    to the file. If not provided, files are not created. When the folder specified
    is not writable, \code{tempdir()} is used. When there are multiple chains,
    an underscore and chain number are appended to the file name.}

  \item{save_dso}{Logical, with default \code{TRUE}, indicating whether the
    dynamic shared object (DSO) compiled from the C++ code for the model
    will be saved or not. If \code{TRUE}, we can draw samples from
    the same model in another \R session using the saved DSO (i.e.,
    without compiling the C++ code again).
    This parameter only takes effect if \code{fit} is not used; with
    \code{fit} defined, the DSO from the previous run is used.
    When \code{save_dso=TRUE}, the fitted object can be loaded from
    what is saved previously and used for sampling, if the compiling is
    done on the same platform, that is, same operating system and same
    architecture (32bits or 64bits).}

  \item{verbose}{\code{TRUE} or \code{FALSE}: flag indicating whether
    to print intermediate output from Stan on the console, which might
    be helpful for model debugging.}

  \item{control}{A named \code{list} of parameters to control the sampler's
    behavior. It defaults to \code{NULL} so all the default values are used.
    First, the following are adaptation parameters for sampling algorithms.
    These are parameters used in Stan with similar names here.
    \itemize{
      \item \code{adapt_engaged} (\code{logical})
      \item \code{adapt_gamma} (\code{double}, positive, defaults to 0.05)
      \item \code{adapt_delta} (\code{double}, between 0 and 1, defaults to 0.8)
      \item \code{adapt_kappa} (\code{double}, positive, defaults to 0.75)
      \item \code{adapt_t0} (\code{double}, positive, defaults to 10)
      \item \code{adapt_init_buffer} (\code{integer}, positive, defaults to 75)
      \item \code{adapt_term_buffer} (\code{integer}, positive, defaults to 50)
      \item \code{adapt_window} (\code{integer}, positive, defaults to 25)
    }

    In addition, algorithm HMC (called 'static HMC' in Stan) and NUTS share the
    following parameters:
    \itemize{
      \item \code{stepsize} (\code{double}, positive, defaults to 1)
        Note: this controls the \emph{initial} stepsize only, unless \code{adapt_engaged=FALSE}.
      \item \code{stepsize_jitter} (\code{double}, [0,1], defaults to 0)
      \item \code{metric} (\code{string}, one of "unit_e", "diag_e", "dense_e",
      defaults to "diag_e")
    }
    For algorithm NUTS, we can also set:
    \itemize{
      \item \code{max_treedepth} (\code{integer}, positive, defaults to 10)
    }
    For algorithm HMC, we can also set:
    \itemize{
      \item \code{int_time} (\code{double}, positive)
    }

    For \code{test_grad} mode, the following parameters can be set:
    \itemize{
      \item \code{epsilon} (\code{double}, defaults to 1e-6)
      \item \code{error} (\code{double}, defaults to 1e-6)
    }
  }

  \item{open_progress}{Logical scalar that only takes effect if
    \code{cores > 1} but is recommended to be \code{TRUE} in interactive
    use so that the progress of the chains will be redirected to a file
    that is automatically opened for inspection. For very short runs, the
    user might prefer \code{FALSE}.}

  \item{\dots}{Other optional parameters:
    \itemize{
      \item \code{chain_id} (\code{integer})
      \item \code{init_r} (\code{double}, positive)
      \item \code{test_grad} (\code{logical})
      \item \code{append_samples} (\code{logical})
      \item \code{refresh}(\code{integer})
      \item \code{save_warmup}(\code{logical})
      \item deprecated: \code{enable_random_init}(\code{logical})
    }

    \code{chain_id} can be a vector to specify the chain_id for all
    chains or an integer. For the former case, they should be unique.
    For the latter, the sequence of integers starting from the given
    \code{chain_id} are used for all chains.

    \code{init_r} is used only for generating random initial values,
    specifically when \code{init="random"} or not all parameters
    are initialized in the user-supplied list or function. If specified,
    the initial values are simulated uniformly from interval
    [-\code{init_r}, \code{init_r}] rather than using the default interval
    (see the manual of (cmd)Stan).

    \code{test_grad} (\code{logical}).
    If \code{test_grad=TRUE}, Stan will not do any sampling. Instead,
    the gradient calculation is tested and printed out and the fitted
    \code{stanfit} object is in test gradient mode.  By default, it is
    \code{FALSE}.

    \code{append_samples} (\code{logical}).
    Only relevant if \code{sample_file} is specified \emph{and} is an
    existing file. In that case, setting \code{append_samples=TRUE} will append
    the samples to the existing file rather than overwriting the contents of
    the file.

    \code{refresh} (\code{integer}) can be used to
    control how often the progress of the sampling is reported (i.e.
    show the progress every \code{refresh} iterations).
    By default, \code{refresh = max(iter/10, 1)}.
    The progress indicator is turned off if \code{refresh <= 0}.

    Deprecated: \code{enable_random_init} (\code{logical}) being \code{TRUE}
    enables specifying initial values randomly when the initial
    values are not fully specified from the user.

    \code{save_warmup} (\code{logical}) indicates whether to
    save draws during the warmup phase and defaults to \code{TRUE}. Some
    memory related problems can be avoided by setting it to \code{FALSE},
    but some diagnostics are more limited if the warmup draws are not
    stored.
  }

  \item{boost_lib}{The path for an alternative version of the Boost C++
    to use instead of the one in the \pkg{BH} package.}

  \item{eigen_lib}{The path for an alternative version of the Eigen C++
     library to the one in \pkg{RcppEigen}.}
}
\details{
  The \code{stan} function does all of the work of fitting a Stan model and
  returning the results as an instance of \code{stanfit}. The steps are
  roughly as follows:

  \enumerate{
  \item Translate the Stan model to C++ code.  (\code{\link{stanc}})
  \item Compile the C++ code into a binary shared object, which
  is loaded into the current \R session (an object
  of S4 class \code{stanmodel} is created).   (\code{\link{stan_model}})
  \item Draw samples and wrap them in an object of S4 class \code{stanfit}. (\code{\link{sampling}})
  }

  The returned object can be used with methods such as \code{print},
  \code{summary}, and \code{plot} to inspect and retrieve the results of
  the fitted model.

  \code{stan} can also be used to sample again from a fitted model under
  different settings (e.g., different \code{iter}, \code{data}, etc.) by
  using the \code{fit} argument to specify an existing \code{stanfit} object.
  In this case, the compiled C++ code for the model is reused.
}
\value{
  An object of S4 class \code{\link{stanfit}}. However, if \code{cores > 1}
  and there is an error for any of the chains, then the error(s) are printed. If
  all chains have errors and an error occurs before or during sampling, the returned
  object does not contain samples. But the compiled binary object for the
  model is still included, so we can reuse the returned object for another
  sampling.
}

\section{Passing data to Stan}{
  The data passed to \code{stan} are preprocessed before being passed to Stan.
  If \code{data} is not a character vector, the data block of the Stan program
  is parsed and \R objects of the same name are searched starting from the
  calling environment. Then, if \code{data} is list-like but not a \code{data.frame}
  the elements of \code{data} take precedence. This behavior is similar to how
  a \code{formula} is evaluated by the \code{\link{lm}} function when \code{data} is
  supplied. In general, each \R object being passed to Stan should be either a numeric
  vector (including the special case of a 'scalar') or a numeric array (matrix).
  The first exception is that an element can be a logical vector: \code{TRUE}'s
  are converted to 1 and \code{FALSE}'s to 0.
  An element can also be a data frame or a specially structured list (see
  details below), both of which will be converted into arrays in the
  preprocessing.  Using a specially structured list is not
  encouraged though it might be convenient sometimes; and when in doubt, just
  use arrays.

  This preprocessing for each element mainly includes the following:
  \enumerate{
    \item Change the data of type from \code{double}
      to \code{integer} if no accuracy is lost. The main
      reason is that by default, \R uses \code{double}
      as data type such as in \code{a <- 3}. But Stan
      will not read data of type \code{int} from \code{real}
      and it reads data from \code{int} if the data
      type is declared as \code{real}.

   \item Check if there is \code{NA} in the data.
     Unlike BUGS, Stan does not allow missing data. Any \code{NA} values
     in supplied data will cause the function to stop and report an error.

   \item Check data types.  Stan allows only numeric data, that is,
     doubles, integers, and arrays of these.  Data of other types (for
     example, characters and factors) are not passed to Stan.

   \item Check whether there are objects in the data list with duplicated names.
     Duplicated names, if found, will cause the function to stop and
     report an error.

   \item Check whether the names of objects in the data list are legal
     Stan names. If illegal names are found, it will stop and
     report an error. See (Cmd)Stan's manual for the rules of variable names.

   \item When an element is of type \code{data.frame}, it will be converted to
     \code{matrix} by function \code{data.matrix}.

   \item When an element is of type \code{list}, it is supposed to make it
     easier to pass data for those declared in Stan code such as
     \code{"vector[J] y1[I]"} and \code{"matrix[J,K] y2[I]"}. Using the latter
     as an example, we can use a list for \code{y2} if the list has "I" elements,
     each of which is an array (matrix) of dimension "J*K". However, it is
     not possible to pass a list for data declared such as
     \code{"vector[K] y3[I,J]"}; the only way for it is to use an array with
     dimension "I*J*K". In addition, technically a \code{data.frame} in R is
     also a list, but it should not be used for the purpose here since a
     \code{data.frame} will be converted to a matrix as described above.
  }

  Stan treats a vector of length 1 in R as a scalar.  So technically
  if, for example, \code{"array[1] real y;"} is defined in the data block, an array
  such as \code{"y = array(1.0, dim = 1)"} in R should be used. This
  is also the case for specifying initial values since the same
  underlying approach for reading data from R in Stan is used, in which
  vector of length 1 is treated as a scalar.

  In general, the higher the optimization level is set, the faster the
  generated binary code for the model runs, which can be set in a Makevars file.
  However, the binary code generated for the model runs fast by using a higher
  optimization level at the cost of longer times to compile the C++ code.
}

\references{
  The Stan Development Team
  \emph{Stan Modeling Language User's Guide and Reference Manual}.
  \url{https://mc-stan.org}.

  The Stan Development Team
  \emph{CmdStan Interface User's Guide}.
  \url{https://mc-stan.org}.
}
\seealso{
  \itemize{
  \item The package vignettes for an example of fitting a model and accessing
  the contents of \code{stanfit} objects (\url{https://mc-stan.org/rstan/articles/}).

  \item \code{\link{stanc}} for translating model code in Stan modeling language to C++,
  \code{\link{sampling}} for sampling, and \code{\linkS4class{stanfit}} for the
  fitted results.

  \item \code{\link{as.array.stanfit}} and \code{\link{extract}} for extracting
  samples from \code{stanfit} objects.
  }
}
\examples{\dontrun{
#### example 1
library(rstan)
scode <- "
parameters {
  array[2] real y;
}
model {
  y[1] ~ normal(0, 1);
  y[2] ~ double_exponential(0, 2);
}
"
fit1 <- stan(model_code = scode, iter = 10, verbose = FALSE)
print(fit1)
fit2 <- stan(fit = fit1, iter = 10000, verbose = FALSE)

## using as.array on the stanfit object to get samples
a2 <- as.array(fit2)

## extract samples as a list of arrays
e2 <- extract(fit2, permuted = FALSE)

#### example 2
#### the result of this package is included in the package

excode <- '
  transformed data {
    array[20] real y;
    y[1] = 0.5796;  y[2] = 0.2276;   y[3]  = -0.2959;
    y[4] = -0.3742; y[5] = 0.3885;   y[6]  = -2.1585;
    y[7] = 0.7111;  y[8] = 1.4424;   y[9]  = 2.5430;
    y[10] = 0.3746; y[11] = 0.4773;  y[12] = 0.1803;
    y[13] = 0.5215; y[14] = -1.6044; y[15] = -0.6703;
    y[16] = 0.9459; y[17] = -0.382;  y[18] = 0.7619;
    y[19] = 0.1006; y[20] = -1.7461;
  }
  parameters {
    real mu;
    real<lower=0, upper=10> sigma;
    vector[2] z[3];
    real<lower=0> alpha;
  }
  model {
    y ~ normal(mu, sigma);
    for (i in 1:3)
      z[i] ~ normal(0, 1);
    alpha ~ exponential(2);
  }
'

exfit <- stan(model_code = excode, save_dso = FALSE, iter = 500)
print(exfit)
plot(exfit)
}
\dontrun{
## examples of specify argument `init` for function stan

## define a function to generate initial values that can
## be fed to function stan's argument `init`
# function form 1 without arguments
initf1 <- function() {
  list(mu = 1, sigma = 4, z = array(rnorm(6), dim = c(3,2)), alpha = 1)
}
# function form 2 with an argument named `chain_id`
initf2 <- function(chain_id = 1) {
  # cat("chain_id =", chain_id, "\n")
  list(mu = 1, sigma = 4, z = array(rnorm(6), dim = c(3,2)), alpha = chain_id)
}

# generate a list of lists to specify initial values
n_chains <- 4
init_ll <- lapply(1:n_chains, function(id) initf2(chain_id = id))

exfit0 <- stan(model_code = excode, init = initf1)
stan(fit = exfit0, init = initf2)
stan(fit = exfit0, init = init_ll, chains = n_chains)
}}
\keyword{rstan}

\name{check_hmc_diagnostics}
\alias{check_hmc_diagnostics}
\alias{check_divergences}
\alias{check_treedepth}
\alias{check_energy}
\alias{get_divergent_iterations}
\alias{get_max_treedepth_iterations}
\alias{get_num_leapfrog_per_iteration}
\alias{get_num_divergent}
\alias{get_num_max_treedepth}
\alias{get_bfmi}
\alias{get_low_bfmi_chains}


\title{Check HMC diagnostics after sampling} 

\description{
These functions print summaries of important HMC diagnostics or extract
those diagnostics from a \code{stanfit} object. See the \strong{Details}
section, below.
}

\usage{
check_hmc_diagnostics(object)
check_divergences(object)
check_treedepth(object)
check_energy(object)

get_divergent_iterations(object)
get_max_treedepth_iterations(object)
get_num_leapfrog_per_iteration(object)

get_num_divergent(object)
get_num_max_treedepth(object)

get_bfmi(object)
get_low_bfmi_chains(object)
} 
 
\arguments{
  \item{object}{A stanfit object.}
} 

\details{
The \code{check_hmc_diagnostics} function calls the other \code{check_*} 
functions internally and prints an overall summary, but the other 
functions can also be called directly: 

\itemize{
\item \code{check_divergences} prints the number (and percentage) of 
iterations that ended with a divergence,
\item \code{check_treedepth} prints the number (and percentage) of iterations 
that saturated the max treedepth, 
\item \code{check_energy} prints E-BFMI values for each chain for which E-BFMI 
is less than 0.2. 
}

The \code{get_*} functions are for programmatic access to the diagnostics. 

\itemize{
\item \code{get_divergent_iterations} and \code{get_max_treedepth_iterations} 
return a logical vector indicating problems for individual iterations,
\item \code{get_num_divergent} and \code{get_num_max_treedepth} return the 
number of offending interations, 
\item \code{get_num_leapfrog_per_iteration} returns an integer vector with the 
number of leapfrog evalutions for each iteration,
\item \code{get_bfmi} returns per-chain E-BFMI values and \code{get_low_bfmi_chains}
returns the indices of chains with low E-BFMI.
}

The following are several of many resources that provide more information on 
these diagnostics:
\itemize{
\item Brief explanations of some of the problems detected by these diagnostics 
can be found in the \href{https://mc-stan.org/misc/warnings.html}{\emph{Brief Guide to Stan's Warnings}}.

\item Betancourt (2017) provides much more depth on these diagnostics
as well as a conceptual introduction to Hamiltonian Monte Carlo in general.

\item Gabry et al. (2018) and the \pkg{bayesplot}
 package \href{https://mc-stan.org/bayesplot/articles/}{vignettes} demonstrate
 various visualizations of these diagnostics that can be made in \R.
}
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org/}.
  
  Betancourt, M. (2017). A conceptual introduction to Hamiltonian Monte Carlo.
  \url{https://arxiv.org/abs/1701.02434}.
  
  Gabry, J., Simpson, D., Vehtari, A., Betancourt, M., and Gelman, A. (2018). 
  Visualization in Bayesian workflow. 
  \emph{Journal of the Royal Statistical Society Series A}, accepted for publication.
  arXiv preprint: https://arxiv.org/abs/1709.01449.
} 

\examples{
\dontrun{
schools <- stan_demo("eight_schools")
check_hmc_diagnostics(schools)
check_divergences(schools)
check_treedepth(schools)
check_energy(schools)
}
} 

\name{rstan.package.skeleton}
\alias{rstan.package.skeleton}
\title{Create a Skeleton for a New Source Package with Stan Programs}
\description{
  This function has been removed from \pkg{rstan}. Please use the new 
  \code{rstan_package_skeleton} function in the \pkg{rstantools} package.
}

\name{stan_version}
\alias{stan_version}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Obtain the version of Stan} 
\description{
  The stan version is in form of \code{major.minor.patch};  the
  first version is 1.0.0, indicating major version 1, minor version
  0 and patch level 0.  Functionality only changes with minor versions
  and backward compatibility will only be affected by major versions.
}
\usage{
  stan_version()
}
\value{
  A character string giving the version of Stan used in this version of RStan. 
}
\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org/}.
}
\seealso{
  \code{\link{stan}} and \code{\link{stan_model}} 
}
\examples{
  stan_version() 
}
\keyword{rstan} 

\name{read_stan_csv}
\alias{read_stan_csv}
\title{Read CSV files of samples generated by (R)Stan into a \code{stanfit} object}
\description{Create a \code{stanfit} object from the saved CSV files that are
  created by Stan or RStan and that include the samples drawn from the
  distribution of interest to facilitate analysis of samples using RStan.  
} 

\usage{
read_stan_csv(csvfiles, col_major = TRUE)
}

\arguments{
  \item{csvfiles}{A character vector providing CSV file names}
  \item{col_major}{The order for array parameters; default to \code{TRUE}}
}

\details{
  Stan and RStan could save the samples to CSV files.  This function
  reads the samples and using the comments (beginning with \code{"#"})
  to create a \code{stanfit} object. The model name is derived from 
  the first CSV file. 

  \code{col_major} specifies how array parameters are ordered in each row of
  the CSV files.  For example, parameter \code{"a[2,2]"} would be ordered as
  \code{"a[1,1], a[2,1], a[1,2], a[2,2]"} if col_major is \code{TRUE}. 
} 

\value{
  A \code{stanfit} object (with invalid \code{stanmodel} slot). This \code{stanfit}
  object cannot be used to re-run the sampler. 
} 


\seealso{
  \code{\linkS4class{stanfit}} 
} 
\examples{
csvfiles <- dir(system.file('misc', package = 'rstan'),
                pattern = 'rstan_doc_ex_[0-9].csv', full.names = TRUE)
fit <- read_stan_csv(csvfiles)
}

\docType{methods}
\name{plot-methods}
\alias{plot,stanfit-method}
\alias{plot,stanfit,missing-method}

\title{Plots for stanfit objects} 
\description{The default plot shows posterior uncertainty intervals and point 
estimates for parameters and generated quantities. The \code{plot} method can 
also be used to call the other \strong{rstan} plotting functions via the 
\code{plotfun} argument (see Examples).}

\usage{
  \S4method{plot}{stanfit,missing}(x, ..., plotfun)
} 

\arguments{
  \item{x}{An instance of class \code{\linkS4class{stanfit}}.}
  \item{plotfun}{A character string naming the plotting function to apply to the
  stanfit object. If \code{plotfun} is missing, the default is to call
  \code{\link{stan_plot}}, which generates a plot of credible intervals and
  point estimates. See \code{\link{rstan-plotting-functions}} for the names and
  descriptions of the other plotting functions. \code{plotfun} can be either the
  full name of the plotting function (e.g. \code{"stan_hist"}) or can be 
  abbreviated to the part of the name following the underscore 
  (e.g. \code{"hist"}).}
  \item{...}{Optional arguments to \code{plotfun}.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object that can be further customized
using the \pkg{ggplot2} package.
}
\note{
Because the \pkg{rstan} plotting functions use \pkg{ggplot2} (and thus the 
resulting plots behave like \code{ggplot} objects), when calling a plotting 
function within a loop or when assigning a plot to a name 
(e.g., \code{graph <- plot(fit, plotfun = "rhat")}),
if you also want the side effect of the plot being displayed you 
must explicity print it (e.g., \code{(graph <- plot(fit, plotfun = "rhat"))}, 
\code{print(graph <- plot(fit, plotfun = "rhat"))}).
}
\seealso{
\code{\link[=rstan-plotting-functions]{List of RStan plotting functions}},
\code{\link[=rstan_gg_options]{Plot options}}
}
\examples{
\dontrun{
library(rstan)
fit <- stan_demo("eight_schools")
plot(fit)
plot(fit, show_density = TRUE, ci_level = 0.5, fill_color = "purple")
plot(fit, plotfun = "hist", pars = "theta", include = FALSE)
plot(fit, plotfun = "trace", pars = c("mu", "tau"), inc_warmup = TRUE)
plot(fit, plotfun = "rhat") + ggtitle("Example of adding title to plot")
}
} 
\keyword{methods}

\name{loo_moment_match.stanfit}
\alias{loo_moment_match.stanfit}
\alias{loo_moment_match,stanfit-method}
\alias{loo_moment_match}

\title{Moment matching for efficient approximate leave-one-out cross-validation (LOO)}

\description{
A \code{\link[loo]{loo_moment_match}} method that
is customized for stanfit objects.
The \code{loo_moment_match} method for stanfit objects ---a wrapper around the
\code{\link[loo]{loo_moment_match}} (\pkg{loo} package)--- updates
a loo object using moment matching (Paananen et al., 2020).
}


\usage{
\method{loo_moment_match}{stanfit}(x,
    loo = loo,
    \dots)
}
\arguments{
\item{x}{An object of S4 class \code{stanfit}.}

\item{loo}{A loo object that is modified.}

\item{\dots}{Further arguments.}
}

\value{
The \code{loo_moment_match()} methods return an updated \code{loo} object.
}



\references{
Paananen, T., Piironen, J., Buerkner, P.-C., Vehtari, A. (2020).
Implicitly Adaptive Importance Sampling.
\href{https://arxiv.org/abs/1906.08850}{preprint arXiv:1906.08850}
}
\seealso{
\code{\link[=loo]{loo()}}, \code{\link[=loo]{loo_moment_match()}}
}

\docType{methods}
\name{extract}
\alias{extract}
\alias{extract,stanfit-method}

\title{Extract samples from a fitted Stan model}
\description{
  Extract samples from a fitted model represented by an 
  instance of class \code{\linkS4class{stanfit}}.
}
\usage{
  \S4method{extract}{stanfit}(object, pars, permuted = TRUE, inc_warmup = FALSE, 
  include = TRUE) 
} 

\section{Methods}{
  \describe{
    \item{extract}{\code{signature(object = "stanfit")}
    Extract samples from a fitted model represented by an instance of class 
    \code{\linkS4class{stanfit}}.  
    }
  }
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{stanfit}}.}
  \item{pars}{An optional character vector providing the parameter
    names (or other quantity names) of interest. If not specified,
    all parameters and other quantities are used. The log-posterior with
    name \code{lp__} is also included by default.} 
  \item{permuted}{A logical scalar indicating whether the draws
    after the \emph{warmup} period in each chain should be \emph{permuted} and 
    \emph{merged}. If \code{FALSE}, the original order is kept. For each 
    \code{stanfit} object, the permutation is fixed (i.e., extracting samples 
    a second time will give the same sequence of iterations).}
  \item{inc_warmup}{A logical scalar indicating whether to include
    the warmup draws. This argument is only relevant if \code{permuted} is 
    \code{FALSE}.}
  \item{include}{A logical scalar indicating whether the parameters
    named in \code{pars} should be included (\code{TRUE}) or excluded 
    (\code{FALSE}).}
}
\value{
  When \code{permuted = TRUE}, this function returns a named list,  
  every element of which is an array representing samples for a parameter
  with all chains merged together.
  
  When \code{permuted = FALSE}, an array is returned; the first
  dimension is for the iterations, the second for the number of chains, the 
  third for the parameters. Vectors and arrays are expanded to one 
  parameter (a scalar) per cell, with names indicating the third dimension.
  See the examples (with comments) below. The \code{\link{monitor}} function
  can be applied to the returned array to obtain a summary 
  (similar to the \code{print} method for \code{\linkS4class{stanfit}} objects).
}

\examples{\dontrun{
ex_model_code <- '
  parameters {
    array[2, 3] real alpha;
    array[2] real beta; 
  } 
  model {
    for (i in 1:2) for (j in 1:3) 
      alpha[i, j] ~ normal(0, 1); 
    for (i in 1:2) 
      beta ~ normal(0, 2); 
  } 
'

## fit the model 
fit <- stan(model_code = ex_model_code, chains = 4) 

## extract alpha and beta with 'permuted = TRUE' 
fit_ss <- extract(fit, permuted = TRUE) # fit_ss is a list 
## list fit_ss should have elements with name 'alpha', 'beta', 'lp__'
alpha <- fit_ss$alpha  
beta <- fit_ss$beta 
## or extract alpha by just specifying pars = 'alpha' 
alpha2 <- extract(fit, pars = 'alpha', permuted = TRUE)$alpha 
print(identical(alpha, alpha2)) 

## or extract alpha by excluding beta and lp__
alpha3 <- extract(fit, pars = c('beta', 'lp__'), 
                  permuted = TRUE, include = FALSE)$alpha
print(identical(alpha, alpha3))

## get the samples for alpha[1,1] and beta[2] 
alpha_11 <- alpha[, 1, 1] 
beta_2 <- beta[, 2] 

## extract samples with 'permuted = FALSE' 
fit_ss2 <- extract(fit, permuted = FALSE) # fit_ss2 is an array  

## the dimensions of fit_ss2 should be  
## "# of iterations * # of chains * # of parameters"
dim(fit_ss2) 

## since the third dimension of `fit_ss2` indicates 
## parameters, the names should be 
##  alpha[1,1], alpha[2,1], alpha[1,2], alpha[2,2], 
##  alpha[1,3], alpha[2,3], beta[1], beta[2], and lp__ 
## `lp__` (the log-posterior) is always included 
## in the samples.  
dimnames(fit_ss2) 
}

# Create a stanfit object from reading CSV files of samples (saved in rstan
# package) generated by funtion stan for demonstration purpose from model as follows. 
# 
excode <- '
  transformed data {
    array[20] real y;
    y[1] <- 0.5796;  y[2]  <- 0.2276;   y[3] <- -0.2959; 
    y[4] <- -0.3742; y[5]  <- 0.3885;   y[6] <- -2.1585;
    y[7] <- 0.7111;  y[8]  <- 1.4424;   y[9] <- 2.5430; 
    y[10] <- 0.3746; y[11] <- 0.4773;   y[12] <- 0.1803; 
    y[13] <- 0.5215; y[14] <- -1.6044;  y[15] <- -0.6703; 
    y[16] <- 0.9459; y[17] <- -0.382;   y[18] <- 0.7619;
    y[19] <- 0.1006; y[20] <- -1.7461;
  }
  parameters {
    real mu;
    real<lower=0, upper=10> sigma;
    vector[2] z[3];
    real<lower=0> alpha;
  } 
  model {
    y ~ normal(mu, sigma);
    for (i in 1:3) 
      z[i] ~ normal(0, 1);
    alpha ~ exponential(2);
  } 
'
# exfit <- stan(model_code = excode, save_dso = FALSE, iter = 200, 
#               sample_file = "rstan_doc_ex.csv")
# 
exfit <- read_stan_csv(dir(system.file('misc', package = 'rstan'),
                       pattern='rstan_doc_ex_[[:digit:]].csv',
                       full.names = TRUE))

ee1 <- extract(exfit, permuted = TRUE)
print(names(ee1))

for (name in names(ee1)) {
  cat(name, "\n")
  print(dim(ee1[[name]]))
}

ee2 <- extract(exfit, permuted = FALSE)
print(dim(ee2))
print(dimnames(ee2))
} 

\seealso{
  S4 class \code{\linkS4class{stanfit}}, \code{\link{as.array.stanfit}}, and
  \code{\link{monitor}}
} 

\keyword{methods}

\name{monitor}
\alias{monitor}
\alias{print.simsummary}
\alias{[.simsummary}
\title{Compute summaries of MCMC draws and monitor convergence} 
\description{
  Similar to the \code{print} method for \code{stanfit} objects, but \code{monitor}
  takes an array of simulations as its argument rather than a \code{stanfit}
  object. For a 3-D array (iterations * chains * parameters) of MCMC draws, 
  \code{monitor} computes means, standard deviations, quantiles, Monte Carlo 
  standard errors, split Rhats, and effective sample sizes. By default, half of 
  the iterations are considered warmup and are excluded.
}

\usage{
monitor(sims, warmup = floor(dim(sims)[1]/2), 
        probs = c(0.025, 0.25, 0.5, 0.75, 0.975), 
        digits_summary = 1, print = TRUE, ...)
\method{print}{simsummary}(x, digits = 3, se = FALSE, ...)
\method{[}{simsummary}(x, i, j, drop = if (missing(i)) TRUE else length(j) == 1)
        
}
\arguments{
  \item{sims}{A 3-D array (iterations * chains * parameters) of MCMC 
    simulations from any MCMC algorithm.}
  \item{warmup}{The number of warmup iterations to be excluded 
    when computing the summaries. The default is half of the total number 
    of iterations. If \code{sims} doesn't include the warmup iterations then 
    \code{warmup} should be set to zero.}
  \item{probs}{A numeric vector specifying quantiles of interest. The 
    defaults is \code{c(0.025,0.25,0.5,0.75,0.975)}.}
  \item{digits_summary}{The number of significant digits to use when printing
    the summary, defaulting to 1. Applies to the quantities other than 
    the effective sample size, which is always rounded to the nearest integer.}
  \item{print}{Logical, indicating whether to print the summary after the 
    computations are performed.} 
  \item{\dots}{Additional arguments passed to the underlying \code{print} method.}
  \item{x}{An object of class \code{simsummary} created by \code{monitor}}
  \item{digits}{An integer scalar defaulting to 3 for the number of decimal
    places to print}
  \item{se}{A logical scalar defaulting to \code{FALSE} indicating whether
    to print the estimated standard errors of the estimates}
  \item{i}{A vector indicating which rows of the object created by \code{monitor}
    to select}
  \item{j}{A vector indicating which columns of the object crated by \code{monitor}
    to select}
  \item{drop}{A logical scalar indicating whether the resulting object 
    should return a vector where possible}
}

\value{
  A 2-D array with rows corresponding to parameters and columns to the 
  summary statistics that can be printed and subset.
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org}.
}

\seealso{
  S4 class \code{\linkS4class{stanfit}} and particularly its
  \code{\link[=print.stanfit]{print}} method.
}

\examples{
csvfiles <- dir(system.file('misc', package = 'rstan'),
                pattern = 'rstan_doc_ex_[0-9].csv', full.names = TRUE)
fit <- read_stan_csv(csvfiles)
# The following is just for the purpose of giving an example
# since print can be used for a stanfit object.
monitor(extract(fit, permuted = FALSE, inc_warmup = TRUE))
}

\name{stan_demo}
\alias{stan_demo} 

\title{Demonstrate examples included in Stan} 

\description{Stan includes a variety of examples and most of the BUGS example models 
  that are translated into Stan modeling language. One example is chosen from a list
  created from matching user input and gets fitted in the demonstration.} 
 
\usage{
  stan_demo(model = character(0), 
            method = c("sampling", "optimizing", "meanfield", "fullrank"), ...)
} 
 
\arguments{
  \item{model}{A character string for model name to specify which model
    will be used for demonstration.  The default is an empty string, which
    prompts the user to select one the available models. If \code{model = 0},
    a character vector of all models is returned without any user intervention.
    If \code{model = i} where \code{i > 0}, then the ith available model is
    chosen without user intervention, which is useful for testing.}
  \item{method}{Whether to call \code{\link{sampling}} (the default),
    \code{\link{optimizing}}, or one of the variants of \code{\link{vb}}
    for the demonstration}
  \item{\dots}{Further arguments passed to \code{method}.}
} 


\value{
  An S4 object of \code{stanfit}, unless \code{model = 0}, in which case a 
  character vector of paths to available models is returned.
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org/}.
} 

\seealso{
  \code{\link{sampling}}, \code{\link{optimizing}}
} 

\examples{
  \dontrun{
     dogsfit <- stan_demo("dogs") # run the dogs model
     fit1 <- stan_demo(1) # run model_names[1]
  }
} 

\name{Rhat}
\alias{Rhat}
\alias{ess_bulk}
\alias{ess_tail}
\title{
Convergence and efficiency diagnostics for Markov Chains
}
\description{
These functions are improved versions of the traditional
Rhat (for convergence) and Effective Sample Size (for
efficiency).
}
\usage{
Rhat(sims)
ess_bulk(sims)
ess_tail(sims)
}
\arguments{
  \item{sims}{
  A two-dimensional array whose rows are equal to the
  number of iterations of the Markov Chain(s) and whose
  columns are equal to the number of Markov Chains
  (preferably more than one). The cells are the realized
  draws for a particular parameter or function of parameters.
}
}
\value{
The \code{Rhat} function produces R-hat convergence diagnostic, which
compares the between- and within-chain estimates for model parameters
and other univariate quantities of interest. If chains have not mixed
well (ie, the between- and within-chain estimates don't agree), R-hat is
larger than 1. We recommend running at least four chains by default and
only using the sample if R-hat is less than 1.05. Stan reports R-hat
which is the maximum of rank normalized split-R-hat and rank normalized
folded-split-R-hat, which works for thick tailed distributions and is
sensitive also to differences in scale.

The \code{ess_bulk} function produces an estimated Bulk Effective Sample
Size (bulk-ESS) using rank normalized draws. Bulk-ESS is useful measure
for sampling efficiency in the bulk of the distribution (related e.g. to
efficiency of mean and median estimates), and is well defined even if
the chains do not have finite mean or variance.

The \code{ess_tail} function produces an estimated Tail Effective Sample
Size (tail-ESS) by computing the minimum of effective sample sizes for
5\% and 95\% quantiles. Tail-ESS is useful measure for sampling
efficiency in the tails of the distribution (related e.g. to efficiency
of variance and tail quantile estimates).

Both bulk-ESS and tail-ESS should be at least \eqn{100} (approximately)
per Markov Chain in order to be reliable and indicate that estimates of
respective posterior quantiles are reliable.
}
\references{
Aki Vehtari, Andrew Gelman, Daniel Simpson, Bob Carpenter, and
Paul-Christian Burkner (2019). Rank-normalization, folding, and
localization: An improved R-hat for assessing convergence of MCMC.
\emph{arXiv preprint} \code{arXiv:1903.08008}.
}
\author{
Paul-Christian Burkner and Aki Vehtari
}
\seealso{
\code{\link{monitor}}
}
\examples{
# pretend these draws came from five actual Markov Chins
sims <- matrix(rnorm(500), nrow = 100, ncol = 5)
Rhat(sims)
ess_bulk(sims)
ess_tail(sims)
}

\name{sflist2stanfit} 
\alias{sflist2stanfit} 

\title{Merge a list of stanfit objects into one} 

\description{This function takes a list of \code{stanfit} objects and returns a
  consolidated \code{stanfit} object. The \code{stanfit} objects to be merged
  need to have the same configuration of iteration, warmup, and thin, besides
  being from the same model.  This could facilitate some parallel usage of RStan.
  For example, if we call \code{\link{stan}} by parallel and it returns a list of
  \code{stanfit} objects, this function can be used to create one \code{stanfit}
  object from the list. 
} 
 
\usage{
  sflist2stanfit(sflist) 
} 
 
\arguments{
  \item{sflist}{A list of \code{stanfit} objects.}
} 


\value{
  An S4 object of \code{stanfit} consolidated from all the input \code{stanfit}
  objects. 
}

\note{
  This function should be called in rare circumstances because 
  \code{\link{sampling}} has a \code{cores} argument that allows multiple
  chains to be executed in parallel. However, if you need to depart from that,
  the best practice is to use \code{sflist2stanfit} on a list of \code{stanfit}
  objects created with the same \code{seed} but different \code{chain_id} (see
  example below). Using the same seed but different chain_id can make sure 
  the random number generations for all chains are not correlated. 

  This function would do some check to see if the \code{stanfit} objects in the input list 
  can be merged. But the check is not sufficient. So generally, it is the
  user's responsibility to make sure the input is correct so that the merging
  makes sense. 

  The date in the new \code{stanfit} object is when it is merged. 

  \code{get_seed} function for the new consolidated \code{stanfit} object only returns
  the seed used in the first chain of the new object. 

  The sampler such as NUTS2 that is displayed in the printout by \code{print} 
  is the sampler used for the first chain. The \code{print} method assumes the samplers
  are the same for all chains. 

  The included \code{stanmodel} object, which includes the compiled model, 
  in the new \code{stanfit} object is from the first element of the input list. 

} 

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org/}.
} 

\seealso{
  \code{\link{stan}} 
} 

\examples{\dontrun{
library(rstan)
scode <- "
data {
  int<lower=1> N;
} 
parameters {
  array[N] real y1;
  array[N] real y2; 
} 
model {
  y1 ~ normal(0, 1);
  y2 ~ double_exponential(0, 2);
} 
"
seed <- 123 # or any other integer 
foo_data <- list(N = 2)
foo <- stan(model_code = scode, data = foo_data, chains = 1, iter = 1)
f1 <- stan(fit = foo, data = foo_data, chains = 1, seed = seed, chain_id = 1) 
f2 <- stan(fit = foo, data = foo_data, chains = 2, seed = seed, chain_id = 2:3) 
f12 <- sflist2stanfit(list(f1, f2)) 

## parallel stan call for unix-like OS
library(parallel)

if (.Platform$OS.type == "unix") {
sflist1 <- 
  mclapply(1:4, mc.cores = 2, 
           function(i) stan(fit = foo, data = foo_data, seed = seed, 
	                    chains = 1, chain_id = i, refresh = -1))
f3 <- sflist2stanfit(sflist1)
}
if (.Platform$OS.type == "windows") { # also works on non-Windows
CL <- makeCluster(2)
clusterExport(cl = CL, c("foo_data", "foo", "seed")) 
sflist1 <- parLapply(CL, 1:4, fun = function(cid) {  
  require(rstan)
  stan(fit = foo, data = foo_data, chains = 1, 
       iter = 2000, seed = seed, chain_id = cid)
})

fit <- sflist2stanfit(sflist1)
print(fit)
stopCluster(CL)
} # end example for Windows 
}} 

\name{loo.stanfit}
\alias{loo.stanfit}
\alias{loo,stanfit-method}
\alias{loo}

\title{Approximate leave-one-out cross-validation}

\description{
A \code{\link[loo]{loo}} method that is customized for stanfit objects.
The \code{loo} method for stanfit objects ---a wrapper around the \code{array}
method for \code{\link[loo]{loo}} in the  \pkg{loo} package --- computes PSIS-LOO CV,
approximate leave-one-out cross-validation using Pareto smoothed importance
sampling (Vehtari, Gelman, and Gabry, 2017a,2017b).
}

\usage{
  \method{loo}{stanfit}(x,
    pars = "log_lik",
    save_psis = FALSE,
    cores = getOption("mc.cores", 1),
    moment_match = FALSE,
    k_threshold = 0.7,
    \dots)
}

\arguments{
  \item{x}{An object of S4 class \code{stanfit}.}
  \item{pars}{Name of transformed parameter or generated quantity in
  the Stan program corresponding to the pointwise log-likelihood. If not
  specified the default behavior is to look for \code{"log_lik"}.}
  \item{save_psis}{Should the intermediate results from \code{\link[loo]{psis}}
  be saved in the returned object? The default is \code{FALSE}. This can be
  useful to avoid repeated computation when using other functions in the
  \pkg{loo} and \pkg{bayesplot} packages.}
  \item{cores}{Number of cores to use for parallelization. The default is 1 unless
  \code{cores} is specified or the \code{mc.cores} \link[=options]{option}
  has been set.}
\item{moment_match}{Logical; Whether to use the moment matching algorithm for
  observations with high Pareto k values to improve accuracy. Note:
  because the moment matching algorithm relies on the \code{unconstrain_pars}
  method in RStan it is only available if run in the same R session as fitting the
  model.}
  \item{k_threshold}{Threshold value for Pareto k values above which
  the moment matching algorithm is used. If \code{moment_match} is \code{FALSE},
  this is ignored.}
  \item{\dots}{Ignored.}
}

\details{
  Stan does not automatically compute and store the log-likelihood. It
  is up to the user to incorporate it into the Stan program if it is to be
  extracted after fitting the model. In a Stan program, the pointwise log
  likelihood can be coded as a vector in the transformed parameters block
  (and then summed up in the model block) or it can be coded entirely in the
  generated quantities block. We recommend using the generated quantities
  block so that the computations are carried out only once per iteration
  rather than once per HMC leapfrog step.

  For example, the following is the \code{generated quantities} block for
  computing and saving the log-likelihood for a linear regression model with
  \code{N} data points, outcome \code{y}, predictor matrix \code{X} (including
  column of 1s for intercept), coefficients \code{beta},
  and standard deviation \code{sigma}:

 \code{vector[N] log_lik;}

 \code{for (n in 1:N) log_lik[n] = normal_lpdf(y[n] | X[n, ] * beta, sigma);}

  This function automatically uses Pareto k diagnostics for assessing
  the accuracy of importance sampling for each observation. When the
  diagnostics indicate that importance sampling for certain observations
  is inaccurate, a moment matching algorithm can be used, which can
  improve the accuracy (Paananen et al., 2020).
}

\value{
  A list with class \code{c("psis_loo", "loo")}, as detailed in the
  \code{\link[loo]{loo}} documentation.
}

\references{
Vehtari, A., Gelman, A., and Gabry, J. (2017a).
Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC.
\emph{Statistics and Computing}. 27(5), 1413-1432.
\code{doi:10.1007/s11222-016-9696-4}.
\url{https://arxiv.org/abs/1507.04544},
\url{https://link.springer.com/article/10.1007/s11222-016-9696-4}

Vehtari, A., Gelman, A., and Gabry, J. (2017b).
Pareto smoothed importance sampling. arXiv preprint:
\url{https://arxiv.org/abs/1507.02646}

Yao, Y., Vehtari, A., Simpson, D., and Gelman, A. (2018).
Using stacking to average Bayesian predictive distributions.
Bayesian Analysis, advance publication, \code{doi:10.1214/17-BA1091}.

Paananen, T., Piironen, J., Buerkner, P.-C., Vehtari, A. (2020).
Implicitly Adaptive Importance Sampling.
arXiv preprint:
\url{https://arxiv.org/abs/1906.08850}.
}

\seealso{
 \itemize{
 \item The \pkg{\link[loo]{loo}} package documentation, including the vignettes for
 many examples (\url{https://mc-stan.org/loo/}).
 \item \code{\link[loo]{loo_moment_match}} for the moment matching algorithm.
 \item \code{\link[loo]{loo_model_weights}} for model averaging/weighting via
 stacking or pseudo-BMA weighting.
 }
}

\examples{
\dontrun{
# Generate a dataset from N(0,1)
N <- 100
y <- rnorm(N, 0, 1)

# Suppose we have three models for y:
#  1) y ~ N(-1, sigma)
#  2) y ~ N(0.5, sigma)
#  3) y ~ N(0.6,sigma)
#
stan_code <- "
data {
  int N;
  vector[N] y;
  real mu_fixed;
}
  parameters {
  real<lower=0> sigma;
}
model {
  sigma ~ exponential(1);
  y ~ normal(mu_fixed, sigma);
}
generated quantities {
  vector[N] log_lik;
  for (n in 1:N) log_lik[n] = normal_lpdf(y[n]| mu_fixed, sigma);
}"

mod <- stan_model(model_code = stan_code)
fit1 <- sampling(mod, data=list(N=N, y=y, mu_fixed=-1))
fit2 <- sampling(mod, data=list(N=N, y=y, mu_fixed=0.5))
fit3 <- sampling(mod, data=list(N=N, y=y, mu_fixed=0.6))

# use the loo method for stanfit objects
loo1 <- loo(fit1, pars = "log_lik")
print(loo1)

# which is equivalent to
LL <- as.array(fit1, pars = "log_lik")
r_eff <- loo::relative_eff(exp(LL))
loo1b <- loo::loo.array(LL, r_eff = r_eff)
print(loo1b)

# compute loo for the other models
loo2 <- loo(fit2)
loo3 <- loo(fit3)

# stacking weights
wts <- loo::loo_model_weights(list(loo1, loo2, loo3), method = "stacking")
print(wts)

# use the moment matching for loo with a stanfit object
loo_mm <- loo(fit1, pars = "log_lik", moment_match = TRUE)
print(loo_mm)
}
}

\name{nlist}
\alias{nlist}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Created named lists} 
\description{
Create a named list using specified names or, if names are omitted, using the 
names of the objects in the list. The code ]\code{list(a = a, b = b)} becomes 
\code{nlist(a,b)} and \code{list(a = a, b = 2)} becomes \code{nlist(a, b = 2)}, 
etc. This is convenient when creating the list of data to pass to Stan. 
}
\usage{
  nlist(...)
}
\arguments{
  \item{...}{The objects to include in the list.}
}
\value{
  A named list.
}
\examples{
# All variables already defined
x <- 1
y <- 2
nlist(x, y)

# Define some variables in the call and take the rest from the environment
nlist(x, y, z = 3)
}
\keyword{rstan} 

\name{stan_rdump}
\alias{stan_rdump}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Dump the data for a Stan model to \R dump file
in the limited format that Stan can read.
%%  ~~function to do ... ~~
}


\description{
  This function takes a vector of names of \R objects and outputs
  text representations of the objects to a file or connection.
  The file created by \code{stan_rdump} is typically used as data 
  input of the Stan package (\url{https://mc-stan.org/}) or
  \code{\link{source}}d into another \R session. The usage of 
  this function is very similar to \code{dump} in \R. 
}

\usage{
stan_rdump(list, file = "", append = FALSE, 
          envir = parent.frame(),
          width = options("width")$width, 
          quiet = FALSE)
}

\arguments{
  \item{list}{A vector of character string: the names of one or more \R objects
    to be dumped. See the note below.}

  \item{file}{Either a character string naming a file or a
    \link{connection}.  \code{""} indicates output to the console.}

  \item{append}{Logical: if \code{TRUE} and \code{file} is a character string,
    output will be appended to \code{file}; otherwise, it will overwrite
    the contents of \code{file}.}

  \item{envir}{The environment to search for objects.}
  \item{width}{The width for maximum characters on a line.  
    The output is broken into lines with \code{width.}}
  \item{quiet}{Whether to suppress warning messages that would appear when 
    a variable is not found or not supported for dumping (not being numeric
    or it would not be converted to numeric) or a variable name is not allowed
    in Stan.} 
}

\value{
  An invisible character vector containing the names of the objects
  that were dumped.
}
\note{
  \code{stan_rdump} only dumps numeric data, which first can be 
  a scalar, vector, matrix, or (multidimensional) array.  Additional types
  supported are \code{logical} (\code{TRUE} and \code{FALSE}), \code{factor},
  \code{data.frame} and a specially structured \code{list}.  

  The conversion for logical variables is to map \code{TRUE} to 1
  and \code{FALSE} to 0.  For \code{factor} variable, function 
  \code{as.integer} is used to do the conversion (If we want to transform a
  factor \code{f} to approximately its original numeric values, see the help of
  function \code{factor} and do the transformation before calling
  \code{stan_rdump}). 
  In the case of \code{data.frame}, function 
  \code{data.matrix} is applied to the data frame before
  dumping. See the notes in \link{stan} for the specially
  structured \code{list}, which will be converted to 
  \code{array} before dumping. 

  \code{stan_rdump} will check whether the names of objects
  are legal variable names in Stan. If an illegal name is 
  found, data will be dumped with a warning. However, passing the 
  name checking does not necessarily mean that the name is 
  legal. More details regarding rules of variable names in Stan can
  be found in Stan's manual. 
  
  If objects with specified names are not found, a warning will
  be issued. 
}

\references{
  The Stan Development Team 
  \emph{Stan Modeling Language User's Guide and Reference Manual}. 
  \url{https://mc-stan.org}.
}

\seealso{
  \code{\link{dump}} 
}
\examples{
# set variables in global environment
a <- 17.5
b <- c(1,2,3)
# write variables a and b to file ab.data.R in temporary directory
stan_rdump(c('a','b'), file.path(tempdir(), "ab.data.R"))
unlink(file.path(tempdir(), "ab.data.R"))

x <- 1; y <- 1:10; z <- array(1:10, dim = c(2,5)) 
stan_rdump(ls(pattern = '^[xyz]'), file.path(tempdir(), "xyz.Rdump"))
cat(paste(readLines(file.path(tempdir(), "xyz.Rdump")), collapse = '\n'), '\n')
unlink(file.path(tempdir(), "xyz.Rdump"))
}
\keyword{rstan} 
